@@ -4,10 +4,10 @@
 
 /* exported gCalendarGeneralPane */
 
-/* import-globals-from ../../../lightning/content/messenger-overlay-preferences.js */
-/* globals labelLong, labelShort */ // From general.xul.
+/* import-globals-from ../calendar-ui-utils.js */
+/* globals Preferences */
 
-var { cal } = ChromeUtils.import("resource://calendar/modules/calUtils.jsm");
+var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 
 Preferences.addAll([
   { id: "calendar.date.format", type: "int" },
@@ -20,8 +20,9 @@ Preferences.addAll([
   { id: "calendar.task.defaultdueoffset", type: "int" },
   { id: "calendar.task.defaultdueoffsetunits", type: "string" },
   { id: "calendar.view.useSystemColors", type: "bool" },
-  { id: "calendar.agendaListbox.soondays", type: "int" },
+  { id: "calendar.agenda.days", type: "int" },
   { id: "calendar.item.editInTab", type: "bool" },
+  { id: "calendar.item.promptDelete", type: "bool" },
 ]);
 
 /**
@@ -32,37 +33,36 @@ var gCalendarGeneralPane = {
    * Initialize the general pref pane. Sets up dialog controls to match the
    * values set in prefs.
    */
-  init: function() {
-    let formatter = Cc["@mozilla.org/calendar/datetime-formatter;1"].getService(
-      Ci.calIDateTimeFormatter
-    );
-
+  init() {
+    let formatter = cal.dtz.formatter;
     let dateFormattedLong = formatter.formatDateLong(cal.dtz.now());
     let dateFormattedShort = formatter.formatDateShort(cal.dtz.now());
 
     // menu items include examples of current date formats.
-    document
-      .getElementById("dateformat-long-menuitem")
-      .setAttribute("label", labelLong + ": " + dateFormattedLong);
-    document
-      .getElementById("dateformat-short-menuitem")
-      .setAttribute("label", labelShort + ": " + dateFormattedShort);
+    document.l10n.setAttributes(
+      document.getElementById("dateformat-long-menuitem"),
+      "dateformat-long",
+      { date: dateFormattedLong }
+    );
+
+    document.l10n.setAttributes(
+      document.getElementById("dateformat-short-menuitem"),
+      "dateformat-short",
+      { date: dateFormattedShort }
+    );
 
     // deselect and reselect to update visible item title
-    updateSelectedLabel("dateformat");
     updateUnitLabelPlural("defaultlength", "defaultlengthunit", "minutes");
     this.updateDefaultTodoDates();
 
     let tzMenuList = document.getElementById("calendar-timezone-menulist");
     let tzMenuPopup = document.getElementById("calendar-timezone-menupopup");
 
-    let tzService = cal.getTimezoneService();
-    let enumerator = tzService.timezoneIds;
     let tzids = {};
     let displayNames = [];
     // don't rely on what order the timezone-service gives you
-    while (enumerator.hasMore()) {
-      let timezone = tzService.getTimezone(enumerator.getNext());
+    for (let timezoneId of cal.timezoneService.timezoneIds) {
+      let timezone = cal.timezoneService.getTimezone(timezoneId);
       if (timezone && !timezone.isFloating && !timezone.isUTC) {
         let displayName = timezone.displayName;
         displayNames.push(displayName);
@@ -81,11 +81,11 @@ var gCalendarGeneralPane = {
     }
     tzMenuList.value = prefValue;
 
-    // Set the soondays menulist preference
+    // Set the agenda length menulist.
     this.initializeTodaypaneMenu();
   },
 
-  updateDefaultTodoDates: function() {
+  updateDefaultTodoDates() {
     let defaultDue = document.getElementById("default_task_due").value;
     let defaultStart = document.getElementById("default_task_start").value;
     let offsetValues = ["offsetcurrent", "offsetnexthour"];
@@ -105,10 +105,10 @@ var gCalendarGeneralPane = {
     updateMenuLabelsPlural("default_task_due_offset_text", "default_task_due_offset_units");
   },
 
-  initializeTodaypaneMenu: function() {
+  initializeTodaypaneMenu() {
     // Assign the labels for the menuitem
-    let soondaysMenu = document.getElementById("soondays-menulist");
-    let items = soondaysMenu.getElementsByTagName("menuitem");
+    let menulist = document.getElementById("agenda-days-menulist");
+    let items = menulist.getElementsByTagName("menuitem");
     for (let menuItem of items) {
       let menuitemValue = Number(menuItem.value);
       if (menuitemValue > 7) {
@@ -118,19 +118,18 @@ var gCalendarGeneralPane = {
       }
     }
 
-    let pref = Preferences.get("calendar.agendaListbox.soondays");
-    let soonpref = pref.value || 5;
+    let pref = Preferences.get("calendar.agenda.days");
+    let value = pref.value;
 
-    // Check if soonDays preference has been edited with a wrong value.
-    if (soonpref > 0 && soonpref <= 28) {
-      if (soonpref % 7 != 0) {
-        let intSoonpref = Math.floor(soonpref / 7) * 7;
-        soonpref = intSoonpref == 0 ? soonpref : intSoonpref;
-        pref.value = soonpref;
+    // Check if the preference has been edited with a wrong value.
+    if (value > 0 && value <= 28) {
+      if (value % 7 != 0) {
+        let intValue = Math.floor(value / 7) * 7;
+        value = intValue == 0 ? value : intValue;
+        pref.value = value;
       }
     } else {
-      soonpref = soonpref > 28 ? 28 : 1;
-      pref.value = soonpref;
+      pref.value = 14;
     }
   },
 };
