@@ -442,7 +442,8 @@ calItemBase.prototype = {
     }
   },
 
-  // readonly attribute nsIJSEnumerator properties;
+  // Each inner array has two elements: a string and a nsIVariant.
+  // readonly attribute Array<Array<jsval> > properties;
   get properties() {
     let properties = this.mProperties;
     if (this.mIsProxy) {
@@ -456,7 +457,7 @@ calItemBase.prototype = {
       );
     }
 
-    return properties.entries();
+    return [...properties.entries()];
   },
 
   // nsIVariant getProperty(in AString name);
@@ -470,7 +471,7 @@ calItemBase.prototype = {
 
   // boolean hasProperty(in AString name);
   hasProperty(aName) {
-    return this.getProperty(aName.toUpperCase()) != null;
+    return this.getProperty(aName) != null;
   },
 
   // void setProperty(in AString name, in nsIVariant value);
@@ -510,20 +511,21 @@ calItemBase.prototype = {
   getPropertyParameter(aPropName, aParamName) {
     let propName = aPropName.toUpperCase();
     let paramName = aParamName.toUpperCase();
-    if (propName in this.mPropertyParams && paramName in this.mPropertyParams[propName]) {
-      // If the property is not in mPropertyParams, then this just means
-      // there are no properties set.
-      return this.mPropertyParams[propName][paramName];
+    if (propName in this.mPropertyParams) {
+      if (paramName in this.mPropertyParams[propName]) {
+        // If the property is not in mPropertyParams, then this just means
+        // there are no properties set.
+        return this.mPropertyParams[propName][paramName];
+      }
+      return null;
     }
-    return null;
+    return this.mIsProxy ? this.mParentItem.getPropertyParameter(propName, paramName) : null;
   },
 
   // boolean hasPropertyParameter(in AString aPropertyName,
   //                              in AString aParameterName);
   hasPropertyParameter(aPropName, aParamName) {
-    let propName = aPropName.toUpperCase();
-    let paramName = aParamName.toUpperCase();
-    return propName in this.mPropertyParams && paramName in this.mPropertyParams[propName];
+    return this.getPropertyParameter(aPropName, aParamName) != null;
   },
 
   // void setPropertyParameter(in AString aPropertyName,
@@ -1020,7 +1022,6 @@ calItemBase.prototype = {
    */
   fillIcalComponentFromBase(icalcomp) {
     this.ensureNotDirty();
-    let icssvc = cal.getIcsService();
 
     this.mapPropsToICS(icalcomp, this.icsBasePropMap);
 
@@ -1048,7 +1049,7 @@ calItemBase.prototype = {
     }
 
     for (let cat of this.getCategories()) {
-      let catprop = icssvc.createIcalProperty("CATEGORIES");
+      let catprop = cal.icsService.createIcalProperty("CATEGORIES");
       catprop.value = cat;
       icalcomp.addProperty(catprop);
     }
@@ -1061,7 +1062,7 @@ calItemBase.prototype = {
 
     let alarmLastAck = this.alarmLastAck;
     if (alarmLastAck) {
-      let lastAck = cal.getIcsService().createIcalProperty("X-MOZ-LASTACK");
+      let lastAck = cal.icsService.createIcalProperty("X-MOZ-LASTACK");
       // - should we further ensure that those are UTC or rely on calAlarmService doing so?
       lastAck.value = alarmLastAck.icalString;
       icalcomp.addProperty(lastAck);
