@@ -10,20 +10,17 @@ const { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 const { cloudFileAccounts } = ChromeUtils.import("resource:///modules/cloudFileAccounts.jsm");
 const { MockFilePicker } = ChromeUtils.import("resource://specialpowers/MockFilePicker.jsm");
 
-let manager = cal.getCalendarManager();
-let _calendar = manager.createCalendar("memory", Services.io.newURI("moz-memory-calendar://"));
-_calendar.name = "Attachments";
-manager.registerCalendar(_calendar);
+let calendar = cal.manager.createCalendar("memory", Services.io.newURI("moz-memory-calendar://"));
+calendar.name = "Attachments";
+cal.manager.registerCalendar(calendar);
 
 // Remove the save prompt observer that head.js added. It's causing trouble here.
 Services.ww.unregisterNotification(savePromptObserver);
 
 registerCleanupFunction(() => {
-  manager.unregisterCalendar(_calendar);
+  cal.manager.unregisterCalendar(calendar);
 });
 
-let calendar = cal.async.promisifyCalendar(_calendar);
-
 async function getEventBox(selector) {
   let itemBox;
   await TestUtils.waitForCondition(() => {
@@ -120,7 +117,6 @@ add_task(async function testAttachWebPage() {
   let eventBox = await getEventBox("calendar-month-day-box-item");
   eventBox.focus();
   EventUtils.synthesizeKey("VK_DELETE", {});
-  await CalendarTestUtils.closeCalendarTab(window);
 });
 
 /**
@@ -146,13 +142,16 @@ add_task(async function testAttachProvider() {
         iconURL,
         configured: true,
         managementURL: "",
-        uploadFile() {
+        uploadFile(window, aFile) {
           return new Promise(resolve =>
             setTimeout(() =>
               resolve({
+                id: 1,
+                path: aFile.path,
+                size: aFile.fileSize,
                 url: fileUrl,
                 // The uploadFile() function should return serviceIcon, serviceName
-                // and serviceURL - either default or user defined values specified
+                // and serviceUrl - either default or user defined values specified
                 // by the onFileUpload event. The item-edit dialog uses only the
                 // serviceIcon.
                 serviceIcon: "chrome://messenger/skin/icons/globe.svg",
@@ -277,7 +276,6 @@ add_task(async function testAttachProvider() {
   let eventBox = await getEventBox("calendar-month-day-box-item");
   eventBox.focus();
   EventUtils.synthesizeKey("VK_DELETE", {});
-  await CalendarTestUtils.closeCalendarTab(window);
 });
 
 registerCleanupFunction(() => {
