@@ -3,6 +3,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalAttendee: "resource:///modules/CalAttendee.jsm",
+  CalEvent: "resource:///modules/CalEvent.jsm",
+});
 
 function run_test() {
   // Test attendee duplicate handling for bug 1204255
@@ -56,10 +62,10 @@ function test_newAttendee() {
     },
   ];
 
-  let event = cal.createEvent();
+  let event = new CalEvent();
   for (let test of data) {
     for (let input of test.input) {
-      let attendee = cal.createAttendee();
+      let attendee = new CalAttendee();
       attendee.id = input.id;
       attendee.participationStatus = input.partstat;
       attendee.commonName = input.cname;
