@@ -4,13 +4,16 @@
 
 do_get_profile();
 
-var { Services } = ChromeUtils.import("resource:///modules/imServices.jsm");
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { IMServices } = ChromeUtils.import("resource:///modules/IMServices.jsm");
 
 var gLogger = {};
 Services.scriptloader.loadSubScript("resource:///modules/logger.jsm", gLogger);
 
-var logDirPath = OS.Path.join(OS.Constants.Path.profileDir, "logs");
+var logDirPath = PathUtils.join(
+  Services.dirsvc.get("ProfD", Ci.nsIFile).path,
+  "logs"
+);
 
 var dummyAccount = {
   name: "dummy-account",
@@ -199,7 +202,7 @@ var test_queueFileOperation = async function() {
 var test_getLogFolderPathForAccount = async function() {
   let path = gLogger.getLogFolderPathForAccount(dummyAccount);
   equal(
-    OS.Path.join(
+    PathUtils.join(
       logDirPath,
       dummyAccount.protocol.normalizedName,
       gLogger.encodeName(dummyAccount.normalizedName)
@@ -210,57 +213,76 @@ var test_getLogFolderPathForAccount = async function() {
 
 // Tests the global function getLogFilePathForConversation in logger.js.
 var test_getLogFilePathForConversation = async function() {
-  let path = gLogger.getLogFilePathForConversation(dummyConv, "format");
-  let expectedPath = OS.Path.join(
+  let path = gLogger.getLogFilePathForConversation(dummyConv);
+  let expectedPath = PathUtils.join(
     logDirPath,
     dummyAccount.protocol.normalizedName,
     gLogger.encodeName(dummyAccount.normalizedName)
   );
-  expectedPath = OS.Path.join(
+  expectedPath = PathUtils.join(
     expectedPath,
     gLogger.encodeName(dummyConv.normalizedName)
   );
-  expectedPath = OS.Path.join(
+  expectedPath = PathUtils.join(
     expectedPath,
-    gLogger.getNewLogFileName("format", dummyConv.startDate / 1000)
+    gLogger.getNewLogFileName(dummyConv.startDate / 1000)
   );
   equal(path, expectedPath);
 };
 
 var test_getLogFilePathForMUC = async function() {
-  let path = gLogger.getLogFilePathForConversation(dummyMUC, "format");
-  let expectedPath = OS.Path.join(
+  let path = gLogger.getLogFilePathForConversation(dummyMUC);
+  let expectedPath = PathUtils.join(
     logDirPath,
     dummyAccount.protocol.normalizedName,
     gLogger.encodeName(dummyAccount.normalizedName)
   );
-  expectedPath = OS.Path.join(
+  expectedPath = PathUtils.join(
     expectedPath,
     gLogger.encodeName(dummyMUC.normalizedName + ".chat")
   );
-  expectedPath = OS.Path.join(
+  expectedPath = PathUtils.join(
     expectedPath,
-    gLogger.getNewLogFileName("format", dummyMUC.startDate / 1000)
+    gLogger.getNewLogFileName(dummyMUC.startDate / 1000)
   );
   equal(path, expectedPath);
 };
 
 var test_appendToFile = async function() {
   const kStringToWrite = "Hello, world!";
-  let path = OS.Path.join(OS.Constants.Path.profileDir, "testFile.txt");
-  let encoder = new TextEncoder();
-  let encodedString = encoder.encode(kStringToWrite);
-  gLogger.appendToFile(path, encodedString);
-  encodedString = encoder.encode(kStringToWrite);
-  gLogger.appendToFile(path, encodedString);
-  let text = new TextDecoder().decode(
-    await gLogger.queueFileOperation(path, () => OS.File.read(path))
+  let path = PathUtils.join(
+    Services.dirsvc.get("ProfD", Ci.nsIFile).path,
+    "testFile.txt"
+  );
+  await IOUtils.write(path, new Uint8Array());
+  gLogger.appendToFile(path, kStringToWrite);
+  gLogger.appendToFile(path, kStringToWrite);
+  ok(await gLogger.queueFileOperation(path, () => IOUtils.exists(path)));
+  let text = await gLogger.queueFileOperation(path, () =>
+    IOUtils.readUTF8(path)
   );
   // The read text should be equal to kStringToWrite repeated twice.
   equal(text, kStringToWrite + kStringToWrite);
-  await OS.File.remove(path);
+  await IOUtils.remove(path);
 };
 
+add_task(async function test_appendToFileHeader() {
+  const kStringToWrite = "Lorem ipsum";
+  let path = PathUtils.join(
+    Services.dirsvc.get("ProfD", Ci.nsIFile).path,
+    "headerTestFile.txt"
+  );
+  await gLogger.appendToFile(path, kStringToWrite, true);
+  await gLogger.appendToFile(path, kStringToWrite, true);
+  let text = await gLogger.queueFileOperation(path, () =>
+    IOUtils.readUTF8(path)
+  );
+  // The read text should be equal to kStringToWrite once, since the second
+  // create should just noop.
+  equal(text, kStringToWrite);
+  await IOUtils.remove(path);
+});
+
 // Tests the getLogPathsForConversation API defined in the imILogger interface.
 var test_getLogPathsForConversation = async function() {
   let logger = new gLogger.Logger();
@@ -271,9 +293,9 @@ var test_getLogPathsForConversation = async function() {
   paths = await logger.getLogPathsForConversation(dummyConv);
   equal(paths.length, 1);
   equal(paths[0], logWriter.currentPath);
-  ok(await OS.File.exists(paths[0]));
+  ok(await IOUtils.exists(paths[0]));
   // Ensure this doesn't interfere with future tests.
-  await OS.File.remove(paths[0]);
+  await IOUtils.remove(paths[0]);
   gLogger.closeLogWriter(dummyConv);
 };
 
@@ -310,6 +332,23 @@ var test_logging = async function() {
         displayMessage: "Nothing much!",
         incoming: true,
       },
+      {
+        time: startTime + 5,
+        who: "personB",
+        displayMessage: "Encrypted msg",
+        remoteId: "identifier",
+        incoming: true,
+        isEncrypted: true,
+      },
+      {
+        time: startTime + 6,
+        who: "personA",
+        displayMessage: "Deleted",
+        remoteId: "otherID",
+        outgoing: true,
+        isEncrypted: true,
+        deleted: true,
+      },
     ];
   };
   let firstDayMsgs = getMsgsForConv(dummyConv);
@@ -334,24 +373,20 @@ var test_logging = async function() {
 
   // Write a zero-length file and a file with incorrect JSON for each day
   // to ensure they are handled correctly.
-  let logDir = OS.Path.dirname(
-    gLogger.getLogFilePathForConversation(dummyConv, "json")
+  let logDir = PathUtils.parent(
+    gLogger.getLogFilePathForConversation(dummyConv)
   );
   let createBadFiles = async function(aConv) {
-    let blankFile = OS.Path.join(
+    let blankFile = PathUtils.join(
       logDir,
-      gLogger.getNewLogFileName("json", (aConv.startDate + oneSec) / 1000)
+      gLogger.getNewLogFileName((aConv.startDate + oneSec) / 1000)
     );
-    let invalidJSONFile = OS.Path.join(
+    let invalidJSONFile = PathUtils.join(
       logDir,
-      gLogger.getNewLogFileName("json", (aConv.startDate + 2 * oneSec) / 1000)
-    );
-    let file = await OS.File.open(blankFile, { truncate: true });
-    await file.close();
-    await OS.File.writeAtomic(
-      invalidJSONFile,
-      new TextEncoder().encode("This isn't JSON!")
+      gLogger.getNewLogFileName((aConv.startDate + 2 * oneSec) / 1000)
     );
+    await IOUtils.write(blankFile, new Uint8Array());
+    await IOUtils.writeUTF8(invalidJSONFile, "This isn't JSON!");
   };
   await createBadFiles(dummyConv);
   await createBadFiles(dummyConv2);
@@ -376,18 +411,6 @@ var test_logging = async function() {
     }
   };
 
-  let logs = await logger.getLogsForConversation(dummyConv);
-  let allLogMsgs = [];
-  for (let log of logs) {
-    let conv = await log.getConversation();
-    if (!conv) {
-      continue;
-    }
-    allLogMsgs = allLogMsgs.concat(conv.getMessages());
-  }
-  // Two session messages, one for each valid log file.
-  testMsgs(allLogMsgs, firstDayMsgs.concat(secondDayMsgs), 2);
-
   // Accepts time in seconds, reduces it to a date, and returns the value in millis.
   let reduceTimeToDate = function(aTime) {
     let date = new Date(aTime * 1000);
@@ -399,10 +422,16 @@ var test_logging = async function() {
 
   // Group expected messages by day.
   let messagesByDay = new Map();
-  messagesByDay.set(reduceTimeToDate(firstDayMsgs[0].time), firstDayMsgs);
-  messagesByDay.set(reduceTimeToDate(secondDayMsgs[0].time), secondDayMsgs);
+  messagesByDay.set(
+    reduceTimeToDate(firstDayMsgs[0].time),
+    firstDayMsgs.filter(msg => !msg.deleted)
+  );
+  messagesByDay.set(
+    reduceTimeToDate(secondDayMsgs[0].time),
+    secondDayMsgs.filter(msg => !msg.deleted)
+  );
 
-  logs = await logger.getLogsForConversation(dummyConv, true);
+  let logs = await logger.getLogsForConversation(dummyConv);
   for (let log of logs) {
     let conv = await log.getConversation();
     let date = reduceTimeToDate(log.time);
@@ -413,23 +442,22 @@ var test_logging = async function() {
   // Remove the created log files, testing forEach in the process.
   await logger.forEach({
     async processLog(aLog) {
-      let info = await OS.File.stat(aLog);
-      ok(!info.isDir);
+      let info = await IOUtils.stat(aLog);
+      notEqual(info.type, "directory");
       ok(aLog.endsWith(".json"));
-      await OS.File.remove(aLog);
+      await IOUtils.remove(aLog);
     },
   });
-  let logFolder = OS.Path.dirname(
+  let logFolder = PathUtils.parent(
     gLogger.getLogFilePathForConversation(dummyConv)
   );
   // The folder should now be empty - this will throw if it isn't.
-  await OS.File.removeEmptyDir(logFolder, { ignoreAbsent: false });
+  await IOUtils.remove(logFolder, { ignoreAbsent: false });
 };
 
 var test_logFileSplitting = async function() {
   // Start clean, remove the log directory.
-  let logFolderPath = OS.Path.join(OS.Constants.Path.profileDir, "logs");
-  await OS.File.removeDir(logFolderPath, { ignoreAbsent: true });
+  await IOUtils.remove(logDirPath, { recursive: true });
   let logWriter = gLogger.getLogWriter(dummyConv);
   let startTime = logWriter._startTime / 1000; // Message times are in seconds.
   let oldPath = logWriter.currentPath;
@@ -456,9 +484,7 @@ var test_logFileSplitting = async function() {
 
   let getCurrentHeader = async function() {
     return JSON.parse(
-      new TextDecoder()
-        .decode(await OS.File.read(logWriter.currentPath))
-        .split("\n")[0]
+      (await IOUtils.readUTF8(logWriter.currentPath)).split("\n")[0]
     );
   };
 
@@ -522,30 +548,313 @@ var test_logFileSplitting = async function() {
   ok(logWriter._startTime > oldStartTime);
 
   // Clean up.
-  await OS.File.removeDir(logFolderPath);
+  await IOUtils.remove(logDirPath, { recursive: true });
+  gLogger.closeLogWriter(dummyConv);
 };
 
-function run_test() {
+add_task(async function test_logWithEdits() {
+  // Start clean, remove the log directory.
+  await IOUtils.remove(logDirPath, { recursive: true });
+  let logger = new gLogger.Logger();
+  let logFilePath = gLogger.getLogFilePathForConversation(dummyConv);
+  await IOUtils.writeUTF8(
+    logFilePath,
+    [
+      {
+        date: "2022-03-04T12:00:03.508Z",
+        name: "test",
+        title: "test",
+        account: "@test:example.com",
+        protocol: "matrix",
+        isChat: false,
+        normalizedName: "!foobar:example.com",
+      },
+      {
+        date: "2022-03-04T11:59:48.000Z",
+        who: "@other:example.com",
+        text: "Decrypting...",
+        flags: ["incoming", "delayed", "isEncrypted"],
+        remoteId: "$AjmS57jkBbYnSnC01r3fXya8BfuHIMAw9mOYQRlnkFk",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:51.000Z",
+        who: "@other:example.com",
+        text: "Decrypting...",
+        flags: ["incoming", "delayed", "isEncrypted"],
+        remoteId: "$00zdmKvErkDR4wMaxZBCFsV1WwqPQRolP0kYiXPIXsQ",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:53.000Z",
+        who: "@other:example.com",
+        text: "Decrypting...",
+        flags: ["incoming", "delayed", "isEncrypted"],
+        remoteId: "$Z6ILSf7cBMRbr_B6Z6DPHJWzf-Utxa8_s0f6vxhR_VQ",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:56.000Z",
+        who: "@other:example.com",
+        text: "Decrypting...",
+        flags: ["incoming", "delayed", "isEncrypted"],
+        remoteId: "$GFlcel-9tWrTvSb7HM_113-WpkzEdB4neglPVpZn3dM",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:56.000Z",
+        who: "@other:example.com",
+        text: "Lorem ipsum dolor sit amet",
+        flags: ["incoming", "isEncrypted"],
+        remoteId: "$GFlcel-9tWrTvSb7HM_113-WpkzEdB4neglPVpZn3dM",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:53.000Z",
+        who: "@other:example.com",
+        text: "consectetur adipiscing elit",
+        flags: ["incoming", "isEncrypted"],
+        remoteId: "$Z6ILSf7cBMRbr_B6Z6DPHJWzf-Utxa8_s0f6vxhR_VQ",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:51.000Z",
+        who: "@other:example.com",
+        text:
+          "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua",
+        flags: ["incoming", "isEncrypted"],
+        remoteId: "$00zdmKvErkDR4wMaxZBCFsV1WwqPQRolP0kYiXPIXsQ",
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:48.000Z",
+        who: "@other:example.com",
+        text: "Ut enim ad minim veniam",
+        flags: ["incoming", "isEncrypted"],
+        remoteId: "$AjmS57jkBbYnSnC01r3fXya8BfuHIMAw9mOYQRlnkFk",
+        alias: "other",
+      },
+    ]
+      .map(message => JSON.stringify(message))
+      .join("\n"),
+    {
+      mode: "create",
+    }
+  );
+  let logs = await logger.getLogsForConversation(dummyConv);
+  equal(logs.length, 1);
+  const conv = await logs[0].getConversation();
+  const messages = conv.getMessages();
+  equal(messages.length, 5);
+  for (const msg of messages) {
+    if (msg.who !== "sessionstart") {
+      notEqual(msg.displayMessage, "Decrypting...");
+    }
+  }
+
+  // Clean up.
+  await IOUtils.remove(logDirPath, { recursive: true });
+});
+
+// Ensure that any message with a remoteId that has a deleted flag in the
+// latest version is not visible in logs.
+add_task(async function test_logWithDeletedMessages() {
+  // Start clean, remove the log directory.
+  await IOUtils.remove(logDirPath, { recursive: true });
+  let logger = new gLogger.Logger();
+  let logFilePath = gLogger.getLogFilePathForConversation(dummyConv);
+  const remoteId = "$GFlcel-9tWrTvSb7HM_113-WpkzEdB4neglPVpZn3dM";
+  await IOUtils.writeUTF8(
+    logFilePath,
+    [
+      {
+        date: "2022-03-04T12:00:03.508Z",
+        name: "test",
+        title: "test",
+        account: "@test:example.com",
+        protocol: "matrix",
+        isChat: false,
+        normalizedName: "!foobar:example.com",
+      },
+      {
+        date: "2022-03-04T11:59:56.000Z",
+        who: "@other:example.com",
+        text: "Decrypting...",
+        flags: ["incoming", "isEncrypted"],
+        remoteId,
+        alias: "other",
+      },
+      {
+        date: "2022-03-04T11:59:56.000Z",
+        who: "@other:example.com",
+        text: "Message was redacted.",
+        flags: ["incoming", "isEncrypted", "deleted"],
+        remoteId,
+        alias: "other",
+      },
+    ]
+      .map(message => JSON.stringify(message))
+      .join("\n"),
+    {
+      mode: "create",
+    }
+  );
+  let logs = await logger.getLogsForConversation(dummyConv);
+  equal(logs.length, 1);
+  const conv = await logs[0].getConversation();
+  const messages = conv.getMessages();
+  equal(messages.length, 1);
+  equal(messages[0].who, "sessionstart");
+
+  // Clean up.
+  await IOUtils.remove(logDirPath, { recursive: true });
+});
+
+add_task(async function test_logDeletedMessageCleanup() {
+  // Start clean, remove the log directory.
+  await IOUtils.remove(logDirPath, { recursive: true });
+  let logger = new gLogger.Logger();
+  let logWriter = gLogger.getLogWriter(dummyConv);
+  let remoteId = "testId";
+
+  let logMessage = async function(aMessage) {
+    logWriter.logMessage(aMessage);
+    await logWriter._initialized;
+    await gLogger.gFilePromises.get(logWriter.currentPath);
+  };
+
+  await logMessage({
+    time: Math.floor(dummyConv.startDate / 1000000) + 10,
+    who: "test",
+    displayMessage: "delete me",
+    remoteId,
+    incoming: true,
+  });
+
+  await logMessage({
+    time: Math.floor(dummyConv.startDate / 1000000) + 20,
+    who: "test",
+    displayMessage: "Message is deleted",
+    remoteId,
+    deleted: true,
+    incoming: true,
+  });
+  ok(gLogger.gPendingCleanup.has(logWriter.currentPath));
+  equal(
+    Services.prefs.getStringPref("chat.logging.cleanup.pending"),
+    JSON.stringify([logWriter.currentPath])
+  );
+
+  await new Promise(resolve => ChromeUtils.idleDispatch(resolve));
+  await (gLogger.gFilePromises.get(logWriter.currentPath) || Promise.resolve());
+
+  ok(!gLogger.gPendingCleanup.has(logWriter.currentPath));
+  equal(Services.prefs.getStringPref("chat.logging.cleanup.pending"), "[]");
+
+  let logs = await logger.getLogsForConversation(dummyConv);
+  equal(logs.length, 1, "Only a single log file for this conversation");
+  let conv = await logs[0].getConversation();
+  let messages = conv.getMessages();
+  equal(messages.length, 1, "Only the log header is left");
+  equal(messages[0].who, "sessionstart");
+
+  // Check that the message contents were removed from the file on disk. The
+  // log parser above removes it either way.
+  let logOnDisk = await IOUtils.readUTF8(logWriter.currentPath);
+  let rawMessages = logOnDisk
+    .split("\n")
+    .filter(Boolean)
+    .map(line => JSON.parse(line));
+  equal(rawMessages.length, 3);
+  equal(rawMessages[1].text, "", "Deleted message content was removed");
+  equal(
+    rawMessages[2].text,
+    "Message is deleted",
+    "Deletion content is unaffected"
+  );
+
+  // Clean up.
+  await IOUtils.remove(logDirPath, { recursive: true });
+
+  gLogger.closeLogWriter(dummyConv);
+});
+
+add_task(async function test_displayOldActionLog() {
+  // Start clean, remove the log directory.
+  await IOUtils.remove(logDirPath, { recursive: true });
+  let logger = new gLogger.Logger();
+  let logFilePath = gLogger.getLogFilePathForConversation(dummyConv);
+  await IOUtils.writeUTF8(
+    logFilePath,
+    [
+      {
+        date: "2022-03-04T12:00:03.508Z",
+        name: "test",
+        title: "test",
+        account: "@test:example.com",
+        protocol: "matrix",
+        isChat: false,
+        normalizedName: "!foobar:example.com",
+      },
+      {
+        date: "2022-03-04T11:59:56.000Z",
+        who: "@other:example.com",
+        text: "/me an old action",
+        flags: ["incoming"],
+      },
+      {
+        date: "2022-03-04T11:59:56.000Z",
+        who: "@other:example.com",
+        text: "a new action",
+        flags: ["incoming", "action"],
+      },
+    ]
+      .map(message => JSON.stringify(message))
+      .join("\n"),
+    {
+      mode: "create",
+    }
+  );
+  let logs = await logger.getLogsForConversation(dummyConv);
+  equal(logs.length, 1);
+  for (let log of logs) {
+    const conv = await log.getConversation();
+    const messages = conv.getMessages();
+    equal(messages.length, 3);
+    for (let message of messages) {
+      if (message.who !== "sessionstart") {
+        ok(message.action, "Message is marked as action");
+        ok(
+          !message.displayMessage.startsWith("/me"),
+          "Message has no leading /me"
+        );
+      }
+    }
+  }
+
+  // Clean up.
+  await IOUtils.remove(logDirPath, { recursive: true });
+});
+
+add_task(function test_encodeName() {
   // Test encodeName().
   for (let i = 0; i < encodeName_input.length; ++i) {
     equal(gLogger.encodeName(encodeName_input[i]), encodeName_output[i]);
   }
+});
 
-  add_task(test_getLogFolderPathForAccount);
+add_task(test_getLogFolderPathForAccount);
 
-  add_task(test_getLogFilePathForConversation);
+add_task(test_getLogFilePathForConversation);
 
-  add_task(test_getLogFilePathForMUC);
+add_task(test_getLogFilePathForMUC);
 
-  add_task(test_queueFileOperation);
+add_task(test_queueFileOperation);
 
-  add_task(test_appendToFile);
+add_task(test_appendToFile);
 
-  add_task(test_getLogPathsForConversation);
+add_task(test_getLogPathsForConversation);
 
-  add_task(test_logging);
+add_task(test_logging);
 
-  add_task(test_logFileSplitting);
-
-  run_next_test();
-}
+add_task(test_logFileSplitting);
