@@ -24,6 +24,9 @@
     initHTMLDocument: "resource:///modules/imThemes.jsm",
     insertHTMLForMessage: "resource:///modules/imThemes.jsm",
     isNextMessage: "resource:///modules/imThemes.jsm",
+    wasNextMessage: "resource:///modules/imThemes.jsm",
+    replaceHTMLForMessage: "resource:///modules/imThemes.jsm",
+    removeMessage: "resource:///modules/imThemes.jsm",
     serializeSelection: "resource:///modules/imThemes.jsm",
     smileTextNode: "resource:///modules/imSmileys.jsm",
   });
@@ -418,6 +421,68 @@
       }
     }
 
+    /**
+     * Replace an existing message in the conversation based on the remote ID.
+     *
+     * @param {imIMessage} msg - Message to use as replacement.
+     */
+    replaceMessage(msg) {
+      if (!msg.remoteId) {
+        // No remote id, nothing existing to replace.
+        return;
+      }
+      if (this._messageDisplayPending || this._pendingMessages.length) {
+        let pendingIndex = this._pendingMessages.findIndex(
+          ({ msg: pendingMsg }) => pendingMsg.remoteId === msg.remoteId
+        );
+        if (
+          pendingIndex > -1 &&
+          pendingIndex >= this._nextPendingMessageIndex
+        ) {
+          this._pendingMessages[pendingIndex].msg = msg;
+        }
+      }
+      if (this.browsingContext.isActive) {
+        msg.message = this.prepareMessageContent(msg);
+        const isNext = LazyModules.wasNextMessage(msg, this.contentDocument);
+        const htmlMessage = LazyModules.getHTMLForMessage(
+          msg,
+          this.theme,
+          isNext,
+          false
+        );
+        let ruler = this.contentDocument.getElementById("unread-ruler");
+        if (ruler?._originalMsg?.remoteId === msg.remoteId) {
+          ruler._originalMsg = msg;
+          ruler.nextMsgHtml = htmlMessage;
+        }
+        LazyModules.replaceHTMLForMessage(
+          msg,
+          htmlMessage,
+          this.contentDocument,
+          isNext
+        );
+      }
+      if (this._lastMessage?.remoteId === msg.remoteId) {
+        this._lastMessage = msg;
+      }
+    }
+
+    /**
+     * Remove an existing message in the conversation based on the remote ID.
+     *
+     * @param {string} remoteId - Remote ID of the message to remove.
+     */
+    removeMessage(remoteId) {
+      if (this.browsingContext.isActive) {
+        LazyModules.removeMessage(remoteId, this.contentDocument);
+      }
+      if (this._lastMessage?.remoteId === remoteId) {
+        // Reset last message info if we removed the last message.
+        this._lastMessage = null;
+      }
+    }
+
     startDisplayingPendingMessages(delayed) {
       if (this._messageDisplayPending) {
         return;
@@ -535,50 +600,11 @@
         }
       }
 
-      let cs = Cc["@mozilla.org/txttohtmlconv;1"].getService(
-        Ci.mozITXTToHTMLConv
-      );
-
-      // kStructPhrase creates tags for plaintext-markup like *bold*,
-      // /italics/, etc. We always use this; the content filter will
-      // filter it out if the user does not want styling.
-      let csFlags = cs.kStructPhrase;
-      // Automatically find and link freetext URLs
-      if (!aMsg.noLinkification) {
-        csFlags |= cs.kURLs;
-      }
-
       if (aFirstUnread) {
         this.setUnreadRuler();
       }
 
-      // Right trim before displaying. This removes any OTR related
-      // whitespace when the extension isn't enabled.
-      let msg = aMsg.displayMessage?.trimRight() ?? "";
-
-      // The slash of a leading '/me' should not be used to
-      // format as italic, so we remove the '/me' text before
-      // scanning the HTML, and we add it back later.
-      let meRegExp = /^((<[^>]+>)*)\/me /;
-      let me = false;
-      if (meRegExp.test(msg)) {
-        me = true;
-        msg = msg.replace(meRegExp, "$1");
-      }
-
-      msg = cs
-        .scanHTML(msg.replace(/&/g, "FROM-DTD-amp"), csFlags)
-        .replace(/FROM-DTD-amp/g, "&");
-
-      if (me) {
-        msg = msg.replace(/^((<[^>]+>)*)/, "$1/me ");
-      }
-
-      aMsg.message = LazyModules.cleanupImMarkup(
-        msg.replace(/\r?\n/g, "<br/>"),
-        null,
-        this._textModifiers
-      );
+      aMsg.message = this.prepareMessageContent(aMsg);
 
       let next =
         (aContext == this._lastMessageIsContext || aMsg.system) &&
@@ -647,6 +673,41 @@
       this._lastMessageIsContext = aContext;
     }
 
+    /**
+     * Prepare the message text for display. Transforms plain text formatting
+     * and removes any unwanted formatting.
+     *
+     * @param {imIMessage} message - Raw message.
+     * @return {string} Message content ready for insertion.
+     */
+    prepareMessageContent(message) {
+      let cs = Cc["@mozilla.org/txttohtmlconv;1"].getService(
+        Ci.mozITXTToHTMLConv
+      );
+
+      // kStructPhrase creates tags for plaintext-markup like *bold*,
+      // /italics/, etc. We always use this; the content filter will
+      // filter it out if the user does not want styling.
+      let csFlags = cs.kStructPhrase;
+      // Automatically find and link freetext URLs
+      if (!message.noLinkification) {
+        csFlags |= cs.kURLs;
+      }
+
+      // Right trim before displaying. This removes any OTR related
+      // whitespace when the extension isn't enabled.
+      let msg = message.displayMessage?.trimRight() ?? "";
+      msg = cs
+        .scanHTML(msg.replace(/&/g, "FROM-DTD-amp"), csFlags)
+        .replace(/FROM-DTD-amp/g, "&");
+
+      return LazyModules.cleanupImMarkup(
+        msg.replace(/\r?\n/g, "<br/>"),
+        null,
+        this._textModifiers
+      );
+    }
+
     setUnreadRuler() {
       // Remove any existing ruler (occurs when the window has lost focus).
       this.removeUnreadRuler();
@@ -692,6 +753,7 @@
           root = root.nextElementSibling
         ) {
           root._originalMsg = ruler._originalMsg;
+          root.dataset.remoteId = ruler._originalMsg.remoteId;
         }
         moveToParent.insertBefore(documentFragment, moveTo);
 
