@@ -740,6 +740,11 @@ var gMailInit = {
       setTimeout(this.showEOYDonationAppeal, 2000);
     }
 
+    if (AppConstants.platform == "win" && this.shouldShowBetaAppeal()) {
+      // A little later than donation appeal.
+      setTimeout(this.showBetaAppeal, 5000);
+    }
+
     // Idle dispatch the telemetry reports.
     Services.tm.idleDispatchToMainThread(() => {
       reportAccountTypes();
@@ -822,6 +827,74 @@ var gMailInit = {
     let currentEOY = Services.prefs.getIntPref("app.donation.eoy.version", 1);
     Services.prefs.setIntPref("app.donation.eoy.version.viewed", currentEOY);
   },
+
+  /**
+   * Open the beta appeal in a new window to ensure the user can download it
+   * correctly.
+   */
+  showBetaAppeal() {
+    let url = Services.urlFormatter.formatURLPref("app.beta_appeal.url");
+    let messenger =
+      window.messenger ||
+      Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger);
+    messenger.launchExternalURL(url);
+
+    this.markBetaAppealVersion();
+  },
+
+  /**
+   * Check if we can trigger the opening of the beta appeal page.
+   *
+   * @returns {boolean} - True if the beta appeal page should be opened.
+   */
+  shouldShowBetaAppeal() {
+    let currentBetaAppeal = Services.prefs.getIntPref(
+      "app.beta_appeal.version",
+      1
+    );
+    let viewedBetaAppeal = Services.prefs.getIntPref(
+      "app.beta_appeal.version.viewed",
+      0
+    );
+
+    // True if the user never saw the beta appeal, this is not a new
+    // profile, and we're not running tests.
+    const applicable =
+      viewedBetaAppeal < currentBetaAppeal &&
+      !specialTabs.shouldShowPolicyNotification() &&
+      !Cu.isInAutomation;
+    if (!applicable) {
+      return false;
+    }
+
+    // expiry date
+    const now = Math.floor(Date.now() / 1000);
+    const expiry = Services.prefs.getIntPref("app.beta_appeal.expiry", now);
+
+    // Never shown, filter to 30% of users.
+    const factor = Math.random() * 100;
+    if (factor > 30 || now > expiry) {
+      this.markBetaAppealVersion();
+      return false;
+    }
+
+    return true;
+  },
+
+  /**
+   * Mark the viewed version of the beta appeal as the latest version
+   * stored in the schema.
+   */
+  markBetaAppealVersion() {
+    let currentBetaAppeal = Services.prefs.getIntPref(
+      "app.beta_appeal.version",
+      1
+    );
+    Services.prefs.setIntPref(
+      "app.beta_appeal.version.viewed",
+      currentBetaAppeal
+    );
+  },
 };
 
 /**
@@ -2230,6 +2303,11 @@ function ThreadPaneOnDragStart(aEvent) {
       new messageFlavorDataProvider(),
       index
     );
+    aEvent.dataTransfer.mozSetDataAt(
+      "application/x-moz-file-promise-dest-filename",
+      msgFileName.replace(/(.{74}).*(.{10})$/u, "$1...$2"),
+      index
+    );
   }
 
   aEvent.dataTransfer.effectAllowed = "copyMove";
@@ -2256,7 +2334,7 @@ messageFlavorDataProvider.prototype = {
     );
     let fileUri = Services.io.newURI(fileUriStr.data);
     let fileUrl = fileUri.QueryInterface(Ci.nsIURL);
-    let fileName = fileUrl.fileName;
+    let fileName = fileUrl.fileName.replace(/(.{74}).*(.{10})$/u, "$1...$2");
 
     let destDirPrimitive = {};
     aTransferable.getTransferData(
