@@ -7,6 +7,12 @@ ChromeUtils.defineModuleGetter(
   "PromiseUtils",
   "resource://gre/modules/PromiseUtils.jsm"
 );
+ChromeUtils.defineModuleGetter(
+  this,
+  "MailE10SUtils",
+  "resource:///modules/MailE10SUtils.jsm"
+);
+
 var { ExtensionError } = ExtensionUtils;
 
 /**
@@ -57,8 +63,8 @@ let tabListener = {
   /**
    * Promise that the given tab completes loading.
    *
-   * @param {NativeTabInfo} nativeTabInfo       The tabInfo describing the tab
-   * @return {Promise<NativeTabInfo>}           Resolves when the tab completes loading
+   * @param {NativeTabInfo} nativeTabInfo - the tabInfo describing the tab
+   * @return {Promise<NativeTabInfo>} - resolves when the tab completes loading
    */
   awaitTabReady(nativeTabInfo) {
     let deferred = this.tabReadyPromises.get(nativeTabInfo);
@@ -161,25 +167,17 @@ class TabsUpdateFilterEventManager extends EventManager {
       };
 
       let listener = event => {
-        let needed = [];
-        if (event.type == "TabAttrModified") {
-          let changed = event.detail.changed;
-          if (
-            changed.includes("image") &&
-            filter.properties.has("favIconUrl")
-          ) {
-            needed.push("favIconUrl");
-          }
-          if (changed.includes("label") && filter.properties.has("title")) {
-            needed.push("title");
-          }
-        }
-
-        let tab = tabManager.getWrapper(event.detail.tabInfo);
-
         let changeInfo = {};
-        for (let prop of needed) {
-          changeInfo[prop] = tab[prop];
+        let tab = tabManager.getWrapper(event.detail.tabInfo);
+        let changed = event.detail.changed;
+        if (
+          changed.includes("favIconUrl") &&
+          filter.properties.has("favIconUrl")
+        ) {
+          changeInfo.favIconUrl = tab.favIconUrl;
+        }
+        if (changed.includes("label") && filter.properties.has("title")) {
+          changeInfo.title = tab.title;
         }
 
         fireForTab(tab, changeInfo);
@@ -241,6 +239,26 @@ class TabsUpdateFilterEventManager extends EventManager {
 }
 
 this.tabs = class extends ExtensionAPI {
+  onShutdown(isAppShutdown) {
+    if (isAppShutdown) {
+      return;
+    }
+    for (let window of Services.wm.getEnumerator("mail:3pane")) {
+      let tabmail = window.document.getElementById("tabmail");
+      for (let i = tabmail.tabInfo.length; i > 0; i--) {
+        let nativeTabInfo = tabmail.tabInfo[i - 1];
+        let uri = nativeTabInfo.browser?.browsingContext.currentURI;
+        if (
+          uri &&
+          uri.scheme == "moz-extension" &&
+          uri.host == this.extension.uuid
+        ) {
+          tabmail.closeTab(nativeTabInfo);
+        }
+      }
+    }
+  }
+
   getAPI(context) {
     let { extension } = context;
     let { tabManager } = extension;
@@ -370,7 +388,7 @@ this.tabs = class extends ExtensionAPI {
           name: "tabs.onMoved",
           register: fire => {
             let moveListener = event => {
-              let nativeTab = event.originalTarget;
+              let nativeTab = event.target;
               let nativeTabInfo = event.detail.tabInfo;
               let tabmail = nativeTab.ownerDocument.getElementById("tabmail");
 
@@ -391,29 +409,11 @@ this.tabs = class extends ExtensionAPI {
         onUpdated: new TabsUpdateFilterEventManager({ context }).api(),
 
         async create(createProperties) {
-          let window = await new Promise((resolve, reject) => {
-            let window =
-              createProperties.windowId === null
-                ? windowTracker.topNormalWindow
-                : windowTracker.getWindow(createProperties.windowId, context);
-            if (!window.delayedStartupFinished) {
-              let obs = (finishedWindow, topic, data) => {
-                if (finishedWindow != window) {
-                  return;
-                }
-                Services.obs.removeObserver(
-                  obs,
-                  "mail-delayed-startup-finished"
-                );
-                resolve(window);
-              };
-              Services.obs.addObserver(obs, "mail-delayed-startup-finished");
-            } else {
-              resolve(window);
-            }
-          });
+          let window = await getNormalWindowReady(
+            context,
+            createProperties.windowId
+          );
           let tabmail = window.document.getElementById("tabmail");
-
           let url;
           if (createProperties.url) {
             url = context.uri.resolve(createProperties.url);
@@ -424,17 +424,17 @@ this.tabs = class extends ExtensionAPI {
           }
 
           let currentTab = tabmail.selectedTab;
-
-          let active = true;
-          if (createProperties.active) {
-            active = createProperties.active;
-          }
-
+          let active = createProperties.active ?? true;
           tabListener.initTabReady();
 
           let nativeTabInfo = tabmail.openTab("contentTab", {
-            contentPage: url || "about:blank",
+            url: url || "about:blank",
+            linkHandler: null,
             background: !active,
+            initialBrowsingContextGroupId:
+              context.extension.policy.browsingContextGroupId,
+            principal: context.extension.principal,
+            duplicate: true,
           });
 
           if (createProperties.index) {
@@ -447,7 +447,6 @@ this.tabs = class extends ExtensionAPI {
             // requested URL is loaded.
             tabListener.initializingTabs.add(nativeTabInfo);
           }
-
           return tabManager.convert(nativeTabInfo, currentTab);
         },
 
@@ -477,6 +476,23 @@ this.tabs = class extends ExtensionAPI {
           let tabmail = getTabTabmail(nativeTabInfo);
 
           if (updateProperties.url) {
+            // Only supported for content tabs and active mail tabs.
+            let supportedTabs =
+              nativeTabInfo == tabmail.currentTabInfo
+                ? ["folder", "contentTab"]
+                : ["contentTab"];
+            if (supportedTabs.includes(nativeTabInfo.mode.name)) {
+              if (nativeTabInfo.messageDisplay) {
+                // Clear the selection before loading a content url into a mail
+                // tab, to reset the messageDisplay.
+                nativeTabInfo.folderDisplay.clearSelection();
+              }
+            } else {
+              throw new ExtensionError(
+                "Updating the displayed url is only supported for content tabs and active mail tabs."
+              );
+            }
+
             let browser = getTabBrowser(nativeTabInfo);
             if (!browser) {
               throw new ExtensionError("Cannot set a URL for this tab.");
@@ -493,15 +509,13 @@ this.tabs = class extends ExtensionAPI {
                 : Ci.nsIWebNavigation.LOAD_FLAGS_NONE,
               triggeringPrincipal: context.principal,
             };
-            browser.loadURI(url, options);
+            MailE10SUtils.loadURI(browser, url, options);
           }
 
+          // The current tab can only be set to active. To set it inactive, another tab has to be set
+          // as active.
           if (updateProperties.active) {
-            if (updateProperties.active) {
-              tabmail.selectedTab = nativeTabInfo;
-            } else {
-              // Not sure what to do here? Which tab should we select?
-            }
+            tabmail.selectedTab = nativeTabInfo;
           }
 
           return tabManager.convert(nativeTabInfo);
@@ -537,7 +551,7 @@ this.tabs = class extends ExtensionAPI {
 
         async query(queryInfo) {
           if (!extension.hasPermission("tabs")) {
-            if (queryInfo.url || queryInfo.title !== null) {
+            if (queryInfo.url !== null || queryInfo.title !== null) {
               return Promise.reject({
                 message:
                   'The "tabs" permission is required to use the query API with the "url" or "title" parameters',
@@ -545,16 +559,8 @@ this.tabs = class extends ExtensionAPI {
             }
           }
 
-          queryInfo = Object.assign({}, queryInfo);
-
-          if (queryInfo.url) {
-            queryInfo.url = new MatchPatternSet([].concat(queryInfo.url));
-          }
-          if (queryInfo.title !== null) {
-            queryInfo.title = new MatchGlob(queryInfo.title);
-          }
-
           // Make ext-tabs-base happy since it does a strict check.
+          queryInfo.cookieStoreId = null;
           queryInfo.screen = null;
 
           return Array.from(tabManager.query(queryInfo, context), tab =>
@@ -691,7 +697,7 @@ this.tabs = class extends ExtensionAPI {
 
           if (mode.tabs.length && mode.tabs.length == mode.maxTabs) {
             throw new ExtensionError(
-              `Maximum number of ${state.mode} tabs reached`
+              `Maximum number of ${state.mode} tabs reached.`
             );
           } else {
             tabmail.restoreTab(state);
