@@ -8,13 +8,15 @@
 
 // Wrap in a block to prevent leaking to window scope.
 {
-  const { Services } = ChromeUtils.import("resource:///modules/imServices.jsm");
+  const { Services } = ChromeUtils.import(
+    "resource://gre/modules/Services.jsm"
+  );
+  const { IMServices } = ChromeUtils.import(
+    "resource:///modules/IMServices.jsm"
+  );
   const { Status } = ChromeUtils.import(
     "resource:///modules/imStatusUtils.jsm"
   );
-  const { MessageFormat } = ChromeUtils.import(
-    "resource:///modules/imTextboxUtils.jsm"
-  );
   const { TextboxSize } = ChromeUtils.import(
     "resource:///modules/imTextboxUtils.jsm"
   );
@@ -41,6 +43,12 @@
     constructor() {
       super();
 
+      ChromeUtils.defineModuleGetter(
+        this,
+        "ChatEncryption",
+        "resource:///modules/ChatEncryption.jsm"
+      );
+
       this.observer = {
         // @see {nsIObserver}
         observe: (subject, topic, data) => {
@@ -58,6 +66,15 @@
             }
 
             Services.obs.removeObserver(this.observer, "conversation-loaded");
+
+            // Report the active chat message theme via telemetry. This is not
+            // inside the conv browser itself, since the browser is also used
+            // for the theme preview in the settings.
+            Services.telemetry.scalarSet(
+              "tb.chat.active_message_theme",
+              `${this.convBrowser.theme.name}:${this.convBrowser.theme.variant}`
+            );
+
             return;
           }
 
@@ -70,6 +87,18 @@
               }
               break;
 
+            case "update-text":
+              if (this.loaded) {
+                this.updateMsg(subject);
+              }
+              break;
+
+            case "remove-text":
+              if (this.loaded) {
+                this.removeMsg(data);
+              }
+              break;
+
             case "status-text-changed":
               this._statusText = data || "";
               this.displayStatusText();
@@ -89,6 +118,7 @@
             // Update the status too.
             case "update-buddy-status":
             case "update-buddy-icon":
+            case "update-conv-icon":
             case "update-conv-chatleft":
               if (this.tab && this._isConversationSelected) {
                 this.updateConvStatus();
@@ -139,11 +169,16 @@
                 this.updateTopic();
               }
               break;
+            case "update-conv-encryption":
+              if (this._isConversationSelected) {
+                this.ChatEncryption.updateEncryptionButton(document, this.conv);
+              }
+              break;
           }
         },
         QueryInterface: ChromeUtils.generateQI([
-          Ci.nsIObserver,
-          Ci.nsISupportsWeakReference,
+          "nsIObserver",
+          "nsISupportsWeakReference",
         ]),
       };
     }
@@ -291,15 +326,13 @@
     }
 
     get msgNotificationBar() {
-      delete this._msgNotificationBar;
-
-      let newNotificationBox = new MozElements.NotificationBox(element => {
-        element.setAttribute("flex", "1");
-        element.setAttribute("notificationside", "top");
-        this.notification.prepend(element);
-      });
-
-      return (this._msgNotificationBar = newNotificationBox);
+      if (!this._notificationBox) {
+        this._notificationBox = new MozElements.NotificationBox(element => {
+          element.setAttribute("notificationside", "top");
+          this.notification.prepend(element);
+        });
+      }
+      return this._notificationBox;
     }
 
     destroy() {
@@ -307,9 +340,6 @@
         this._forgetConv();
       }
 
-      if ("MessageFormat" in window) {
-        MessageFormat.unregisterTextbox(this.inputBox);
-      }
       Services.prefs.removeObserver(
         "mail.spellcheck.inline",
         this.prefObserver
@@ -432,6 +462,57 @@
       return isTabFocused;
     }
 
+    /**
+     * Updates an existing message with the matching remote ID.
+     *
+     * @param {imIMessage} aMsg - Message to update.
+     */
+    updateMsg(aMsg) {
+      if (!this.loaded) {
+        throw new Error("Calling updateMsg before the browser is ready?");
+      }
+
+      var conv = aMsg.conversation;
+      if (!conv) {
+        // The conversation has already been destroyed,
+        // probably because the window was closed.
+        // Return without doing anything.
+        return;
+      }
+
+      // Update buddy color.
+      // Ugly hack... :(
+      if (!aMsg.system && conv.isChat) {
+        let name = aMsg.who;
+        let color;
+        if (this.buddies.has(name)) {
+          let buddy = this.buddies.get(name);
+          color = buddy.color;
+          buddy.removeAttribute("inactive");
+          this._activeBuddies[name] = true;
+        } else {
+          // Buddy no longer in the room
+          color = this._computeColor(name);
+        }
+        aMsg.color = "color: hsl(" + color + ", 100%, 40%);";
+      }
+
+      this.convBrowser.replaceMessage(aMsg);
+    }
+
+    /**
+     * Removes an existing message with matching remote ID.
+     *
+     * @param {string} remoteId - Remote ID of the message to remove.
+     */
+    removeMsg(remoteId) {
+      if (!this.loaded) {
+        throw new Error("Calling removeMsg before the browser is ready?");
+      }
+
+      this.convBrowser.removeMessage(remoteId);
+    }
+
     sendMsg(aMsg) {
       if (!aMsg) {
         return;
@@ -453,7 +534,7 @@
         if (aMsg.match(/^\/(?:say)? .*/)) {
           aMsg = aMsg.slice(aMsg.indexOf(" ") + 1);
         } else if (
-          Services.cmd.executeCommand(aMsg, this._conv.target, convToFocus)
+          IMServices.cmd.executeCommand(aMsg, this._conv.target, convToFocus)
         ) {
           this._conv.sendTyping("");
           this.resetInput();
@@ -475,88 +556,8 @@
         }
       }
 
-      let msg = Cc["@mozilla.org/txttohtmlconv;1"]
-        .getService(Ci.mozITXTToHTMLConv)
-        .scanTXT(aMsg, 0);
+      this._conv.sendMsg(aMsg, false, false);
 
-      if (account.HTMLEnabled) {
-        msg = msg.replace(/\n/g, "<br/>");
-        if (Services.prefs.getBoolPref("messenger.conversations.sendFormat")) {
-          let style = MessageFormat.getMessageStyle();
-          let proto = this._conv.account.protocol.id;
-          if (proto == "prpl-msn") {
-            if ("color" in style) {
-              msg = '<font color="' + style.color + '">' + msg + "</font>";
-            }
-            if ("fontFamily" in style) {
-              msg = '<font face="' + style.fontFamily + '">' + msg + "</font>";
-            }
-            // MSN doesn't support font size info in messages...
-          } else if (proto == "prpl-aim" || proto == "prpl-icq") {
-            let styleAttributes = "";
-            if ("color" in style) {
-              styleAttributes += ' color="' + style.color + '"';
-            }
-            if ("fontFamily" in style) {
-              styleAttributes += ' face="' + style.fontFamily + '"';
-            }
-            if ("fontSize" in style) {
-              let size = style.fontSize - style.defaultFontSize;
-              if (size < -4) {
-                size = 1;
-              } else if (size < 0) {
-                size = 2;
-              } else if (size < 3) {
-                size = 3;
-              } else if (size < 7) {
-                size = 4;
-              } else if (size < 15) {
-                size = 5;
-              } else if (size < 25) {
-                size = 6;
-              } else {
-                size = 7;
-              }
-              styleAttributes +=
-                ' size="' +
-                size +
-                '"' +
-                ' style="font-size: ' +
-                style.fontSize +
-                'px;"';
-            }
-            if (styleAttributes) {
-              msg = "<font" + styleAttributes + ">" + msg + "</font>";
-            }
-          } else {
-            let styleProperties = [];
-            if ("color" in style) {
-              styleProperties.push("color: " + style.color);
-            }
-            if ("fontFamily" in style) {
-              styleProperties.push("font-family: " + style.fontFamily);
-            }
-            if ("fontSize" in style) {
-              styleProperties.push("font-size: " + style.fontSize + "px");
-            }
-            style = styleProperties.join("; ");
-            if (style) {
-              msg = '<span style="' + style + '">' + msg + "</span>";
-            }
-          }
-        }
-        this._conv.sendMsg(msg);
-      } else {
-        msg = account.HTMLEscapePlainText ? msg : aMsg;
-
-        if (account.noNewlines) {
-          // 'Illegal operation on WrappedNative prototype object' if the this
-          // object is not specified (since this._conv implements nsIClassInfo)
-          msg.split("\n").forEach(this._conv.sendMsg, this._conv);
-        } else {
-          this._conv.sendMsg(msg);
-        }
-      }
       // reset the textbox to its original size
       this.resetInput();
     }
@@ -609,8 +610,6 @@
     }
 
     initTextboxFormat() {
-      MessageFormat.registerTextbox(this.inputBox);
-
       // Init the textbox size
       this.calculateTextboxDefaultHeight();
       this.inputBox.parentNode.height =
@@ -648,19 +647,7 @@
           (event.keyCode == KeyEvent.DOM_VK_PAGE_UP ||
             event.keyCode == KeyEvent.DOM_VK_PAGE_DOWN))
       ) {
-        let newEvent = document.createEvent("KeyboardEvent");
-        newEvent.initKeyEvent(
-          "keypress",
-          event.bubbles,
-          event.cancelable,
-          null,
-          event.ctrlKey,
-          event.altKey,
-          event.shiftKey,
-          event.metaKey,
-          event.keyCode,
-          event.charCode
-        );
+        let newEvent = new KeyboardEvent("keypress", event);
         event.preventDefault();
         event.stopPropagation();
         // Keyboard events must be sent to the focused element for bubbling to work.
@@ -807,7 +794,7 @@
         // Check if we are completing a command.
         let completingCommand = isFirstWord && word[0] == "/";
         if (completingCommand) {
-          for (let cmd of Services.cmd.listCommandsForConversation(
+          for (let cmd of IMServices.cmd.listCommandsForConversation(
             this._conv
           )) {
             // It's possible to have a global and a protocol specific command
@@ -1206,11 +1193,6 @@
       for (let node = event.target; node; node = node.parentNode) {
         if (node._originalMsg) {
           let msg = node._originalMsg;
-          let actions = msg.getActions();
-          if (actions.length >= 1) {
-            actions[0].run();
-            return;
-          }
           if (
             msg.system ||
             msg.outgoing ||
@@ -1266,25 +1248,29 @@
      */
     setBuddyAttributes(aItem) {
       let buddy = aItem.chatBuddy;
-      let image;
-      if (!buddy.noFlags) {
-        if (buddy.op) {
-          image = "operator";
-        } else if (buddy.halfOp) {
-          image = "half-operator";
-        } else if (buddy.voiced) {
-          image = "voice";
-        } else if (buddy.founder) {
-          image = "founder";
-        }
+      let src;
+      let l10nId;
+      if (buddy.founder) {
+        src = "chrome://messenger/skin/icons/founder.png";
+        l10nId = "chat-participant-owner-role-icon2";
+      } else if (buddy.admin) {
+        src = "chrome://messenger/skin/icons/operator.png";
+        l10nId = "chat-participant-administrator-role-icon2";
+      } else if (buddy.moderator) {
+        src = "chrome://messenger/skin/icons/half-operator.png";
+        l10nId = "chat-participant-moderator-role-icon2";
+      } else if (buddy.voiced) {
+        src = "chrome://messenger/skin/icons/voice.png";
+        l10nId = "chat-participant-voiced-role-icon2";
       }
-      if (image) {
-        aItem.firstElementChild.setAttribute(
-          "src",
-          "chrome://messenger/skin/" + image + ".png"
-        );
+      let imageEl = aItem.querySelector(".conv-nicklist-image");
+      if (src) {
+        imageEl.setAttribute("src", src);
+        document.l10n.setAttributes(imageEl, l10nId);
       } else {
-        aItem.firstElementChild.removeAttribute("src");
+        imageEl.removeAttribute("src");
+        imageEl.removeAttribute("data-l10n-id");
+        imageEl.removeAttribute("alt");
       }
     }
 
@@ -1341,8 +1327,10 @@
 
       this.trackNick(name);
 
-      let image = document.createXULElement("image");
+      let image = document.createElement("img");
+      image.classList.add("conv-nicklist-image");
       let label = document.createXULElement("label");
+      label.classList.add("conv-nicklist-label");
       label.setAttribute("value", name);
       label.setAttribute("flex", "1");
       label.setAttribute("crop", "end");
@@ -1416,6 +1404,9 @@
       }
 
       if (!aOldName) {
+        if (!this._isConversationSelected) {
+          return;
+        }
         // If aOldName is null, we are changing the flags of the buddy
         let item = this.buddies.get(name);
         item.chatBuddy = aBuddy;
@@ -1537,26 +1528,17 @@
       }.bind(this);
     }
 
+    /**
+     * Display the topic and topic editable flag for the current MUC in the
+     * conversation header.
+     */
     updateTopic() {
       let cti = document.getElementById("conv-top-info");
-      if (this._conv.topicSettable) {
-        cti.setAttribute("topicEditable", "true");
-      } else {
-        cti.removeAttribute("topicEditable");
-      }
+      let editable = !!this._conv.topicSettable;
 
-      let topic = this._conv.topic;
-      if (topic) {
-        cti.setAttribute("statusTooltiptext", topic);
-        cti.removeAttribute("noTopic");
-      } else {
-        topic = this._conv.noTopicString;
-        cti.setAttribute("noTopic", "true");
-        cti.setAttribute("statusTooltiptext", topic);
-      }
-      cti.setAttribute("statusMessage", topic);
-      cti.setAttribute("statusMessageWithDash", " - " + topic);
-      cti.removeAttribute("userIcon");
+      let topicText = this._conv.topic;
+      let noTopic = !topicText;
+      cti.setAsChat(topicText || this._conv.noTopicString, noTopic, editable);
     }
 
     focus() {
@@ -1573,44 +1555,6 @@
       this._conv.markAsRead();
     }
 
-    updateTyping() {
-      let typingState = this._conv.typingState;
-      let cti = document.getElementById("conv-top-info");
-      cti.removeAttribute("typing");
-
-      let name = this._currentTypingName;
-      if (!this._currentTypingName) {
-        name = this._conv.title; // .replace(/^([a-zA-Z0-9.]+)[@\s].*/, "$1");
-      }
-      if (typingState == Ci.prplIConvIM.TYPING) {
-        cti.setAttribute("typing", "active");
-        let typingMsg = this.bundle.formatStringFromName(
-          "chat.contactIsTyping",
-          [name],
-          1
-        );
-        cti.setAttribute("statusTypeTooltiptext", typingMsg);
-        cti.setAttribute("statusTooltiptext", typingMsg);
-        cti.setAttribute(
-          "statusMessage",
-          this.bundle.GetStringFromName("chat.isTyping")
-        );
-      } else if (typingState == Ci.prplIConvIM.TYPED) {
-        cti.setAttribute("typing", "paused");
-        let typedMsg = this.bundle.formatStringFromName(
-          "chat.contactHasStoppedTyping",
-          [name],
-          1
-        );
-        cti.setAttribute("statusTypeTooltiptext", typedMsg);
-        cti.setAttribute("statusTooltiptext", typedMsg);
-        cti.setAttribute(
-          "statusMessage",
-          this.bundle.GetStringFromName("chat.hasStoppedTyping")
-        );
-      }
-    }
-
     switchingToPanel() {
       if (this._visibleTimer) {
         return;
@@ -1649,14 +1593,13 @@
 
     updateConvStatus() {
       let cti = document.getElementById("conv-top-info");
-      cti.setAttribute(
-        "prplIcon",
-        this._conv.account.protocol.iconBaseURI + "icon.png"
-      );
+      cti.setProtocol(this._conv.account.protocol);
+
+      // Set the icon, potentially showing a fallback icon if this is an IM.
+      cti.setUserIcon(this._conv.convIconFilename, !this._conv.isChat);
 
       if (this._conv.isChat) {
         this.updateTopic();
-        cti.setAttribute("status", "chat");
         cti.setAttribute("displayName", this._conv.title);
       } else {
         let displayName = this._conv.title;
@@ -1664,28 +1607,41 @@
         let statusType = Ci.imIStatusInfo.STATUS_UNKNOWN;
 
         let buddy = this._conv.buddy;
-        if (!buddy || !buddy.account.connected) {
-          cti.removeAttribute("userIcon");
-        } else {
+        if (buddy?.account.connected) {
           displayName = buddy.displayName;
           statusText = buddy.statusText;
           statusType = buddy.statusType;
-          cti.setAttribute("userIcon", buddy.buddyIconFilename);
         }
-
         cti.setAttribute("displayName", displayName);
-        if (statusText) {
-          statusText = " - " + statusText;
+
+        let statusName;
+
+        let typingState = this._conv.typingState;
+        let typingName = this._currentTypingName || this._conv.title;
+
+        switch (typingState) {
+          case Ci.prplIConvIM.TYPING:
+            statusName = "active-typing";
+            statusText = this.bundle.formatStringFromName(
+              "chat.contactIsTyping",
+              [typingName],
+              1
+            );
+            break;
+          case Ci.prplIConvIM.TYPED:
+            statusName = "paused-typing";
+            statusText = this.bundle.formatStringFromName(
+              "chat.contactHasStoppedTyping",
+              [typingName],
+              1
+            );
+            break;
+          default:
+            statusName = Status.toAttribute(statusType);
+            statusText = Status.toLabel(statusType, statusText);
+            break;
         }
-        cti.setAttribute("statusMessageWithDash", statusText);
-        let statusString = Status.toLabel(statusType);
-        cti.setAttribute("statusMessage", statusString + statusText);
-        cti.setAttribute("status", Status.toAttribute(statusType));
-        cti.setAttribute("statusTypeTooltiptext", statusString);
-        cti.setAttribute("statusTooltiptext", statusString + statusText);
-        cti.removeAttribute("topicEditable");
-        cti.removeAttribute("noTopic");
-        this.updateTyping();
+        cti.setStatus(statusName, statusText);
       }
     }
 
@@ -1709,14 +1665,17 @@
       }
     }
 
+    /**
+     * Set up the shared conversation specific components (conversation browser
+     * references, status header, participants list, text input) for this
+     * conversation.
+     */
     initConversationUI() {
+      this._activeBuddies = {};
       if (this._conv.isChat) {
-        this.updateTopic();
         let cti = document.getElementById("conv-top-info");
         cti.setAttribute("displayName", this._conv.title);
-        cti.setAttribute("status", "chat");
 
-        this._activeBuddies = {};
         this.showParticipants();
 
         if (Services.prefs.getBoolPref("messenger.conversations.showNicks")) {
@@ -1734,6 +1693,20 @@
       this.initTextboxFormat();
     }
 
+    /**
+     * Change the UI Conversation attached to this component and its browser.
+     * Does not clear any existing messages in the conversation browser.
+     *
+     * @param {imIConversation} conv
+     */
+    changeConversation(conv) {
+      this._conv.removeObserver(this.observer);
+      this._conv = conv;
+      this._conv.addObserver(this.observer);
+      this.convBrowser._conv = conv;
+      this.initConversationUI();
+    }
+
     get editor() {
       return this.inputBox;
     }
@@ -1763,13 +1736,12 @@
         // tab. Forget the prplConversation so that it isn't
         // closed when destroying this binding.
         this._forgetConv();
-        return val;
+        return;
       }
       this._conv = val;
       this._conv.addObserver(this.observer);
       this.convBrowser.init(this._conv);
       this.initConversationUI();
-      return val;
     }
 
     get contentWindow() {
