@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* eslint-env webextensions */
+
 let { AddonManager } = ChromeUtils.import(
   "resource://gre/modules/AddonManager.jsm"
 );
@@ -12,6 +14,142 @@ let { MockRegistrar } = ChromeUtils.import(
   "resource://testing-common/MockRegistrar.jsm"
 );
 
+function ManagementScript() {
+  browser.test.onMessage.addListener((message, assertMessage, browserStyle) => {
+    if (message !== "check-style") {
+      return;
+    }
+    function verifyButton(buttonElement, expected) {
+      let buttonStyle = window.getComputedStyle(buttonElement);
+      let buttonBackgroundColor = buttonStyle.backgroundColor;
+      if (browserStyle && expected.hasBrowserStyleClass) {
+        browser.test.assertEq(
+          "rgb(9, 150, 248)",
+          buttonBackgroundColor,
+          assertMessage
+        );
+      } else {
+        browser.test.assertTrue(
+          buttonBackgroundColor !== "rgb(9, 150, 248)",
+          assertMessage
+        );
+      }
+    }
+
+    function verifyCheckboxOrRadio(element, expected) {
+      let style = window.getComputedStyle(element);
+      let styledBackground = element.checked
+        ? "rgb(9, 150, 248)"
+        : "rgb(255, 255, 255)";
+      if (browserStyle && expected.hasBrowserStyleClass) {
+        browser.test.assertEq(
+          styledBackground,
+          style.backgroundColor,
+          assertMessage
+        );
+      } else {
+        browser.test.assertTrue(
+          style.backgroundColor != styledBackground,
+          assertMessage
+        );
+      }
+    }
+
+    let normalButton = document.getElementById("normalButton");
+    let browserStyleButton = document.getElementById("browserStyleButton");
+    verifyButton(normalButton, { hasBrowserStyleClass: false });
+    verifyButton(browserStyleButton, { hasBrowserStyleClass: true });
+
+    let normalCheckbox1 = document.getElementById("normalCheckbox1");
+    let normalCheckbox2 = document.getElementById("normalCheckbox2");
+    let browserStyleCheckbox = document.getElementById("browserStyleCheckbox");
+    verifyCheckboxOrRadio(normalCheckbox1, { hasBrowserStyleClass: false });
+    verifyCheckboxOrRadio(normalCheckbox2, { hasBrowserStyleClass: false });
+    verifyCheckboxOrRadio(browserStyleCheckbox, {
+      hasBrowserStyleClass: true,
+    });
+
+    let normalRadio1 = document.getElementById("normalRadio1");
+    let normalRadio2 = document.getElementById("normalRadio2");
+    let browserStyleRadio = document.getElementById("browserStyleRadio");
+    verifyCheckboxOrRadio(normalRadio1, { hasBrowserStyleClass: false });
+    verifyCheckboxOrRadio(normalRadio2, { hasBrowserStyleClass: false });
+    verifyCheckboxOrRadio(browserStyleRadio, { hasBrowserStyleClass: true });
+
+    browser.test.notifyPass("management-ui-browser_style");
+  });
+  browser.test.sendMessage("management-ui-ready");
+}
+
+let extension;
+async function startExtension(browser_style) {
+  let cloud_file = {
+    name: "Mochitest",
+    management_url: "management.html",
+  };
+
+  switch (browser_style) {
+    case "true":
+      cloud_file.browser_style = true;
+      break;
+    case "false":
+      cloud_file.browser_style = false;
+      break;
+  }
+
+  extension = ExtensionTestUtils.loadExtension({
+    async background() {
+      browser.test.onMessage.addListener(async message => {
+        if (message != "set-configured") {
+          return;
+        }
+        let accounts = await browser.cloudFile.getAllAccounts();
+        for (let account of accounts) {
+          await browser.cloudFile.updateAccount(account.id, {
+            configured: true,
+          });
+        }
+        browser.test.sendMessage("ready");
+      });
+    },
+    files: {
+      "management.html": `<html>
+        <body>
+          <a id="a" href="https://www.example.com/">Click me!</a>
+          <button id="normalButton" name="button" class="default">Default</button>
+          <button id="browserStyleButton" name="button" class="browser-style default">Default</button>
+
+          <input id="normalCheckbox1" type="checkbox"/>
+          <input id="normalCheckbox2" type="checkbox"/><label>Checkbox</label>
+          <div class="browser-style">
+            <input id="browserStyleCheckbox" type="checkbox"><label for="browserStyleCheckbox">Checkbox</label>
+          </div>
+
+          <input id="normalRadio1" type="radio"/>
+          <input id="normalRadio2" type="radio"/><label>Radio</label>
+          <div class="browser-style">
+            <input id="browserStyleRadio" checked="" type="radio"><label for="browserStyleRadio">Radio</label>
+          </div>
+        </body>
+        <script src="management.js" type="text/javascript"></script>
+      </html>`,
+      "management.js": ManagementScript,
+    },
+    manifest: {
+      cloud_file,
+      applications: { gecko: { id: "cloudfile@mochitest" } },
+    },
+  });
+
+  info("Starting extension");
+  await extension.startup();
+
+  if (accountIsConfigured) {
+    extension.sendMessage("set-configured");
+    await extension.awaitMessage("ready");
+  }
+}
+
 add_task(async () => {
   // Register a fake provider representing a built-in provider. We don't
   // currently ship any built-in providers, but if we did, we should check
@@ -26,29 +164,7 @@ add_task(async () => {
   });
 });
 
-const ICON_URL = getRootDirectory(gTestPath) + "files/icon.svg";
-const MANAGEMENT_URL = getRootDirectory(gTestPath) + "files/management.html";
 let accountIsConfigured = false;
-let provider = {
-  type: "Mochitest",
-  displayName: "Mochitest",
-  iconURL: ICON_URL,
-  initAccount(accountKey) {
-    return {
-      accountKey,
-      type: "Mochitest",
-      get displayName() {
-        return Services.prefs.getCharPref(
-          `mail.cloud_files.accounts.${this.accountKey}.displayName`,
-          "Mochitest Account"
-        );
-      },
-      iconURL: ICON_URL,
-      configured: accountIsConfigured,
-      managementURL: MANAGEMENT_URL,
-    };
-  },
-};
 
 // Mock the prompt service. We're going to be asked if we're sure
 // we want to remove an account, so let's say yes.
@@ -60,7 +176,7 @@ let mockPromptService = {
     this.confirmCount++;
     return true;
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsIPromptService]),
+  QueryInterface: ChromeUtils.generateQI(["nsIPromptService"]),
 };
 /** @implements {nsIExternalProtocolService} */
 let mockExternalProtocolService = {
@@ -77,20 +193,19 @@ let mockExternalProtocolService = {
   urlLoaded(aURL) {
     return this._loadedURLs.includes(aURL);
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsIExternalProtocolService]),
+  QueryInterface: ChromeUtils.generateQI(["nsIExternalProtocolService"]),
 };
 
-let mockPromptServiceCID = MockRegistrar.register(
-  "@mozilla.org/embedcomp/prompt-service;1",
-  mockPromptService
-);
+let originalPromptService = Services.prompt;
+Services.prompt = mockPromptService;
+
 let mockExternalProtocolServiceCID = MockRegistrar.register(
   "@mozilla.org/uriloader/external-protocol-service;1",
   mockExternalProtocolService
 );
 
 registerCleanupFunction(() => {
-  MockRegistrar.unregister(mockPromptServiceCID);
+  Services.prompt = originalPromptService;
   MockRegistrar.unregister(mockExternalProtocolServiceCID);
 });
 
@@ -129,15 +244,17 @@ add_task(async function addRemoveAccounts() {
   let removeButton = prefsDocument.getElementById("removeCloudFileAccount");
   ok(removeButton.disabled);
 
-  let settingsDeck = prefsDocument.getElementById("cloudFileSettingsDeck");
-  is(settingsDeck.selectedPanel.id, "cloudFileDefaultPanel");
+  let cloudFileDefaultPanel = prefsDocument.getElementById(
+    "cloudFileDefaultPanel"
+  );
+  ok(!cloudFileDefaultPanel.hidden);
 
-  let iframeWrapper = prefsDocument.getElementById("cloudFileSettingsWrapper");
-  is(iframeWrapper.childElementCount, 0);
+  let browserWrapper = prefsDocument.getElementById("cloudFileSettingsWrapper");
+  is(browserWrapper.childElementCount, 0);
 
   // Register our test provider.
 
-  cloudFileAccounts.registerProvider("Mochitest", provider);
+  await startExtension();
   is(cloudFileAccounts.providers.length, 2);
   is(cloudFileAccounts.accounts.length, 0);
 
@@ -148,16 +265,25 @@ add_task(async function addRemoveAccounts() {
     buttonList.children[0].getAttribute("value"),
     "ext-fake@extensions.thunderbird.net"
   );
-  is(buttonList.children[1].getAttribute("value"), "Mochitest");
-  is(buttonList.children[1].style.listStyleImage, `url("${ICON_URL}")`);
+  is(buttonList.children[1].getAttribute("value"), "ext-cloudfile@mochitest");
+  is(
+    buttonList.children[1].style.listStyleImage,
+    `url("chrome://messenger/content/extension.svg")`
+  );
 
   is(menuButton.itemCount, 2);
   is(
     menuButton.getItemAtIndex(0).getAttribute("value"),
     "ext-fake@extensions.thunderbird.net"
   );
-  is(menuButton.getItemAtIndex(1).getAttribute("value"), "Mochitest");
-  is(menuButton.getItemAtIndex(1).getAttribute("image"), ICON_URL);
+  is(
+    menuButton.getItemAtIndex(1).getAttribute("value"),
+    "ext-cloudfile@mochitest"
+  );
+  is(
+    menuButton.getItemAtIndex(1).getAttribute("image"),
+    "chrome://messenger/content/extension.svg"
+  );
 
   // Create a new account.
 
@@ -171,7 +297,7 @@ add_task(async function addRemoveAccounts() {
 
   let account = cloudFileAccounts.accounts[0];
   let accountKey = account.accountKey;
-  is(cloudFileAccounts.accounts[0].type, "Mochitest");
+  is(cloudFileAccounts.accounts[0].type, "ext-cloudfile@mochitest");
 
   // Check prefs were updated.
 
@@ -179,11 +305,11 @@ add_task(async function addRemoveAccounts() {
     Services.prefs.getCharPref(
       `mail.cloud_files.accounts.${accountKey}.displayName`
     ),
-    "Mochitest Account"
+    "Mochitest"
   );
   is(
     Services.prefs.getCharPref(`mail.cloud_files.accounts.${accountKey}.type`),
-    "Mochitest"
+    "ext-cloudfile@mochitest"
   );
 
   // Check UI was updated.
@@ -194,28 +320,40 @@ add_task(async function addRemoveAccounts() {
 
   let accountListItem = accountList.selectedItem;
   is(accountListItem.getAttribute("value"), accountKey);
-  is(accountListItem.style.listStyleImage, `url("${ICON_URL}")`);
-  is(accountListItem.querySelector("label").value, "Mochitest Account");
-  is(accountListItem.querySelector("image.configuredWarning").hidden, false);
-
-  is(settingsDeck.selectedPanel.id, "cloudFileSettingsWrapper");
-  is(iframeWrapper.childElementCount, 1);
-
-  let iframe = iframeWrapper.firstElementChild;
-  is(iframe.src, `${MANAGEMENT_URL}?accountId=${accountKey}`);
-  if (iframe.contentDocument.readyState != "complete") {
-    await BrowserTestUtils.waitForEvent(iframe, "load");
-  }
-  is(iframe.contentDocument.readyState, "complete");
-  EventUtils.synthesizeMouseAtCenter(
-    iframe.contentDocument.getElementById("a"),
-    {},
-    iframe.contentWindow
+  is(
+    accountListItem.querySelector(".typeIcon:not(.configuredWarning)").src,
+    "chrome://messenger/content/extension.svg"
   );
+  is(accountListItem.querySelector("label").value, "Mochitest");
+  is(accountListItem.querySelector(".configuredWarning").hidden, false);
+
+  ok(cloudFileDefaultPanel.hidden);
+  is(browserWrapper.childElementCount, 1);
+
+  let browser = browserWrapper.firstElementChild;
+  if (
+    browser.webProgress?.isLoadingDocument ||
+    browser.currentURI?.spec == "about:blank"
+  ) {
+    await BrowserTestUtils.browserLoaded(browser);
+  }
+  is(
+    browser.currentURI.pathQueryRef,
+    `/management.html?accountId=${accountKey}`
+  );
+  await extension.awaitMessage("management-ui-ready");
+
+  let tabmail = document.getElementById("tabmail");
+  let tabCount = tabmail.tabInfo.length;
+  BrowserTestUtils.synthesizeMouseAtCenter("a", {}, browser);
+  // It might take a moment to get to the external protocol service.
+  // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+  await new Promise(resolve => setTimeout(resolve, 500));
   ok(
     mockExternalProtocolService.urlLoaded("https://www.example.com/"),
     "Link click sent to external protocol service."
   );
+  is(tabmail.tabInfo.length, tabCount, "No new tab opened");
 
   // Rename the account.
 
@@ -233,7 +371,7 @@ add_task(async function addRemoveAccounts() {
   );
   ok(accountListItem.querySelector("label").hidden);
   ok(!accountListItem.querySelector("input").hidden);
-  is(accountListItem.querySelector("input").value, "Mochitest Account");
+  is(accountListItem.querySelector("input").value, "Mochitest");
   EventUtils.synthesizeKey("VK_RIGHT", undefined, prefsWindow);
   EventUtils.synthesizeKey("!", undefined, prefsWindow);
   EventUtils.synthesizeKey("VK_RETURN", undefined, prefsWindow);
@@ -242,13 +380,13 @@ add_task(async function addRemoveAccounts() {
 
   is(prefsDocument.activeElement, accountList);
   ok(!accountListItem.querySelector("label").hidden);
-  is(accountListItem.querySelector("label").value, "Mochitest Account!");
+  is(accountListItem.querySelector("label").value, "Mochitest!");
   ok(accountListItem.querySelector("input").hidden);
   is(
     Services.prefs.getCharPref(
       `mail.cloud_files.accounts.${accountKey}.displayName`
     ),
-    "Mochitest Account!"
+    "Mochitest!"
   );
 
   // Start to rename the account, but bail out.
@@ -275,13 +413,13 @@ add_task(async function addRemoveAccounts() {
 
   is(prefsDocument.activeElement, accountList);
   ok(!accountListItem.querySelector("label").hidden);
-  is(accountListItem.querySelector("label").value, "Mochitest Account!");
+  is(accountListItem.querySelector("label").value, "Mochitest!");
   ok(accountListItem.querySelector("input").hidden);
   is(
     Services.prefs.getCharPref(
       `mail.cloud_files.accounts.${accountKey}.displayName`
     ),
-    "Mochitest Account!"
+    "Mochitest!"
   );
 
   // Configure the account.
@@ -292,13 +430,14 @@ add_task(async function addRemoveAccounts() {
 
   await new Promise(resolve => prefsWindow.requestAnimationFrame(resolve));
 
-  is(accountListItem.querySelector("image.configuredWarning").hidden, true);
+  is(accountListItem.querySelector(".configuredWarning").hidden, true);
   is(cloudFileAccounts.accounts.length, 1);
   is(cloudFileAccounts.configuredAccounts.length, 1);
 
-  // Remove the test provider. The list item, button, and iframe should disappear.
+  // Remove the test provider. The list item, button, and browser should disappear.
 
-  cloudFileAccounts.unregisterProvider("Mochitest");
+  info("Stopping extension");
+  await extension.unload();
   is(cloudFileAccounts.providers.length, 1);
   is(cloudFileAccounts.accounts.length, 0);
 
@@ -315,12 +454,13 @@ add_task(async function addRemoveAccounts() {
     "ext-fake@extensions.thunderbird.net"
   );
   is(accountList.itemCount, 0);
-  is(settingsDeck.selectedPanel.id, "cloudFileDefaultPanel");
-  is(iframeWrapper.childElementCount, 0);
+  ok(!cloudFileDefaultPanel.hidden);
+  is(browserWrapper.childElementCount, 0);
 
   // Re-add the test provider.
 
-  cloudFileAccounts.registerProvider("Mochitest", provider);
+  await startExtension();
+
   is(cloudFileAccounts.providers.length, 2);
   is(cloudFileAccounts.accounts.length, 1);
   is(cloudFileAccounts.configuredAccounts.length, 1);
@@ -332,14 +472,17 @@ add_task(async function addRemoveAccounts() {
     buttonList.children[0].getAttribute("value"),
     "ext-fake@extensions.thunderbird.net"
   );
-  is(buttonList.children[1].getAttribute("value"), "Mochitest");
+  is(buttonList.children[1].getAttribute("value"), "ext-cloudfile@mochitest");
 
   is(menuButton.itemCount, 2);
   is(
     menuButton.getItemAtIndex(0).getAttribute("value"),
     "ext-fake@extensions.thunderbird.net"
   );
-  is(menuButton.getItemAtIndex(1).getAttribute("value"), "Mochitest");
+  is(
+    menuButton.getItemAtIndex(1).getAttribute("value"),
+    "ext-cloudfile@mochitest"
+  );
 
   is(accountList.itemCount, 1);
   is(accountList.selectedIndex, -1);
@@ -350,7 +493,7 @@ add_task(async function addRemoveAccounts() {
     Services.prefs.getCharPref(
       `mail.cloud_files.accounts.${accountKey}.displayName`
     ),
-    "Mochitest Account!"
+    "Mochitest!"
   );
 
   EventUtils.synthesizeMouseAtCenter(
@@ -380,7 +523,8 @@ add_task(async function addRemoveAccounts() {
   is(cloudFileAccounts.providers.length, 2);
   is(cloudFileAccounts.accounts.length, 0);
 
-  cloudFileAccounts.unregisterProvider("Mochitest", provider);
+  info("Stopping extension");
+  await extension.unload();
   is(cloudFileAccounts.providers.length, 1);
   is(cloudFileAccounts.accounts.length, 0);
 
@@ -389,13 +533,153 @@ add_task(async function addRemoveAccounts() {
   await closePrefsTab();
 });
 
+async function subtestBrowserStyle(assertMessage, expected) {
+  is(cloudFileAccounts.providers.length, 1);
+  is(cloudFileAccounts.accounts.length, 0);
+
+  // Load the preferences tab.
+
+  let { prefsDocument, prefsWindow } = await openNewPrefsTab(
+    "paneCompose",
+    "compositionAttachmentsCategory"
+  );
+
+  // Minimal check everything is as it should be.
+
+  let accountList = prefsDocument.getElementById("cloudFileView");
+  is(accountList.itemCount, 0);
+
+  let buttonList = prefsDocument.getElementById("addCloudFileAccountButtons");
+  ok(!buttonList.hidden);
+
+  let browserWrapper = prefsDocument.getElementById("cloudFileSettingsWrapper");
+  is(browserWrapper.childElementCount, 0);
+
+  // Register our test provider.
+
+  await startExtension(expected.browser_style);
+  is(cloudFileAccounts.providers.length, 2);
+  is(cloudFileAccounts.accounts.length, 0);
+
+  await new Promise(resolve => prefsWindow.requestAnimationFrame(resolve));
+
+  is(buttonList.childElementCount, 2);
+  is(buttonList.children[1].getAttribute("value"), "ext-cloudfile@mochitest");
+
+  // Create a new account.
+
+  EventUtils.synthesizeMouseAtCenter(
+    buttonList.children[1],
+    { clickCount: 1 },
+    prefsWindow
+  );
+  is(cloudFileAccounts.accounts.length, 1);
+  is(cloudFileAccounts.configuredAccounts.length, 0);
+
+  let account = cloudFileAccounts.accounts[0];
+  let accountKey = account.accountKey;
+  is(cloudFileAccounts.accounts[0].type, "ext-cloudfile@mochitest");
+
+  // Minimal check UI was updated.
+
+  is(accountList.itemCount, 1);
+  is(accountList.selectedIndex, 0);
+
+  let accountListItem = accountList.selectedItem;
+  is(accountListItem.getAttribute("value"), accountKey);
+
+  is(browserWrapper.childElementCount, 1);
+  let browser = browserWrapper.firstElementChild;
+  if (
+    browser.webProgress?.isLoadingDocument ||
+    browser.currentURI?.spec == "about:blank"
+  ) {
+    await BrowserTestUtils.browserLoaded(browser);
+  }
+  is(
+    browser.currentURI.pathQueryRef,
+    `/management.html?accountId=${accountKey}`
+  );
+  await extension.awaitMessage("management-ui-ready");
+
+  // Test browser_style
+
+  extension.sendMessage(
+    "check-style",
+    assertMessage,
+    expected.browser_style == "true"
+  );
+  await extension.awaitFinish("management-ui-browser_style");
+
+  // Remove the account
+
+  accountListItem = accountList.getItemAtIndex(0);
+  EventUtils.synthesizeMouseAtCenter(
+    accountList.getItemAtIndex(0),
+    { clickCount: 1 },
+    prefsWindow
+  );
+
+  let removeButton = prefsDocument.getElementById("removeCloudFileAccount");
+  ok(!removeButton.disabled);
+  EventUtils.synthesizeMouseAtCenter(
+    removeButton,
+    { clickCount: 1 },
+    prefsWindow
+  );
+  is(mockPromptService.confirmCount, expected.confirmCount);
+
+  is(cloudFileAccounts.providers.length, 2);
+  is(cloudFileAccounts.accounts.length, 0);
+
+  info("Stopping extension");
+  await extension.unload();
+  is(cloudFileAccounts.providers.length, 1);
+  is(cloudFileAccounts.accounts.length, 0);
+
+  // Close the preferences tab.
+
+  await closePrefsTab();
+}
+
+add_task(async function test_without_setting_browser_style() {
+  await subtestBrowserStyle(
+    "Expected correct style when browser_style is excluded",
+    {
+      confirmCount: 2,
+      browser_style: "default",
+    }
+  );
+});
+
+add_task(async function test_with_browser_style_set_to_true() {
+  await subtestBrowserStyle(
+    "Expected correct style when browser_style is set to `true`",
+    {
+      confirmCount: 3,
+      browser_style: "true",
+    }
+  );
+});
+
+add_task(async function test_with_browser_style_set_to_false() {
+  await subtestBrowserStyle(
+    "Expected no style when browser_style is set to `false`",
+    {
+      confirmCount: 4,
+      browser_style: "false",
+    }
+  );
+});
+
 add_task(async function accountListOverflow() {
   is(cloudFileAccounts.providers.length, 1);
   is(cloudFileAccounts.accounts.length, 0);
 
   // Register our test provider.
 
-  cloudFileAccounts.registerProvider("Mochitest", provider);
+  await startExtension();
+
   is(cloudFileAccounts.providers.length, 2);
   is(cloudFileAccounts.accounts.length, 0);
 
@@ -412,7 +696,7 @@ add_task(async function accountListOverflow() {
   let buttonList = prefsDocument.getElementById("addCloudFileAccountButtons");
   ok(!buttonList.hidden);
   is(buttonList.childElementCount, 2);
-  is(buttonList.children[0].getAttribute("value"), "Mochitest");
+  is(buttonList.children[0].getAttribute("value"), "ext-cloudfile@mochitest");
 
   let menuButton = prefsDocument.getElementById("addCloudFileAccount");
   ok(menuButton.hidden);
@@ -422,12 +706,15 @@ add_task(async function accountListOverflow() {
 
   let count = 0;
   do {
+    let readyPromise = extension.awaitMessage("management-ui-ready");
     EventUtils.synthesizeMouseAtCenter(
       buttonList.children[0],
       { clickCount: 1 },
       prefsWindow
     );
-    await new Promise(resolve => setTimeout(resolve));
+    await readyPromise;
+    // eslint-disable-next-line mozilla/no-arbitrary-setTimeout
+    await new Promise(r => setTimeout(r, 500));
     if (buttonList.hidden) {
       break;
     }
@@ -460,7 +747,8 @@ add_task(async function accountListOverflow() {
   // Close the preferences tab.
 
   await closePrefsTab();
-  cloudFileAccounts.unregisterProvider("Mochitest");
+  info("Stopping extension");
+  await extension.unload();
   Services.prefs.deleteBranch("mail.cloud_files.accounts");
 });
 
@@ -476,7 +764,7 @@ add_task(async function accountListOrder() {
   ]) {
     Services.prefs.setCharPref(
       `mail.cloud_files.accounts.${key}.type`,
-      "Mochitest"
+      "ext-cloudfile@mochitest"
     );
     Services.prefs.setCharPref(
       `mail.cloud_files.accounts.${key}.displayName`,
@@ -486,7 +774,8 @@ add_task(async function accountListOrder() {
 
   // Register our test provider.
 
-  cloudFileAccounts.registerProvider("Mochitest", provider);
+  await startExtension();
+
   is(cloudFileAccounts.providers.length, 2);
   is(cloudFileAccounts.accounts.length, 4);
 
@@ -504,6 +793,7 @@ add_task(async function accountListOrder() {
   is(accountList.getItemAtIndex(3).value, "someKey1");
 
   await closePrefsTab();
-  cloudFileAccounts.unregisterProvider("Mochitest");
+  info("Stopping extension");
+  await extension.unload();
   Services.prefs.deleteBranch("mail.cloud_files.accounts");
 });
