@@ -1192,6 +1192,12 @@ Enigmail.hdrView = {
     },
 
     handleSMimeMessage(uri) {
+      if (
+        Enigmail.hdrView.msgSignedStateString != null ||
+        Enigmail.hdrView.msgEncryptedStateString != null
+      ) {
+        return;
+      }
       if (this.isCurrentMessage(uri)) {
         EnigmailVerify.unregisterContentTypeHandler();
         Enigmail.msg.messageReload(false);
