@@ -2,13 +2,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-// Uses: chrome://openpgp/content/ui/enigmailCommon.js
-
 "use strict";
 
-// Modules
-/* global EnigmailApp: false, EnigmailKeyRing: false, GetEnigmailSvc: false,
-   EnigSavePrefs: false, EnigmailWindows: false, PgpSqliteDb2: false */
+/* global GetEnigmailSvc */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
@@ -17,21 +13,27 @@ var { MailServices } = ChromeUtils.import(
 var { AppConstants } = ChromeUtils.import(
   "resource://gre/modules/AppConstants.jsm"
 );
-var EnigmailCryptoAPI = ChromeUtils.import(
+var { EnigmailCryptoAPI } = ChromeUtils.import(
   "chrome://openpgp/content/modules/cryptoAPI.jsm"
-).EnigmailCryptoAPI;
-var { EnigmailFiles } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/files.jsm"
 );
-var OpenPGPMasterpass = ChromeUtils.import(
+var { OpenPGPMasterpass } = ChromeUtils.import(
   "chrome://openpgp/content/modules/masterpass.jsm"
-).OpenPGPMasterpass;
-var EnigmailDialog = ChromeUtils.import(
+);
+var { EnigmailDialog } = ChromeUtils.import(
   "chrome://openpgp/content/modules/dialog.jsm"
-).EnigmailDialog;
+);
 var { EnigmailKey } = ChromeUtils.import(
   "chrome://openpgp/content/modules/key.jsm"
 );
+var { EnigmailKeyRing } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/keyRing.jsm"
+);
+var { EnigmailWindows } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/windows.jsm"
+);
+var { PgpSqliteDb2 } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/sqliteDb.jsm"
+);
 var { RNP } = ChromeUtils.import("chrome://openpgp/content/modules/RNP.jsm");
 
 // UI variables.
@@ -118,6 +120,11 @@ async function init() {
   // Switch directly to the import screen if requested by the user.
   if (window.arguments[0].isImport) {
     document.getElementById("openPgpKeyChoices").value = 1;
+
+    // Disable the "Continue" button so the user can't accidentally click on it.
+    // See bug 1689980.
+    kDialog.getButton("accept").setAttribute("disabled", true);
+
     switchSection();
   }
 }
@@ -176,15 +183,15 @@ async function setIdentity() {
     gIdentity = MailServices.accounts.getIdentity(
       gIdentityList.selectedItem.getAttribute("identitykey")
     );
-  }
 
-  document.l10n.setAttributes(
-    document.documentElement,
-    "key-wizard-dialog-window",
-    {
-      identity: gIdentity.email,
-    }
-  );
+    document.l10n.setAttributes(
+      document.documentElement,
+      "key-wizard-dialog-window",
+      {
+        identity: gIdentity.email,
+      }
+    );
+  }
 }
 
 /**
@@ -321,8 +328,10 @@ async function addImportWarningNotification() {
   let wrapper = document.createXULElement("hbox");
   wrapper.classList.add("inline-notification-wrapper", "align-center");
 
-  let image = document.createXULElement("image");
-  image.classList.add("notification-image", "notification-image-error");
+  let image = document.createElement("img");
+  image.classList.add("notification-image");
+  image.setAttribute("src", "chrome://global/skin/icons/warning.svg");
+  image.setAttribute("alt", "");
 
   let description = document.createXULElement("description");
 
@@ -530,7 +539,7 @@ function resizeDialog() {
   if (gSubDialog && gSubDialog._topDialog) {
     gSubDialog._topDialog.resizeVertically();
   } else {
-    sizeToContent();
+    window.sizeToContent();
   }
 }
 
@@ -604,22 +613,19 @@ async function openPgpKeygenConfirm() {
 
   kGenerating = true;
 
-  let cApi;
+  let cApi = EnigmailCryptoAPI();
   try {
     let newId = null;
-    cApi = EnigmailCryptoAPI();
     let pass = await OpenPGPMasterpass.retrieveOpenPGPPassword();
-    newId = cApi.sync(
-      cApi.genKey(
-        `${gIdentity.fullName} <${gIdentity.email}>`,
-        document.getElementById("keyType").value,
-        Number(document.getElementById("keySize").value),
-        document.getElementById("openPgpKeygeExpiry").value == 1
-          ? 0
-          : Number(document.getElementById("expireInput").value) *
-              Number(document.getElementById("timeScale").value),
-        pass
-      )
+    newId = await cApi.genKey(
+      `${gIdentity.fullName} <${gIdentity.email}>`,
+      document.getElementById("keyType").value,
+      Number(document.getElementById("keySize").value),
+      document.getElementById("openPgpKeygeExpiry").value == 1
+        ? 0
+        : Number(document.getElementById("expireInput").value) *
+            Number(document.getElementById("timeScale").value),
+      pass
     );
     console.log("created new key with id: " + newId);
     gGeneratedKey = newId;
@@ -646,13 +652,13 @@ async function openPgpKeygenConfirm() {
   }
 
   console.debug("saving new key id " + gGeneratedKey);
-  EnigSavePrefs();
+  Services.prefs.savePrefFile(null);
 
   // Hide wizard overlay at the end of the generation process.
   closeOverlay();
   EnigmailKeyRing.clearCache();
 
-  let rev = cApi.sync(cApi.getNewRevocation(`0x${gGeneratedKey}`));
+  let rev = await cApi.getNewRevocation(`0x${gGeneratedKey}`);
   if (!rev) {
     openPgpWarning.collapsed = false;
     document.l10n.setAttributes(
@@ -675,16 +681,16 @@ async function openPgpKeygenConfirm() {
     revocationFilePrefix2 +
     rev;
 
-  let revFile = EnigmailApp.getProfileDirectory();
+  let revFile = Services.dirsvc.get("ProfD", Ci.nsIFile);
   revFile.append(`0x${gGeneratedKey}_rev.asc`);
 
-  // Create a revokation cert in the Thunderbird profile directoy.
-  EnigmailFiles.writeFileContents(revFile, revFull, DEFAULT_FILE_PERMS);
+  // Create a revokation cert in the Thunderbird profile directory.
+  await IOUtils.writeUTF8(revFile.path, revFull);
 
-  // Key succesfully created. Assign the new key to the current identity, close
-  // the dialog and show a confirmation message.
-  gIdentity.setUnicharAttribute("openpgp_key_id", gGeneratedKey);
-  window.arguments[0].okCallback();
+  // Key successfully created. Close the dialog and show a confirmation message.
+  // Assigning the key to an identity is the responsibility of the caller,
+  // so we pass back what we created.
+  window.arguments[0].okCallback(gGeneratedKey);
   window.close();
 }
 
@@ -789,7 +795,7 @@ async function importSecretKey() {
 
     let errorMsgObj = {};
     // Fetch the list of all the available keys inside the selected file.
-    let importKeys = EnigmailKey.getKeyListFromKeyFile(
+    let importKeys = await EnigmailKey.getKeyListFromKeyFile(
       file,
       errorMsgObj,
       false,
@@ -801,7 +807,9 @@ async function importSecretKey() {
       document.l10n.setAttributes(
         await addImportWarningNotification(),
         "import-error-failed",
-        { error: errorMsgObj.value }
+        {
+          error: errorMsgObj.value,
+        }
       );
       continue;
     }
@@ -816,8 +824,10 @@ async function importSecretKey() {
   // Update the list count recap and show the container.
   document.l10n.setAttributes(
     document.getElementById("keyListCount"),
-    "openpgp-import-key-list-amount",
-    { count: keyCount }
+    "openpgp-import-key-list-amount-2",
+    {
+      count: keyCount,
+    }
   );
 
   document.getElementById("importKeyListContainer").collapsed = !keyCount;
@@ -899,14 +909,12 @@ async function openPgpImportStart() {
     let resultKeys = {};
     let errorMsgObj = {};
 
-    let exitCode = EnigmailKeyRing.importKeyFromFile(
+    let exitCode = await EnigmailKeyRing.importSecKeyFromFile(
       window,
       passphrasePromptCallback,
       file,
       errorMsgObj,
-      resultKeys,
-      false,
-      true
+      resultKeys
     );
 
     // Skip this file if something went wrong.
@@ -914,7 +922,9 @@ async function openPgpImportStart() {
       document.l10n.setAttributes(
         await addImportWarningNotification(),
         "openpgp-import-keys-failed",
-        { error: errorMsgObj.value }
+        {
+          error: errorMsgObj.value,
+        }
       );
       continue;
     }
@@ -1104,12 +1114,8 @@ function openPgpExternalComplete() {
   gIdentity.setBoolAttribute("is_gnupg_key_id", true);
 
   let externalKey = document.getElementById("externalKey").value;
-  gIdentity.setUnicharAttribute(
-    "last_entered_external_gnupg_key_id",
-    externalKey
-  );
   gIdentity.setUnicharAttribute("openpgp_key_id", externalKey);
 
-  window.arguments[0].okExternalCallback();
+  window.arguments[0].okExternalCallback(externalKey);
   window.close();
 }
