@@ -23,13 +23,11 @@ var {
   add_cloud_attachments,
   close_compose_window,
   open_compose_new_mail,
+  delete_attachment,
 } = ChromeUtils.import("resource://testing-common/mozmill/ComposeHelpers.jsm");
 var { mc } = ChromeUtils.import(
   "resource://testing-common/mozmill/FolderDisplayHelpers.jsm"
 );
-var mozmill = ChromeUtils.import(
-  "resource://testing-common/mozmill/mozmill.jsm"
-);
 var {
   assert_notification_displayed,
   close_notification,
@@ -43,11 +41,7 @@ var { gMockPromptService } = ChromeUtils.import(
 );
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { cloudFileAccounts } = ChromeUtils.import(
-  "resource:///modules/cloudFileAccounts.jsm"
-);
 
-var controller = mozmill.getMail3PaneController();
 var maxSize, oldInsertNotificationPref;
 
 var kOfferThreshold = "mail.compose.big_attachments.threshold_kb";
@@ -56,7 +50,7 @@ var kInsertNotificationPref =
 
 var kBoxId = "compose-notification-bottom";
 
-add_task(function setupModule(module) {
+add_setup(function() {
   requestLongerTimeout(2);
 
   gMockCloudfileManager.register();
@@ -69,7 +63,7 @@ add_task(function setupModule(module) {
   Services.prefs.setBoolPref(kInsertNotificationPref, true);
 });
 
-registerCleanupFunction(function teardownModule(module) {
+registerCleanupFunction(function() {
   gMockCloudfileManager.unregister();
   gMockFilePickReg.unregister();
   Services.prefs.setBoolPref(
@@ -302,9 +296,10 @@ add_task(function test_link_insertion_goes_away_on_error() {
 
 /**
  * Test that we do not show the Filelink offer notification if we convert
- * a Filelink back into a normal attachment.
+ * a Filelink back into a normal attachment. Also test, that the privacy
+ * notification is correctly shown and hidden.
  */
-add_task(function test_no_offer_on_conversion() {
+add_task(async function test_no_offer_on_conversion() {
   const kFiles = ["./data/testFile1", "./data/testFile2"];
   // Set the notification threshold to 0 to ensure that we get it.
   Services.prefs.setIntPref(kOfferThreshold, 0);
@@ -317,9 +312,12 @@ add_task(function test_no_offer_on_conversion() {
   // Override uploadFile to succeed instantaneously so that we don't have
   // to worry about waiting for the onStopRequest method being called
   // asynchronously.
-  provider.uploadFile = function(aFile) {
+  provider.uploadFile = function(window, aFile) {
     return Promise.resolve({
       id: 1,
+      url: "https://some.cloud.net/1",
+      path: aFile.path,
+      size: aFile.fileSize,
     });
   };
 
@@ -327,11 +325,20 @@ add_task(function test_no_offer_on_conversion() {
   add_cloud_attachments(cw, provider, false);
 
   assert_cloudfile_notification_displayed(cw, false);
+  assert_privacy_warning_notification_displayed(cw, true);
+
   // Now convert the file back into a normal attachment
   select_attachments(cw, 0);
-  cw.window.convertSelectedToRegularAttachment();
-
+  await cw.window.convertSelectedToRegularAttachment();
   assert_cloudfile_notification_displayed(cw, false);
+  assert_privacy_warning_notification_displayed(cw, true);
+
+  // Convert also the other file, the privacy notification should no longer
+  // be shown as well.
+  select_attachments(cw, 1);
+  await cw.window.convertSelectedToRegularAttachment();
+  assert_cloudfile_notification_displayed(cw, false);
+  assert_privacy_warning_notification_displayed(cw, false);
 
   close_compose_window(cw);
 
@@ -343,7 +350,7 @@ add_task(function test_no_offer_on_conversion() {
  * Test that when we kick off an upload via the offer notification, then
  * the upload notification is shown.
  */
-add_task(function test_offer_then_upload_notifications() {
+add_task(async function test_offer_then_upload_notifications() {
   const kFiles = ["./data/testFile1", "./data/testFile2"];
   // Set the notification threshold to 0 to ensure that we get it.
   Services.prefs.setIntPref(kOfferThreshold, 0);
@@ -362,9 +369,12 @@ add_task(function test_offer_then_upload_notifications() {
   // Override uploadFile to succeed instantaneously so that we don't have
   // to worry about waiting for the onStopRequest method being called
   // asynchronously.
-  provider.uploadFile = function(aFile) {
+  provider.uploadFile = function(window, aFile) {
     return Promise.resolve({
       id: 1,
+      url: "https://some.cloud.net/1",
+      path: aFile.path,
+      size: aFile.fileSize,
     });
   };
 
@@ -379,7 +389,7 @@ add_task(function test_offer_then_upload_notifications() {
   // them.
   select_attachments(cw, 0, 1);
   // Convert them.
-  cw.window.convertSelectedToCloudAttachment(provider);
+  await cw.window.convertSelectedToCloudAttachment(provider);
 
   // The offer should now be gone...
   assert_cloudfile_notification_displayed(cw, false);
@@ -432,6 +442,42 @@ add_task(function test_privacy_warning_notification() {
   gMockPromptService.unregister();
 });
 
+/**
+ * Test that when all cloud attachments are removed, the privacy warning will
+ * be removed as well.
+ */
+add_task(function test_privacy_warning_notification() {
+  gMockPromptService.register();
+  gMockPromptService.returnValue = false;
+  gMockFilePicker.returnFiles = collectFiles([
+    "./data/testFile1",
+    "./data/testFile2",
+  ]);
+  let provider = new MockCloudfileAccount();
+  provider.init("aKey");
+
+  let cwc = open_compose_new_mail(mc);
+  add_cloud_attachments(cwc, provider, false);
+
+  wait_for_notification_to_show(cwc, kBoxId, "bigAttachmentUploading");
+  gMockCloudfileManager.resolveUploads();
+  wait_for_notification_to_stop(cwc, kBoxId, "bigAttachmentUploading");
+
+  // Assert that the warning is displayed.
+  assert_privacy_warning_notification_displayed(cwc, true);
+
+  // Assert that the warning is still displayed, if one attachment is removed.
+  delete_attachment(cwc, 1);
+  assert_privacy_warning_notification_displayed(cwc, true);
+
+  // Assert that the warning is not displayed, after both attachments are removed.
+  delete_attachment(cwc, 0);
+  assert_privacy_warning_notification_displayed(cwc, false);
+
+  close_compose_window(cwc);
+  gMockPromptService.unregister();
+});
+
 /**
  * Test that the privacy warning notification does not persist when closing
  * and re-opening a compose window.
