@@ -6,7 +6,6 @@
 #include "nsAbDirProperty.h"
 #include "nsAbBaseCID.h"
 #include "nsIAbCard.h"
-#include "nsDirPrefs.h"
 #include "nsIPrefService.h"
 #include "nsIPrefLocalizedString.h"
 #include "nsServiceManagerUtils.h"
@@ -16,13 +15,18 @@
 #include "nsArrayUtils.h"
 #include "nsIUUIDGenerator.h"
 #include "mozilla/Services.h"
+#include "nsIObserverService.h"
+#include "mozilla/dom/Promise.h"
+
+using mozilla::ErrorResult;
+using mozilla::dom::Promise;
 using namespace mozilla;
 
 // From nsDirPrefs
 #define kDefaultPosition 1
 
 nsAbDirProperty::nsAbDirProperty(void)
-    : m_LastModifiedDate(0), mIsValidURI(false), mIsQueryURI(false) {
+    : m_LastModifiedDate(0), mIsValidURI(false) {
   m_IsMailList = false;
   mUID = EmptyCString();
 }
@@ -45,17 +49,6 @@ nsAbDirProperty::~nsAbDirProperty(void) {
 
 NS_IMPL_ISUPPORTS(nsAbDirProperty, nsIAbDirectory, nsISupportsWeakReference)
 
-NS_IMETHODIMP nsAbDirProperty::GetUuid(nsACString& uuid) {
-  // XXX: not all directories have a dirPrefId...
-  nsresult rv = GetDirPrefId(uuid);
-  NS_ENSURE_SUCCESS(rv, rv);
-  uuid.Append('&');
-  nsString dirName;
-  GetDirName(dirName);
-  uuid.Append(NS_ConvertUTF16toUTF8(dirName));
-  return rv;
-}
-
 NS_IMETHODIMP nsAbDirProperty::GetPropertiesChromeURI(nsACString& aResult) {
   aResult.AssignLiteral(
       "chrome://messenger/content/addressbook/abAddressBookNameDialog.xhtml");
@@ -93,10 +86,6 @@ NS_IMETHODIMP nsAbDirProperty::GetDirName(nsAString& aDirName) {
   return NS_OK;
 }
 
-// XXX Although mailing lists could use the NotifyItemPropertyChanged
-// mechanism here, it requires some rework on how we write/save data
-// relating to mailing lists, so we're just using the old method of a
-// local variable to store the mailing list name.
 NS_IMETHODIMP nsAbDirProperty::SetDirName(const nsAString& aDirName) {
   if (m_DirPrefId.IsEmpty()) {
     m_ListDirName = aDirName;
@@ -115,16 +104,19 @@ NS_IMETHODIMP nsAbDirProperty::SetDirName(const nsAString& aDirName) {
   nsCOMPtr<nsIAbManager> abManager =
       do_GetService(NS_ABMANAGER_CONTRACTID, &rv);
 
-  if (NS_SUCCEEDED(rv))
+  if (NS_SUCCEEDED(rv)) {
+    nsCOMPtr<nsIObserverService> observerService =
+        mozilla::services::GetObserverService();
     // We inherit from nsIAbDirectory, so this static cast should be safe.
-    abManager->NotifyItemPropertyChanged(static_cast<nsIAbDirectory*>(this),
-                                         "DirName", oldDirName, aDirName);
+    observerService->NotifyObservers(static_cast<nsIAbDirectory*>(this),
+                                     "addrbook-directory-updated", u"DirName");
+  }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP nsAbDirProperty::GetDirType(int32_t* aDirType) {
-  return GetIntValue("dirType", LDAPDirectory, aDirType);
+  return GetIntValue("dirType", nsIAbManager::LDAP_DIRECTORY_TYPE, aDirType);
 }
 
 NS_IMETHODIMP nsAbDirProperty::GetFileName(nsACString& aFileName) {
@@ -237,38 +229,42 @@ NS_IMETHODIMP nsAbDirProperty::CopyMailList(nsIAbDirectory* srcList) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsAbDirProperty::GetIsQuery(bool* aResult) {
-  NS_ENSURE_ARG_POINTER(aResult);
-  // Mailing lists are not queries by default, individual directory types
-  // will override this.
-  *aResult = false;
+NS_IMETHODIMP
+nsAbDirProperty::Init(const char* aURI) {
+  mURI = aURI;
+  mIsValidURI = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsAbDirProperty::Init(const char* aURI) {
-  mURINoQuery = aURI;
-  mURI = aURI;
-  mIsValidURI = true;
-
-  int32_t searchCharLocation = mURINoQuery.FindChar('?');
-  if (searchCharLocation >= 0) {
-    mQueryString = Substring(mURINoQuery, searchCharLocation + 1);
-    mURINoQuery.SetLength(searchCharLocation);
-    mIsQueryURI = true;
+nsAbDirProperty::CleanUp(JSContext* cx, Promise** retval) {
+  nsIGlobalObject* globalObject =
+      xpc::NativeGlobal(JS::CurrentGlobalOrNull(cx));
+  if (NS_WARN_IF(!globalObject)) {
+    return NS_ERROR_FAILURE;
   }
 
+  ErrorResult result;
+  RefPtr<Promise> promise = Promise::Create(globalObject, result);
+  promise->MaybeResolveWithUndefined();
+  promise.forget(retval);
+
   return NS_OK;
 }
 
 // nsIAbDirectory NOT IMPLEMENTED methods
 NS_IMETHODIMP
-nsAbDirProperty::GetChildNodes(nsISimpleEnumerator** childList) {
+nsAbDirProperty::GetChildNodes(nsTArray<RefPtr<nsIAbDirectory>>& childList) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsAbDirProperty::GetChildCards(nsISimpleEnumerator** childCards) {
+nsAbDirProperty::GetChildCardCount(uint32_t* count) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsAbDirProperty::GetChildCards(nsTArray<RefPtr<nsIAbCard>>& childCards) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -288,38 +284,17 @@ nsAbDirProperty::HasDirectory(nsIAbDirectory* dir, bool* hasDir) {
 }
 
 NS_IMETHODIMP
-nsAbDirProperty::HasMailListWithName(const char16_t* aName, bool* aHasList) {
-  NS_ENSURE_ARG_POINTER(aName);
+nsAbDirProperty::HasMailListWithName(const nsAString& aName, bool* aHasList) {
   NS_ENSURE_ARG_POINTER(aHasList);
 
   *aHasList = false;
+  nsCOMPtr<nsIAbDirectory> aDir;
+  nsresult rv = GetMailListFromName(aName, getter_AddRefs(aDir));
 
-  bool supportsLists = false;
-  nsresult rv = GetSupportsMailingLists(&supportsLists);
-  if (NS_FAILED(rv) || !supportsLists) return NS_OK;
-
-  if (m_IsMailList) return NS_OK;
-
-  if (!m_AddressList) {
-    nsresult rv;
-    m_AddressList = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  uint32_t listCount = 0;
-  rv = m_AddressList->GetLength(&listCount);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  for (uint32_t i = 0; i < listCount; i++) {
-    nsCOMPtr<nsIAbDirectory> listDir(do_QueryElementAt(m_AddressList, i, &rv));
-    if (NS_SUCCEEDED(rv) && listDir) {
-      nsAutoString listName;
-      rv = listDir->GetDirName(listName);
-      if (NS_SUCCEEDED(rv) && listName.Equals(aName)) {
-        *aHasList = true;
-        return NS_OK;
-      }
-    }
+  if (aDir) {
+    *aHasList = true;
   }
   return NS_OK;
 }
@@ -366,10 +341,46 @@ NS_IMETHODIMP nsAbDirProperty::GetCardFromProperty(const char* aProperty,
 
 NS_IMETHODIMP nsAbDirProperty::GetCardsFromProperty(
     const char* aProperty, const nsACString& aValue, bool caseSensitive,
-    nsISimpleEnumerator** result) {
+    nsTArray<RefPtr<nsIAbCard>>& result) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+nsAbDirProperty::GetMailListFromName(const nsAString& aName,
+                                     nsIAbDirectory** aResult) {
+  NS_ENSURE_ARG_POINTER(aResult);
+
+  *aResult = nullptr;
+  bool supportsLists = false;
+  nsresult rv = GetSupportsMailingLists(&supportsLists);
+  if (NS_FAILED(rv) || !supportsLists) return NS_OK;
+
+  if (m_IsMailList) return NS_OK;
+
+  if (!m_AddressList) {
+    nsresult rv;
+    m_AddressList = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  uint32_t listCount = 0;
+  rv = m_AddressList->GetLength(&listCount);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  for (uint32_t i = 0; i < listCount; i++) {
+    nsCOMPtr<nsIAbDirectory> listDir(do_QueryElementAt(m_AddressList, i, &rv));
+    if (NS_SUCCEEDED(rv) && listDir) {
+      nsAutoString listName;
+      rv = listDir->GetDirName(listName);
+      if (NS_SUCCEEDED(rv) && listName.Equals(aName)) {
+        listDir.forget(aResult);
+        return NS_OK;
+      }
+    }
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAbDirProperty::GetSupportsMailingLists(
     bool* aSupportsMailingsLists) {
   NS_ENSURE_ARG_POINTER(aSupportsMailingsLists);
@@ -426,16 +437,6 @@ NS_IMETHODIMP nsAbDirProperty::GetDirPrefId(nsACString& aDirPrefId) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsAbDirProperty::SetDirPrefId(const nsACString& aDirPrefId) {
-  if (!m_DirPrefId.Equals(aDirPrefId)) {
-    m_DirPrefId.Assign(aDirPrefId);
-    // Clear the directory pref branch so that it is re-initialized next
-    // time its required.
-    m_DirectoryPrefs = nullptr;
-  }
-  return NS_OK;
-}
-
 nsresult nsAbDirProperty::InitDirectoryPrefs() {
   if (m_DirPrefId.IsEmpty()) return NS_ERROR_NOT_INITIALIZED;
 
@@ -565,6 +566,7 @@ NS_IMETHODIMP nsAbDirProperty::SetLocalizedStringValue(
 }
 
 NS_IMETHODIMP nsAbDirProperty::Search(const nsAString& query,
+                                      const nsAString& searchString,
                                       nsIAbDirSearchListener* listener) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
