@@ -3,14 +3,19 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { MailE10SUtils } = ChromeUtils.import(
+  "resource:///modules/MailE10SUtils.jsm"
+);
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 var gLogView;
 var gLogFile;
 
+window.addEventListener("DOMContentLoaded", onLoad);
+
 function onLoad() {
   gLogView = document.getElementById("logView");
-  gLogView.docShell.allowJavascript = false; // for security, disable JS
+  gLogView.browsingContext.allowJavascript = false; // for security, disable JS
 
   gLogView.addEventListener("load", () => {
     addStyling();
@@ -19,7 +24,7 @@ function onLoad() {
   gLogFile = Services.dirsvc.get("ProfD", Ci.nsIFile);
   gLogFile.append("junklog.html");
   if (gLogFile.exists()) {
-    gLogView.setAttribute("src", Services.io.newFileURI(gLogFile).spec);
+    MailE10SUtils.loadURI(gLogView, Services.io.newFileURI(gLogFile).spec);
   } else {
     addStyling(); // set style for initial about:blank
   }
@@ -38,7 +43,7 @@ function addStyling() {
   style.sheet.insertRule(
     `@media (prefers-color-scheme: dark) {
        :root { scrollbar-color: rgba(249, 249, 250, .4) rgba(20, 20, 25, .3);}
-       body { color: #f9f9fa;}
+       body { color: #f9f9fa; }
      }`
   );
 }
