@@ -1,5 +1,4 @@
-/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -12,14 +11,12 @@ var gMsgWindow;
 
 var gInitialFolderStates = {};
 
+window.addEventListener("DOMContentLoaded", onLoad);
+
 document.addEventListener("dialogaccept", syncOkButton);
 
-function OnLoad() {
-  if (window.arguments && window.arguments[0]) {
-    if (window.arguments[0].msgWindow) {
-      gParentMsgWindow = window.arguments[0].msgWindow;
-    }
-  }
+function onLoad() {
+  gParentMsgWindow = window.arguments?.[0]?.msgWindow;
 
   document.getElementById("syncMail").checked = Services.prefs.getBoolPref(
     "mailnews.offline_sync_mail"
@@ -33,8 +30,6 @@ function OnLoad() {
   document.getElementById("workOffline").checked = Services.prefs.getBoolPref(
     "mailnews.offline_sync_work_offline"
   );
-
-  return true;
 }
 
 function syncOkButton() {
