@@ -1,10 +1,14 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-ChromeUtils.import("resource:///modules/mailServices.js");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
+
+window.addEventListener("DOMContentLoaded", onLoad);
+document.addEventListener("dialogaccept", onOk);
 
 // pull stuff out of window.arguments
 var gServerSettings = window.arguments[0];
@@ -13,34 +17,31 @@ var gFirstDeferredAccount;
 // initialize the controls with the "gServerSettings" argument
 
 var gControls;
-function getControls()
-{
-  if (!gControls)
+function getControls() {
+  if (!gControls) {
     gControls = document.getElementsByAttribute("amsa_persist", "true");
+  }
   return gControls;
 }
 
-function getLocalFoldersAccount()
-{
-  return MailServices.accounts
-    .FindAccountForServer(MailServices.accounts.localFoldersServer);
+function getLocalFoldersAccount() {
+  return MailServices.accounts.FindAccountForServer(
+    MailServices.accounts.localFoldersServer
+  );
 }
 
-function onLoad()
-{
+function onLoad() {
   var prettyName = gServerSettings.serverPrettyName;
 
-  if (prettyName)
-    document.getElementById("serverPrettyName").value =
-      document.getElementById("bundle_prefs")
-              .getFormattedString("forAccount", [prettyName]);
-
-  if (gServerSettings.serverType == "imap")
-  {
-    document.getElementById("pop3Panel").hidden = true;
+  if (prettyName) {
+    document.getElementById("serverPrettyName").value = document
+      .getElementById("bundle_prefs")
+      .getFormattedString("forAccount", [prettyName]);
   }
-  else if (gServerSettings.serverType == "pop3")
-  {
+
+  if (gServerSettings.serverType == "imap") {
+    document.getElementById("pop3Panel").hidden = true;
+  } else if (gServerSettings.serverType == "pop3") {
     document.getElementById("imapPanel").hidden = true;
     let radioGroup = document.getElementById("folderStorage");
 
@@ -50,19 +51,18 @@ function onLoad()
     // The current account should not be shown in the folder picker
     // of the "other account" option.
     folderPopup._teardown();
-    folderPopup.setAttribute("excludeServers",
-                             gServerSettings.account.incomingServer.key);
+    folderPopup.setAttribute(
+      "excludeServers",
+      gServerSettings.account.incomingServer.key
+    );
     folderPopup._ensureInitialized();
 
-    if (gFirstDeferredAccount.length)
-    {
+    if (gFirstDeferredAccount.length) {
       // The current account is deferred.
       let account = MailServices.accounts.getAccount(gFirstDeferredAccount);
       radioGroup.value = "otherAccount";
       folderPopup.selectFolder(account.incomingServer.rootFolder);
-    }
-    else
-    {
+    } else {
       // Current account is not deferred.
       radioGroup.value = "currentAccount";
       // If there are no suitable accounts to defer to, then the menulist is
@@ -81,77 +81,73 @@ function onLoad()
 
   var controls = getControls();
 
-  for (var i = 0; i < controls.length; i++)
-  {
+  for (let i = 0; i < controls.length; i++) {
     var slot = controls[i].id;
-    if (slot in gServerSettings)
-    {
-      if (controls[i].localName == "checkbox")
+    if (slot in gServerSettings) {
+      if (controls[i].localName == "checkbox") {
         controls[i].checked = gServerSettings[slot];
-      else
+      } else {
         controls[i].value = gServerSettings[slot];
+      }
     }
   }
 }
 
-function onOk()
-{
+function onOk(event) {
   // Handle account deferral settings for POP3 accounts.
-  if (gServerSettings.serverType == "pop3")
-  {
+  if (gServerSettings.serverType == "pop3") {
     var radioGroup = document.getElementById("folderStorage");
     var gPrefsBundle = document.getElementById("bundle_prefs");
     let picker = document.getElementById("deferredServerFolderPicker");
 
     // This account wasn't previously deferred, but is now deferred.
-    if (radioGroup.value != "currentAccount" && !gFirstDeferredAccount.length)
-    {
+    if (radioGroup.value != "currentAccount" && !gFirstDeferredAccount.length) {
       // If the user hasn't selected a folder, keep the default.
-      if (!picker.selectedItem)
-        return true;
+      if (!picker.selectedItem) {
+        return;
+      }
 
-      var confirmDeferAccount =
-        gPrefsBundle.getString("confirmDeferAccountWarning");
+      var confirmDeferAccount = gPrefsBundle.getString(
+        "confirmDeferAccountWarning"
+      );
 
       var confirmTitle = gPrefsBundle.getString("confirmDeferAccountTitle");
 
-      if (!Services.prompt.confirm(window, confirmTitle, confirmDeferAccount))
-        return false;
+      if (!Services.prompt.confirm(window, confirmTitle, confirmDeferAccount)) {
+        event.preventDefault();
+        return;
+      }
     }
-    switch (radioGroup.value)
-    {
+    switch (radioGroup.value) {
       case "currentAccount":
-        gServerSettings['deferredToAccount'] = "";
+        gServerSettings.deferredToAccount = "";
         break;
       case "otherAccount":
         let server = picker.selectedItem._folder.server;
         let account = MailServices.accounts.FindAccountForServer(server);
-        gServerSettings['deferredToAccount'] = account.key;
+        gServerSettings.deferredToAccount = account.key;
         break;
     }
   }
 
   // Save the controls back to the "gServerSettings" array.
   var controls = getControls();
-  for (var i = 0; i < controls.length; i++)
-  {
+  for (let i = 0; i < controls.length; i++) {
     var slot = controls[i].id;
-    if (slot in gServerSettings)
-    {
-      if (controls[i].localName == "checkbox")
+    if (slot in gServerSettings) {
+      if (controls[i].localName == "checkbox") {
         gServerSettings[slot] = controls[i].checked;
-      else
+      } else {
         gServerSettings[slot] = controls[i].value;
+      }
     }
   }
-
-  return true;
 }
 
-
 // Set radio element choices and picker states
-function updateInboxAccount(enablePicker)
-{
-  document.getElementById("deferredServerFolderPicker").disabled = !enablePicker;
+function updateInboxAccount(enablePicker) {
+  document.getElementById(
+    "deferredServerFolderPicker"
+  ).disabled = !enablePicker;
   document.getElementById("deferGetNewMail").disabled = !enablePicker;
 }
