@@ -1,19 +1,12 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* import-globals-from am-prefs.js */
 /* import-globals-from amUtils.js */
 
-var { fixIterator } = ChromeUtils.import(
-  "resource:///modules/iteratorUtils.jsm"
-);
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
-var { BrowserUtils } = ChromeUtils.import(
-  "resource://gre/modules/BrowserUtils.jsm"
-);
 
 var gServer;
 var gOriginalStoreType;
@@ -32,7 +25,7 @@ function clickStoreTypeMenu(aStoreTypeElement) {
   // otherwise 'response.newRootFolder' will be null.
   let response = { newRootFolder: null };
   // Send 'response' as an argument to converterDialog.xhtml.
-  window.docShell.rootTreeItem.domWindow.openDialog(
+  window.browsingContext.topChromeWindow.openDialog(
     "converterDialog.xhtml",
     "mailnews:mailstoreconverter",
     "modal,centerscreen,resizable=no,width=700,height=130",
@@ -67,7 +60,7 @@ function changeStoreType(aResponse) {
     document.getElementById("server.localPath").value = aResponse.newRootFolder;
     gOriginalStoreType = document.getElementById("server.storeTypeMenulist")
       .value;
-    BrowserUtils.restartApplication();
+    MailUtils.restartApplication();
   } else {
     // The conversion failed or was cancelled.
     // Restore selected item to what was selected before conversion.
@@ -390,9 +383,8 @@ function onAdvanced() {
 
   parent.gSubDialog.open(
     "chrome://messenger/content/am-server-advanced.xhtml",
-    null,
-    serverSettings,
-    onCloseAdvanced
+    { closingCallback: onCloseAdvanced },
+    serverSettings
   );
 }
 
@@ -506,36 +498,42 @@ function BrowseForNewsrc() {
 
 function setupImapDeleteUI(aServerId) {
   // read delete_model preference
-  var deleteModel = document
+  let deleteModel = document
     .getElementById("imap.deleteModel")
     .getAttribute("value");
   selectImapDeleteModel(deleteModel);
 
   // read trash folder path preference
-  var trashFolderName = getTrashFolderName();
+  let trashFolderName = getTrashFolderName();
 
   // set folderPicker menulist
-  var trashPopup = document.getElementById("msgTrashFolderPopup");
+  let trashPopup = document.getElementById("msgTrashFolderPopup");
   trashPopup._teardown();
   trashPopup._parentFolder = MailUtils.getOrCreateFolder(aServerId);
   trashPopup._ensureInitialized();
 
-  // Convert the folder path in Unicode to MUTF-7.
-  let manager = Cc["@mozilla.org/charset-converter-manager;1"].getService(
-    Ci.nsICharsetConverterManager
-  );
   // Escape backslash and double-quote with another backslash before encoding.
-  let trashMutf7 = manager.unicodeToMutf7(
-    trashFolderName.replace(/([\\"])/g, "\\$1")
+  let trashEscaped = trashFolderName.replace(/([\\"])/g, "\\$1");
+
+  // Convert the folder path from JS Unicode to MUTF-7 if necessary.
+  let imapServer = trashPopup._parentFolder.server.QueryInterface(
+    Ci.nsIImapIncomingServer
   );
-  // TODO: There is something wrong here, selectFolder() fails even if the
-  // folder does exist. Try to fix in bug 802609.
-  let trashFolder = MailUtils.getOrCreateFolder(aServerId + "/" + trashMutf7);
-  try {
-    trashPopup.selectFolder(trashFolder);
-  } catch (ex) {
-    trashPopup.parentNode.setAttribute("label", trashFolder.prettyName);
+
+  let trashFolder;
+  if (imapServer.utf8AcceptEnabled) {
+    // Trash folder with UTF8=ACCEPT capability in effect.
+    trashFolder = MailUtils.getOrCreateFolder(aServerId + "/" + trashEscaped);
+  } else {
+    // Traditional MUTF-7.
+    let manager = Cc["@mozilla.org/charset-converter-manager;1"].getService(
+      Ci.nsICharsetConverterManager
+    );
+    trashFolder = MailUtils.getOrCreateFolder(
+      aServerId + "/" + manager.unicodeToMutf7(trashEscaped)
+    );
   }
+  trashPopup.selectFolder(trashFolder);
   trashPopup.parentNode.folder = trashFolder;
 }
 
@@ -570,33 +568,51 @@ function selectImapDeleteModel(choice) {
 
 // Capture any menulist changes from folderPicker
 function folderPickerChange(aEvent) {
-  var folder = aEvent.target._folder;
+  let folder = aEvent.target._folder;
   // Since we need to deal with localised folder names, we simply use
   // the path of the URI like we do in nsImapIncomingServer::DiscoveryDone().
   // Note that the path is returned with a leading slash which we need to remove.
-  var folderPath = Services.io.newURI(folder.URI).pathQueryRef.substring(1);
-  // We need to convert that from MUTF-7 to Unicode.
-  var manager = Cc["@mozilla.org/charset-converter-manager;1"].getService(
-    Ci.nsICharsetConverterManager
-  );
-  var trashUnicode = manager.mutf7ToUnicode(
-    Services.netUtils.unescapeString(folderPath, Ci.nsINetUtil.ESCAPE_URL_PATH)
+  let folderPath = Services.io.newURI(folder.URI).pathQueryRef.substring(1);
+  let folderPathUnescaped = Services.io.unescapeString(
+    folderPath,
+    Ci.nsINetUtil.ESCAPE_URL_PATH
   );
 
+  // Convert the folder path from MUTF-7 or UTF-8 to Unicode.
+  let imapServer = folder.server.QueryInterface(Ci.nsIImapIncomingServer);
+
+  let trashUnicode;
+  if (imapServer.utf8AcceptEnabled) {
+    // UTF8=ACCEPT capability in effect. Unescaping has brought back
+    // raw UTF-8 bytes, so convert them to JS Unicode.
+    let typedarray = new Uint8Array(folderPathUnescaped.length);
+    for (let i = 0; i < folderPathUnescaped.length; i++) {
+      typedarray[i] = folderPathUnescaped.charCodeAt(i);
+    }
+    let utf8Decoder = new TextDecoder("utf-8");
+    trashUnicode = utf8Decoder.decode(typedarray);
+  } else {
+    // We need to convert that from MUTF-7 to Unicode.
+    let manager = Cc["@mozilla.org/charset-converter-manager;1"].getService(
+      Ci.nsICharsetConverterManager
+    );
+    trashUnicode = manager.mutf7ToUnicode(folderPathUnescaped);
+  }
+
   // Set the value to be persisted.
   document
     .getElementById("imap.trashFolderName")
     .setAttribute("value", trashUnicode);
 
   // Update the widget to show/do correct things even for subfolders.
-  var trashFolderPicker = document.getElementById("msgTrashFolderPicker");
+  let trashFolderPicker = document.getElementById("msgTrashFolderPicker");
   trashFolderPicker.menupopup.selectFolder(folder);
 }
 
 // Get trash_folder_name from prefs. Despite its name this returns
 // a folder path, for example INBOX/Trash.
 function getTrashFolderName() {
-  var trashFolderName = document
+  let trashFolderName = document
     .getElementById("imap.trashFolderName")
     .getAttribute("value");
   // if the preference hasn't been set, set it to a sane default
