@@ -24,7 +24,7 @@ class nsMsgGroupView : public nsMsgDBView {
   NS_IMETHOD Open(nsIMsgFolder* folder, nsMsgViewSortTypeValue sortType,
                   nsMsgViewSortOrderValue sortOrder,
                   nsMsgViewFlagsTypeValue viewFlags, int32_t* pCount) override;
-  NS_IMETHOD OpenWithHdrs(nsISimpleEnumerator* aHeaders,
+  NS_IMETHOD OpenWithHdrs(nsIMsgEnumerator* aHeaders,
                           nsMsgViewSortTypeValue aSortType,
                           nsMsgViewSortOrderValue aSortOrder,
                           nsMsgViewFlagsTypeValue aViewFlags,
@@ -72,16 +72,8 @@ class nsMsgGroupView : public nsMsgDBView {
   virtual nsMsgGroupThread* CreateGroupThread(nsIMsgDatabase* db);
 
   nsInterfaceHashtable<nsStringHashKey, nsIMsgThread> m_groupsTable;
-  PRExplodedTime m_lastCurExplodedTime;
+  PRExplodedTime m_lastCurExplodedTime{0};
   bool m_dayChanged;
-
- private:
-  nsString m_kTodayString;
-  nsString m_kYesterdayString;
-  nsString m_kLastWeekString;
-  nsString m_kTwoWeeksAgoString;
-  nsString m_kOldMailString;
-  nsString m_kFutureDateString;
 };
 
 #endif
