@@ -32,6 +32,7 @@
 #include "nsContentUtils.h"
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
+#include "nsIChannel.h"
 
 nsMsgMailNewsUrl::nsMsgMailNewsUrl() {
   // nsIURI specific state
@@ -285,7 +286,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetServer(
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIMsgIncomingServer> server;
-    rv = accountManager->FindServerByURI(url, false, aIncomingServer);
+    rv = accountManager->FindServerByURI(url, aIncomingServer);
     if (!*aIncomingServer && scheme.EqualsLiteral("imap")) {
       // look for any imap server with this host name so clicking on
       // other users folder urls will work. We could override this method
@@ -293,7 +294,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetServer(
       // just set the server in the imap code for this case.
       rv = NS_MutateURI(url).SetUserPass(EmptyCString()).Finalize(url);
       NS_ENSURE_SUCCESS(rv, rv);
-      rv = accountManager->FindServerByURI(url, false, aIncomingServer);
+      rv = accountManager->FindServerByURI(url, aIncomingServer);
     }
   }
 
@@ -1013,8 +1014,8 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetSaveAsListener(
                                         (void**)aSaveListener);
 }
 
-nsresult nsMsgMailNewsUrl::SetFailedSecInfoInternal(
-    nsITransportSecurityInfo* secInfo) {
+NS_IMETHODIMP
+nsMsgMailNewsUrl::SetFailedSecInfo(nsITransportSecurityInfo* secInfo) {
   mFailedSecInfo = secInfo;
   return NS_OK;
 }
@@ -1051,7 +1052,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetIsMessageUri(bool* aIsMessageUri) {
   NS_ENSURE_ARG(aIsMessageUri);
   nsAutoCString scheme;
   m_baseURL->GetScheme(scheme);
-  *aIsMessageUri = StringEndsWith(scheme, NS_LITERAL_CSTRING("-message"));
+  *aIsMessageUri = StringEndsWith(scheme, "-message"_ns);
   return NS_OK;
 }
 
