@@ -31,6 +31,7 @@
 #include "mozilla/Services.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/Components.h"
+#include "nsFocusManager.h"
 
 NS_IMPL_ISUPPORTS(nsMsgMailSession, nsIMsgMailSession, nsIFolderListener)
 
@@ -82,69 +83,81 @@ NS_IMETHODIMP nsMsgMailSession::RemoveFolderListener(
   PR_END_MACRO
 
 NS_IMETHODIMP
-nsMsgMailSession::OnItemPropertyChanged(nsIMsgFolder* aItem,
-                                        const nsACString& aProperty,
-                                        const nsACString& aOldValue,
-                                        const nsACString& aNewValue) {
-  NOTIFY_FOLDER_LISTENERS(propertyChanged, OnItemPropertyChanged,
+nsMsgMailSession::OnFolderPropertyChanged(nsIMsgFolder* aItem,
+                                          const nsACString& aProperty,
+                                          const nsACString& aOldValue,
+                                          const nsACString& aNewValue) {
+  NOTIFY_FOLDER_LISTENERS(propertyChanged, OnFolderPropertyChanged,
                           (aItem, aProperty, aOldValue, aNewValue));
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgMailSession::OnItemUnicharPropertyChanged(nsIMsgFolder* aItem,
-                                               const nsACString& aProperty,
-                                               const nsAString& aOldValue,
-                                               const nsAString& aNewValue) {
-  NOTIFY_FOLDER_LISTENERS(unicharPropertyChanged, OnItemUnicharPropertyChanged,
+nsMsgMailSession::OnFolderUnicharPropertyChanged(nsIMsgFolder* aItem,
+                                                 const nsACString& aProperty,
+                                                 const nsAString& aOldValue,
+                                                 const nsAString& aNewValue) {
+  NOTIFY_FOLDER_LISTENERS(unicharPropertyChanged,
+                          OnFolderUnicharPropertyChanged,
                           (aItem, aProperty, aOldValue, aNewValue));
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgMailSession::OnItemIntPropertyChanged(nsIMsgFolder* aItem,
-                                           const nsACString& aProperty,
-                                           int64_t aOldValue,
-                                           int64_t aNewValue) {
-  NOTIFY_FOLDER_LISTENERS(intPropertyChanged, OnItemIntPropertyChanged,
+nsMsgMailSession::OnFolderIntPropertyChanged(nsIMsgFolder* aItem,
+                                             const nsACString& aProperty,
+                                             int64_t aOldValue,
+                                             int64_t aNewValue) {
+  NOTIFY_FOLDER_LISTENERS(intPropertyChanged, OnFolderIntPropertyChanged,
                           (aItem, aProperty, aOldValue, aNewValue));
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgMailSession::OnItemBoolPropertyChanged(nsIMsgFolder* aItem,
-                                            const nsACString& aProperty,
-                                            bool aOldValue, bool aNewValue) {
-  NOTIFY_FOLDER_LISTENERS(boolPropertyChanged, OnItemBoolPropertyChanged,
+nsMsgMailSession::OnFolderBoolPropertyChanged(nsIMsgFolder* aItem,
+                                              const nsACString& aProperty,
+                                              bool aOldValue, bool aNewValue) {
+  NOTIFY_FOLDER_LISTENERS(boolPropertyChanged, OnFolderBoolPropertyChanged,
                           (aItem, aProperty, aOldValue, aNewValue));
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMsgMailSession::OnItemPropertyFlagChanged(nsIMsgDBHdr* aItem,
-                                            const nsACString& aProperty,
-                                            uint32_t aOldValue,
-                                            uint32_t aNewValue) {
-  NOTIFY_FOLDER_LISTENERS(propertyFlagChanged, OnItemPropertyFlagChanged,
+nsMsgMailSession::OnFolderPropertyFlagChanged(nsIMsgDBHdr* aItem,
+                                              const nsACString& aProperty,
+                                              uint32_t aOldValue,
+                                              uint32_t aNewValue) {
+  NOTIFY_FOLDER_LISTENERS(propertyFlagChanged, OnFolderPropertyFlagChanged,
                           (aItem, aProperty, aOldValue, aNewValue));
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgMailSession::OnItemAdded(nsIMsgFolder* aParentItem,
-                                            nsISupports* aItem) {
-  NOTIFY_FOLDER_LISTENERS(added, OnItemAdded, (aParentItem, aItem));
+NS_IMETHODIMP nsMsgMailSession::OnFolderAdded(nsIMsgFolder* parent,
+                                              nsIMsgFolder* child) {
+  NOTIFY_FOLDER_LISTENERS(added, OnFolderAdded, (parent, child));
+  return NS_OK;
+}
+NS_IMETHODIMP nsMsgMailSession::OnMessageAdded(nsIMsgFolder* parent,
+                                               nsIMsgDBHdr* msg) {
+  NOTIFY_FOLDER_LISTENERS(added, OnMessageAdded, (parent, msg));
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgMailSession::OnItemRemoved(nsIMsgFolder* aParentItem,
-                                              nsISupports* aItem) {
-  NOTIFY_FOLDER_LISTENERS(removed, OnItemRemoved, (aParentItem, aItem));
+NS_IMETHODIMP nsMsgMailSession::OnFolderRemoved(nsIMsgFolder* parent,
+                                                nsIMsgFolder* child) {
+  NOTIFY_FOLDER_LISTENERS(removed, OnFolderRemoved, (parent, child));
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgMailSession::OnItemEvent(nsIMsgFolder* aFolder,
-                                            const nsACString& aEvent) {
-  NOTIFY_FOLDER_LISTENERS(event, OnItemEvent, (aFolder, aEvent));
+NS_IMETHODIMP nsMsgMailSession::OnMessageRemoved(nsIMsgFolder* parent,
+                                                 nsIMsgDBHdr* msg) {
+  NOTIFY_FOLDER_LISTENERS(removed, OnMessageRemoved, (parent, msg));
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgMailSession::OnFolderEvent(nsIMsgFolder* aFolder,
+                                              const nsACString& aEvent) {
+  NOTIFY_FOLDER_LISTENERS(event, OnFolderEvent, (aFolder, aEvent));
   return NS_OK;
 }
 
@@ -173,7 +186,7 @@ NS_IMETHODIMP
 nsMsgMailSession::AlertUser(const nsAString& aMessage,
                             nsIMsgMailNewsUrl* aUrl) {
   bool listenersNotified = false;
-  nsTObserverArray<nsCOMPtr<nsIMsgUserFeedbackListener> >::ForwardIterator iter(
+  nsTObserverArray<nsCOMPtr<nsIMsgUserFeedbackListener>>::ForwardIterator iter(
       mFeedbackListeners);
   nsCOMPtr<nsIMsgUserFeedbackListener> listener;
 
@@ -252,26 +265,41 @@ nsresult nsMsgMailSession::GetTopmostMsgWindow(nsIMsgWindow** aMsgWindow) {
       NS_ENSURE_SUCCESS(rv, rv);
       NS_ENSURE_TRUE(windowSupports, NS_ERROR_FAILURE);
 
-      topMostWindow = do_QueryInterface(windowSupports, &rv);
+      nsCOMPtr<nsPIDOMWindowOuter> window =
+          do_QueryInterface(windowSupports, &rv);
       NS_ENSURE_SUCCESS(rv, rv);
-      NS_ENSURE_TRUE(topMostWindow, NS_ERROR_FAILURE);
+      NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
 
-      mozilla::dom::Document* domDocument = topMostWindow->GetDoc();
+      mozilla::dom::Document* domDocument = window->GetDoc();
       NS_ENSURE_TRUE(domDocument, NS_ERROR_FAILURE);
 
       mozilla::dom::Element* domElement = domDocument->GetDocumentElement();
       NS_ENSURE_TRUE(domElement, NS_ERROR_FAILURE);
 
-      domElement->GetAttribute(NS_LITERAL_STRING("windowtype"), windowType);
+      domElement->GetAttribute(u"windowtype"_ns, windowType);
       if (windowType.EqualsLiteral("mail:3pane") ||
-          windowType.EqualsLiteral("mail:messageWindow"))
-        break;
+          windowType.EqualsLiteral("mail:messageWindow")) {
+        // topMostWindow is the last 3pane/messageWindow found, not necessarily
+        // the top most.
+        topMostWindow = window;
+        RefPtr<nsFocusManager> fm = nsFocusManager::GetFocusManager();
+        nsCOMPtr<mozIDOMWindowProxy> currentWindow =
+            do_QueryInterface(windowSupports, &rv);
+        NS_ENSURE_SUCCESS(rv, rv);
+        nsCOMPtr<mozIDOMWindowProxy> activeWindow;
+        rv = fm->GetActiveWindow(getter_AddRefs(activeWindow));
+        NS_ENSURE_SUCCESS(rv, rv);
+        if (currentWindow == activeWindow) {
+          // We are sure topMostWindow is really the top most now.
+          break;
+        }
+      }
 
       windowEnum->HasMoreElements(&more);
     }
 
     // identified the top most window
-    if (more) {
+    if (topMostWindow) {
       // use this for the match
       nsIDocShell* topDocShell = topMostWindow->GetDocShell();
 
@@ -341,26 +369,19 @@ NS_IMETHODIMP nsMsgMailSession::IsFolderOpenInWindow(nsIMsgFolder* folder,
 }
 
 NS_IMETHODIMP
-nsMsgMailSession::ConvertMsgURIToMsgURL(const char* aURI,
-                                        nsIMsgWindow* aMsgWindow, char** aURL) {
-  NS_ENSURE_ARG_POINTER(aURI);
-  NS_ENSURE_ARG_POINTER(aURL);
-
+nsMsgMailSession::ConvertMsgURIToMsgURL(const nsACString& aURI,
+                                        nsIMsgWindow* aMsgWindow,
+                                        nsACString& aURL) {
   // convert the rdf msg uri into a url that represents the message...
   nsCOMPtr<nsIMsgMessageService> msgService;
-  nsresult rv = GetMessageServiceFromURI(nsDependentCString(aURI),
-                                         getter_AddRefs(msgService));
+  nsresult rv = GetMessageServiceFromURI(aURI, getter_AddRefs(msgService));
   NS_ENSURE_SUCCESS(rv, NS_ERROR_NULL_POINTER);
 
   nsCOMPtr<nsIURI> tURI;
-  rv = msgService->GetUrlForUri(aURI, getter_AddRefs(tURI), aMsgWindow);
+  rv = msgService->GetUrlForUri(aURI, aMsgWindow, getter_AddRefs(tURI));
   NS_ENSURE_SUCCESS(rv, NS_ERROR_NULL_POINTER);
 
-  nsAutoCString urlString;
-  if (NS_SUCCEEDED(tURI->GetSpec(urlString))) {
-    *aURL = ToNewCString(urlString);
-    NS_ENSURE_ARG_POINTER(aURL);
-  }
+  rv = tURI->GetSpec(aURL);
   return rv;
 }
 
@@ -410,7 +431,8 @@ NS_IMPL_ISUPPORTS(nsMsgShutdownService, nsIMsgShutdownService, nsIUrlListener,
                   nsIObserver)
 
 nsMsgShutdownService::nsMsgShutdownService()
-    : mQuitMode(nsIAppStartup::eAttemptQuit),
+    : mTaskIndex(0),
+      mQuitMode(nsIAppStartup::eAttemptQuit),
       mProcessedShutdown(false),
       mQuitForced(false),
       mReadyToQuit(false) {
@@ -483,7 +505,8 @@ void nsMsgShutdownService::AttemptShutdown() {
     nsCOMPtr<nsIAppStartup> appStartup =
         mozilla::components::AppStartup::Service();
     NS_ENSURE_TRUE_VOID(appStartup);
-    NS_ENSURE_SUCCESS_VOID(appStartup->Quit(mQuitMode));
+    bool userAllowedQuit = true;
+    NS_ENSURE_SUCCESS_VOID(appStartup->Quit(mQuitMode, 0, &userAllowedQuit));
   }
 }
 
