@@ -53,14 +53,11 @@
 #include "nsIProtocolProxyCallback.h"
 #include "nsICancelable.h"
 #include "nsIMsgDatabase.h"
-#include "nsIMutableArray.h"
 #include "nsIMsgMailNewsUrl.h"
-#include "nsArrayUtils.h"
 #include "nsIStringBundle.h"
 #include "nsIMsgWindow.h"
 #include "nsIWindowWatcher.h"
 #include "nsIPrompt.h"
-#include "nsIArray.h"
 #include "nsIMsgSearchTerm.h"
 #include "nsTextFormatter.h"
 #include "nsIStreamListener.h"
@@ -69,7 +66,6 @@
 #include "nsIParserUtils.h"
 #include "nsICharsetConverterManager.h"
 #include "nsIDocumentEncoder.h"
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/Services.h"
 #include "locale.h"
 #include "nsStringStream.h"
@@ -152,10 +148,10 @@ nsresult GetMessageServiceFromURI(const nsACString& uri,
   return rv;
 }
 
-nsresult GetMsgDBHdrFromURI(const char* uri, nsIMsgDBHdr** msgHdr) {
+nsresult GetMsgDBHdrFromURI(const nsACString& uri, nsIMsgDBHdr** msgHdr) {
   nsCOMPtr<nsIMsgMessageService> msgMessageService;
-  nsresult rv = GetMessageServiceFromURI(nsDependentCString(uri),
-                                         getter_AddRefs(msgMessageService));
+  nsresult rv =
+      GetMessageServiceFromURI(uri, getter_AddRefs(msgMessageService));
   NS_ENSURE_SUCCESS(rv, rv);
   if (!msgMessageService) return NS_ERROR_FAILURE;
 
@@ -288,7 +284,7 @@ int32_t MsgFindCharInSet(const nsCString& aString, const char* aChars,
   return aString.FindCharInSet(aChars, aOffset);
 }
 
-int32_t MsgFindCharInSet(const nsString& aString, const char* aChars,
+int32_t MsgFindCharInSet(const nsString& aString, const char16_t* aChars,
                          uint32_t aOffset) {
   return aString.FindCharInSet(aChars, aOffset);
 }
@@ -322,10 +318,10 @@ nsresult NS_MsgHashIfNecessary(nsAutoCString& name) {
   // Need to check the first ('.') and last ('.', '~' and ' ') char
   if (illegalCharacterIndex == -1) {
     int32_t lastIndex = str.Length() - 1;
-    if (NS_LITERAL_CSTRING(ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER)
+    if (nsLiteralCString(ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER)
             .FindChar(str[0]) != -1)
       illegalCharacterIndex = 0;
-    else if (NS_LITERAL_CSTRING(ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER)
+    else if (nsLiteralCString(ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER)
                  .FindChar(str[lastIndex]) != -1)
       illegalCharacterIndex = lastIndex;
     else
@@ -364,16 +360,16 @@ nsresult NS_MsgHashIfNecessary(nsAutoCString& name) {
 nsresult NS_MsgHashIfNecessary(nsAutoString& name) {
   if (name.IsEmpty()) return NS_OK;  // Nothing to do.
   int32_t illegalCharacterIndex = MsgFindCharInSet(
-      name, FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS ILLEGAL_FOLDER_CHARS,
-      0);
+      name,
+      u"" FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS ILLEGAL_FOLDER_CHARS, 0);
 
   // Need to check the first ('.') and last ('.', '~' and ' ') char
   if (illegalCharacterIndex == -1) {
     int32_t lastIndex = name.Length() - 1;
-    if (NS_LITERAL_STRING(ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER)
+    if (NS_LITERAL_STRING_FROM_CSTRING(ILLEGAL_FOLDER_CHARS_AS_FIRST_LETTER)
             .FindChar(name[0]) != -1)
       illegalCharacterIndex = 0;
-    else if (NS_LITERAL_STRING(ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER)
+    else if (NS_LITERAL_STRING_FROM_CSTRING(ILLEGAL_FOLDER_CHARS_AS_LAST_LETTER)
                  .FindChar(name[lastIndex]) != -1)
       illegalCharacterIndex = lastIndex;
     else
@@ -403,12 +399,8 @@ nsresult NS_MsgHashIfNecessary(nsAutoString& name) {
 
 nsresult FormatFileSize(int64_t size, bool useKB, nsAString& formattedSize) {
   const char* sizeAbbrNames[] = {
-      "byteAbbreviation2",
-      "kiloByteAbbreviation2",
-      "megaByteAbbreviation2",
-      "gigaByteAbbreviation2",
-      "teraByteAbbreviation2",
-      "petaByteAbbreviation2",
+      "byteAbbreviation2",     "kiloByteAbbreviation2", "megaByteAbbreviation2",
+      "gigaByteAbbreviation2", "teraByteAbbreviation2", "petaByteAbbreviation2",
   };
 
   nsresult rv;
@@ -758,8 +750,10 @@ nsresult EscapeFromSpaceLine(nsIOutputStream* outputStream, char* start,
 
     if (pChar < end) {
       // Found a line so check if it's a qualified "From " line.
-      if (IsAFromSpaceLine(start, pChar))
+      if (IsAFromSpaceLine(start, pChar)) {
         rv = outputStream->Write(">", 1, &written);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
       int32_t lineTerminatorCount = (*(pChar + 1) == '\n') ? 2 : 1;
       rv = outputStream->Write(start, pChar - start + lineTerminatorCount,
                                &written);
@@ -768,8 +762,10 @@ nsresult EscapeFromSpaceLine(nsIOutputStream* outputStream, char* start,
       start = pChar;
     } else if (start < end) {
       // Check and flush out the remaining data and we're done.
-      if (IsAFromSpaceLine(start, end))
+      if (IsAFromSpaceLine(start, end)) {
         rv = outputStream->Write(">", 1, &written);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
       rv = outputStream->Write(start, end - start, &written);
       NS_ENSURE_SUCCESS(rv, rv);
       break;
@@ -843,7 +839,7 @@ nsresult GetOrCreateJunkFolder(const nsACString& aURI,
 
       // force the junk folder name to be Junk so it gets created on disk
       // correctly...
-      msgFolder->SetName(NS_LITERAL_STRING("Junk"));
+      msgFolder->SetName(u"Junk"_ns);
       msgFolder->SetFlag(nsMsgFolderFlags::Junk);
       rv = msgFolder->CreateStorageIfMissing(aListener);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -891,7 +887,7 @@ nsresult IsRSSArticle(nsIURI* aMsgURI, bool* aIsRSSArticle) {
   if (NS_FAILED(rv)) return rv;
 
   nsCString resourceURI;
-  msgUrl->GetUri(getter_Copies(resourceURI));
+  msgUrl->GetUri(resourceURI);
 
   // get the msg service for this URI
   nsCOMPtr<nsIMsgMessageService> msgService;
@@ -901,8 +897,7 @@ nsresult IsRSSArticle(nsIURI* aMsgURI, bool* aIsRSSArticle) {
   // Check if the message is a feed message, regardless of folder.
   uint32_t flags;
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  rv =
-      msgService->MessageURIToMsgHdr(resourceURI.get(), getter_AddRefs(msgHdr));
+  rv = msgService->MessageURIToMsgHdr(resourceURI, getter_AddRefs(msgHdr));
   NS_ENSURE_SUCCESS(rv, rv);
   msgHdr->GetFlags(&flags);
   if (flags & nsMsgMessageFlags::FeedMsg) {
@@ -1120,7 +1115,7 @@ NS_MSG_BASE nsresult NS_SetPersistentFile(const char* relPrefName,
   nsCOMPtr<nsIRelativeFilePref> relFilePref = new nsRelativeFilePref();
   mozilla::Unused << relFilePref->SetFile(aFile);
   mozilla::Unused << relFilePref->SetRelativeToKey(
-      NS_LITERAL_CSTRING(NS_APP_USER_PROFILE_50_DIR));
+      nsLiteralCString(NS_APP_USER_PROFILE_50_DIR));
 
   nsresult rv2 = prefBranch->SetComplexValue(
       relPrefName, NS_GET_IID(nsIRelativeFilePref), relFilePref);
@@ -1278,21 +1273,11 @@ nsresult MsgCleanupTempFiles(const char* fileName, const char* extension) {
 }
 
 nsresult MsgGetFileStream(nsIFile* file, nsIOutputStream** fileStream) {
-  nsMsgFileStream* newFileStream = new nsMsgFileStream;
-  NS_ENSURE_TRUE(newFileStream, NS_ERROR_OUT_OF_MEMORY);
+  RefPtr<nsMsgFileStream> newFileStream = new nsMsgFileStream;
   nsresult rv = newFileStream->InitWithFile(file);
-  if (NS_SUCCEEDED(rv))
-    rv = newFileStream->QueryInterface(NS_GET_IID(nsIOutputStream),
-                                       (void**)fileStream);
-  return rv;
-}
-
-nsresult MsgReopenFileStream(nsIFile* file, nsIInputStream* fileStream) {
-  nsMsgFileStream* msgFileStream = static_cast<nsMsgFileStream*>(fileStream);
-  if (msgFileStream)
-    return msgFileStream->InitWithFile(file);
-  else
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_SUCCESS(rv, rv);
+  newFileStream.forget(fileStream);
+  return NS_OK;
 }
 
 nsresult MsgNewBufferedFileOutputStream(nsIOutputStream** aResult,
@@ -1477,40 +1462,9 @@ NS_MSG_BASE nsresult MsgEscapeURL(const nsACString& aStr, uint32_t aFlags,
   return nu->EscapeURL(aStr, aFlags, aResult);
 }
 
-NS_MSG_BASE nsresult MsgGetHeadersFromKeys(nsIMsgDatabase* aDB,
-                                           const nsTArray<nsMsgKey>& aMsgKeys,
-                                           nsIMutableArray* aHeaders) {
-  NS_ENSURE_ARG_POINTER(aDB);
-
-  uint32_t count = aMsgKeys.Length();
-  nsresult rv = NS_OK;
-
-  for (uint32_t kindex = 0; kindex < count; kindex++) {
-    nsMsgKey key = aMsgKeys.ElementAt(kindex);
-
-    bool hasKey;
-    rv = aDB->ContainsKey(key, &hasKey);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    // This function silently skips when the key is not found. This is an
-    // expected case.
-    if (hasKey) {
-      nsCOMPtr<nsIMsgDBHdr> msgHdr;
-      rv = aDB->GetMsgHdrForKey(key, getter_AddRefs(msgHdr));
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      aHeaders->AppendElement(msgHdr);
-    }
-  }
-
-  return rv;
-}
-
-// Stopgap while we complete removal of nsIArray usage (Bug 1583030).
-// Then this function can replace MsgGetHeadersFromKeys().
 NS_MSG_BASE nsresult
-MsgGetHeadersFromKeys2(nsIMsgDatabase* aDB, const nsTArray<nsMsgKey>& aMsgKeys,
-                       nsTArray<RefPtr<nsIMsgDBHdr>>& aHeaders) {
+MsgGetHeadersFromKeys(nsIMsgDatabase* aDB, const nsTArray<nsMsgKey>& aMsgKeys,
+                      nsTArray<RefPtr<nsIMsgDBHdr>>& aHeaders) {
   NS_ENSURE_ARG_POINTER(aDB);
   aHeaders.Clear();
   aHeaders.SetCapacity(aMsgKeys.Length());
@@ -1638,18 +1592,13 @@ NS_MSG_BASE PRTime MsgConvertAgeInDaysToCutoffDate(int32_t ageInDays) {
   return now - PR_USEC_PER_DAY * ageInDays;
 }
 
-NS_MSG_BASE nsresult MsgTermListToString(nsIArray* aTermList,
-                                         nsCString& aOutString) {
-  uint32_t count;
-  aTermList->GetLength(&count);
+NS_MSG_BASE nsresult
+MsgTermListToString(nsTArray<RefPtr<nsIMsgSearchTerm>> const& aTermList,
+                    nsCString& aOutString) {
   nsresult rv = NS_OK;
-
-  for (uint32_t searchIndex = 0; searchIndex < count; searchIndex++) {
+  for (nsIMsgSearchTerm* term : aTermList) {
     nsAutoCString stream;
 
-    nsCOMPtr<nsIMsgSearchTerm> term = do_QueryElementAt(aTermList, searchIndex);
-    if (!term) continue;
-
     if (aOutString.Length() > 1) aOutString += ' ';
 
     bool booleanAnd;
@@ -1744,7 +1693,7 @@ NS_MSG_BASE nsresult MsgStreamMsgHeaders(nsIInputStream* aInputStream,
   rv = NS_NewInputStreamPump(getter_AddRefs(pump), hdrsStream.forget());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return pump->AsyncRead(aConsumer, nullptr);
+  return pump->AsyncRead(aConsumer);
 }
 
 NS_MSG_BASE nsresult MsgDetectCharsetFromFile(nsIFile* aFile,
@@ -1790,7 +1739,7 @@ NS_MSG_BASE nsresult MsgDetectCharsetFromFile(nsIFile* aFile,
   numRead = 0;
   while (NS_SUCCEEDED(inputStream->Read(buffer, sizeof(buffer), &numRead))) {
     mozilla::Span<const uint8_t> src =
-        mozilla::AsBytes(mozilla::MakeSpan(buffer, numRead));
+        mozilla::AsBytes(mozilla::Span(buffer, numRead));
     Unused << detector->Feed(src, false);
     if (numRead == 0) {
       break;
@@ -1809,7 +1758,7 @@ NS_MSG_BASE nsresult MsgDetectCharsetFromFile(nsIFile* aFile,
  * unknown or deemed of no importance NULL could be passed.
  */
 NS_MSG_BASE nsresult ConvertBufToPlainText(nsString& aConBuf, bool formatFlowed,
-                                           bool delsp, bool formatOutput,
+                                           bool formatOutput,
                                            bool disallowBreaks) {
   if (aConBuf.IsEmpty()) return NS_OK;
 
@@ -1827,7 +1776,6 @@ NS_MSG_BASE nsresult ConvertBufToPlainText(nsString& aConBuf, bool formatFlowed,
 
   uint32_t converterFlags = nsIDocumentEncoder::OutputPersistNBSP;
   if (formatFlowed) converterFlags |= nsIDocumentEncoder::OutputFormatFlowed;
-  if (delsp) converterFlags |= nsIDocumentEncoder::OutputFormatDelSp;
   if (formatOutput) converterFlags |= nsIDocumentEncoder::OutputFormatted;
   if (disallowBreaks)
     converterFlags |= nsIDocumentEncoder::OutputDisallowLineBreaking;
@@ -1881,16 +1829,47 @@ void MsgRemoveQueryPart(nsCString& aSpec) {
   if (ind != kNotFound) aSpec.SetLength(ind);
 }
 
-void MsgHdrsToTArray(nsIArray* messages, nsTArray<RefPtr<nsIMsgDBHdr>>& out) {
-  if (!messages) {
-    out.Clear();
-    return;
-  }
-  uint32_t count;
-  messages->GetLength(&count);
-  out.Clear();
-  out.SetCapacity(count);
-  for (uint32_t i = 0; i < count; ++i) {
-    out.AppendElement(do_QueryElementAt(messages, i));
+// Perform C-style string escaping.
+// e.g. "foo\r\n" => "foo\\r\\n"
+// (See also CEscape(), in protobuf, for similar function).
+nsCString CEscapeString(nsACString const& s) {
+  nsCString out;
+  for (size_t i = 0; i < s.Length(); ++i) {
+    char c = s[i];
+    if (c & 0x80) {
+      out.AppendPrintf("\\x%02x", (uint8_t)c);
+      continue;
+    }
+    switch (c) {
+      case '\a':
+        out += "\\a";
+        break;
+      case '\b':
+        out += "\\b";
+        break;
+      case '\f':
+        out += "\\f";
+        break;
+      case '\n':
+        out += "\\n";
+        break;
+      case '\r':
+        out += "\\r";
+        break;
+      case '\t':
+        out += "\\t";
+        break;
+      case '\v':
+        out += "\\v";
+        break;
+      default:
+        if (c < ' ') {
+          out.AppendPrintf("\\x%02x", (uint8_t)c);
+        } else {
+          out += c;
+        }
+        break;
+    }
   }
+  return out;
 }
