@@ -20,8 +20,6 @@
 #include "nsIStringBundle.h"
 #include "mozilla/Services.h"
 #include "mozilla/mailnews/MimeHeaderParser.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "nsMailDirServiceDefs.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsDirectoryServiceDefs.h"
@@ -31,6 +29,7 @@
 #include "nsIAbManager.h"
 #include "nsIMsgAccountManager.h"
 #include "nsMsgBaseCID.h"
+#include "mozilla/intl/AppDateTimeFormat.h"
 
 using namespace mozilla::mailnews;
 
@@ -42,13 +41,14 @@ nsSpamSettings::nsSpamSettings() {
   mPurgeInterval = 14;  // 14 days
 
   mServerFilterTrustFlags = 0;
-
+  mInhibitWhiteListingIdentityUser = false;
+  mInhibitWhiteListingIdentityDomain = false;
   mUseWhiteList = false;
   mUseServerFilter = false;
 
   nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
                                        getter_AddRefs(mLogFile));
-  if (NS_SUCCEEDED(rv)) mLogFile->Append(NS_LITERAL_STRING("junklog.html"));
+  if (NS_SUCCEEDED(rv)) mLogFile->Append(u"junklog.html"_ns);
 }
 
 nsSpamSettings::~nsSpamSettings() {}
@@ -125,38 +125,36 @@ NS_IMPL_GETSET(nsSpamSettings, Purge, bool, mPurge)
 NS_IMPL_GETSET(nsSpamSettings, UseWhiteList, bool, mUseWhiteList)
 NS_IMPL_GETSET(nsSpamSettings, UseServerFilter, bool, mUseServerFilter)
 
-NS_IMETHODIMP nsSpamSettings::GetWhiteListAbURI(char** aWhiteListAbURI) {
-  NS_ENSURE_ARG_POINTER(aWhiteListAbURI);
-  *aWhiteListAbURI = ToNewCString(mWhiteListAbURI);
+NS_IMETHODIMP nsSpamSettings::GetWhiteListAbURI(nsACString& aWhiteListAbURI) {
+  aWhiteListAbURI = mWhiteListAbURI;
   return NS_OK;
 }
-NS_IMETHODIMP nsSpamSettings::SetWhiteListAbURI(const char* aWhiteListAbURI) {
+NS_IMETHODIMP nsSpamSettings::SetWhiteListAbURI(
+    const nsACString& aWhiteListAbURI) {
   mWhiteListAbURI = aWhiteListAbURI;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsSpamSettings::GetActionTargetAccount(
-    char** aActionTargetAccount) {
-  NS_ENSURE_ARG_POINTER(aActionTargetAccount);
-  *aActionTargetAccount = ToNewCString(mActionTargetAccount);
+    nsACString& aActionTargetAccount) {
+  aActionTargetAccount = mActionTargetAccount;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsSpamSettings::SetActionTargetAccount(
-    const char* aActionTargetAccount) {
+    const nsACString& aActionTargetAccount) {
   mActionTargetAccount = aActionTargetAccount;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsSpamSettings::GetActionTargetFolder(
-    char** aActionTargetFolder) {
-  NS_ENSURE_ARG_POINTER(aActionTargetFolder);
-  *aActionTargetFolder = ToNewCString(mActionTargetFolder);
+    nsACString& aActionTargetFolder) {
+  aActionTargetFolder = mActionTargetFolder;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsSpamSettings::SetActionTargetFolder(
-    const char* aActionTargetFolder) {
+    const nsACString& aActionTargetFolder) {
   mActionTargetFolder = aActionTargetFolder;
   return NS_OK;
 }
@@ -250,13 +248,13 @@ NS_IMETHODIMP nsSpamSettings::Initialize(nsIMsgIncomingServer* aServer) {
   rv =
       aServer->GetCharValue("spamActionTargetAccount", spamActionTargetAccount);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = SetActionTargetAccount(spamActionTargetAccount.get());
+  rv = SetActionTargetAccount(spamActionTargetAccount);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCString spamActionTargetFolder;
   rv = aServer->GetCharValue("spamActionTargetFolder", spamActionTargetFolder);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = SetActionTargetFolder(spamActionTargetFolder.get());
+  rv = SetActionTargetFolder(spamActionTargetFolder);
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool useWhiteList;
@@ -268,7 +266,7 @@ NS_IMETHODIMP nsSpamSettings::Initialize(nsIMsgIncomingServer* aServer) {
   nsCString whiteListAbURI;
   rv = aServer->GetCharValue("whiteListAbURI", whiteListAbURI);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = SetWhiteListAbURI(whiteListAbURI.get());
+  rv = SetWhiteListAbURI(whiteListAbURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool purgeSpam;
@@ -396,7 +394,7 @@ nsresult nsSpamSettings::UpdateJunkFolderState() {
   // if the spam folder uri changed on us, we need to unset the junk flag
   // on the old spam folder
   nsCString newJunkFolderURI;
-  rv = GetSpamFolderURI(getter_Copies(newJunkFolderURI));
+  rv = GetSpamFolderURI(newJunkFolderURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!mCurrentJunkFolderURI.IsEmpty() &&
@@ -449,18 +447,17 @@ NS_IMETHODIMP nsSpamSettings::Clone(nsISpamSettings* aSpamSettings) {
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCString actionTargetAccount;
-  rv =
-      aSpamSettings->GetActionTargetAccount(getter_Copies(actionTargetAccount));
+  rv = aSpamSettings->GetActionTargetAccount(actionTargetAccount);
   NS_ENSURE_SUCCESS(rv, rv);
   mActionTargetAccount = actionTargetAccount;
 
   nsCString actionTargetFolder;
-  rv = aSpamSettings->GetActionTargetFolder(getter_Copies(actionTargetFolder));
+  rv = aSpamSettings->GetActionTargetFolder(actionTargetFolder);
   NS_ENSURE_SUCCESS(rv, rv);
   mActionTargetFolder = actionTargetFolder;
 
   nsCString whiteListAbURI;
-  rv = aSpamSettings->GetWhiteListAbURI(getter_Copies(whiteListAbURI));
+  rv = aSpamSettings->GetWhiteListAbURI(whiteListAbURI);
   NS_ENSURE_SUCCESS(rv, rv);
   mWhiteListAbURI = whiteListAbURI;
 
@@ -470,16 +467,14 @@ NS_IMETHODIMP nsSpamSettings::Clone(nsISpamSettings* aSpamSettings) {
   return rv;
 }
 
-NS_IMETHODIMP nsSpamSettings::GetSpamFolderURI(char** aSpamFolderURI) {
-  NS_ENSURE_ARG_POINTER(aSpamFolderURI);
-
+NS_IMETHODIMP nsSpamSettings::GetSpamFolderURI(nsACString& aSpamFolderURI) {
   if (mMoveTargetMode == nsISpamSettings::MOVE_TARGET_MODE_FOLDER)
     return GetActionTargetFolder(aSpamFolderURI);
 
   // if the mode is nsISpamSettings::MOVE_TARGET_MODE_ACCOUNT
   // the spam folder URI = account uri + "/Junk"
   nsCString folderURI;
-  nsresult rv = GetActionTargetAccount(getter_Copies(folderURI));
+  nsresult rv = GetActionTargetAccount(folderURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // we might be trying to get the old spam folder uri
@@ -520,11 +515,8 @@ NS_IMETHODIMP nsSpamSettings::GetSpamFolderURI(char** aSpamFolderURI) {
     if (!folderUriWithNamespace.IsEmpty()) folderURI = folderUriWithNamespace;
   }
 
-  *aSpamFolderURI = ToNewCString(folderURI);
-  if (!*aSpamFolderURI)
-    return NS_ERROR_OUT_OF_MEMORY;
-  else
-    return rv;
+  aSpamFolderURI = folderURI;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsSpamSettings::GetServerFilterName(nsACString& aFilterName) {
@@ -609,9 +601,10 @@ NS_IMETHODIMP nsSpamSettings::LogJunkHit(nsIMsgDBHdr* aMsgHdr,
   PRExplodedTime exploded;
   PR_ExplodeTime(date, PR_LocalTimeParameters, &exploded);
 
-  mozilla::DateTimeFormat::FormatPRExplodedTime(mozilla::kDateFormatShort,
-                                                mozilla::kTimeFormatSeconds,
-                                                &exploded, dateValue);
+  mozilla::intl::DateTimeFormat::StyleBag style;
+  style.date = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Short);
+  style.time = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Long);
+  mozilla::intl::AppDateTimeFormat::Format(style, &exploded, dateValue);
 
   (void)aMsgHdr->GetMime2DecodedAuthor(authorValue);
   (void)aMsgHdr->GetMime2DecodedSubject(subjectValue);
@@ -645,7 +638,7 @@ NS_IMETHODIMP nsSpamSettings::LogJunkHit(nsIMsgDBHdr* aMsgHdr,
     aMsgHdr->GetMessageId(getter_Copies(msgId));
 
     nsCString junkFolderURI;
-    rv = GetSpamFolderURI(getter_Copies(junkFolderURI));
+    rv = GetSpamFolderURI(junkFolderURI);
     NS_ENSURE_SUCCESS(rv, rv);
 
     AutoTArray<nsString, 2> logMoveFormatStrings;
@@ -674,9 +667,10 @@ NS_IMETHODIMP nsSpamSettings::LogJunkString(const char* string) {
   PRExplodedTime exploded;
   PR_ExplodeTime(PR_Now(), PR_LocalTimeParameters, &exploded);
 
-  mozilla::DateTimeFormat::FormatPRExplodedTime(mozilla::kDateFormatShort,
-                                                mozilla::kTimeFormatSeconds,
-                                                &exploded, dateValue);
+  mozilla::intl::DateTimeFormat::StyleBag style;
+  style.date = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Short);
+  style.time = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Long);
+  mozilla::intl::AppDateTimeFormat::Format(style, &exploded, dateValue);
 
   nsCString timestampString(LOG_ENTRY_TIMESTAMP);
   timestampString.ReplaceSubstring("$S",
@@ -727,7 +721,7 @@ NS_IMETHODIMP nsSpamSettings::OnStartRunningUrl(nsIURI* aURL) {
 NS_IMETHODIMP nsSpamSettings::OnStopRunningUrl(nsIURI* aURL,
                                                nsresult exitCode) {
   nsCString junkFolderURI;
-  nsresult rv = GetSpamFolderURI(getter_Copies(junkFolderURI));
+  nsresult rv = GetSpamFolderURI(junkFolderURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (junkFolderURI.IsEmpty()) return NS_ERROR_UNEXPECTED;
