@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -6,7 +5,9 @@
 /* Tests for platform-independent code to count new and unread messages and pass the
  * information to platform-specific notification modules */
 
-ChromeUtils.import("resource:///modules/mailServices.js");
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
 
 var iNMNS = Ci.mozINewMailNotificationService;
 
@@ -16,7 +17,7 @@ var iNMNS = Ci.mozINewMailNotificationService;
  */
 add_test(function testListeners() {
   let notif = MailServices.newMailNotification.wrappedJSObject;
-  let listener = {a: 1};
+  let listener = { a: 1 };
 
   notif.addListener(listener, iNMNS.count);
   let list = notif._listenersForFlag(iNMNS.count);
@@ -39,10 +40,10 @@ add_test(function testListeners() {
  */
 add_test(function testMultiListeners() {
   let notif = MailServices.newMailNotification.wrappedJSObject;
-  let l1 = {a: 1};
-  let l2 = {b: 2};
+  let l1 = { a: 1 };
+  let l2 = { b: 2 };
 
-  notif.addListener(l1, (iNMNS.count | iNMNS.messages));
+  notif.addListener(l1, iNMNS.count | iNMNS.messages);
   // do_check_eq(notif._listeners.length, 1);
   notif.addListener(l2, iNMNS.messages);
   // do_check_eq(notif._listeners.length, 2);
@@ -70,46 +71,44 @@ var countInboxesPref = "mail.notification.count.inbox_only";
 add_test(function testNotifyInbox() {
   let notified = false;
   let mockListener = {
-    onCountChanged: function TNU_onCountChanged(count) {notified = true;}
+    onCountChanged: function TNU_onCountChanged(count) {
+      notified = true;
+    },
   };
   let folder = {
     URI: "Test Inbox",
-    flags: Ci.nsMsgFolderFlags.Mail | Ci.nsMsgFolderFlags.Inbox
+    flags: Ci.nsMsgFolderFlags.Mail | Ci.nsMsgFolderFlags.Inbox,
   };
 
   let notif = MailServices.newMailNotification.wrappedJSObject;
   notif.addListener(mockListener, iNMNS.count);
 
-  notif.OnItemIntPropertyChanged(folder, "TotalUnreadMessages", 0, 2);
+  notif.onFolderIntPropertyChanged(folder, "TotalUnreadMessages", 0, 2);
   Assert.ok(notified);
 
   // Special folders should never count
   let special = {
     URI: "Test Special",
-    flags: Ci.nsMsgFolderFlags.Mail | Ci.nsMsgFolderFlags.Junk
+    flags: Ci.nsMsgFolderFlags.Mail | Ci.nsMsgFolderFlags.Junk,
   };
   notified = false;
-  notif.OnItemIntPropertyChanged(special, "TotalUnreadMessages", 0, 2);
+  notif.onFolderIntPropertyChanged(special, "TotalUnreadMessages", 0, 2);
   Assert.ok(!notified);
 
   // by default, non-inbox should not count
   let nonInbox = {
     URI: "Test Non-Inbox",
-    flags: Ci.nsMsgFolderFlags.Mail
+    flags: Ci.nsMsgFolderFlags.Mail,
   };
   notified = false;
-  notif.OnItemIntPropertyChanged(nonInbox, "TotalUnreadMessages", 0, 2);
+  notif.onFolderIntPropertyChanged(nonInbox, "TotalUnreadMessages", 0, 2);
   Assert.ok(!notified);
 
   // Try setting the pref to count non-inboxes and notifying a non-inbox
   Services.prefs.setBoolPref(countInboxesPref, false);
   notified = false;
-  notif.OnItemIntPropertyChanged(nonInbox, "TotalUnreadMessages", 0, 2);
+  notif.onFolderIntPropertyChanged(nonInbox, "TotalUnreadMessages", 0, 2);
   Assert.ok(notified);
 
   run_next_test();
 });
-
-function run_test() {
-  run_next_test();
-}
