@@ -4,13 +4,16 @@
  * files.
  */
 
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 function run_test() {
-  const items = [ { key: "MailD", value: "Mail" },
-                  { key: "IMapMD", value: "ImapMail" },
-                  { key: "NewsD", value: "News" },
-                  { key: "MFCaF", value: "panacea.dat" } ];
+  const items = [
+    { key: "MailD", value: "Mail" },
+    { key: "IMapMD", value: "ImapMail" },
+    { key: "NewsD", value: "News" },
+    { key: "MLFCaF", value: "panacea.dat" }, // Legacy folder cache.
+    { key: "MFCaF", value: "folderCache.json" },
+  ];
 
   items.forEach(function(item) {
     var dir = Services.dirsvc.get(item.key, Ci.nsIFile);
@@ -19,4 +22,4 @@ function run_test() {
 
     Assert.equal(dir.leafName, item.value);
   });
-};
+}
