@@ -52,7 +52,7 @@ function sendMessage(fieldParams, identity, opts = {}, attachments = []) {
     progressListener.reject = reject;
   });
   progress.registerListener(progressListener);
-  msgCompose.SendMsg(
+  msgCompose.sendMsg(
     Ci.nsIMsgSend.nsMsgDeliverNow,
     identity,
     "",
@@ -81,10 +81,17 @@ function checkMessageHeaders(msgData, expectedHeaders, partNum = "") {
       for (let header in expectedHeaders) {
         let expected = expectedHeaders[header];
         if (expected === undefined) {
-          Assert.ok(!headers.has(header));
+          Assert.ok(
+            !headers.has(header),
+            `Should not have header named "${header}"`
+          );
         } else {
           let value = headers.getRawHeader(header);
-          Assert.equal(value.length, 1);
+          Assert.equal(
+            value && value.length,
+            1,
+            `Should have exactly one header named "${header}"`
+          );
           value[0] = value[0].replace(/boundary=[^;]*(;|$)/, "boundary=.");
           Assert.equal(value[0], expected);
         }
@@ -339,7 +346,10 @@ async function testSendHeaders() {
     getBasicSmtpServer()
   );
   identity.setCharAttribute("headers", "bah,humbug");
-  identity.setCharAttribute("header.bah", "X-Custom-1: A header value");
+  identity.setCharAttribute(
+    "header.bah",
+    "X-Custom-1: A header value: with a colon"
+  );
   identity.setUnicharAttribute("header.humbug", "X-Custom-2: Enchanté");
   identity.setCharAttribute("subscribed_mailing_lists", "list@test.invalid");
   identity.setCharAttribute(
@@ -350,8 +360,8 @@ async function testSendHeaders() {
   fields.cc = "not-list@test.invalid";
   await richCreateMessage(fields, [], identity);
   checkDraftHeaders({
-    "X-Custom-1": "A header value",
-    "X-Custom-2": "=?UTF-8?Q?_Enchant=c3=a9?=",
+    "X-Custom-1": "A header value: with a colon",
+    "X-Custom-2": "=?UTF-8?B?RW5jaGFudMOp?=",
     "Mail-Followup-To": "list@test.invalid, not-list@test.invalid",
     "Mail-Reply-To": undefined,
   });
@@ -361,8 +371,8 @@ async function testSendHeaders() {
   fields.cc = "";
   await richCreateMessage(fields, [], identity);
   checkDraftHeaders({
-    "X-Custom-1": "A header value",
-    "X-Custom-2": "=?UTF-8?Q?_Enchant=c3=a9?=",
+    "X-Custom-1": "A header value: with a colon",
+    "X-Custom-2": "=?UTF-8?B?RW5jaGFudMOp?=",
     "Mail-Reply-To": "from@tinderbox.invalid",
     "Mail-Followup-To": undefined,
   });
@@ -391,15 +401,6 @@ async function testContentHeaders() {
     "Content-Transfer-Encoding": "8bit",
   });
 
-  // What if we change the message charset?
-  fields.characterSet = "ISO-8859-1";
-  fields.body = "Archæologist";
-  await richCreateMessage(fields, [], identity);
-  checkDraftHeaders({
-    "Content-Type": "text/html; charset=ISO-8859-1",
-    "Content-Transfer-Encoding": "8bit",
-  });
-
   // Attachments
   fields.body = "";
   let plainAttachment = makeAttachment({
@@ -414,7 +415,7 @@ async function testContentHeaders() {
   await richCreateMessage(fields, [plainAttachment], identity);
   checkDraftHeaders(
     {
-      "Content-Type": "text/html; charset=ISO-8859-1",
+      "Content-Type": "text/html; charset=UTF-8",
       "Content-Transfer-Encoding": "7bit",
     },
     "1"
@@ -440,35 +441,34 @@ async function testContentHeaders() {
   let httpAttachmentHeaders = {
     "Content-Type": "text/html; charset=UTF-8",
     "Content-Disposition": 'attachment; filename="attachment.html"',
-    "Content-Base": '"data:text/html,<html></html>"',
-    "Content-Location": '"data:text/html,<html></html>"',
+    "Content-Location": "data:text/html,<html></html>",
   };
   await richCreateMessage(fields, [httpAttachment], identity);
   checkDraftHeaders(
     {
-      "Content-Base": undefined,
       "Content-Location": undefined,
     },
     "1"
   );
   checkDraftHeaders(httpAttachmentHeaders, "2");
 
-  fields.characterSet = "UTF-8";
   let cloudAttachment = makeAttachment({
     url: Services.io.newFileURI(do_get_file("data/test-UTF-8.txt")).spec,
     sendViaCloud: true,
+    htmlAnnotation:
+      "<html><body>This is an html placeholder file.</body></html>",
     cloudFileAccountKey: "akey",
+    cloudPartHeaderData: "0123456789ABCDE",
     name: "attachment.html",
     contentLocation: "http://localhost.invalid/",
   });
   let cloudAttachmentHeaders = {
-    "Content-Type": "application/octet-stream",
+    "Content-Type": "text/html",
     "X-Mozilla-Cloud-Part":
-      "cloudFile; url=http://localhost.invalid/; " +
+      "cloudFile; " +
+      "url=http://localhost.invalid/; " +
       "provider=akey; " +
-      "file=" +
-      cloudAttachment.url +
-      '; name="attachment.html"',
+      'data="0123456789ABCDE"',
   };
   await richCreateMessage(fields, [cloudAttachment], identity);
   checkDraftHeaders(cloudAttachmentHeaders, "2");
@@ -477,18 +477,20 @@ async function testContentHeaders() {
   cloudAttachment = makeAttachment({
     url: Services.io.newFileURI(do_get_file("data/test-UTF-8.txt")).spec,
     sendViaCloud: true,
+    htmlAnnotation:
+      "<html><body>This is an html placeholder file.</body></html>",
     cloudFileAccountKey: "akey",
+    cloudPartHeaderData: "0123456789ABCDE",
     name: "ファイル.txt",
     contentLocation: "http://localhost.invalid/",
   });
   cloudAttachmentHeaders = {
-    "Content-Type": "application/octet-stream",
+    "Content-Type": "text/html",
     "X-Mozilla-Cloud-Part":
-      "cloudFile; url=http://localhost.invalid/; " +
+      "cloudFile; " +
+      "url=http://localhost.invalid/; " +
       "provider=akey; " +
-      "file=" +
-      cloudAttachment.url +
-      "; name*=UTF-8''%E3%83%95%E3%82%A1%E3%82%A4%E3%83%AB%2E%74%78%74",
+      'data="0123456789ABCDE"',
   };
   await richCreateMessage(fields, [cloudAttachment], identity);
   checkDraftHeaders(cloudAttachmentHeaders, "2");
@@ -576,9 +578,9 @@ async function testContentHeaders() {
     },
     "1.2"
   );
-  checkDraftHeaders(cloudAttachmentHeaders, "2");
-  checkDraftHeaders(plainAttachmentHeaders, "3");
-  checkDraftHeaders(httpAttachmentHeaders, "4");
+  checkDraftHeaders(plainAttachmentHeaders, "2");
+  checkDraftHeaders(httpAttachmentHeaders, "3");
+  checkDraftHeaders(cloudAttachmentHeaders, "4");
 
   // Test a request for plain text with text/html.
   fields.forcePlainText = true;
@@ -608,6 +610,7 @@ async function testSentMessage() {
       {},
       []
     );
+    server.performTest();
     checkMessageHeaders(daemon.post, {
       From: "test@tinderbox.invalid",
       To: "Nobody <nobody@tinderbox.invalid>",
@@ -619,10 +622,13 @@ async function testSentMessage() {
       "X-Mozilla-Draft-Info": undefined,
       Fcc: undefined,
     });
+    server.resetTest();
     await sendMessage({ bcc: "Somebody <test@tinderbox.invalid" }, identity);
+    server.performTest();
     checkMessageHeaders(daemon.post, {
       To: "undisclosed-recipients: ;",
     });
+    server.resetTest();
     await sendMessage(
       {
         to: "Somebody <test@tinderbox.invalid>",
@@ -631,26 +637,31 @@ async function testSentMessage() {
       },
       identity
     );
+    server.performTest();
     checkMessageHeaders(daemon.post, {
       "Disposition-Notification-To": "test@tinderbox.invalid",
       "Return-Receipt-To": "test@tinderbox.invalid",
     });
+    server.resetTest();
     let cloudAttachment = makeAttachment({
       url: Services.io.newFileURI(do_get_file("data/test-UTF-8.txt")).spec,
       sendViaCloud: true,
+      htmlAnnotation:
+        "<html><body>This is an html placeholder file.</body></html>",
       cloudFileAccountKey: "akey",
+      cloudPartHeaderData: "0123456789ABCDE",
       name: "attachment.html",
       contentLocation: "http://localhost.invalid/",
     });
     await sendMessage({ to: "test@tinderbox.invalid" }, identity, {}, [
       cloudAttachment,
     ]);
+    server.performTest();
     checkMessageHeaders(
       daemon.post,
       {
-        "Content-Type": "application/octet-stream",
-        "X-Mozilla-Cloud-Part":
-          'cloudFile; url=http://localhost.invalid/; name="attachment.html"',
+        "Content-Type": "text/html",
+        "X-Mozilla-Cloud-Part": "cloudFile; url=http://localhost.invalid/",
       },
       "2"
     );
