@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -13,7 +13,9 @@
 #include "nsMsgMimeCID.h"
 #include "mozilla/Attributes.h"
 #include "nsStringEnumerator.h"
-
+#ifdef DEBUG
+#  include "nsPrintfCString.h"
+#endif
 using namespace mozilla::mailnews;
 
 NS_IMPL_ISUPPORTS(nsMsgHdr, nsIMsgDBHdr)
@@ -23,7 +25,7 @@ NS_IMPL_ISUPPORTS(nsMsgHdr, nsIMsgDBHdr)
 #define REFERENCES_INITED 0x4
 #define THREAD_PARENT_INITED 0x8
 
-nsMsgHdr::nsMsgHdr(nsMsgDatabase *db, nsIMdbRow *dbRow) {
+nsMsgHdr::nsMsgHdr(nsMsgDatabase* db, nsIMdbRow* dbRow) {
   m_mdb = db;
   Init();
   m_mdbRow = dbRow;
@@ -32,14 +34,13 @@ nsMsgHdr::nsMsgHdr(nsMsgDatabase *db, nsIMdbRow *dbRow) {
     mdbOid outOid;
     if (dbRow && NS_SUCCEEDED(dbRow->GetOid(m_mdb->GetEnv(), &outOid))) {
       m_messageKey = outOid.mOid_Id;
-      m_mdb->AddHdrToUseCache((nsIMsgDBHdr *)this, m_messageKey);
+      m_mdb->AddHdrToUseCache((nsIMsgDBHdr*)this, m_messageKey);
     }
   }
 }
 
 void nsMsgHdr::Init() {
   m_initedValues = 0;
-  m_statusOffset = 0xffffffff;
   m_messageKey = nsMsgKey_None;
   m_messageSize = 0;
   m_date = 0;
@@ -91,13 +92,13 @@ nsMsgHdr::~nsMsgHdr() {
   if (m_mdbRow) {
     if (m_mdb) {
       NS_RELEASE(m_mdbRow);
-      m_mdb->RemoveHdrFromUseCache((nsIMsgDBHdr *)this, m_messageKey);
+      m_mdb->RemoveHdrFromUseCache((nsIMsgDBHdr*)this, m_messageKey);
     }
   }
   NS_IF_RELEASE(m_mdb);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMessageKey(nsMsgKey *result) {
+NS_IMETHODIMP nsMsgHdr::GetMessageKey(nsMsgKey* result) {
   if (m_messageKey == nsMsgKey_None && m_mdbRow != NULL) {
     mdbOid outOid;
     if (NS_SUCCEEDED(m_mdbRow->GetOid(m_mdb->GetEnv(), &outOid)))
@@ -107,7 +108,7 @@ NS_IMETHODIMP nsMsgHdr::GetMessageKey(nsMsgKey *result) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetThreadId(nsMsgKey *result) {
+NS_IMETHODIMP nsMsgHdr::GetThreadId(nsMsgKey* result) {
   if (!(m_initedValues & CACHED_VALUES_INITED)) InitCachedValues();
 
   if (result) {
@@ -127,13 +128,13 @@ NS_IMETHODIMP nsMsgHdr::SetMessageKey(nsMsgKey value) {
   return NS_OK;
 }
 
-nsresult nsMsgHdr::GetRawFlags(uint32_t *result) {
+nsresult nsMsgHdr::GetRawFlags(uint32_t* result) {
   if (!(m_initedValues & FLAGS_INITED)) InitFlags();
   *result = m_flags;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetFlags(uint32_t *result) {
+NS_IMETHODIMP nsMsgHdr::GetFlags(uint32_t* result) {
   if (!(m_initedValues & FLAGS_INITED)) InitFlags();
   if (m_mdb)
     *result = m_mdb->GetStatusFlags(this, m_flags);
@@ -158,14 +159,14 @@ NS_IMETHODIMP nsMsgHdr::SetFlags(uint32_t flags) {
                          m_mdb->m_flagsColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::OrFlags(uint32_t flags, uint32_t *result) {
+NS_IMETHODIMP nsMsgHdr::OrFlags(uint32_t flags, uint32_t* result) {
   if (!(m_initedValues & FLAGS_INITED)) InitFlags();
   if ((m_flags & flags) != flags) SetFlags(m_flags | flags);
   *result = m_flags;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::AndFlags(uint32_t flags, uint32_t *result) {
+NS_IMETHODIMP nsMsgHdr::AndFlags(uint32_t flags, uint32_t* result) {
   if (!(m_initedValues & FLAGS_INITED)) InitFlags();
   if ((m_flags & flags) != m_flags) SetFlags(m_flags & flags);
   *result = m_flags;
@@ -206,51 +207,51 @@ NS_IMETHODIMP nsMsgHdr::MarkFlagged(bool bFlagged) {
   return rv;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetProperty(const char *propertyName,
-                                    nsAString &resultProperty) {
+NS_IMETHODIMP nsMsgHdr::GetProperty(const char* propertyName,
+                                    nsAString& resultProperty) {
   NS_ENSURE_ARG_POINTER(propertyName);
   if (!m_mdb || !m_mdbRow) return NS_ERROR_NULL_POINTER;
   return m_mdb->GetPropertyAsNSString(m_mdbRow, propertyName, resultProperty);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetProperty(const char *propertyName,
-                                    const nsAString &propertyStr) {
+NS_IMETHODIMP nsMsgHdr::SetProperty(const char* propertyName,
+                                    const nsAString& propertyStr) {
   NS_ENSURE_ARG_POINTER(propertyName);
   if (!m_mdb || !m_mdbRow) return NS_ERROR_NULL_POINTER;
   return m_mdb->SetPropertyFromNSString(m_mdbRow, propertyName, propertyStr);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetStringProperty(const char *propertyName,
-                                          const char *propertyValue) {
+NS_IMETHODIMP nsMsgHdr::SetStringProperty(const char* propertyName,
+                                          const char* propertyValue) {
   NS_ENSURE_ARG_POINTER(propertyName);
   if (!m_mdb || !m_mdbRow) return NS_ERROR_NULL_POINTER;
   return m_mdb->SetProperty(m_mdbRow, propertyName, propertyValue);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetStringProperty(const char *propertyName,
-                                          char **aPropertyValue) {
+NS_IMETHODIMP nsMsgHdr::GetStringProperty(const char* propertyName,
+                                          char** aPropertyValue) {
   NS_ENSURE_ARG_POINTER(propertyName);
   if (!m_mdb || !m_mdbRow) return NS_ERROR_NULL_POINTER;
   return m_mdb->GetProperty(m_mdbRow, propertyName, aPropertyValue);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetUint32Property(const char *propertyName,
-                                          uint32_t *pResult) {
+NS_IMETHODIMP nsMsgHdr::GetUint32Property(const char* propertyName,
+                                          uint32_t* pResult) {
   NS_ENSURE_ARG_POINTER(propertyName);
   if (!m_mdb || !m_mdbRow) return NS_ERROR_NULL_POINTER;
   return m_mdb->GetUint32Property(m_mdbRow, propertyName, pResult);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetUint32Property(const char *propertyName,
+NS_IMETHODIMP nsMsgHdr::SetUint32Property(const char* propertyName,
                                           uint32_t value) {
   NS_ENSURE_ARG_POINTER(propertyName);
   if (!m_mdb || !m_mdbRow) return NS_ERROR_NULL_POINTER;
   return m_mdb->SetUint32Property(m_mdbRow, propertyName, value);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetNumReferences(uint16_t *result) {
+NS_IMETHODIMP nsMsgHdr::GetNumReferences(uint16_t* result) {
   if (!(m_initedValues & REFERENCES_INITED)) {
-    const char *references;
+    const char* references;
     if (NS_SUCCEEDED(m_mdb->RowCellColumnToConstCharPtr(
             GetMDBRow(), m_mdb->m_referencesColumnToken, &references)))
       ParseReferences(references);
@@ -263,8 +264,8 @@ NS_IMETHODIMP nsMsgHdr::GetNumReferences(uint16_t *result) {
   return NS_OK;
 }
 
-nsresult nsMsgHdr::ParseReferences(const char *references) {
-  const char *startNextRef = references;
+nsresult nsMsgHdr::ParseReferences(const char* references) {
+  const char* startNextRef = references;
   nsAutoCString resultReference;
   nsCString messageId;
   GetMessageId(getter_Copies(messageId));
@@ -280,7 +281,7 @@ nsresult nsMsgHdr::ParseReferences(const char *references) {
 }
 
 NS_IMETHODIMP nsMsgHdr::GetStringReference(int32_t refNum,
-                                           nsACString &resultReference) {
+                                           nsACString& resultReference) {
   nsresult err = NS_OK;
 
   if (!(m_initedValues & REFERENCES_INITED))
@@ -293,18 +294,18 @@ NS_IMETHODIMP nsMsgHdr::GetStringReference(int32_t refNum,
   return err;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetDate(PRTime *result) {
+NS_IMETHODIMP nsMsgHdr::GetDate(PRTime* result) {
   if (!(m_initedValues & CACHED_VALUES_INITED)) InitCachedValues();
 
   *result = m_date;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetDateInSeconds(uint32_t *aResult) {
+NS_IMETHODIMP nsMsgHdr::GetDateInSeconds(uint32_t* aResult) {
   return GetUInt32Column(m_mdb->m_dateColumnToken, aResult);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetMessageId(const char *messageId) {
+NS_IMETHODIMP nsMsgHdr::SetMessageId(const char* messageId) {
   if (messageId && *messageId == '<') {
     nsAutoCString tempMessageID(messageId + 1);
     if (tempMessageID.CharAt(tempMessageID.Length() - 1) == '>')
@@ -314,15 +315,15 @@ NS_IMETHODIMP nsMsgHdr::SetMessageId(const char *messageId) {
   return SetStringColumn(messageId, m_mdb->m_messageIdColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetSubject(const char *subject) {
+NS_IMETHODIMP nsMsgHdr::SetSubject(const char* subject) {
   return SetStringColumn(subject, m_mdb->m_subjectColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetAuthor(const char *author) {
+NS_IMETHODIMP nsMsgHdr::SetAuthor(const char* author) {
   return SetStringColumn(author, m_mdb->m_senderColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetReferences(const char *references) {
+NS_IMETHODIMP nsMsgHdr::SetReferences(const char* references) {
   NS_ENSURE_ARG_POINTER(references);
   m_references.Clear();
   ParseReferences(references);
@@ -332,16 +333,16 @@ NS_IMETHODIMP nsMsgHdr::SetReferences(const char *references) {
   return SetStringColumn(references, m_mdb->m_referencesColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetRecipients(const char *recipients) {
+NS_IMETHODIMP nsMsgHdr::SetRecipients(const char* recipients) {
   // need to put in rfc822 address parsing code here (or make caller do it...)
   return SetStringColumn(recipients, m_mdb->m_recipientsColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetCcList(const char *ccList) {
+NS_IMETHODIMP nsMsgHdr::SetCcList(const char* ccList) {
   return SetStringColumn(ccList, m_mdb->m_ccListColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetBccList(const char *bccList) {
+NS_IMETHODIMP nsMsgHdr::SetBccList(const char* bccList) {
   return SetStringColumn(bccList, m_mdb->m_bccListColumnToken);
 }
 
@@ -351,7 +352,7 @@ NS_IMETHODIMP nsMsgHdr::SetMessageSize(uint32_t messageSize) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetOfflineMessageSize(uint32_t *result) {
+NS_IMETHODIMP nsMsgHdr::GetOfflineMessageSize(uint32_t* result) {
   uint32_t size;
   nsresult res = GetUInt32Column(m_mdb->m_offlineMessageSizeColumnToken, &size);
 
@@ -368,10 +369,6 @@ NS_IMETHODIMP nsMsgHdr::SetLineCount(uint32_t lineCount) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::SetStatusOffset(uint32_t statusOffset) {
-  return SetUInt32Column(statusOffset, m_mdb->m_statusOffsetColumnToken);
-}
-
 NS_IMETHODIMP nsMsgHdr::SetDate(PRTime date) {
   m_date = date;
   uint32_t seconds;
@@ -379,20 +376,12 @@ NS_IMETHODIMP nsMsgHdr::SetDate(PRTime date) {
   return SetUInt32Column((uint32_t)seconds, m_mdb->m_dateColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetStatusOffset(uint32_t *result) {
-  uint32_t offset = 0;
-  nsresult res = GetUInt32Column(m_mdb->m_statusOffsetColumnToken, &offset);
-
-  *result = offset;
-  return res;
-}
-
 NS_IMETHODIMP nsMsgHdr::SetPriority(nsMsgPriorityValue priority) {
   SetUInt32Column((uint32_t)priority, m_mdb->m_priorityColumnToken);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetPriority(nsMsgPriorityValue *result) {
+NS_IMETHODIMP nsMsgHdr::GetPriority(nsMsgPriorityValue* result) {
   if (!result) return NS_ERROR_NULL_POINTER;
 
   uint32_t priority = 0;
@@ -408,7 +397,7 @@ NS_IMETHODIMP nsMsgHdr::SetLabel(nsMsgLabelValue label) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetLabel(nsMsgLabelValue *result) {
+NS_IMETHODIMP nsMsgHdr::GetLabel(nsMsgLabelValue* result) {
   NS_ENSURE_ARG_POINTER(result);
 
   return GetUInt32Column(m_mdb->m_labelColumnToken, result);
@@ -419,23 +408,38 @@ NS_IMETHODIMP nsMsgHdr::GetLabel(nsMsgLabelValue *result) {
 // This might be problematic when a message gets moved...
 // And I'm not sure if we should short circuit it here,
 // or at a higher level where it might be more efficient.
-NS_IMETHODIMP nsMsgHdr::SetAccountKey(const char *aAccountKey) {
+NS_IMETHODIMP nsMsgHdr::SetAccountKey(const char* aAccountKey) {
   return SetStringProperty("account", aAccountKey);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetAccountKey(char **aResult) {
+NS_IMETHODIMP nsMsgHdr::GetAccountKey(char** aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
 
   return GetStringProperty("account", aResult);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMessageOffset(uint64_t *result) {
+NS_IMETHODIMP nsMsgHdr::GetMessageOffset(uint64_t* result) {
   NS_ENSURE_ARG(result);
 
-  // if there is a message offset, use it, otherwise, we'll use the message key.
   (void)GetUInt64Column(m_mdb->m_offlineMsgOffsetColumnToken, result,
                         (unsigned)-1);
-  if (*result == (unsigned)-1) *result = m_messageKey;
+  if (*result == (unsigned)-1) {
+    // It's unset. Unfortunately there's not much we can do here. There's
+    // a lot of code which relies on being able to read .messageOffset,
+    // even if it doesn't require it to return anything sensible.
+    // (For example - in js unit tests - Assert.equals() stringifies the
+    // attributes of it's expected/actual values to produce an error
+    // message even if the assert passes).
+#ifdef DEBUG
+    nsCString tok;
+    GetStringProperty("storeToken", getter_Copies(tok));
+    nsPrintfCString err("Missing .messageOffset (key=%u, storeToken='%s')",
+                        m_messageKey, tok.get());
+    NS_WARNING(err.get());
+#endif
+    // Return something obviously broken.
+    *result = 12345678;
+  }
   return NS_OK;
 }
 
@@ -444,7 +448,7 @@ NS_IMETHODIMP nsMsgHdr::SetMessageOffset(uint64_t offset) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMessageSize(uint32_t *result) {
+NS_IMETHODIMP nsMsgHdr::GetMessageSize(uint32_t* result) {
   uint32_t size;
   nsresult res = GetUInt32Column(m_mdb->m_messageSizeColumnToken, &size);
 
@@ -452,14 +456,14 @@ NS_IMETHODIMP nsMsgHdr::GetMessageSize(uint32_t *result) {
   return res;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetLineCount(uint32_t *result) {
+NS_IMETHODIMP nsMsgHdr::GetLineCount(uint32_t* result) {
   uint32_t linecount;
   nsresult res = GetUInt32Column(m_mdb->m_numLinesColumnToken, &linecount);
   *result = linecount;
   return res;
 }
 
-NS_IMETHODIMP nsMsgHdr::SetPriorityString(const char *priority) {
+NS_IMETHODIMP nsMsgHdr::SetPriorityString(const char* priority) {
   nsMsgPriorityValue priorityVal = nsMsgPriority::Default;
 
   // We can ignore |NS_MsgGetPriorityFromString()| return value,
@@ -469,79 +473,78 @@ NS_IMETHODIMP nsMsgHdr::SetPriorityString(const char *priority) {
   return SetPriority(priorityVal);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetAuthor(char **resultAuthor) {
+NS_IMETHODIMP nsMsgHdr::GetAuthor(char** resultAuthor) {
   return m_mdb->RowCellColumnToCharPtr(GetMDBRow(), m_mdb->m_senderColumnToken,
                                        resultAuthor);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetSubject(char **resultSubject) {
+NS_IMETHODIMP nsMsgHdr::GetSubject(char** resultSubject) {
   return m_mdb->RowCellColumnToCharPtr(GetMDBRow(), m_mdb->m_subjectColumnToken,
                                        resultSubject);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetRecipients(char **resultRecipients) {
+NS_IMETHODIMP nsMsgHdr::GetRecipients(char** resultRecipients) {
   return m_mdb->RowCellColumnToCharPtr(
       GetMDBRow(), m_mdb->m_recipientsColumnToken, resultRecipients);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetCcList(char **resultCCList) {
+NS_IMETHODIMP nsMsgHdr::GetCcList(char** resultCCList) {
   return m_mdb->RowCellColumnToCharPtr(GetMDBRow(), m_mdb->m_ccListColumnToken,
                                        resultCCList);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetBccList(char **resultBCCList) {
+NS_IMETHODIMP nsMsgHdr::GetBccList(char** resultBCCList) {
   return m_mdb->RowCellColumnToCharPtr(GetMDBRow(), m_mdb->m_bccListColumnToken,
                                        resultBCCList);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMessageId(char **resultMessageId) {
+NS_IMETHODIMP nsMsgHdr::GetMessageId(char** resultMessageId) {
   return m_mdb->RowCellColumnToCharPtr(
       GetMDBRow(), m_mdb->m_messageIdColumnToken, resultMessageId);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMime2DecodedAuthor(nsAString &resultAuthor) {
+NS_IMETHODIMP nsMsgHdr::GetMime2DecodedAuthor(nsAString& resultAuthor) {
   return m_mdb->RowCellColumnToMime2DecodedString(
       GetMDBRow(), m_mdb->m_senderColumnToken, resultAuthor);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMime2DecodedSubject(nsAString &resultSubject) {
+NS_IMETHODIMP nsMsgHdr::GetMime2DecodedSubject(nsAString& resultSubject) {
   return m_mdb->RowCellColumnToMime2DecodedString(
       GetMDBRow(), m_mdb->m_subjectColumnToken, resultSubject);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetMime2DecodedRecipients(nsAString &resultRecipients) {
+NS_IMETHODIMP nsMsgHdr::GetMime2DecodedRecipients(nsAString& resultRecipients) {
   return m_mdb->RowCellColumnToMime2DecodedString(
       GetMDBRow(), m_mdb->m_recipientsColumnToken, resultRecipients);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetAuthorCollationKey(uint32_t *len,
-                                              uint8_t **resultAuthor) {
+NS_IMETHODIMP nsMsgHdr::GetAuthorCollationKey(nsTArray<uint8_t>& resultAuthor) {
   return m_mdb->RowCellColumnToAddressCollationKey(
-      GetMDBRow(), m_mdb->m_senderColumnToken, resultAuthor, len);
+      GetMDBRow(), m_mdb->m_senderColumnToken, resultAuthor);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetSubjectCollationKey(uint32_t *len,
-                                               uint8_t **resultSubject) {
+NS_IMETHODIMP nsMsgHdr::GetSubjectCollationKey(
+    nsTArray<uint8_t>& resultSubject) {
   return m_mdb->RowCellColumnToCollationKey(
-      GetMDBRow(), m_mdb->m_subjectColumnToken, resultSubject, len);
+      GetMDBRow(), m_mdb->m_subjectColumnToken, resultSubject);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetRecipientsCollationKey(uint32_t *len,
-                                                  uint8_t **resultRecipients) {
+NS_IMETHODIMP nsMsgHdr::GetRecipientsCollationKey(
+    nsTArray<uint8_t>& resultRecipients) {
   return m_mdb->RowCellColumnToCollationKey(
-      GetMDBRow(), m_mdb->m_recipientsColumnToken, resultRecipients, len);
+      GetMDBRow(), m_mdb->m_recipientsColumnToken, resultRecipients);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetCharset(char **aCharset) {
+NS_IMETHODIMP nsMsgHdr::GetCharset(char** aCharset) {
   return m_mdb->RowCellColumnToCharPtr(
       GetMDBRow(), m_mdb->m_messageCharSetColumnToken, aCharset);
 }
 
-NS_IMETHODIMP nsMsgHdr::SetCharset(const char *aCharset) {
+NS_IMETHODIMP nsMsgHdr::SetCharset(const char* aCharset) {
   return SetStringColumn(aCharset, m_mdb->m_messageCharSetColumnToken);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetEffectiveCharset(nsACString &resultCharset) {
+NS_IMETHODIMP nsMsgHdr::GetEffectiveCharset(nsACString& resultCharset) {
   return m_mdb->GetEffectiveCharset(m_mdbRow, resultCharset);
 }
 
@@ -553,7 +556,7 @@ NS_IMETHODIMP nsMsgHdr::SetThreadParent(nsMsgKey inKey) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetThreadParent(nsMsgKey *result) {
+NS_IMETHODIMP nsMsgHdr::GetThreadParent(nsMsgKey* result) {
   nsresult res;
   if (!(m_initedValues & THREAD_PARENT_INITED)) {
     res = GetUInt32Column(m_mdb->m_threadParentColumnToken, &m_threadParent,
@@ -564,7 +567,7 @@ NS_IMETHODIMP nsMsgHdr::GetThreadParent(nsMsgKey *result) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetFolder(nsIMsgFolder **result) {
+NS_IMETHODIMP nsMsgHdr::GetFolder(nsIMsgFolder** result) {
   NS_ENSURE_ARG(result);
 
   if (m_mdb && m_mdb->m_folder) {
@@ -574,7 +577,7 @@ NS_IMETHODIMP nsMsgHdr::GetFolder(nsIMsgFolder **result) {
   return NS_OK;
 }
 
-nsresult nsMsgHdr::SetStringColumn(const char *str, mdb_token token) {
+nsresult nsMsgHdr::SetStringColumn(const char* str, mdb_token token) {
   NS_ENSURE_ARG_POINTER(str);
   return m_mdb->CharPtrToRowCellColumn(m_mdbRow, token, str);
 }
@@ -583,7 +586,7 @@ nsresult nsMsgHdr::SetUInt32Column(uint32_t value, mdb_token token) {
   return m_mdb->UInt32ToRowCellColumn(m_mdbRow, token, value);
 }
 
-nsresult nsMsgHdr::GetUInt32Column(mdb_token token, uint32_t *pvalue,
+nsresult nsMsgHdr::GetUInt32Column(mdb_token token, uint32_t* pvalue,
                                    uint32_t defaultValue) {
   return m_mdb->RowCellColumnToUInt32(GetMDBRow(), token, pvalue, defaultValue);
 }
@@ -592,7 +595,7 @@ nsresult nsMsgHdr::SetUInt64Column(uint64_t value, mdb_token token) {
   return m_mdb->UInt64ToRowCellColumn(m_mdbRow, token, value);
 }
 
-nsresult nsMsgHdr::GetUInt64Column(mdb_token token, uint64_t *pvalue,
+nsresult nsMsgHdr::GetUInt64Column(mdb_token token, uint64_t* pvalue,
                                    uint64_t defaultValue) {
   return m_mdb->RowCellColumnToUInt64(GetMDBRow(), token, pvalue, defaultValue);
 }
@@ -632,15 +635,21 @@ nsresult nsMsgHdr::GetUInt64Column(mdb_token token, uint64_t *pvalue,
  *     function, as if you are around to make a second call, it means we found
  *     a properly formatted message-id and so we should only look for more
  *     properly formatted message-ids.
+ *     NOTE: this option will also strip off a single leading '<' if there is
+ *     one. Some examples:
+ *        "   foo" => "foo"
+ *        "  <bar" => "bar"
+ *        "<<<foo" => "<<foo"
+ *        "<foo@bar>" => "foo@bar"  (completed message-id)
  * @returns The next starting position of this routine, which may be pointing at
  *     a nul '\0' character to indicate termination.
  */
-const char *nsMsgHdr::GetNextReference(const char *startNextRef,
-                                       nsCString &reference,
+const char* nsMsgHdr::GetNextReference(const char* startNextRef,
+                                       nsCString& reference,
                                        bool acceptNonDelimitedReferences) {
-  const char *ptr = startNextRef;
-  const char *whitespaceEndedAt = nullptr;
-  const char *firstMessageIdChar = nullptr;
+  const char* ptr = startNextRef;
+  const char* whitespaceEndedAt = nullptr;
+  const char* firstMessageIdChar = nullptr;
 
   // make the reference result string empty by default; we will set it to
   //  something valid if the time comes.
@@ -664,12 +673,13 @@ const char *nsMsgHdr::GetNextReference(const char *startNextRef,
         // do nothing, make default case mean you didn't get whitespace
         break;
       case '<':
-        firstMessageIdChar = ++ptr;  // skip over the '<'
-        foundLessThan = true;        // (flag to stop)
-        // intentional fallthrough so whitespaceEndedAt will definitely have
+        firstMessageIdChar = ptr + 1;  // skip over the '<'
+        foundLessThan = true;          // (flag to stop)
+        // Ensure whitespaceEndedAt skips the leading '<' and is set to
         //  a non-NULL value, just in case the message-id is not valid (no '>')
         //  and the old-school support is desired.
-        MOZ_FALLTHROUGH;
+        if (!whitespaceEndedAt) whitespaceEndedAt = ptr + 1;
+        break;
       default:
         if (!whitespaceEndedAt) whitespaceEndedAt = ptr;
         break;
@@ -692,7 +702,7 @@ const char *nsMsgHdr::GetNextReference(const char *startNextRef,
   return ptr;
 }
 
-bool nsMsgHdr::IsParentOf(nsIMsgDBHdr *possibleChild) {
+bool nsMsgHdr::IsParentOf(nsIMsgDBHdr* possibleChild) {
   uint16_t referenceToCheck = 0;
   possibleChild->GetNumReferences(&referenceToCheck);
   nsAutoCString reference;
@@ -715,10 +725,10 @@ bool nsMsgHdr::IsParentOf(nsIMsgDBHdr *possibleChild) {
   return false;
 }
 
-bool nsMsgHdr::IsAncestorOf(nsIMsgDBHdr *possibleChild) {
-  const char *references;
-  nsMsgHdr *curHdr =
-      static_cast<nsMsgHdr *>(possibleChild);  // closed system, cast ok
+bool nsMsgHdr::IsAncestorOf(nsIMsgDBHdr* possibleChild) {
+  const char* references;
+  nsMsgHdr* curHdr =
+      static_cast<nsMsgHdr*>(possibleChild);  // closed system, cast ok
   m_mdb->RowCellColumnToConstCharPtr(
       curHdr->GetMDBRow(), m_mdb->m_referencesColumnToken, &references);
   if (!references) return false;
@@ -729,21 +739,21 @@ bool nsMsgHdr::IsAncestorOf(nsIMsgDBHdr *possibleChild) {
   return (strstr(references, messageId.get()) != nullptr);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetIsRead(bool *isRead) {
+NS_IMETHODIMP nsMsgHdr::GetIsRead(bool* isRead) {
   NS_ENSURE_ARG_POINTER(isRead);
   if (!(m_initedValues & FLAGS_INITED)) InitFlags();
   *isRead = !!(m_flags & nsMsgMessageFlags::Read);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetIsFlagged(bool *isFlagged) {
+NS_IMETHODIMP nsMsgHdr::GetIsFlagged(bool* isFlagged) {
   NS_ENSURE_ARG_POINTER(isFlagged);
   if (!(m_initedValues & FLAGS_INITED)) InitFlags();
   *isFlagged = !!(m_flags & nsMsgMessageFlags::Marked);
   return NS_OK;
 }
 
-void nsMsgHdr::ReparentInThread(nsIMsgThread *thread) {
+void nsMsgHdr::ReparentInThread(nsIMsgThread* thread) {
   NS_WARNING("Borked message header, attempting to fix!");
   uint32_t numChildren;
   thread->GetNumChildren(&numChildren);
@@ -757,7 +767,7 @@ void nsMsgHdr::ReparentInThread(nsIMsgThread *thread) {
     for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++) {
       thread->GetChildHdrAt(childIndex, getter_AddRefs(curHdr));
       // closed system, cast ok
-      nsMsgHdr *curMsgHdr = static_cast<nsMsgHdr *>(curHdr.get());
+      nsMsgHdr* curMsgHdr = static_cast<nsMsgHdr*>(curHdr.get());
       if (curHdr && curMsgHdr->IsParentOf(this)) {
         nsMsgKey curHdrKey;
         curHdr->GetMessageKey(&curHdrKey);
@@ -830,7 +840,7 @@ bool nsMsgHdr::IsAncestorKilled(uint32_t ancestorsToCheck) {
           return false;
         }
         // closed system, cast ok
-        nsMsgHdr *parent = static_cast<nsMsgHdr *>(parentHdr.get());
+        nsMsgHdr* parent = static_cast<nsMsgHdr*>(parentHdr.get());
         return parent->IsAncestorKilled(ancestorsToCheck - 1);
       }
     }
@@ -838,7 +848,7 @@ bool nsMsgHdr::IsAncestorKilled(uint32_t ancestorsToCheck) {
   return isKilled;
 }
 
-NS_IMETHODIMP nsMsgHdr::GetIsKilled(bool *isKilled) {
+NS_IMETHODIMP nsMsgHdr::GetIsKilled(bool* isKilled) {
   NS_ENSURE_ARG_POINTER(isKilled);
   *isKilled = false;
   nsCOMPtr<nsIMsgThread> thread;
@@ -857,7 +867,6 @@ NS_IMETHODIMP nsMsgHdr::GetIsKilled(bool *isKilled) {
 ////////////////////////////////////////////////////////////////////////////////
 
 #include "nsIStringEnumerator.h"
-#include "nsAutoPtr.h"
 #define NULL_MORK_COLUMN 0
 class nsMsgPropertyEnumerator : public nsStringEnumeratorBase {
  public:
@@ -866,7 +875,7 @@ class nsMsgPropertyEnumerator : public nsStringEnumeratorBase {
 
   using nsStringEnumeratorBase::GetNext;
 
-  explicit nsMsgPropertyEnumerator(nsMsgHdr *aHdr);
+  explicit nsMsgPropertyEnumerator(nsMsgHdr* aHdr);
   void PrefetchNext();
 
  protected:
@@ -882,7 +891,7 @@ class nsMsgPropertyEnumerator : public nsStringEnumeratorBase {
   mdb_column mNextColumn;
 };
 
-nsMsgPropertyEnumerator::nsMsgPropertyEnumerator(nsMsgHdr *aHdr)
+nsMsgPropertyEnumerator::nsMsgPropertyEnumerator(nsMsgHdr* aHdr)
     : mNextPrefetched(false), mNextColumn(NULL_MORK_COLUMN) {
   RefPtr<nsMsgDatabase> mdb;
   nsCOMPtr<nsIMdbRow> mdbRow;
@@ -903,7 +912,7 @@ nsMsgPropertyEnumerator::~nsMsgPropertyEnumerator() {
 NS_IMPL_ISUPPORTS(nsMsgPropertyEnumerator, nsIUTF8StringEnumerator,
                   nsIStringEnumerator)
 
-NS_IMETHODIMP nsMsgPropertyEnumerator::GetNext(nsACString &aItem) {
+NS_IMETHODIMP nsMsgPropertyEnumerator::GetNext(nsACString& aItem) {
   PrefetchNext();
   if (mNextColumn == NULL_MORK_COLUMN)
     return NS_ERROR_FAILURE;  // call HasMore first
@@ -915,11 +924,11 @@ NS_IMETHODIMP nsMsgPropertyEnumerator::GetNext(nsACString &aItem) {
   nsresult rv = m_mdbStore->TokenToString(m_mdbEnv, mNextColumn, &colYarn);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  aItem.Assign(static_cast<char *>(colYarn.mYarn_Buf), colYarn.mYarn_Fill);
+  aItem.Assign(static_cast<char*>(colYarn.mYarn_Buf), colYarn.mYarn_Fill);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgPropertyEnumerator::HasMore(bool *aResult) {
+NS_IMETHODIMP nsMsgPropertyEnumerator::HasMore(bool* aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
 
   PrefetchNext();
@@ -945,7 +954,7 @@ void nsMsgPropertyEnumerator::PrefetchNext(void) {
 ////////////////////////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP nsMsgHdr::GetPropertyEnumerator(
-    nsIUTF8StringEnumerator **_result) {
+    nsIUTF8StringEnumerator** _result) {
   NS_ADDREF(*_result = new nsMsgPropertyEnumerator(this));
   return NS_OK;
 }
