@@ -4,15 +4,17 @@
 
 // Tests corpus management functions using nsIMsgCorpus
 
-var msgCorpus =
-  Cc["@mozilla.org/messenger/filter-plugin;1?name=bayesianfilter"]
-    .getService(Ci.nsIMsgCorpus);
+var { MailServices } = ChromeUtils.import(
+  "resource:///modules/MailServices.jsm"
+);
+
+var msgCorpus = MailServices.junk.QueryInterface(Ci.nsIMsgCorpus);
 
 // tokens found in the test corpus file. trait 1001 was trained with
 // 2 messages, and trait 1003 with 1.
 
 var tokenData = [
-// [traitid, count, token]
+  // [traitid, count, token]
   [1001, 0, "iDoNotExist"],
   [1001, 1, "linecount"],
   [1001, 2, "envelope-to:kenttest@caspia.com"],
@@ -21,12 +23,11 @@ var tokenData = [
   [1003, 1, "linecount"],
   [1003, 1, "subject:test"],
   [1003, 1, "envelope-to:kenttest@caspia.com"],
-]
+];
 
 // list of tests
 
-var gTests =
-[
+var gTests = [
   // train two different combinations of messages
   function checkLoadOnce() {
     let fileName = "msgCorpus.dat";
@@ -70,7 +71,7 @@ var gTests =
   function loadWithRemap() {
     let fileName = "msgCorpus.dat";
     let file = do_get_file("resources/" + fileName);
-    msgCorpus.updateData(file, true, 2, [1001, 1003], [1, 3]);
+    msgCorpus.updateData(file, true, [1001, 1003], [1, 3]);
 
     for (let i = 0; i < tokenData.length; i++) {
       let id = tokenData[i][0] - 1000;
@@ -124,17 +125,14 @@ var gTests =
       Assert.equal(id == 1001 ? 0 : count, msgCorpus.getTokenCount(word, id));
     }
   },
-
-]
+];
 
 // main test
-function run_test()
-{
+function run_test() {
   do_test_pending();
-  while(1)
-  {
-    if (!gTests.length)       // Do we have more commands?
-    {
+  while (true) {
+    if (!gTests.length) {
+      // Do we have more commands?
       // no, all done
       do_test_finished();
       return;
