@@ -30,8 +30,6 @@
 #include "nsNetUtil.h"
 #include "nsIMsgDatabase.h"
 #include "mozilla/Services.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "mozilla/mailnews/MimeHeaderParser.h"
 #include "mozilla/Unused.h"
 
@@ -82,17 +80,16 @@ char DispositionTypes[7][16] = {
 
 NS_IMPL_ISUPPORTS(nsMsgMdnGenerator, nsIMsgMdnGenerator, nsIUrlListener)
 
-nsMsgMdnGenerator::nsMsgMdnGenerator() {
-  m_disposeType = eDisplayed;
-  m_outputStream = nullptr;
-  m_reallySendMdn = false;
-  m_autoSend = false;
-  m_autoAction = false;
-  m_mdnEnabled = false;
-  m_notInToCcOp = eNeverSendOp;
-  m_outsideDomainOp = eNeverSendOp;
-  m_otherOp = eNeverSendOp;
-}
+nsMsgMdnGenerator::nsMsgMdnGenerator()
+    : m_disposeType(eDisplayed),
+      m_key(nsMsgKey_None),
+      m_notInToCcOp(eNeverSendOp),
+      m_outsideDomainOp(eNeverSendOp),
+      m_otherOp(eNeverSendOp),
+      m_reallySendMdn(false),
+      m_autoSend(false),
+      m_autoAction(false),
+      m_mdnEnabled(false) {}
 
 nsMsgMdnGenerator::~nsMsgMdnGenerator() {}
 
@@ -420,8 +417,8 @@ nsresult nsMsgMdnGenerator::CreateFirstPart() {
   // convert fullName to UTF8 before passing it to MakeMimeAddress
   MakeMimeAddress(NS_ConvertUTF16toUTF8(fullName), m_email, fullAddress);
 
-  convbuf = nsMsgI18NEncodeMimePartIIStr(fullAddress.get(), true,
-                                         m_charset.get(), 0, conformToStandard);
+  convbuf = nsMsgI18NEncodeMimePartIIStr(fullAddress.get(), true, "UTF-8", 0,
+                                         conformToStandard);
 
   parm = PR_smprintf("From: %s" CRLF, convbuf ? convbuf : m_email.get());
 
@@ -476,8 +473,8 @@ nsresult nsMsgMdnGenerator::CreateFirstPart() {
   nsCString subject;
   m_headers->ExtractHeader(HEADER_SUBJECT, false, subject);
   convbuf = nsMsgI18NEncodeMimePartIIStr(
-      subject.Length() ? subject.get() : "[no subject]", false, m_charset.get(),
-      0, conformToStandard);
+      subject.Length() ? subject.get() : "[no subject]", false, "UTF-8", 0,
+      conformToStandard);
   tmpBuffer = PR_smprintf(
       "Subject: %s%s" CRLF, encodedReceiptString,
       (convbuf ? convbuf
@@ -487,8 +484,8 @@ nsresult nsMsgMdnGenerator::CreateFirstPart() {
   PR_Free(convbuf);
   PR_Free(encodedReceiptString);
 
-  convbuf = nsMsgI18NEncodeMimePartIIStr(m_dntRrt.get(), true, m_charset.get(),
-                                         0, conformToStandard);
+  convbuf = nsMsgI18NEncodeMimePartIIStr(m_dntRrt.get(), true, "UTF-8", 0,
+                                         conformToStandard);
   tmpBuffer = PR_smprintf("To: %s" CRLF, convbuf ? convbuf : m_dntRrt.get());
   PUSH_N_FREE_STRING(tmpBuffer);
 
@@ -621,8 +618,8 @@ nsresult nsMsgMdnGenerator::CreateSecondPart() {
   compUtils = do_GetService(NS_MSGCOMPUTILS_CONTRACTID, &rv);
   if (compUtils) compUtils->GetMsgMimeConformToStandard(&conformToStandard);
 
-  convbuf = nsMsgI18NEncodeMimePartIIStr(m_email.get(), true, m_charset.get(),
-                                         0, conformToStandard);
+  convbuf = nsMsgI18NEncodeMimePartIIStr(m_email.get(), true, "UTF-8", 0,
+                                         conformToStandard);
   tmpBuffer = PR_smprintf("Final-Recipient: rfc822;%s" CRLF,
                           convbuf ? convbuf : m_email.get());
   PUSH_N_FREE_STRING(tmpBuffer);
@@ -733,8 +730,9 @@ nsresult nsMsgMdnGenerator::OutputAllHeaders() {
       } else {
         NS_ASSERTION(*end == 0, "content of end should be null");
         rv = WriteString(start);
-        if (NS_FAILED(rv)) return rv;
+        NS_ENSURE_SUCCESS(rv, rv);
         rv = WriteString(CRLF);
+        NS_ENSURE_SUCCESS(rv, rv);
         while (end < buf_end && (*end == '\n' || *end == '\r' || *end == 0))
           end++;
         start = end;
@@ -752,12 +750,14 @@ nsresult nsMsgMdnGenerator::SendMdnMsg() {
       do_GetService(NS_SMTPSERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsCOMPtr<nsIURI> aUri;
   nsCOMPtr<nsIRequest> aRequest;
   nsCString identEmail;
   m_identity->GetEmail(identEmail);
-  smtpService->SendMailMessage(
-      m_file, m_dntRrt.get(), m_identity, identEmail.get(), EmptyString(), this,
-      nullptr, nullptr, false, nullptr, getter_AddRefs(aRequest));
+  smtpService->SendMailMessage(m_file, m_dntRrt.get(), m_identity,
+                               identEmail.get(), EmptyString(), this, nullptr,
+                               nullptr, false, ""_ns, getter_AddRefs(aUri),
+                               getter_AddRefs(aRequest));
 
   return NS_OK;
 }
@@ -857,7 +857,6 @@ nsresult nsMsgMdnGenerator::InitAndProcess(bool* needToAskUser) {
     }
   }
 
-  rv = m_folder->GetCharset(m_charset);
   if (m_mdnEnabled) {
     m_headers->ExtractHeader(HEADER_DISPOSITION_NOTIFICATION_TO, false,
                              m_dntRrt);
