@@ -19,9 +19,7 @@
 #include "nsIMsgCopyService.h"
 #include "nsImapProtocol.h"
 #include "nsMsgUtils.h"
-#include "nsIMutableArray.h"
 #include "nsIAutoSyncManager.h"
-#include "nsArrayUtils.h"
 #include "mozilla/Unused.h"
 
 NS_IMPL_ISUPPORTS(nsImapOfflineSync, nsIUrlListener, nsIMsgCopyServiceListener,
@@ -206,8 +204,6 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation* op) {
 
   imapMessageFlagsType matchingFlags;
   currentOp->GetNewFlags(&matchingFlags);
-  imapMessageFlagsType flagOperation;
-  imapMessageFlagsType newFlags;
   bool flagsMatch = true;
   do {  // loop for all messages with the same flags
     if (flagsMatch) {
@@ -218,6 +214,8 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation* op) {
       m_currentOpsToClear.AppendObject(currentOp);
     }
     currentOp = nullptr;
+    imapMessageFlagsType newFlags = kNoImapMsgFlag;
+    imapMessageFlagsType flagOperation = kNoImapMsgFlag;
     if (++currentKeyIndex < m_CurrentKeys.Length())
       m_currentDB->GetOfflineOpForKey(m_CurrentKeys[currentKeyIndex], false,
                                       getter_AddRefs(currentOp));
@@ -359,7 +357,7 @@ void nsImapOfflineSync::ProcessAppendMsgOperation(
   bool setPlayingBack = false;
   do {
     nsCString moveDestination;
-    currentOp->GetDestinationFolderURI(getter_Copies(moveDestination));
+    currentOp->GetDestinationFolderURI(moveDestination);
 
     nsCOMPtr<nsIMsgFolder> destFolder;
     rv = GetOrCreateFolder(moveDestination, getter_AddRefs(destFolder));
@@ -459,7 +457,7 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation* op) {
   nsTArray<nsMsgKey> matchingFlagKeys;
   uint32_t currentKeyIndex = m_KeyIndex;
   nsCString moveDestination;
-  op->GetDestinationFolderURI(getter_Copies(moveDestination));
+  op->GetDestinationFolderURI(moveDestination);
   bool moveMatches = true;
   nsCOMPtr<nsIMsgOfflineImapOperation> currentOp = op;
   do {  // loop for all messages with the same destination
@@ -481,7 +479,7 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation* op) {
         nsOfflineImapOperationType opType;
         currentOp->GetOperation(&opType);
         if (opType & nsIMsgOfflineImapOperation::kMsgMoved) {
-          currentOp->GetDestinationFolderURI(getter_Copies(nextDestination));
+          currentOp->GetDestinationFolderURI(nextDestination);
           moveMatches = moveDestination.Equals(nextDestination);
         }
       }
@@ -505,40 +503,38 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation* op) {
                                       m_window);
   } else {
     nsresult rv;
-    nsCOMPtr<nsIMutableArray> messages(
-        do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-    if (NS_SUCCEEDED(rv)) {
-      for (uint32_t keyIndex = 0; keyIndex < matchingFlagKeys.Length();
-           keyIndex++) {
-        nsCOMPtr<nsIMsgDBHdr> mailHdr = nullptr;
-        rv = m_currentFolder->GetMessageHeader(
-            matchingFlagKeys.ElementAt(keyIndex), getter_AddRefs(mailHdr));
-        if (NS_SUCCEEDED(rv) && mailHdr) {
-          uint32_t msgSize;
-          // in case of a move, the header has already been deleted,
-          // so we've really got a fake header. We need to get its flags and
-          // size from the offline op to have any chance of doing the move.
-          mailHdr->GetMessageSize(&msgSize);
-          if (!msgSize) {
-            imapMessageFlagsType newImapFlags;
-            uint32_t msgFlags = 0;
-            op->GetMsgSize(&msgSize);
-            op->GetNewFlags(&newImapFlags);
-            // first three bits are the same
-            msgFlags |= (newImapFlags & 0x07);
-            if (newImapFlags & kImapMsgForwardedFlag)
-              msgFlags |= nsMsgMessageFlags::Forwarded;
-            mailHdr->SetFlags(msgFlags);
-            mailHdr->SetMessageSize(msgSize);
-          }
-          messages->AppendElement(mailHdr);
+    nsTArray<RefPtr<nsIMsgDBHdr>> messages;
+    for (uint32_t keyIndex = 0; keyIndex < matchingFlagKeys.Length();
+         keyIndex++) {
+      nsCOMPtr<nsIMsgDBHdr> mailHdr = nullptr;
+      rv = m_currentFolder->GetMessageHeader(
+          matchingFlagKeys.ElementAt(keyIndex), getter_AddRefs(mailHdr));
+      if (NS_SUCCEEDED(rv) && mailHdr) {
+        uint32_t msgSize;
+        // in case of a move, the header has already been deleted,
+        // so we've really got a fake header. We need to get its flags and
+        // size from the offline op to have any chance of doing the move.
+        mailHdr->GetMessageSize(&msgSize);
+        if (!msgSize) {
+          imapMessageFlagsType newImapFlags;
+          uint32_t msgFlags = 0;
+          op->GetMsgSize(&msgSize);
+          op->GetNewFlags(&newImapFlags);
+          // first three bits are the same
+          msgFlags |= (newImapFlags & 0x07);
+          if (newImapFlags & kImapMsgForwardedFlag)
+            msgFlags |= nsMsgMessageFlags::Forwarded;
+          mailHdr->SetFlags(msgFlags);
+          mailHdr->SetMessageSize(msgSize);
         }
+        messages.AppendElement(mailHdr);
       }
-      nsCOMPtr<nsIMsgCopyService> copyService =
-          do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
-      if (copyService)
-        copyService->CopyMessages(m_currentFolder, messages, destFolder, true,
-                                  this, m_window, false);
+    }
+    nsCOMPtr<nsIMsgCopyService> copyService =
+        do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+    if (copyService) {
+      copyService->CopyMessages(m_currentFolder, messages, destFolder, true,
+                                this, m_window, false);
     }
   }
 }
@@ -611,22 +607,19 @@ void nsImapOfflineSync::ProcessCopyOperation(
     rv = imapFolder->ReplayOfflineMoveCopy(matchingFlagKeys, false, destFolder,
                                            this, m_window);
   } else {
-    nsCOMPtr<nsIMutableArray> messages(
-        do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-    if (messages && NS_SUCCEEDED(rv)) {
-      for (uint32_t keyIndex = 0; keyIndex < matchingFlagKeys.Length();
-           keyIndex++) {
-        nsCOMPtr<nsIMsgDBHdr> mailHdr = nullptr;
-        rv = m_currentFolder->GetMessageHeader(
-            matchingFlagKeys.ElementAt(keyIndex), getter_AddRefs(mailHdr));
-        if (NS_SUCCEEDED(rv) && mailHdr) messages->AppendElement(mailHdr);
-      }
-      nsCOMPtr<nsIMsgCopyService> copyService =
-          do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
-      if (copyService)
-        copyService->CopyMessages(m_currentFolder, messages, destFolder, false,
-                                  this, m_window, false);
+    nsTArray<RefPtr<nsIMsgDBHdr>> messages;
+    for (uint32_t keyIndex = 0; keyIndex < matchingFlagKeys.Length();
+         keyIndex++) {
+      nsCOMPtr<nsIMsgDBHdr> mailHdr = nullptr;
+      rv = m_currentFolder->GetMessageHeader(
+          matchingFlagKeys.ElementAt(keyIndex), getter_AddRefs(mailHdr));
+      if (NS_SUCCEEDED(rv) && mailHdr) messages.AppendElement(mailHdr);
     }
+    nsCOMPtr<nsIMsgCopyService> copyService =
+        do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+    if (copyService)
+      copyService->CopyMessages(m_currentFolder, messages, destFolder, false,
+                                this, m_window, false);
   }
 }
 
@@ -1102,6 +1095,7 @@ void nsImapOfflineSync::ClearDB() {
 
 NS_IMETHODIMP
 nsImapOfflineSync::OnHdrPropertyChanged(nsIMsgDBHdr* aHdrToChange,
+                                        const nsACString& property,
                                         bool aPreChange, uint32_t* aStatus,
                                         nsIDBChangeListener* aInstigator) {
   return NS_OK;
