@@ -82,9 +82,8 @@ function run_test() {
 var actionTestOffline = {
   id: "mailnews@mozilla.org#testOffline",
   name: "test if offline",
-  apply(aMsgHdrs, aActionValue, aListener, aType, aMsgWindow) {
-    for (let i = 0; i < aMsgHdrs.length; i++) {
-      var msgHdr = aMsgHdrs.queryElementAt(i, Ci.nsIMsgDBHdr);
+  applyAction(aMsgHdrs, aActionValue, aListener, aType, aMsgWindow) {
+    for (let msgHdr of aMsgHdrs) {
       let isOffline = msgHdr.flags & Ci.nsMsgMessageFlags.Offline;
       Assert.equal(!!isOffline, aActionValue == "true");
     }
