@@ -6,35 +6,76 @@
  * Tests imap save and detach attachments.
  */
 
-/* import-globals-from ../../../test/resources/logHelper.js */
-/* import-globals-from ../../../test/resources/asyncTestUtils.js */
-/* import-globals-from ../../../test/resources/messageGenerator.js */
-load("../../../resources/logHelper.js");
-load("../../../resources/asyncTestUtils.js");
-load("../../../resources/messageGenerator.js");
-
-// javascript mime emitter functions
-var mimeMsg = {};
-ChromeUtils.import("resource:///modules/gloda/mimemsg.js", mimeMsg);
-var { MailServices } = ChromeUtils.import(
-  "resource:///modules/MailServices.jsm"
+var { MessageGenerator } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageGenerator.jsm"
+);
+var { PromiseTestUtils } = ChromeUtils.import(
+  "resource://testing-common/mailnews/PromiseTestUtils.jsm"
 );
 
-// IMAP pump
+// javascript mime emitter functions
+var { MsgHdrToMimeMessage } = ChromeUtils.import(
+  "resource:///modules/gloda/MimeMessage.jsm"
+);
 
 var kAttachFileName = "bob.txt";
+function SaveAttachmentCallback() {
+  this.attachments = null;
+  this._promise = new Promise((resolve, reject) => {
+    this._resolve = resolve;
+    this._reject = reject;
+  });
+}
 
-setupIMAPPump();
+SaveAttachmentCallback.prototype = {
+  callback: function saveAttachmentCallback_callback(aMsgHdr, aMimeMessage) {
+    this.attachments = aMimeMessage.allAttachments;
+    this._resolve();
+  },
+  get promise() {
+    return this._promise;
+  },
+};
+var gCallbackObject = new SaveAttachmentCallback();
 
 // Dummy message window so we can say the inbox is open in a window.
 var dummyMsgWindow = Cc["@mozilla.org/messenger/msgwindow;1"].createInstance(
   Ci.nsIMsgWindow
 );
 
-var tests = [loadImapMessage, startMime, startDetach, testDetach, endTest];
+function MsgsDeletedListener() {
+  this._promise = new Promise(resolve => (this._resolve = resolve));
+}
+MsgsDeletedListener.prototype = {
+  msgsDeleted(aMsgArray) {
+    this._resolve();
+  },
+  get promise() {
+    return this._promise;
+  },
+};
+var trackDeletionMessageListener = new MsgsDeletedListener();
+
+add_task(function setupTest() {
+  setupIMAPPump();
+
+  // Add folder listeners that will capture async events
+  MailServices.mfn.addListener(
+    trackDeletionMessageListener,
+    Ci.nsIMsgFolderNotificationService.msgsDeleted
+  );
+
+  // We need to register the dummyMsgWindow so that when we've finished running
+  // the append url, in nsImapMailFolder::OnStopRunningUrl, we'll think the
+  // Inbox is open in a folder and update it, which the detach code relies
+  // on to finish the detach.
+
+  dummyMsgWindow.openFolder = IMAPPump.inbox;
+  MailServices.mailSession.AddMsgWindow(dummyMsgWindow);
+});
 
 // load and update a message in the imap fake server
-function* loadImapMessage() {
+add_task(async function loadImapMessage() {
   let gMessageGenerator = new MessageGenerator();
   // create a synthetic message with attachment
   let smsg = gMessageGenerator.makeMessage({
@@ -47,30 +88,29 @@ function* loadImapMessage() {
   let imapInbox = IMAPPump.daemon.getMailbox("INBOX");
   let message = new imapMessage(msgURI.spec, imapInbox.uidnext++, []);
   IMAPPump.mailbox.addMessage(message);
-  IMAPPump.inbox.updateFolderWithListener(null, asyncUrlListener);
-  yield false;
+  let listener = new PromiseTestUtils.PromiseUrlListener();
+  IMAPPump.inbox.updateFolderWithListener(null, listener);
+  await listener.promise;
   Assert.equal(1, IMAPPump.inbox.getTotalMessages(false));
   let msgHdr = mailTestUtils.firstMsgHdr(IMAPPump.inbox);
   Assert.ok(msgHdr instanceof Ci.nsIMsgDBHdr);
-
-  yield true;
-}
+});
 
 // process the message through mime
-function* startMime() {
+add_task(async function startMime() {
   let msgHdr = mailTestUtils.firstMsgHdr(IMAPPump.inbox);
 
-  mimeMsg.MsgHdrToMimeMessage(
+  MsgHdrToMimeMessage(
     msgHdr,
     gCallbackObject,
     gCallbackObject.callback,
-    true /* allowDownload */
+    true // allowDownload
   );
-  yield false;
-}
+  await gCallbackObject.promise;
+});
 
 // detach any found attachments
-function* startDetach() {
+add_task(async function startDetach() {
   let msgHdr = mailTestUtils.firstMsgHdr(IMAPPump.inbox);
   let msgURI = msgHdr.folder.generateMessageURI(msgHdr.messageKey);
 
@@ -81,7 +121,6 @@ function* startDetach() {
 
   messenger.detachAttachmentsWOPrompts(
     do_get_profile(),
-    1,
     [attachment.contentType],
     [attachment.url],
     [attachment.name],
@@ -89,14 +128,11 @@ function* startDetach() {
     null
   );
   // deletion of original message should kick async_driver.
-  yield false;
-}
+  await trackDeletionMessageListener.promise;
+});
 
 // test that the detachment was successful
-function* testDetach() {
-  // This test seems to fail on Linux without the following delay.
-  mailTestUtils.do_timeout_function(200, async_driver);
-  yield false;
+add_task(function testDetach() {
   // Check that the file attached to the message now exists in the profile
   // directory.
   let checkFile = do_get_profile().clone();
@@ -111,40 +147,12 @@ function* testDetach() {
   Assert.ok(msgHdr !== null);
   let messageContent = getContentFromMessage(msgHdr);
   Assert.ok(messageContent.includes("AttachmentDetached"));
-}
+});
 
 // Cleanup
-function endTest() {
+add_task(function endTest() {
   teardownIMAPPump();
-}
-
-function SaveAttachmentCallback() {
-  this.attachments = null;
-}
-
-SaveAttachmentCallback.prototype = {
-  callback: function saveAttachmentCallback_callback(aMsgHdr, aMimeMessage) {
-    this.attachments = aMimeMessage.allAttachments;
-    async_driver();
-  },
-};
-var gCallbackObject = new SaveAttachmentCallback();
-
-function run_test() {
-  // Add folder listeners that will capture async events
-  const nsIMFNService = Ci.nsIMsgFolderNotificationService;
-  MailServices.mfn.addListener(mfnListener, nsIMFNService.msgsDeleted);
-
-  // We need to register the dummyMsgWindow so that when we've finished running
-  // the append url, in nsImapMailFolder::OnStopRunningUrl, we'll think the
-  // Inbox is open in a folder and update it, which the detach code relies
-  // on to finish the detach.
-
-  dummyMsgWindow.openFolder = IMAPPump.inbox;
-  MailServices.mailSession.AddMsgWindow(dummyMsgWindow);
-
-  async_run_tests(tests);
-}
+});
 
 /*
  * Get the full message content.
@@ -163,7 +171,7 @@ function getContentFromMessage(aMsgHdr) {
   let streamListener = Cc[
     "@mozilla.org/network/sync-stream-listener;1"
   ].createInstance(Ci.nsISyncStreamListener);
-  // pass true for aLocalOnly since message should be in offline store.
+  // Pass true for aLocalOnly since message should be in offline store.
   messenger
     .messageServiceFromURI(msgUri)
     .streamMessage(msgUri, streamListener, null, null, false, "", true);
@@ -175,10 +183,3 @@ function getContentFromMessage(aMsgHdr) {
   sis.close();
   return content;
 }
-
-var mfnListener = {
-  msgsDeleted(aMsgArray) {
-    dump("msg deleted\n");
-    async_driver();
-  },
-};
