@@ -21,7 +21,6 @@
 #include "nsIOutputStream.h"
 
 #include "nsMsgCompCID.h"
-#include "nsIMutableArray.h"
 #include "MapiDbgLog.h"
 #include "MapiApi.h"
 
@@ -36,6 +35,9 @@
 
 #include "mozilla/Encoding.h"
 
+#include <stdlib.h>
+#include <tuple>
+
 // needed for the call the OpenStreamOnFile
 extern LPMAPIALLOCATEBUFFER gpMapiAllocateBuffer;
 extern LPMAPIFREEBUFFER gpMapiFreeBuffer;
@@ -184,7 +186,6 @@ DEFINE_OLEGUID(PSETID_Common, MAKELONG(0x2000 + (8), 0x0006), 0, 0);
 
 void CMapiMessage::GetDownloadState() {
   // See http://support.microsoft.com/kb/912239
-  HRESULT hRes = S_OK;
   ULONG ulVal = 0;
   LPSPropValue lpPropVal = NULL;
   LPSPropTagArray lpNamedPropTag = NULL;
@@ -196,14 +197,14 @@ void CMapiMessage::GetDownloadState() {
   NamedID.Kind.lID = dispidHeaderItem;
   lpNamedID = &NamedID;
 
-  hRes = m_lpMsg->GetIDsFromNames(1, &lpNamedID, NULL, &lpNamedPropTag);
+  m_lpMsg->GetIDsFromNames(1, &lpNamedID, NULL, &lpNamedPropTag);
 
   if (lpNamedPropTag && 1 == lpNamedPropTag->cValues) {
     lpNamedPropTag->aulPropTag[0] =
         CHANGE_PROP_TYPE(lpNamedPropTag->aulPropTag[0], PT_LONG);
 
     // Get the value of the property.
-    hRes = m_lpMsg->GetProps(lpNamedPropTag, 0, &ulVal, &lpPropVal);
+    m_lpMsg->GetProps(lpNamedPropTag, 0, &ulVal, &lpPropVal);
     if (lpPropVal && 1 == ulVal && PT_LONG == PROP_TYPE(lpPropVal->ulPropTag) &&
         lpPropVal->Value.ul)
       m_dldStateHeadersOnly = true;
@@ -567,13 +568,12 @@ bool CMapiMessage::CheckBodyInCharsetRange(const char* charset) {
   auto encoder = encoding->NewEncoder();
 
   uint8_t buffer[512];
-  auto src = mozilla::MakeSpan(m_body);
-  auto dst = mozilla::MakeSpan(buffer);
+  auto src = mozilla::Span(m_body);
+  auto dst = mozilla::Span(buffer);
   while (true) {
     uint32_t result;
     size_t read;
-    size_t written;
-    mozilla::Tie(result, read, written) =
+    std::tie(result, read, std::ignore) =
         encoder->EncodeFromUTF16WithoutReplacement(src, dst, false);
     if (result == mozilla::kInputEmpty) {
       // All converted successfully.
@@ -584,7 +584,6 @@ bool CMapiMessage::CheckBodyInCharsetRange(const char* charset) {
       return false;
     }
     src = src.From(read);
-    // dst = dst.From(written); // Just overwrite output since we don't need it.
   }
 
   return true;
@@ -713,13 +712,6 @@ bool CMapiMessage::FetchBody(void) {
       if (bFoundCharset) m_mimeCharset.Assign(charset);
     }
   }
-  if (!bFoundCharset) {  // Use system default
-    const nsCString charset(nsMsgI18NFileSystemCharset());
-    if (!charset.IsEmpty()) {
-      bFoundCharset = CheckBodyInCharsetRange(charset.get());
-      if (bFoundCharset) m_mimeCharset = charset;
-    }
-  }
   if (!bFoundCharset)  // Everything else failed, let's use the lossless
                        // utf-8...
     m_mimeCharset.AssignLiteral("utf-8");
@@ -1129,14 +1121,14 @@ void CMapiMessage::ProcessAttachments() {
   pTable->Release();
 }
 
-nsresult CMapiMessage::GetAttachments(nsIArray** aArray) {
-  nsresult rv;
-  nsCOMPtr<nsIMutableArray> attachments(
-      do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
+nsresult CMapiMessage::GetAttachments(
+    nsTArray<RefPtr<nsIMsgAttachedFile>>& attachments) {
+  attachments.Clear();
+  attachments.SetCapacity(m_stdattachments.size());
 
   for (std::vector<attach_data*>::const_iterator it = m_stdattachments.begin();
        it != m_stdattachments.end(); it++) {
+    nsresult rv;
     nsCOMPtr<nsIMsgAttachedFile> a(
         do_CreateInstance(NS_MSGATTACHEDFILE_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1145,16 +1137,15 @@ nsresult CMapiMessage::GetAttachments(nsIArray** aArray) {
     a->SetEncoding(nsDependentCString((*it)->encoding));
     a->SetRealName(nsDependentCString((*it)->real_name));
     a->SetType(nsDependentCString((*it)->type));
-    attachments->AppendElement(a);
+    attachments.AppendElement(a);
   }
-  attachments.forget(aArray);
-  return rv;
+  return NS_OK;
 }
 
 bool CMapiMessage::GetEmbeddedAttachmentInfo(unsigned int i, nsIURI** uri,
                                              const char** cid,
                                              const char** name) const {
-  if ((i < 0) || (i >= m_embattachments.size())) return false;
+  if (i >= m_embattachments.size()) return false;
   attach_data* data = m_embattachments[i];
   if (!data) return false;
   *uri = data->orig_url;
