@@ -14,7 +14,6 @@
 #include "nsIImportFieldMap.h"
 #include "nsIImportMailboxDescriptor.h"
 #include "nsIImportABDescriptor.h"
-#include "nsIMutableArray.h"
 #include "nsOutlookStringBundle.h"
 #include "nsAbBaseCID.h"
 #include "nsIAbCard.h"
@@ -100,19 +99,14 @@ nsOutlookMail::~nsOutlookMail() {
   //  EmptyAttachments();
 }
 
-nsresult nsOutlookMail::GetMailFolders(nsIArray** pArray) {
+nsresult nsOutlookMail::GetMailFolders(
+    nsTArray<RefPtr<nsIImportMailboxDescriptor>>& boxes) {
   if (!m_haveMapi) {
     IMPORT_LOG0("GetMailFolders called before Mapi is initialized\n");
     return NS_ERROR_FAILURE;
   }
-
   nsresult rv;
-  nsCOMPtr<nsIMutableArray> array(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-  if (NS_FAILED(rv)) {
-    IMPORT_LOG0(
-        "FAILED to allocate the nsIMutableArray for the mail folder list\n");
-    return rv;
-  }
+  boxes.Clear();
 
   nsCOMPtr<nsIImportService> impSvc(
       do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
@@ -165,11 +159,9 @@ nsresult nsOutlookMail::GetMailFolders(nsIArray** pArray) {
       pID->SetDisplayName(name.get());
 
       pID->SetSize(1000);
-      nsCOMPtr<nsISupports> pInterface(do_QueryInterface(pID));
-      array->AppendElement(pInterface);
+      boxes.AppendElement(pID);
     }
   }
-  array.forget(pArray);
   return NS_OK;
 }
 
@@ -200,20 +192,14 @@ void nsOutlookMail::MakeAddressBookNameUnique(nsString& name, nsString& list) {
   list.AppendLiteral("],");
 }
 
-nsresult nsOutlookMail::GetAddressBooks(nsIArray** pArray) {
+nsresult nsOutlookMail::GetAddressBooks(
+    nsTArray<RefPtr<nsIImportABDescriptor>>& books) {
+  books.Clear();
   if (!m_haveMapi) {
     IMPORT_LOG0("GetAddressBooks called before Mapi is initialized\n");
     return NS_ERROR_FAILURE;
   }
-
   nsresult rv;
-  nsCOMPtr<nsIMutableArray> array(do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-  if (NS_FAILED(rv)) {
-    IMPORT_LOG0(
-        "FAILED to allocate the nsIMutableArray for the address book list\n");
-    return rv;
-  }
-
   nsCOMPtr<nsIImportService> impSvc(
       do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv));
   if (NS_FAILED(rv)) return rv;
@@ -262,12 +248,10 @@ nsresult nsOutlookMail::GetAddressBooks(nsIArray** pArray) {
         MakeAddressBookNameUnique(name, list);
         pID->SetPreferredName(name);
         pID->SetSize(100);
-        nsCOMPtr<nsISupports> pInterface(do_QueryInterface(pID));
-        array->AppendElement(pInterface);
+        books.AppendElement(pID);
       }
     }
   }
-  array.forget(pArray);
   return NS_OK;
 }
 
@@ -353,7 +337,7 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
   mCaller->OpenMessageStore(pFolder);
   if (!mCaller->m_lpMdb) {
     IMPORT_LOG1("*** Unable to obtain mapi message store for mailbox: %S\n",
-                mName);
+                (const wchar_t*)mName);
     mResult = NS_ERROR_FAILURE;
     return NS_OK;  // Sync runnable must return OK.
   }
@@ -379,7 +363,7 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
 
   while (!done) {
     if (!contents.GetNext(&cbEid, &lpEid, &oType, &done)) {
-      IMPORT_LOG1("*** Error iterating mailbox: %S\n", mName);
+      IMPORT_LOG1("*** Error iterating mailbox: %S\n", (const wchar_t*)mName);
       mResult = NS_ERROR_FAILURE;
       return NS_OK;  // Sync runnable must return OK.
     }
@@ -391,7 +375,8 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
                                          &reusable,
                                          getter_AddRefs(outputStream));
     if (NS_FAILED(rv)) {
-      IMPORT_LOG1("*** Error getting nsIOutputStream of mailbox: %S\n", mName);
+      IMPORT_LOG1("*** Error getting nsIOutputStream of mailbox: %S\n",
+                  (const wchar_t*)mName);
       mResult = rv;
       return NS_OK;  // Sync runnable must return OK.
     }
@@ -407,7 +392,8 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
     if (!done && (oType == MAPI_MESSAGE)) {
       if (!mCaller->m_mapi.OpenMdbEntry(mCaller->m_lpMdb, cbEid, lpEid,
                                         (LPUNKNOWN*)&lpMsg)) {
-        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", mName);
+        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n",
+                    (const wchar_t*)mName);
         mResult = NS_ERROR_FAILURE;
         return NS_OK;  // Sync runnable must return OK.
       }
@@ -425,7 +411,8 @@ NS_IMETHODIMP ImportMailboxRunnable::Run() {
         (*mMsgCount)++;
         msgStore->FinishNewMessage(outputStream, msgHdr);
       } else {
-        IMPORT_LOG1("*** Error reading message from mailbox: %S\n", mName);
+        IMPORT_LOG1("*** Error reading message from mailbox: %S\n",
+                    (const wchar_t*)mName);
         msgStore->DiscardNewMessage(outputStream, msgHdr);
       }
       if (!reusable) outputStream->Close();
@@ -487,7 +474,7 @@ nsresult ImportMailboxRunnable::ImportMessage(LPMESSAGE lpMsg,
 }
 
 BOOL nsOutlookMail::WriteData(nsIOutputStream* pDest, const char* pData,
-                              int32_t len) {
+                              uint32_t len) {
   uint32_t written;
   nsresult rv = pDest->Write(pData, len, &written);
   return NS_SUCCEEDED(rv) && written == len;
@@ -527,7 +514,7 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
   if (!m_lpMdb) {
     IMPORT_LOG1(
         "*** Unable to obtain mapi message store for address book: %S\n",
-        pName);
+        (const wchar_t*)pName);
     return NS_ERROR_FAILURE;
   }
 
@@ -559,7 +546,8 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
     (*pCount)++;
 
     if (!contents.GetNext(&cbEid, &lpEid, &oType, &done)) {
-      IMPORT_LOG1("*** Error iterating address book: %S\n", pName);
+      IMPORT_LOG1("*** Error iterating address book: %S\n",
+                  (const wchar_t*)pName);
       return NS_ERROR_FAILURE;
     }
 
@@ -567,7 +555,8 @@ nsresult nsOutlookMail::ImportAddresses(uint32_t* pCount, uint32_t* pTotal,
 
     if (!done && (oType == MAPI_MESSAGE)) {
       if (!m_mapi.OpenMdbEntry(m_lpMdb, cbEid, lpEid, (LPUNKNOWN*)&lpMsg)) {
-        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName);
+        IMPORT_LOG1("*** Error opening messages in mailbox: %S\n",
+                    (const wchar_t*)pName);
         return NS_ERROR_FAILURE;
       }
 
@@ -659,7 +648,8 @@ nsresult nsOutlookMail::CreateList(const nsString& pName,
     cbEid = sa->lpbin[idx].cb;
 
     if (!m_mapi.OpenEntry(cbEid, lpEid, (LPUNKNOWN*)&lpMsg)) {
-      IMPORT_LOG1("*** Error opening messages in mailbox: %S\n", pName.get());
+      IMPORT_LOG1("*** Error opening messages in mailbox: %S\n",
+                  static_cast<const wchar_t*>(pName.get()));
       m_mapi.MAPIFreeBuffer(value);
       return NS_ERROR_FAILURE;
     }
@@ -685,7 +675,7 @@ nsresult nsOutlookMail::CreateList(const nsString& pName,
 }
 
 void nsOutlookMail::SanitizeValue(nsString& val) {
-  val.ReplaceSubstring(NS_LITERAL_STRING("\r\n"), NS_LITERAL_STRING(", "));
+  val.ReplaceSubstring(u"\r\n"_ns, u", "_ns);
   val.ReplaceChar("\r\n", ',');
 }
 
