@@ -125,14 +125,14 @@ bool WMSettings::DoImport(nsIMsgAccount** ppAccount) {
 
   nsresult rv;
   nsCOMPtr<nsIWindowsRegKey> subKey;
-  if (NS_SUCCEEDED(key->OpenChild(NS_LITERAL_STRING("mail"),
+  if (NS_SUCCEEDED(key->OpenChild(u"mail"_ns,
                                   nsIWindowsRegKey::ACCESS_QUERY_VALUE,
                                   getter_AddRefs(subKey)))) {
-    uint32_t dwordResult = -1;
-    rv = subKey->ReadIntValue(NS_LITERAL_STRING("Poll For Mail"),
+    uint32_t dwordResult = 0xffffffff;
+    rv = subKey->ReadIntValue(u"Poll For Mail"_ns,
                               &dwordResult);  // reg_dword
     subKey->Close();
-    if (NS_SUCCEEDED(rv) && dwordResult != -1) {
+    if (NS_SUCCEEDED(rv) && dwordResult != 0xffffffff) {
       checkNewMail = true;
       checkNewMailTime = dwordResult / 60000;
     }
@@ -141,9 +141,9 @@ bool WMSettings::DoImport(nsIMsgAccount** ppAccount) {
   // (less than 64 chars) e.g.
   // account{4A18B81E-83CA-472A-8D7F-5301C0B97B8D}.oeaccount
   nsAutoString defMailAcct, defNewsAcct;
-  key->ReadStringValue(NS_LITERAL_STRING("Default Mail Account"),
+  key->ReadStringValue(u"Default Mail Account"_ns,
                        defMailAcct);  // ref_sz
-  key->ReadStringValue(NS_LITERAL_STRING("Default News Account"),
+  key->ReadStringValue(u"Default News Account"_ns,
                        defNewsAcct);  // ref_sz
 
   nsCOMPtr<nsIMsgAccountManager> accMgr =
@@ -210,19 +210,19 @@ bool WMSettings::DoIMAPServer(nsIMsgAccountManager* pMgr,
   bool result = false;
   // I now have a user name/server name pair, find out if it already exists?
   nsCOMPtr<nsIMsgIncomingServer> in;
-  nsresult rv = pMgr->FindServer(
-      NS_ConvertUTF16toUTF8(userName), NS_ConvertUTF16toUTF8(serverName),
-      NS_LITERAL_CSTRING("imap"), getter_AddRefs(in));
+  nsresult rv = pMgr->FindServer(NS_ConvertUTF16toUTF8(userName),
+                                 NS_ConvertUTF16toUTF8(serverName), "imap"_ns,
+                                 0, getter_AddRefs(in));
   if (NS_FAILED(rv) || (in == nullptr)) {
     // Create the incoming server and an account for it?
-    rv = pMgr->CreateIncomingServer(
-        NS_ConvertUTF16toUTF8(userName), NS_ConvertUTF16toUTF8(serverName),
-        NS_LITERAL_CSTRING("imap"), getter_AddRefs(in));
+    rv = pMgr->CreateIncomingServer(NS_ConvertUTF16toUTF8(userName),
+                                    NS_ConvertUTF16toUTF8(serverName),
+                                    "imap"_ns, getter_AddRefs(in));
     if (NS_SUCCEEDED(rv) && in) {
       nsCOMPtr<nsIImapIncomingServer> imapServer = do_QueryInterface(in);
       if (!imapServer) {
         IMPORT_LOG1("*** Failed to create nsIImapIncomingServer for %S!\n",
-                    serverName.get());
+                    static_cast<const wchar_t*>(serverName.get()));
         return false;
       }
       if (NS_SUCCEEDED(
@@ -253,7 +253,8 @@ bool WMSettings::DoIMAPServer(nsIMsgAccountManager* pMgr,
       in->SetBiffMinutes(checkNewMailTime);
 
       IMPORT_LOG2("Created IMAP server named: %S, userName: %S\n",
-                  serverName.get(), userName.get());
+                  static_cast<const wchar_t*>(serverName.get()),
+                  static_cast<const wchar_t*>(userName.get()));
 
       // We have a server, create an account.
       nsCOMPtr<nsIMsgAccount> account;
@@ -305,19 +306,19 @@ bool WMSettings::DoPOP3Server(nsIMsgAccountManager* pMgr,
   bool result = false;
   // I now have a user name/server name pair, find out if it already exists?
   nsCOMPtr<nsIMsgIncomingServer> in;
-  nsresult rv = pMgr->FindServer(
-      NS_ConvertUTF16toUTF8(userName), NS_ConvertUTF16toUTF8(serverName),
-      NS_LITERAL_CSTRING("pop3"), getter_AddRefs(in));
+  nsresult rv = pMgr->FindServer(NS_ConvertUTF16toUTF8(userName),
+                                 NS_ConvertUTF16toUTF8(serverName), "pop3"_ns,
+                                 0, getter_AddRefs(in));
   if (NS_FAILED(rv) || (in == nullptr)) {
     // Create the incoming server and an account for it?
-    rv = pMgr->CreateIncomingServer(
-        NS_ConvertUTF16toUTF8(userName), NS_ConvertUTF16toUTF8(serverName),
-        NS_LITERAL_CSTRING("pop3"), getter_AddRefs(in));
+    rv = pMgr->CreateIncomingServer(NS_ConvertUTF16toUTF8(userName),
+                                    NS_ConvertUTF16toUTF8(serverName),
+                                    "pop3"_ns, getter_AddRefs(in));
     if (NS_SUCCEEDED(rv) && in) {
       nsCOMPtr<nsIPop3IncomingServer> pop3Server = do_QueryInterface(in);
       if (!pop3Server) {
         IMPORT_LOG1("*** Failed to create nsIPop3IncomingServer for %S!\n",
-                    serverName.get());
+                    static_cast<const wchar_t*>(serverName.get()));
         return false;
       }
 
@@ -400,7 +401,8 @@ bool WMSettings::DoPOP3Server(nsIMsgAccountManager* pMgr,
       }
 
       IMPORT_LOG2("Created POP3 server named: %S, userName: %S\n",
-                  serverName.get(), userName.get());
+                  static_cast<const wchar_t*>(serverName.get()),
+                  static_cast<const wchar_t*>(userName.get()));
 
       // We have a server, create an account.
       nsCOMPtr<nsIMsgAccount> account;
@@ -419,7 +421,8 @@ bool WMSettings::DoPOP3Server(nsIMsgAccountManager* pMgr,
     }
   } else if (NS_SUCCEEDED(rv) && in) {
     IMPORT_LOG2("Existing POP3 server named: %S, userName: %S\n",
-                serverName.get(), userName.get());
+                static_cast<const wchar_t*>(serverName.get()),
+                static_cast<const wchar_t*>(userName.get()));
     // for an existing server we create another identity,
     // TB listed under 'manage identities'
     nsCOMPtr<nsIMsgAccount> account;
@@ -456,17 +459,17 @@ bool WMSettings::DoNNTPServer(nsIMsgAccountManager* pMgr,
   nsCOMPtr<nsIMsgIncomingServer> in;
   nsresult rv =
       pMgr->FindServer(EmptyCString(), NS_ConvertUTF16toUTF8(serverName),
-                       NS_LITERAL_CSTRING("nntp"), getter_AddRefs(in));
+                       "nntp"_ns, 0, getter_AddRefs(in));
   if (NS_FAILED(rv) || (in == nullptr)) {
     // Create the incoming server and an account for it?
-    rv = pMgr->CreateIncomingServer(
-        nsDependentCString(""), NS_ConvertUTF16toUTF8(serverName),
-        NS_LITERAL_CSTRING("nntp"), getter_AddRefs(in));
+    rv = pMgr->CreateIncomingServer(nsDependentCString(""),
+                                    NS_ConvertUTF16toUTF8(serverName),
+                                    "nntp"_ns, getter_AddRefs(in));
     if (NS_SUCCEEDED(rv) && in) {
       nsCOMPtr<nsINntpIncomingServer> nntpServer = do_QueryInterface(in);
       if (!nntpServer) {
         IMPORT_LOG1("*** Failed to create nsINnntpIncomingServer for %S!\n",
-                    serverName.get());
+                    static_cast<const wchar_t*>(serverName.get()));
         return false;
       }
       if (!userName.IsEmpty()) {  // if username req'd then auth req'd
@@ -493,7 +496,8 @@ bool WMSettings::DoNNTPServer(nsIMsgAccountManager* pMgr,
       }
 
       IMPORT_LOG2("Created NNTP server named: %S, userName: %S\n",
-                  serverName.get(), userName.get());
+                  static_cast<const wchar_t*>(serverName.get()),
+                  static_cast<const wchar_t*>(userName.get()));
 
       // We have a server, create an account.
       nsCOMPtr<nsIMsgAccount> account;
@@ -535,18 +539,17 @@ void WMSettings::SetIdentities(nsIMsgAccountManager* pMgr, nsIMsgAccount* pAcc,
                                nsAutoString& inUserName,
                                int32_t authMethodIncoming, bool isNNTP) {
   // Get the relevant information for an identity
-  nsresult rv;
   nsAutoString value;
 
   nsCOMPtr<nsIMsgIdentity> id;
-  rv = pMgr->CreateIdentity(getter_AddRefs(id));
+  pMgr->CreateIdentity(getter_AddRefs(id));
   if (id) {
     IMPORT_LOG0("Created identity and added to the account\n");
     if (NS_SUCCEEDED(nsWMUtils::GetValueForTag(
             xmlDoc, isNNTP ? "NNTP_Display_Name" : "SMTP_Display_Name",
             value))) {
       id->SetFullName(value);
-      IMPORT_LOG1("\tname: %S\n", value.get());
+      IMPORT_LOG1("\tname: %S\n", static_cast<const wchar_t*>(value.get()));
     }
 
     if (NS_SUCCEEDED(nsWMUtils::GetValueForTag(
@@ -560,7 +563,7 @@ void WMSettings::SetIdentities(nsIMsgAccountManager* pMgr, nsIMsgAccount* pAcc,
             xmlDoc, isNNTP ? "NNTP_Email_Address" : "SMTP_Email_Address",
             value))) {
       id->SetEmail(NS_ConvertUTF16toUTF8(value));
-      IMPORT_LOG1("\temail: %S\n", value.get());
+      IMPORT_LOG1("\temail: %S\n", static_cast<const wchar_t*>(value.get()));
     }
 
     if (NS_SUCCEEDED(
