@@ -13,7 +13,7 @@
 #include "nsMsgDBCID.h"
 #include "nsIMsgMailNewsUrl.h"
 #include "nsIMsgFolder.h"
-#include "nsICopyMsgStreamListener.h"
+#include "nsICopyMessageStreamListener.h"
 #include "prtime.h"
 #include "mozilla/Logging.h"
 #include "mozilla/SlicedInputStream.h"
@@ -43,7 +43,12 @@ static LazyLogModule MAILBOX("Mailbox");
  */
 #define OUTPUT_BUFFER_SIZE (4096 * 2)
 
-nsMailboxProtocol::nsMailboxProtocol(nsIURI* aURI) : nsMsgProtocol(aURI) {}
+nsMailboxProtocol::nsMailboxProtocol(nsIURI* aURI)
+    : nsMsgProtocol(aURI),
+      m_mailboxAction(nsIMailboxUrl::ActionParseMailbox),
+      m_nextState(MAILBOX_UNINITIALIZED),
+      m_initialState(MAILBOX_UNINITIALIZED),
+      mCurrentProgress(0) {}
 
 nsMailboxProtocol::~nsMailboxProtocol() {}
 
@@ -262,7 +267,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest* request,
             nsCOMPtr<nsICopyMessageStreamListener> listener =
                 do_QueryInterface(m_channelListener, &rv);
             if (listener) {
-              listener->EndCopy(m_runningUrl, aStatus);
+              listener->EndCopy(mailnewsUrl, aStatus);
               listener->StartMessage();  // start next message.
             }
           }
@@ -283,8 +288,8 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest* request,
               nsCOMPtr<nsIMsgMessageUrl> msgUrl =
                   do_QueryInterface(m_runningUrl);
               if (msgUrl) {
-                msgUrl->SetOriginalSpec(uri.get());
-                msgUrl->SetUri(uri.get());
+                msgUrl->SetOriginalSpec(uri);
+                msgUrl->SetUri(uri);
 
                 uint64_t msgOffset;
                 nextMsg->GetMessageOffset(&msgOffset);
@@ -336,7 +341,7 @@ NS_IMETHODIMP nsMailboxProtocol::OnStopRequest(nsIRequest* request,
                     rv = NS_NewInputStreamPump(getter_AddRefs(pump),
                                                stream.forget());
                     if (NS_SUCCEEDED(rv)) {
-                      rv = pump->AsyncRead(this, nullptr);
+                      rv = pump->AsyncRead(this);
                       if (NS_SUCCEEDED(rv)) m_request = pump;
                     }
                   }
@@ -417,7 +422,6 @@ nsresult nsMailboxProtocol::SetupMessageExtraction() {
       if (NS_SUCCEEDED(rv) && msgHdr) {
         msgHdr->GetMessageSize(&messageSize);
         m_runningUrl->SetMessageSize(messageSize);
-        msgHdr->GetMessageOffset(&m_msgOffset);
       } else
         NS_ASSERTION(false, "couldn't get message header");
     }
