@@ -27,10 +27,11 @@
 #include "nsIMsgHdr.h"
 #include "nsIFileURL.h"
 #include "mozilla/RefPtr.h"
+#include "mozilla/LoadInfo.h"
 #include "nsDocShellLoadState.h"
 #include "nsContentUtils.h"
 
-nsMailboxService::nsMailboxService() { mPrintingOperation = false; }
+nsMailboxService::nsMailboxService() {}
 
 nsMailboxService::~nsMailboxService() {}
 
@@ -83,7 +84,7 @@ nsresult nsMailboxService::ParseMailbox(nsIMsgWindow* aMsgWindow,
   return rv;
 }
 
-nsresult nsMailboxService::CopyMessage(const char* aSrcMailboxURI,
+nsresult nsMailboxService::CopyMessage(const nsACString& aSrcMailboxURI,
                                        nsIStreamListener* aMailboxCopyHandler,
                                        bool moveMessage,
                                        nsIUrlListener* aUrlListener,
@@ -92,7 +93,7 @@ nsresult nsMailboxService::CopyMessage(const char* aSrcMailboxURI,
   nsMailboxAction mailboxAction = nsIMailboxUrl::ActionMoveMessage;
   if (!moveMessage) mailboxAction = nsIMailboxUrl::ActionCopyMessage;
   return FetchMessage(aSrcMailboxURI, aMailboxCopyHandler, aMsgWindow,
-                      aUrlListener, nullptr, mailboxAction, nullptr, aURL);
+                      aUrlListener, nullptr, mailboxAction, false, aURL);
 }
 
 nsresult nsMailboxService::CopyMessages(
@@ -115,7 +116,7 @@ nsresult nsMailboxService::CopyMessages(
     if (msgHdr) {
       nsCString uri;
       srcFolder->GetUriForMsg(msgHdr, uri);
-      rv = PrepareMessageUrl(uri.get(), aUrlListener, actionToUse,
+      rv = PrepareMessageUrl(uri, aUrlListener, actionToUse,
                              getter_AddRefs(mailboxurl), aMsgWindow);
 
       if (NS_SUCCEEDED(rv)) {
@@ -135,11 +136,10 @@ nsresult nsMailboxService::CopyMessages(
 }
 
 nsresult nsMailboxService::FetchMessage(
-    const char* aMessageURI, nsISupports* aDisplayConsumer,
+    const nsACString& aMessageURI, nsISupports* aDisplayConsumer,
     nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener,
     const char* aFileName, /* only used by open attachment... */
-    nsMailboxAction mailboxAction, const char* aCharsetOverride,
-    nsIURI** aURL) {
+    nsMailboxAction mailboxAction, bool aAutodetectCharset, nsIURI** aURL) {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIMailboxUrl> mailboxurl;
   nsMailboxAction actionToUse = mailboxAction;
@@ -147,7 +147,7 @@ nsresult nsMailboxService::FetchMessage(
   nsCOMPtr<nsIMsgMailNewsUrl> msgUrl;
   nsAutoCString uriString(aMessageURI);
 
-  if (!strncmp(aMessageURI, "file:", 5)) {
+  if (StringBeginsWith(aMessageURI, "file:"_ns)) {
     int64_t fileSize;
     nsCOMPtr<nsIURI> fileUri;
     rv = NS_NewURI(getter_AddRefs(fileUri), aMessageURI);
@@ -158,7 +158,7 @@ nsresult nsMailboxService::FetchMessage(
     rv = fileUrl->GetFile(getter_AddRefs(file));
     NS_ENSURE_SUCCESS(rv, rv);
     file->GetFileSize(&fileSize);
-    uriString.Replace(0, 5, NS_LITERAL_CSTRING("mailbox:"));
+    uriString.Replace(0, 5, "mailbox:"_ns);
     uriString.AppendLiteral("&number=0");
     rv = NS_NewURI(getter_AddRefs(url), uriString);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -200,7 +200,7 @@ nsresult nsMailboxService::FetchMessage(
   }
 
   nsCOMPtr<nsIMsgI18NUrl> i18nurl(do_QueryInterface(msgUrl));
-  if (i18nurl) i18nurl->SetCharsetOverRide(aCharsetOverride);
+  if (i18nurl) i18nurl->SetAutodetectCharset(aAutodetectCharset);
 
   // instead of running the mailbox url like we used to, let's try to run the
   // url in the docshell...
@@ -229,7 +229,7 @@ nsresult nsMailboxService::FetchMessage(
 }
 
 NS_IMETHODIMP nsMailboxService::FetchMimePart(
-    nsIURI* aURI, const char* aMessageURI, nsISupports* aDisplayConsumer,
+    nsIURI* aURI, const nsACString& aMessageURI, nsISupports* aDisplayConsumer,
     nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener, nsIURI** aURL) {
   nsresult rv;
   nsCOMPtr<nsIMsgMailNewsUrl> msgUrl(do_QueryInterface(aURI, &rv));
@@ -243,19 +243,20 @@ NS_IMETHODIMP nsMailboxService::FetchMimePart(
   return RunMailboxUrl(msgUrl, aDisplayConsumer);
 }
 
-NS_IMETHODIMP nsMailboxService::DisplayMessage(const char* aMessageURI,
+NS_IMETHODIMP nsMailboxService::DisplayMessage(const nsACString& aMessageURI,
                                                nsISupports* aDisplayConsumer,
                                                nsIMsgWindow* aMsgWindow,
                                                nsIUrlListener* aUrlListener,
-                                               const char* aCharsetOveride,
+                                               bool aOverideCharset,
                                                nsIURI** aURL) {
   return FetchMessage(aMessageURI, aDisplayConsumer, aMsgWindow, aUrlListener,
                       nullptr, nsIMailboxUrl::ActionFetchMessage,
-                      aCharsetOveride, aURL);
+                      aOverideCharset, aURL);
 }
 
 NS_IMETHODIMP
-nsMailboxService::StreamMessage(const char* aMessageURI, nsISupports* aConsumer,
+nsMailboxService::StreamMessage(const nsACString& aMessageURI,
+                                nsISupports* aConsumer,
                                 nsIMsgWindow* aMsgWindow,
                                 nsIUrlListener* aUrlListener,
                                 bool /* aConvertData */,
@@ -272,16 +273,14 @@ nsMailboxService::StreamMessage(const char* aMessageURI, nsISupports* aConsumer,
     aURIString += aAdditionalHeader;
   }
 
-  return FetchMessage(aURIString.get(), aConsumer, aMsgWindow, aUrlListener,
-                      nullptr, nsIMailboxUrl::ActionFetchMessage, nullptr,
-                      aURL);
+  return FetchMessage(aURIString, aConsumer, aMsgWindow, aUrlListener, nullptr,
+                      nsIMailboxUrl::ActionFetchMessage, false, aURL);
 }
 
-NS_IMETHODIMP nsMailboxService::StreamHeaders(const char* aMessageURI,
+NS_IMETHODIMP nsMailboxService::StreamHeaders(const nsACString& aMessageURI,
                                               nsIStreamListener* aConsumer,
                                               nsIUrlListener* aUrlListener,
                                               bool aLocalOnly, nsIURI** aURL) {
-  NS_ENSURE_ARG_POINTER(aMessageURI);
   NS_ENSURE_ARG_POINTER(aConsumer);
   nsAutoCString folderURI;
   nsMsgKey msgKey;
@@ -290,16 +289,15 @@ NS_IMETHODIMP nsMailboxService::StreamHeaders(const char* aMessageURI,
       DecomposeMailboxURI(aMessageURI, getter_AddRefs(folder), &msgKey);
   if (msgKey == nsMsgKey_None) return NS_MSG_MESSAGE_NOT_FOUND;
 
+  nsCOMPtr<nsIMsgDatabase> db;
+  rv = folder->GetMsgDatabase(getter_AddRefs(db));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIMsgDBHdr> msgHdr;
+  rv = db->GetMsgHdrForKey(msgKey, getter_AddRefs(msgHdr));
+  NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIInputStream> inputStream;
-  int64_t messageOffset;
-  uint32_t messageSize;
-  rv = folder->GetOfflineFileStream(msgKey, &messageOffset, &messageSize,
-                                    getter_AddRefs(inputStream));
+  rv = folder->GetLocalMsgStream(msgHdr, getter_AddRefs(inputStream));
   NS_ENSURE_SUCCESS(rv, rv);
-
-  // GetOfflineFileStream returns NS_OK but null inputStream when there is an
-  // error getting the database
-  if (!inputStream) return NS_ERROR_FAILURE;
   return MsgStreamMsgHeaders(inputStream, aConsumer);
 }
 
@@ -309,10 +307,13 @@ NS_IMETHODIMP nsMailboxService::IsMsgInMemCache(nsIURI* aUrl,
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsMailboxService::OpenAttachment(
-    const char* aContentType, const char* aFileName, const char* aUrl,
-    const char* aMessageUri, nsISupports* aDisplayConsumer,
-    nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener) {
+NS_IMETHODIMP nsMailboxService::OpenAttachment(const nsACString& aContentType,
+                                               const nsACString& aFileName,
+                                               const nsACString& aUrl,
+                                               const nsACString& aMessageUri,
+                                               nsISupports* aDisplayConsumer,
+                                               nsIMsgWindow* aMsgWindow,
+                                               nsIUrlListener* aUrlListener) {
   nsCOMPtr<nsIURI> URL;
   nsAutoCString urlString(aUrl);
   urlString += "&type=";
@@ -341,8 +342,8 @@ NS_IMETHODIMP nsMailboxService::OpenAttachment(
 }
 
 NS_IMETHODIMP
-nsMailboxService::SaveMessageToDisk(const char* aMessageURI, nsIFile* aFile,
-                                    bool aAddDummyEnvelope,
+nsMailboxService::SaveMessageToDisk(const nsACString& aMessageURI,
+                                    nsIFile* aFile, bool aAddDummyEnvelope,
                                     nsIUrlListener* aUrlListener, nsIURI** aURL,
                                     bool canonicalLineEnding,
                                     nsIMsgWindow* aMsgWindow) {
@@ -370,13 +371,14 @@ nsMailboxService::SaveMessageToDisk(const char* aMessageURI, nsIFile* aFile,
   return rv;
 }
 
-NS_IMETHODIMP nsMailboxService::GetUrlForUri(const char* aMessageURI,
-                                             nsIURI** aURL,
-                                             nsIMsgWindow* aMsgWindow) {
+NS_IMETHODIMP nsMailboxService::GetUrlForUri(const nsACString& aMessageURI,
+                                             nsIMsgWindow* aMsgWindow,
+                                             nsIURI** aURL) {
   NS_ENSURE_ARG_POINTER(aURL);
-  if (!strncmp(aMessageURI, "file:", 5) ||
-      PL_strstr(aMessageURI, "type=application/x-message-display") ||
-      !strncmp(aMessageURI, "mailbox:", 8))
+  if (StringBeginsWith(aMessageURI, "file:"_ns) ||
+      PL_strstr(PromiseFlatCString(aMessageURI).get(),
+                "type=application/x-message-display") ||
+      StringBeginsWith(aMessageURI, "mailbox:"_ns))
     return NS_NewURI(aURL, aMessageURI);
 
   nsresult rv = NS_OK;
@@ -393,30 +395,25 @@ NS_IMETHODIMP nsMailboxService::GetUrlForUri(const char* aMessageURI,
 nsresult nsMailboxService::RunMailboxUrl(nsIURI* aMailboxUrl,
                                          nsISupports* aDisplayConsumer) {
   // create a protocol instance to run the url..
-  nsresult rv = NS_OK;
   RefPtr<nsMailboxProtocol> protocol = new nsMailboxProtocol(aMailboxUrl);
-
-  if (protocol) {
-    rv = protocol->Initialize(aMailboxUrl);
-    if (NS_FAILED(rv)) {
-      protocol = nullptr;
-      return rv;
-    }
-    rv = protocol->LoadUrl(aMailboxUrl, aDisplayConsumer);
-  }
-
-  return rv;
+  // It implements nsIChannel, and all channels require loadInfo.
+  protocol->SetLoadInfo(new mozilla::net::LoadInfo(
+      nsContentUtils::GetSystemPrincipal(), nullptr, nullptr,
+      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
+      nsIContentPolicy::TYPE_OTHER));
+  nsresult rv = protocol->Initialize(aMailboxUrl);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return protocol->LoadUrl(aMailboxUrl, aDisplayConsumer);
 }
 
 // This function takes a message uri, converts it into a file path & msgKey
 // pair. It then turns that into a mailbox url object. It also registers a url
 // listener if appropriate. AND it can take in a mailbox action and set that
 // field on the returned url as well.
-nsresult nsMailboxService::PrepareMessageUrl(const char* aSrcMsgMailboxURI,
-                                             nsIUrlListener* aUrlListener,
-                                             nsMailboxAction aMailboxAction,
-                                             nsIMailboxUrl** aMailboxUrl,
-                                             nsIMsgWindow* msgWindow) {
+nsresult nsMailboxService::PrepareMessageUrl(
+    const nsACString& aSrcMsgMailboxURI, nsIUrlListener* aUrlListener,
+    nsMailboxAction aMailboxAction, nsIMailboxUrl** aMailboxUrl,
+    nsIMsgWindow* msgWindow) {
   nsresult rv = CallCreateInstance(NS_MAILBOXURL_CONTRACTID, aMailboxUrl);
   if (NS_SUCCEEDED(rv) && aMailboxUrl && *aMailboxUrl) {
     // okay now generate the url string
@@ -424,8 +421,9 @@ nsresult nsMailboxService::PrepareMessageUrl(const char* aSrcMsgMailboxURI,
     nsAutoCString folderURI;
     nsMsgKey msgKey;
     nsCString folderPath;
-    const char* part = PL_strstr(aSrcMsgMailboxURI, "part=");
-    const char* header = PL_strstr(aSrcMsgMailboxURI, "header=");
+    const nsPromiseFlatCString& flat = PromiseFlatCString(aSrcMsgMailboxURI);
+    const char* part = PL_strstr(flat.get(), "part=");
+    const char* header = PL_strstr(flat.get(), "header=");
     rv = nsParseLocalMessageURI(aSrcMsgMailboxURI, folderURI, &msgKey);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = nsLocalURI2Path(kMailboxRootURI, folderURI.get(), folderPath);
@@ -437,10 +435,7 @@ nsresult nsMailboxService::PrepareMessageUrl(const char* aSrcMsgMailboxURI,
           folderPath,
           nsINetUtil::ESCAPE_URL_DIRECTORY | nsINetUtil::ESCAPE_URL_FORCED,
           buf);
-      if (mPrintingOperation)
-        urlSpec = PR_smprintf("mailbox://%s?number=%lu&header=print", buf.get(),
-                              msgKey);
-      else if (part)
+      if (part)
         urlSpec =
             PR_smprintf("mailbox://%s?number=%lu&%s", buf.get(), msgKey, part);
       else if (header)
@@ -533,6 +528,7 @@ NS_IMETHODIMP nsMailboxService::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
                                            nsIChannel** _retval) {
   NS_ENSURE_ARG_POINTER(aURI);
   NS_ENSURE_ARG_POINTER(_retval);
+  MOZ_ASSERT(aLoadInfo);
   nsresult rv = NS_OK;
   nsAutoCString spec;
   rv = aURI->GetSpec(spec);
@@ -540,7 +536,7 @@ NS_IMETHODIMP nsMailboxService::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
 
   if (spec.Find("?uidl=") >= 0 || spec.Find("&uidl=") >= 0) {
     nsCOMPtr<nsIProtocolHandler> handler =
-        do_GetService(NS_POP3SERVICE_CONTRACTID1, &rv);
+        do_GetService(NS_POP3SERVICE_CONTRACTID2, &rv);
     if (NS_SUCCEEDED(rv)) {
       nsCOMPtr<nsIURI> pop3Uri;
 
@@ -562,32 +558,32 @@ NS_IMETHODIMP nsMailboxService::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
   rv = protocol->SetLoadInfo(aLoadInfo);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  // Add the attachment disposition. This forces docShell to open the
+  // attachment instead of displaying it. Content types we have special
+  // handlers for are white-listed. This white list also exists in
+  // nsImapService::NewChannel and nsNntpService::NewChannel, so if you're
+  // changing this, update those too.
+  if (spec.Find("part=") >= 0 && spec.Find("type=message/rfc822") < 0 &&
+      spec.Find("type=application/x-message-display") < 0 &&
+      spec.Find("type=application/pdf") < 0) {
+    rv = protocol->SetContentDisposition(nsIChannel::DISPOSITION_ATTACHMENT);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   protocol.forget(_retval);
   return NS_OK;
 }
 
-nsresult nsMailboxService::DisplayMessageForPrinting(
-    const char* aMessageURI, nsISupports* aDisplayConsumer,
-    nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener, nsIURI** aURL) {
-  mPrintingOperation = true;
-  nsresult rv =
-      FetchMessage(aMessageURI, aDisplayConsumer, aMsgWindow, aUrlListener,
-                   nullptr, nsIMailboxUrl::ActionFetchMessage, nullptr, aURL);
-  mPrintingOperation = false;
-  return rv;
-}
-
 NS_IMETHODIMP nsMailboxService::Search(nsIMsgSearchSession* aSearchSession,
                                        nsIMsgWindow* aMsgWindow,
                                        nsIMsgFolder* aMsgFolder,
-                                       const char* aMessageUri) {
+                                       const nsACString& aMessageUri) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-nsresult nsMailboxService::DecomposeMailboxURI(const char* aMessageURI,
+nsresult nsMailboxService::DecomposeMailboxURI(const nsACString& aMessageURI,
                                                nsIMsgFolder** aFolder,
                                                nsMsgKey* aMsgKey) {
-  NS_ENSURE_ARG_POINTER(aMessageURI);
   NS_ENSURE_ARG_POINTER(aFolder);
   NS_ENSURE_ARG_POINTER(aMsgKey);
 
@@ -600,8 +596,8 @@ nsresult nsMailboxService::DecomposeMailboxURI(const char* aMessageURI,
 }
 
 NS_IMETHODIMP
-nsMailboxService::MessageURIToMsgHdr(const char* uri, nsIMsgDBHdr** _retval) {
-  NS_ENSURE_ARG_POINTER(uri);
+nsMailboxService::MessageURIToMsgHdr(const nsACString& uri,
+                                     nsIMsgDBHdr** _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
 
   nsresult rv = NS_OK;
