@@ -14,6 +14,7 @@
 #include "nsMsgFolderFlags.h"
 #include "nsMsgMessageFlags.h"
 #include "nsIMsgLocalMailFolder.h"
+#include "nsIInputStream.h"
 #include "nsCOMArray.h"
 #include "nsIFile.h"
 #include "nsIDirectoryEnumerator.h"
@@ -24,19 +25,16 @@
 #include "nsMsgUtils.h"
 #include "nsMsgDBCID.h"
 #include "nsIDBFolderInfo.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "nsMsgLocalFolderHdrs.h"
 #include "nsMailHeaders.h"
-#include "nsReadLine.h"
 #include "nsParseMailbox.h"
 #include "nsIMailboxService.h"
 #include "nsMsgLocalCID.h"
 #include "nsIMsgFolderCompactor.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
+#include "nsQuarantinedOutputStream.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/UniquePtr.h"
 #include "prprf.h"
 #include <cstdlib>  // for std::abs(int/long)
 #include <cmath>    // for std::abs(float/double)
@@ -175,7 +173,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::HasSpaceAvailable(nsIMsgFolder* aFolder,
   }
 
   *aResult = DiskSpaceAvailableInStore(pathFile, aSpaceRequested);
-  if (!*aResult) return NS_ERROR_FILE_DISK_FULL;
+  if (!*aResult) return NS_ERROR_FILE_NO_DEVICE_SPACE;
 
   return NS_OK;
 }
@@ -469,8 +467,8 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(
   rv = aSrcFolder->MatchOrChangeFilterDestination(newMsgFolder, true, &changed);
   if (changed) aSrcFolder->AlertFilterChanged(aMsgWindow);
 
-  nsCOMPtr<nsISimpleEnumerator> enumerator;
-  rv = aSrcFolder->GetSubFolders(getter_AddRefs(enumerator));
+  nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+  rv = aSrcFolder->GetSubFolders(subFolders);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Copy subfolders to the new location
@@ -478,15 +476,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(
   nsCOMPtr<nsIMsgLocalMailFolder> localNewFolder(
       do_QueryInterface(newMsgFolder, &rv));
   if (NS_SUCCEEDED(rv)) {
-    bool hasMore;
-    while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore &&
-           NS_SUCCEEDED(copyStatus)) {
-      nsCOMPtr<nsISupports> item;
-      enumerator->GetNext(getter_AddRefs(item));
-
-      nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(item));
-      if (!folder) continue;
-
+    for (nsIMsgFolder* folder : subFolders) {
       copyStatus =
           localNewFolder->CopyFolderLocal(folder, false, aMsgWindow, aListener);
       // Test if the call succeeded, if not we have to stop recursive call
@@ -509,7 +499,7 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::CopyFolder(
 
     // Notify the "folder" that was dragged and dropped has been created. No
     // need to do this for its subfolders. isMoveFolder will be true for folder.
-    aDstFolder->NotifyItemAdded(newMsgFolder);
+    aDstFolder->NotifyFolderAdded(newMsgFolder);
 
     nsCOMPtr<nsIMsgFolder> msgParent;
     aSrcFolder->GetParent(getter_AddRefs(msgParent));
@@ -559,6 +549,38 @@ nsMsgBrkMBoxStore::GetNewMsgOutputStream(nsIMsgFolder* aFolder,
                                          nsIMsgDBHdr** aNewMsgHdr,
                                          bool* aReusable,
                                          nsIOutputStream** aResult) {
+  bool quarantining = false;
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  if (prefBranch) {
+    prefBranch->GetBoolPref("mailnews.downloadToTempFile", &quarantining);
+  }
+
+  if (!quarantining) {
+    // Caller will write directly to mbox.
+    return InternalGetNewMsgOutputStream(aFolder, aNewMsgHdr, aReusable,
+                                         aResult);
+  }
+
+  // Quarantining is on, so we want to write the new message to a temp file
+  // and let the virus checker have at it before we append it to the mbox.
+  // We'll wrap the mboxStream with an nsQuarantinedOutputStream and return
+  // that.
+  nsCOMPtr<nsIOutputStream> mboxStream;
+  bool reusable;
+  nsresult rv = InternalGetNewMsgOutputStream(aFolder, aNewMsgHdr, &reusable,
+                                              getter_AddRefs(mboxStream));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  RefPtr<nsQuarantinedOutputStream> qStream =
+      new nsQuarantinedOutputStream(mboxStream);
+  *aReusable = false;
+  qStream.forget(aResult);
+  return NS_OK;
+}
+
+nsresult nsMsgBrkMBoxStore::InternalGetNewMsgOutputStream(
+    nsIMsgFolder* aFolder, nsIMsgDBHdr** aNewMsgHdr, bool* aReusable,
+    nsIOutputStream** aResult) {
   NS_ENSURE_ARG_POINTER(aFolder);
   NS_ENSURE_ARG_POINTER(aNewMsgHdr);
   NS_ENSURE_ARG_POINTER(aReusable);
@@ -606,7 +628,7 @@ nsMsgBrkMBoxStore::GetNewMsgOutputStream(nsIMsgFolder* aFolder,
     NS_ENSURE_SUCCESS(rv, rv);
     rv = seekable->Seek(nsISeekableStream::NS_SEEK_END, 0);
     NS_ENSURE_SUCCESS(rv, rv);
-    m_outputStreams.Put(URI, *aResult);
+    m_outputStreams.InsertOrUpdate(URI, *aResult);
   }
   int64_t filePos;
   seekable->Tell(&filePos);
@@ -632,6 +654,12 @@ nsMsgBrkMBoxStore::DiscardNewMessage(nsIOutputStream* aOutputStream,
 #ifdef _DEBUG
   m_streamOutstandingFolder = nullptr;
 #endif
+  nsCOMPtr<nsISafeOutputStream> safe = do_QueryInterface(aOutputStream);
+  if (safe) {
+    // nsISafeOutputStream only writes upon finish(), so no cleanup required.
+    return aOutputStream->Close();
+  }
+  // Truncate the mbox back to where we started writing.
   uint64_t hdrOffset;
   aNewHdr->GetMessageOffset(&hdrOffset);
   aOutputStream->Close();
@@ -647,11 +675,17 @@ nsMsgBrkMBoxStore::DiscardNewMessage(nsIOutputStream* aOutputStream,
 NS_IMETHODIMP
 nsMsgBrkMBoxStore::FinishNewMessage(nsIOutputStream* aOutputStream,
                                     nsIMsgDBHdr* aNewHdr) {
+  NS_ENSURE_ARG_POINTER(aOutputStream);
 #ifdef _DEBUG
   m_streamOutstandingFolder = nullptr;
 #endif
-  NS_ENSURE_ARG_POINTER(aOutputStream);
-  //  NS_ENSURE_ARG_POINTER(aNewHdr);
+  // Quarantining is implemented using a nsISafeOutputStream.
+  // It requires an explicit commit, or the data will be discarded.
+  nsCOMPtr<nsISafeOutputStream> safe = do_QueryInterface(aOutputStream);
+  if (safe) {
+    return safe->Finish();
+  }
+  aOutputStream->Close();
   return NS_OK;
 }
 
@@ -669,29 +703,25 @@ nsMsgBrkMBoxStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr* aNewHdr,
 NS_IMETHODIMP
 nsMsgBrkMBoxStore::GetMsgInputStream(nsIMsgFolder* aMsgFolder,
                                      const nsACString& aMsgToken,
-                                     int64_t* aOffset, nsIMsgDBHdr* aMsgHdr,
                                      bool* aReusable,
                                      nsIInputStream** aResult) {
-  NS_ENSURE_ARG_POINTER(aMsgFolder);
-  NS_ENSURE_ARG_POINTER(aResult);
-  NS_ENSURE_ARG_POINTER(aOffset);
+  MOZ_ASSERT(aMsgFolder);
+  MOZ_ASSERT(aResult);
+  MOZ_ASSERT(!aMsgToken.IsEmpty());
 
-  // If there is no store token, then we set it to the existing message offset.
-  if (aMsgToken.IsEmpty()) {
-    uint64_t offset;
-    NS_ENSURE_ARG_POINTER(aMsgHdr);
-    aMsgHdr->GetMessageOffset(&offset);
-    *aOffset = int64_t(offset);
-    char storeToken[100];
-    PR_snprintf(storeToken, sizeof(storeToken), "%lld", *aOffset);
-    aMsgHdr->SetStringProperty("storeToken", storeToken);
-  } else
-    *aOffset = ParseUint64Str(PromiseFlatCString(aMsgToken).get());
   *aReusable = true;
+  uint64_t offset = ParseUint64Str(PromiseFlatCString(aMsgToken).get());
   nsCOMPtr<nsIFile> mboxFile;
   nsresult rv = aMsgFolder->GetFilePath(getter_AddRefs(mboxFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  return NS_NewLocalFileInputStream(aResult, mboxFile);
+  nsCOMPtr<nsIInputStream> msgStream;
+  rv = NS_NewLocalFileInputStream(getter_AddRefs(msgStream), mboxFile);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsISeekableStream> seekable(do_QueryInterface(msgStream));
+  rv = seekable->Seek(PR_SEEK_SET, offset);
+  NS_ENSURE_SUCCESS(rv, rv);
+  msgStream.forget(aResult);
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::DeleteMessages(
@@ -700,17 +730,17 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::DeleteMessages(
 }
 
 NS_IMETHODIMP
-nsMsgBrkMBoxStore::CopyMessages(bool isMove, nsIArray* aHdrArray,
+nsMsgBrkMBoxStore::CopyMessages(bool isMove,
+                                const nsTArray<RefPtr<nsIMsgDBHdr>>& aHdrArray,
                                 nsIMsgFolder* aDstFolder,
-                                nsIMsgCopyServiceListener* aListener,
-                                nsIArray** aDstHdrs,
+                                nsTArray<RefPtr<nsIMsgDBHdr>>& aDstHdrs,
                                 nsITransaction** aUndoAction, bool* aCopyDone) {
-  NS_ENSURE_ARG_POINTER(aHdrArray);
   NS_ENSURE_ARG_POINTER(aDstFolder);
-  NS_ENSURE_ARG_POINTER(aDstHdrs);
   NS_ENSURE_ARG_POINTER(aCopyDone);
-  *aDstHdrs = nullptr;
+  aDstHdrs.Clear();
   *aUndoAction = nullptr;
+  // We return false to indicate there's no shortcut. The calling code will
+  // just have to perform the copy the hard way.
   *aCopyDone = false;
   return NS_OK;
 }
@@ -725,18 +755,10 @@ nsMsgBrkMBoxStore::GetSupportsCompaction(bool* aSupportsCompaction) {
 NS_IMETHODIMP nsMsgBrkMBoxStore::CompactFolder(nsIMsgFolder* aFolder,
                                                nsIUrlListener* aListener,
                                                nsIMsgWindow* aMsgWindow) {
-  NS_ENSURE_ARG_POINTER(aFolder);
-  nsresult rv;
-  nsCOMPtr<nsIMsgFolderCompactor> folderCompactor =
-      do_CreateInstance(NS_MSGLOCALFOLDERCOMPACTOR_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  int64_t expungedBytes = 0;
-  aFolder->GetExpungedBytes(&expungedBytes);
-  // check if we need to compact the folder
-  return (expungedBytes > 0)
-             ? folderCompactor->Compact(aFolder, false, aListener, aMsgWindow)
-             : aFolder->NotifyCompactCompleted();
+  // Eventually, folder compaction should be managed by nsMsgBrkMBoxStore, but
+  // for now it's separate (see nsMsgFolderCompactor) and invoked via the
+  // folder methods Compact() and CompactAll().
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP nsMsgBrkMBoxStore::RebuildIndex(nsIMsgFolder* aFolder,
@@ -793,7 +815,7 @@ nsresult nsMsgBrkMBoxStore::GetOutputStream(
     NS_ENSURE_SUCCESS(rv, rv);
     rv = MsgGetFileStream(mboxFile, getter_AddRefs(outputStream));
     seekableStream = do_QueryInterface(outputStream);
-    if (NS_SUCCEEDED(rv)) m_outputStreams.Put(URI, outputStream);
+    if (NS_SUCCEEDED(rv)) m_outputStreams.InsertOrUpdate(URI, outputStream);
   }
   return rv;
 }
@@ -822,10 +844,26 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeFlags(
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr;
   for (auto msgHdr : aHdrArray) {
-    // Seek to x-mozilla-status offset and rewrite value.
-    rv = UpdateFolderFlag(msgHdr, aSet, aFlags, outputStream);
+    // Work out the flags we want to write.
+    uint32_t flags = 0;
+    (void)msgHdr->GetFlags(&flags);
+    flags &= ~(nsMsgMessageFlags::RuntimeOnly | nsMsgMessageFlags::Offline);
+    if (aSet) {
+      flags |= aFlags;
+    } else {
+      flags &= ~aFlags;
+    }
+
+    // Rewrite flags into X-Mozilla-Status headers.
+    nsCOMPtr<nsISeekableStream> seekable(do_QueryInterface(outputStream, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+    uint64_t msgOffset;
+    rv = msgHdr->GetMessageOffset(&msgOffset);
+    NS_ENSURE_SUCCESS(rv, rv);
+    seekable->Seek(nsISeekableStream::NS_SEEK_SET, msgOffset);
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = RewriteMsgFlags(seekable, flags);
     if (NS_FAILED(rv)) {
-      NS_WARNING("updateFolderFlag failed");
       break;
     }
   }
@@ -842,55 +880,47 @@ NS_IMETHODIMP nsMsgBrkMBoxStore::ChangeKeywords(
     bool aAdd) {
   if (aHdrArray.IsEmpty()) return NS_ERROR_INVALID_ARG;
 
-  nsCOMPtr<nsIOutputStream> outputStream;
-  nsCOMPtr<nsISeekableStream> seekableStream;
+  nsTArray<nsCString> keywordsToAdd;
+  nsTArray<nsCString> keywordsToRemove;
+  if (aAdd) {
+    ParseString(aKeywords, ' ', keywordsToAdd);
+  } else {
+    ParseString(aKeywords, ' ', keywordsToRemove);
+  }
+
+  // Get the (possibly-cached) seekable & writable stream for this mbox.
+  nsCOMPtr<nsIOutputStream> output;
+  nsCOMPtr<nsISeekableStream> seekable;
   int64_t restoreStreamPos;
-
-  nsresult rv = GetOutputStream(aHdrArray[0], outputStream, seekableStream,
-                                restoreStreamPos);
+  nsresult rv =
+      GetOutputStream(aHdrArray[0], output, seekable, restoreStreamPos);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIInputStream> inputStream = do_QueryInterface(outputStream, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+  for (auto msgHdr : aHdrArray) {
+    uint64_t msgStart;
+    msgHdr->GetMessageOffset(&msgStart);
+    seekable->Seek(nsISeekableStream::NS_SEEK_SET, msgStart);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-  mozilla::UniquePtr<nsLineBuffer<char>> lineBuffer(new nsLineBuffer<char>);
+    bool notEnoughRoom;
+    rv = ChangeKeywordsHelper(seekable, keywordsToAdd, keywordsToRemove,
+                              notEnoughRoom);
 
-  // For each message, we seek to the beginning of the x-mozilla-status header,
-  // and start reading lines, looking for x-mozilla-keys: headers; If we're
-  // adding the keyword and we find
-  // a header with the desired keyword already in it, we don't need to
-  // do anything. Likewise, if removing keyword and we don't find it,
-  // we don't need to do anything. Otherwise, if adding, we need to
-  // see if there's an x-mozilla-keys
-  // header with room for the new keyword. If so, we replace the
-  // corresponding number of spaces with the keyword. If no room,
-  // we can't do anything until the folder is compacted and another
-  // x-mozilla-keys header is added. In that case, we set a property
-  // on the header, which the compaction code will check.
-
-  nsTArray<nsCString> keywordArray;
-  ParseString(aKeywords, ' ', keywordArray);
-
-  nsCOMPtr<nsIMsgDBHdr> msgHdr;
-  for (auto msgHdr : aHdrArray)  // for each message
-  {
-    uint64_t messageOffset;
-    msgHdr->GetMessageOffset(&messageOffset);
-    uint32_t statusOffset = 0;
-    (void)msgHdr->GetStatusOffset(&statusOffset);
-    uint64_t desiredOffset = messageOffset + statusOffset;
-
-    ChangeKeywordsHelper(msgHdr, desiredOffset, *lineBuffer, keywordArray, aAdd,
-                         outputStream, seekableStream, inputStream);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (notEnoughRoom) {
+      // The growKeywords property indicates that the X-Mozilla-Keys header
+      // doesn't have enough space, and should be rebuilt during the next
+      // folder compaction.
+      msgHdr->SetUint32Property("growKeywords", 1);
+    }
   }
-  lineBuffer.reset();
-  if (restoreStreamPos != -1)
-    seekableStream->Seek(nsISeekableStream::NS_SEEK_SET, restoreStreamPos);
-  else if (outputStream)
-    outputStream->Close();
-  if (!aHdrArray.IsEmpty()) {
-    SetDBValid(aHdrArray[0]);
+
+  if (restoreStreamPos != -1) {
+    seekable->Seek(nsISeekableStream::NS_SEEK_SET, restoreStreamPos);
+  } else if (output) {
+    output->Close();
   }
+  SetDBValid(aHdrArray[0]);
   return NS_OK;
 }
 
