@@ -14,6 +14,7 @@
 #include "nsIMsgFolderNotificationService.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIDirectoryEnumerator.h"
+#include "nsIInputStream.h"
 #include "nsMsgFolderFlags.h"
 #include "nsCOMArray.h"
 #include "nsIFile.h"
@@ -23,8 +24,6 @@
 #include "nsMsgUtils.h"
 #include "nsMsgDBCID.h"
 #include "nsIDBFolderInfo.h"
-#include "nsIMutableArray.h"
-#include "nsArrayUtils.h"
 #include "nsMailHeaders.h"
 #include "nsParseMailbox.h"
 #include "nsMsgLocalCID.h"
@@ -170,14 +169,14 @@ nsresult nsMsgMaildirStore::CreateMaildir(nsIFile* path) {
 
   leaf->InitWithFile(path);
 
-  leaf->AppendNative(NS_LITERAL_CSTRING("tmp"));
+  leaf->AppendNative("tmp"_ns);
   rv = leaf->Create(nsIFile::DIRECTORY_TYPE, 0700);
   if (NS_FAILED(rv) && rv != NS_ERROR_FILE_ALREADY_EXISTS) {
     NS_WARNING("Could not create tmp directory for message folder");
     return rv;
   }
 
-  leaf->SetNativeLeafName(NS_LITERAL_CSTRING("cur"));
+  leaf->SetNativeLeafName("cur"_ns);
   rv = leaf->Create(nsIFile::DIRECTORY_TYPE, 0700);
   if (NS_FAILED(rv) && rv != NS_ERROR_FILE_ALREADY_EXISTS) {
     NS_WARNING("Could not create cur directory for message folder");
@@ -268,7 +267,7 @@ NS_IMETHODIMP nsMsgMaildirStore::HasSpaceAvailable(nsIMsgFolder* aFolder,
   NS_ENSURE_SUCCESS(rv, rv);
 
   *aResult = DiskSpaceAvailableInStore(pathFile, aSpaceRequested);
-  if (!*aResult) return NS_ERROR_FILE_DISK_FULL;
+  if (!*aResult) return NS_ERROR_FILE_NO_DEVICE_SPACE;
 
   return NS_OK;
 }
@@ -288,7 +287,7 @@ NS_IMETHODIMP nsMsgMaildirStore::IsSummaryFileValid(nsIMsgFolder* aFolder,
     nsCOMPtr<nsIFile> newFile;
     rv = aFolder->GetFilePath(getter_AddRefs(newFile));
     NS_ENSURE_SUCCESS(rv, rv);
-    newFile->Append(NS_LITERAL_STRING("cur"));
+    newFile->Append(u"cur"_ns);
 
     // If the "cur" sub-dir doesn't exist, and there are no messages
     // in the db, then the folder is probably new and the db is valid.
@@ -471,8 +470,8 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(
   rv = aSrcFolder->MatchOrChangeFilterDestination(newMsgFolder, true, &changed);
   if (changed) aSrcFolder->AlertFilterChanged(aMsgWindow);
 
-  nsCOMPtr<nsISimpleEnumerator> enumerator;
-  rv = aSrcFolder->GetSubFolders(getter_AddRefs(enumerator));
+  nsTArray<RefPtr<nsIMsgFolder>> subFolders;
+  rv = aSrcFolder->GetSubFolders(subFolders);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Copy subfolders to the new location
@@ -480,15 +479,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(
   nsCOMPtr<nsIMsgLocalMailFolder> localNewFolder(
       do_QueryInterface(newMsgFolder, &rv));
   if (NS_SUCCEEDED(rv)) {
-    bool hasMore;
-    while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore &&
-           NS_SUCCEEDED(copyStatus)) {
-      nsCOMPtr<nsISupports> item;
-      enumerator->GetNext(getter_AddRefs(item));
-
-      nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(item));
-      if (!folder) continue;
-
+    for (nsIMsgFolder* folder : subFolders) {
       copyStatus =
           localNewFolder->CopyFolderLocal(folder, false, aMsgWindow, aListener);
       // Test if the call succeeded, if not we have to stop recursive call
@@ -512,7 +503,7 @@ NS_IMETHODIMP nsMsgMaildirStore::CopyFolder(
     // Notify that the folder that was dragged and dropped has been created.
     // No need to do this for its subfolders - isMoveFolder will be true for
     // folder.
-    aDstFolder->NotifyItemAdded(newMsgFolder);
+    aDstFolder->NotifyFolderAdded(newMsgFolder);
 
     nsCOMPtr<nsIMsgFolder> msgParent;
     aSrcFolder->GetParent(getter_AddRefs(msgParent));
@@ -585,7 +576,7 @@ nsMsgMaildirStore::GetNewMsgOutputStream(nsIMsgFolder* aFolder,
   nsCOMPtr<nsIFile> newFile;
   rv = aFolder->GetFilePath(getter_AddRefs(newFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  newFile->Append(NS_LITERAL_STRING("tmp"));
+  newFile->Append(u"tmp"_ns);
 
   // let's check if the folder exists
   // XXX TODO: kill this and make sure maildir creation includes cur/tmp
@@ -635,7 +626,7 @@ nsMsgMaildirStore::DiscardNewMessage(nsIOutputStream* aOutputStream,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // path to the message download folder
-  path->Append(NS_LITERAL_STRING("tmp"));
+  path->Append(u"tmp"_ns);
   path->AppendNative(fileName);
 
   return path->Remove(false);
@@ -668,7 +659,7 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream* aOutputStream,
   // path to the new destination
   nsCOMPtr<nsIFile> curPath;
   folderPath->Clone(getter_AddRefs(curPath));
-  curPath->Append(NS_LITERAL_STRING("cur"));
+  curPath->Append(u"cur"_ns);
 
   // let's check if the folder exists
   // XXX TODO: kill this and make sure maildir creation includes cur/tmp
@@ -682,7 +673,7 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream* aOutputStream,
   // path to the downloaded message
   nsCOMPtr<nsIFile> fromPath;
   folderPath->Clone(getter_AddRefs(fromPath));
-  fromPath->Append(NS_LITERAL_STRING("tmp"));
+  fromPath->Append(u"tmp"_ns);
   fromPath->AppendNative(tmpName);
 
   // Check that the message is still in tmp.
@@ -703,7 +694,7 @@ nsMsgMaildirStore::FinishNewMessage(nsIOutputStream* aOutputStream,
       return NS_OK;
 
     NS_ERROR("FinishNewMessage - oops! file does not exist!");
-    return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
+    return NS_ERROR_FILE_NOT_FOUND;
   }
 
   nsCString msgID;
@@ -783,7 +774,7 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr* aHdr,
   // path to the downloaded message
   nsCOMPtr<nsIFile> fromPath;
   folderPath->Clone(getter_AddRefs(fromPath));
-  fromPath->Append(NS_LITERAL_STRING("cur"));
+  fromPath->Append(u"cur"_ns);
   fromPath->AppendNative(fileName);
 
   // let's check if the tmp file exists
@@ -798,7 +789,7 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr* aHdr,
   nsCOMPtr<nsIFile> toPath;
   aDestFolder->GetFilePath(getter_AddRefs(folderPath));
   folderPath->Clone(getter_AddRefs(toPath));
-  toPath->Append(NS_LITERAL_STRING("cur"));
+  toPath->Append(u"cur"_ns);
 
   // let's check if the folder exists
   // XXX TODO: kill this and make sure maildir creation includes cur/tmp
@@ -874,9 +865,7 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr* aHdr,
 
     // Notify the message was moved.
     if (notifier) {
-      notifier->NotifyItemEvent(
-          folder, NS_LITERAL_CSTRING("UnincorporatedMessageMoved"), newHdr,
-          EmptyCString());
+      notifier->NotifyMsgUnincorporatedMoved(folder, newHdr);
     }
   }
 
@@ -894,15 +883,12 @@ nsMsgMaildirStore::MoveNewlyDownloadedMessage(nsIMsgDBHdr* aHdr,
 NS_IMETHODIMP
 nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder* aMsgFolder,
                                      const nsACString& aMsgToken,
-                                     int64_t* aOffset, nsIMsgDBHdr* aMsgHdr,
                                      bool* aReusable,
                                      nsIInputStream** aResult) {
   NS_ENSURE_ARG_POINTER(aMsgFolder);
-  NS_ENSURE_ARG_POINTER(aOffset);
   NS_ENSURE_ARG_POINTER(aResult);
 
   *aReusable = false;  // message per file
-  *aOffset = 0;
 
   // construct path to file
   nsCOMPtr<nsIFile> path;
@@ -915,7 +901,7 @@ nsMsgMaildirStore::GetMsgInputStream(nsIMsgFolder* aMsgFolder,
     return NS_ERROR_FAILURE;
   }
 
-  path->Append(NS_LITERAL_STRING("cur"));
+  path->Append(u"cur"_ns);
 
   // let's check if the folder exists
   // XXX TODO: kill this and make sure maildir creation includes cur/tmp
@@ -951,7 +937,7 @@ NS_IMETHODIMP nsMsgMaildirStore::DeleteMessages(
       continue;
     }
 
-    path->Append(NS_LITERAL_STRING("cur"));
+    path->Append(u"cur"_ns);
     path->AppendNative(fileName);
 
     // Let's check if the message exists.
@@ -969,22 +955,22 @@ NS_IMETHODIMP nsMsgMaildirStore::DeleteMessages(
 }
 
 NS_IMETHODIMP
-nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray* aHdrArray,
+nsMsgMaildirStore::CopyMessages(bool aIsMove,
+                                const nsTArray<RefPtr<nsIMsgDBHdr>>& aHdrArray,
                                 nsIMsgFolder* aDstFolder,
-                                nsIMsgCopyServiceListener* aListener,
-                                nsIArray** aDstHdrs,
+                                nsTArray<RefPtr<nsIMsgDBHdr>>& aDstHdrs,
                                 nsITransaction** aUndoAction, bool* aCopyDone) {
-  NS_ENSURE_ARG_POINTER(aHdrArray);
   NS_ENSURE_ARG_POINTER(aDstFolder);
   NS_ENSURE_ARG_POINTER(aCopyDone);
   NS_ENSURE_ARG_POINTER(aUndoAction);
 
   *aCopyDone = false;
-
+  if (aHdrArray.IsEmpty()) {
+    return NS_ERROR_INVALID_ARG;
+  }
   nsCOMPtr<nsIMsgFolder> srcFolder;
   nsresult rv;
-  nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(aHdrArray, 0, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsIMsgDBHdr* msgHdr = aHdrArray[0];
   rv = msgHdr->GetFolder(getter_AddRefs(srcFolder));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1017,12 +1003,12 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray* aHdrArray,
   aDstFolder->GetMsgDatabase(getter_AddRefs(destDB));
   rv = aDstFolder->GetFilePath(getter_AddRefs(destFolderPath));
   NS_ENSURE_SUCCESS(rv, rv);
-  destFolderPath->Append(NS_LITERAL_STRING("cur"));
+  destFolderPath->Append(u"cur"_ns);
 
   nsCOMPtr<nsIFile> srcFolderPath;
   rv = srcFolder->GetFilePath(getter_AddRefs(srcFolderPath));
   NS_ENSURE_SUCCESS(rv, rv);
-  srcFolderPath->Append(NS_LITERAL_STRING("cur"));
+  srcFolderPath->Append(u"cur"_ns);
 
   nsCOMPtr<nsIMsgDatabase> srcDB;
   srcFolder->GetMsgDatabase(getter_AddRefs(srcDB));
@@ -1035,21 +1021,10 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray* aHdrArray,
       msgTxn->SetTransactionType(nsIMessenger::eCopyMsg);
   }
 
-  if (aListener) aListener->OnStartCopy();
+  aDstHdrs.Clear();
+  aDstHdrs.SetCapacity(aHdrArray.Length());
 
-  nsCOMPtr<nsIMutableArray> dstHdrs(
-      do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-  uint32_t messageCount;
-  rv = aHdrArray->GetLength(&messageCount);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  for (uint32_t i = 0; i < messageCount; i++) {
-    nsCOMPtr<nsIMsgDBHdr> srcHdr = do_QueryElementAt(aHdrArray, i, &rv);
-    if (NS_FAILED(rv)) {
-      MOZ_LOG(MailDirLog, mozilla::LogLevel::Info, ("srcHdr null"));
-      continue;
-    }
+  for (auto srcHdr : aHdrArray) {
     nsMsgKey srcKey;
     srcHdr->GetMessageKey(&srcKey);
     msgTxn->AddSrcKey(srcKey);
@@ -1088,39 +1063,30 @@ nsMsgMaildirStore::CopyMessages(bool aIsMove, nsIArray* aHdrArray,
                                           getter_AddRefs(destHdr));
       NS_ENSURE_SUCCESS(rv, rv);
       destHdr->SetStringProperty("storeToken", fileName.get());
-      dstHdrs->AppendElement(destHdr);
+      aDstHdrs.AppendElement(destHdr);
       nsMsgKey dstKey;
       destHdr->GetMessageKey(&dstKey);
       msgTxn->AddDstKey(dstKey);
-      if (aListener) aListener->SetMessageKey(dstKey);
     }
   }
   nsCOMPtr<nsIMsgFolderNotificationService> notifier(
       do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID));
   if (notifier) {
-    nsTArray<RefPtr<nsIMsgDBHdr>> srcTmp;
-    nsTArray<RefPtr<nsIMsgDBHdr>> destTmp;
-    MsgHdrsToTArray(aHdrArray, srcTmp);
-    MsgHdrsToTArray(dstHdrs, destTmp);
-    notifier->NotifyMsgsMoveCopyCompleted(aIsMove, srcTmp, aDstFolder, destTmp);
+    notifier->NotifyMsgsMoveCopyCompleted(aIsMove, aHdrArray, aDstFolder,
+                                          aDstHdrs);
   }
 
   // For now, we only support local dest folders, and for those we are done and
   // can delete the messages. Perhaps this should be moved into the folder
   // when we try to support other folder types.
   if (aIsMove) {
-    for (uint32_t i = 0; i < messageCount; ++i) {
-      nsCOMPtr<nsIMsgDBHdr> msgDBHdr(do_QueryElementAt(aHdrArray, i, &rv));
-      rv = srcDB->DeleteHeader(msgDBHdr, nullptr, false, true);
+    for (auto msgDBHdr : aHdrArray) {
+      srcDB->DeleteHeader(msgDBHdr, nullptr, false, true);
     }
   }
 
   *aCopyDone = true;
-  nsCOMPtr<nsISupports> srcSupports(do_QueryInterface(srcFolder));
-  if (destLocalFolder) destLocalFolder->OnCopyCompleted(srcSupports, true);
-  if (aListener) aListener->OnStopCopy(NS_OK);
   msgTxn.forget(aUndoAction);
-  dstHdrs.forget(aDstHdrs);
   return NS_OK;
 }
 
@@ -1189,7 +1155,6 @@ nsresult MaildirStoreParser::ParseNextMessage(nsIFile* aFile) {
     aFile->GetFileSize(&fileSize);
     msgParser->SetNewMsgHdr(newMsgHdr);
     msgParser->SetState(nsIMsgParseMailMsgState::ParseHeadersState);
-    msgParser->SetEnvelopePos(0);
     bool needMoreData = false;
     char* newLine = nullptr;
     uint32_t numBytesInLine = 0;
@@ -1271,7 +1236,7 @@ NS_IMETHODIMP nsMsgMaildirStore::RebuildIndex(nsIMsgFolder* aFolder,
   nsCOMPtr<nsIFile> path;
   nsresult rv = aFolder->GetFilePath(getter_AddRefs(path));
   NS_ENSURE_SUCCESS(rv, rv);
-  path->Append(NS_LITERAL_STRING("cur"));
+  path->Append(u"cur"_ns);
 
   nsCOMPtr<nsIDirectoryEnumerator> directoryEnumerator;
   rv = path->GetDirectoryEntries(getter_AddRefs(directoryEnumerator));
@@ -1293,8 +1258,21 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeFlags(
     nsCOMPtr<nsIOutputStream> outputStream;
     nsresult rv = GetOutputStream(msgHdr, outputStream);
     NS_ENSURE_SUCCESS(rv, rv);
-    // Seek to x-mozilla-status offset and rewrite value.
-    rv = UpdateFolderFlag(msgHdr, aSet, aFlags, outputStream);
+
+    // Work out the flags we want to write.
+    uint32_t flags = 0;
+    (void)msgHdr->GetFlags(&flags);
+    flags &= ~(nsMsgMessageFlags::RuntimeOnly | nsMsgMessageFlags::Offline);
+    if (aSet) {
+      flags |= aFlags;
+    } else {
+      flags &= ~aFlags;
+    }
+
+    // Rewrite X-Mozilla-Status headers.
+    nsCOMPtr<nsISeekableStream> seekable(do_QueryInterface(outputStream, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = RewriteMsgFlags(seekable, flags);
     if (NS_FAILED(rv)) NS_WARNING("updateFolderFlag failed");
   }
   return NS_OK;
@@ -1318,7 +1296,7 @@ nsresult nsMsgMaildirStore::GetOutputStream(
 
   nsCOMPtr<nsIFile> maildirFile;
   folderPath->Clone(getter_AddRefs(maildirFile));
-  maildirFile->Append(NS_LITERAL_STRING("cur"));
+  maildirFile->Append(u"cur"_ns);
   maildirFile->AppendNative(fileName);
 
   return MsgGetFileStream(maildirFile, getter_AddRefs(aOutputStream));
@@ -1329,35 +1307,34 @@ NS_IMETHODIMP nsMsgMaildirStore::ChangeKeywords(
     bool aAdd) {
   if (aHdrArray.IsEmpty()) return NS_ERROR_INVALID_ARG;
 
-  nsCOMPtr<nsIOutputStream> outputStream;
-  nsCOMPtr<nsISeekableStream> seekableStream;
+  nsTArray<nsCString> keywordsToAdd;
+  nsTArray<nsCString> keywordsToRemove;
+  if (aAdd) {
+    ParseString(aKeywords, ' ', keywordsToAdd);
+  } else {
+    ParseString(aKeywords, ' ', keywordsToRemove);
+  }
 
-  mozilla::UniquePtr<nsLineBuffer<char>> lineBuffer(new nsLineBuffer<char>);
-
-  nsTArray<nsCString> keywordArray;
-  ParseString(aKeywords, ' ', keywordArray);
-
-  for (auto message : aHdrArray)  // for each message
-  {
-    // get output stream for header
-    nsCOMPtr<nsIOutputStream> outputStream;
-    nsresult rv = GetOutputStream(message, outputStream);
+  for (auto msgHdr : aHdrArray) {
+    // Open the message file.
+    nsCOMPtr<nsIOutputStream> output;
+    nsresult rv = GetOutputStream(msgHdr, output);
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCOMPtr<nsIInputStream> inputStream = do_QueryInterface(outputStream, &rv);
+    nsCOMPtr<nsISeekableStream> seekable(do_QueryInterface(output, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCOMPtr<nsISeekableStream> seekableStream(
-        do_QueryInterface(inputStream, &rv));
-    NS_ENSURE_SUCCESS(rv, rv);
-    uint32_t statusOffset = 0;
-    (void)message->GetStatusOffset(&statusOffset);
-    uint64_t desiredOffset = statusOffset;
 
-    ChangeKeywordsHelper(message, desiredOffset, *lineBuffer, keywordArray,
-                         aAdd, outputStream, seekableStream, inputStream);
-    if (inputStream) inputStream->Close();
-    // ### TODO - if growKeywords property is set on the message header,
-    // we need to rewrite the message file with extra room for the keywords,
-    // or schedule some sort of background task to do this.
+    bool notEnoughRoom;
+    rv = ChangeKeywordsHelper(seekable, keywordsToAdd, keywordsToRemove,
+                              notEnoughRoom);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (notEnoughRoom) {
+      // The growKeywords property indicates that the X-Mozilla-Keys header
+      // doesn't have enough space, and should be rebuilt during the next
+      // folder compaction.
+      // TODO: For maildir there is no compaction, so this'll have no effect!
+      msgHdr->SetUint32Property("growKeywords", 1);
+    }
+    output->Close();
   }
   return NS_OK;
 }
