@@ -43,8 +43,6 @@
 #include "nsIMsgCopyService.h"
 #include "nsICryptoHash.h"
 #include "nsIStringBundle.h"
-#include "nsIMutableArray.h"
-#include "nsArrayUtils.h"
 #include "nsIMsgFilterCustomAction.h"
 #include <ctype.h>
 #include "nsIMsgPluggableStore.h"
@@ -74,8 +72,6 @@ NS_IMETHODIMP nsMsgMailboxParser::OnDataAvailable(nsIRequest* request,
 }
 
 NS_IMETHODIMP nsMsgMailboxParser::OnStartRequest(nsIRequest* request) {
-  m_startTime = PR_Now();
-
   // extract the appropriate event sinks from the url and initialize them in our
   // protocol data the URL should be queried for a nsIMailboxURL. If it doesn't
   // support a mailbox URL interface then we have an error.
@@ -156,8 +152,6 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest* request,
                                                 nsresult aStatus) {
   DoneParsingFolder(aStatus);
   // what can we do? we can close the stream?
-  m_urlInProgress =
-      false;  // don't close the connection...we may be re-using it.
 
   if (m_mailDB) m_mailDB->RemoveListener(this);
   // and we want to mark ourselves for deletion or some how inform our protocol
@@ -174,8 +168,8 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest* request,
 
 NS_IMETHODIMP
 nsParseMailMessageState::OnHdrPropertyChanged(
-    nsIMsgDBHdr* aHdrToChange, bool aPreChange, uint32_t* aStatus,
-    nsIDBChangeListener* aInstigator) {
+    nsIMsgDBHdr* aHdrToChange, const nsACString& property, bool aPreChange,
+    uint32_t* aStatus, nsIDBChangeListener* aInstigator) {
   return NS_OK;
 }
 
@@ -242,20 +236,16 @@ nsParseMailMessageState::OnJunkScoreChanged(nsIDBChangeListener* instigator) {
   return NS_OK;
 }
 
-nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nullptr, false) {
-  Init();
-}
+nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer() { Init(); }
 
 nsMsgMailboxParser::nsMsgMailboxParser(nsIMsgFolder* aFolder)
-    : nsMsgLineBuffer(nullptr, false) {
+    : nsMsgLineBuffer() {
   m_folder = do_GetWeakReference(aFolder);
 }
 
 nsMsgMailboxParser::~nsMsgMailboxParser() { ReleaseFolderLock(); }
 
 nsresult nsMsgMailboxParser::Init() {
-  m_obuffer = nullptr;
-  m_obuffer_size = 0;
   m_graph_progress_total = 0;
   m_graph_progress_received = 0;
   return AcquireFolderLock();
@@ -311,8 +301,8 @@ nsresult nsMsgMailboxParser::ProcessMailboxInputStream(nsIInputStream* aIStream,
 }
 
 void nsMsgMailboxParser::DoneParsingFolder(nsresult status) {
-  /* End of file.  Flush out any partial line remaining in the buffer. */
-  FlushLastLine();
+  // End of file. Flush out any data remaining in the buffer.
+  Flush();
   PublishMsgHeader(nullptr);
 
   // only mark the db valid if we've succeeded.
@@ -328,17 +318,6 @@ void nsMsgMailboxParser::DoneParsingFolder(nsresult status) {
     if (folder) folder->RemoveBackupMsgDatabase();
     m_backupMailDB = nullptr;
   }
-
-  //  if (m_folder != nullptr)
-  //    m_folder->SummaryChanged();
-  FreeBuffers();
-}
-
-void nsMsgMailboxParser::FreeBuffers() {
-  /* We're done reading the folder - we don't need these things
-   any more. */
-  PR_FREEIF(m_obuffer);
-  m_obuffer_size = 0;
 }
 
 void nsMsgMailboxParser::UpdateDBFolderInfo() { UpdateDBFolderInfo(m_mailDB); }
@@ -355,6 +334,7 @@ int32_t nsMsgMailboxParser::PublishMsgHeader(nsIMsgWindow* msgWindow) {
     char storeToken[100];
     PR_snprintf(storeToken, sizeof(storeToken), "%lld", m_envelope_pos);
     m_newMsgHdr->SetStringProperty("storeToken", storeToken);
+    m_newMsgHdr->SetMessageOffset(m_envelope_pos);
 
     uint32_t flags;
     (void)m_newMsgHdr->GetFlags(&flags);
@@ -466,7 +446,6 @@ NS_IMPL_ISUPPORTS(nsParseMailMessageState, nsIMsgParseMailMsgState,
 nsParseMailMessageState::nsParseMailMessageState() {
   m_position = 0;
   m_new_key = nsMsgKey_None;
-  m_IgnoreXMozillaStatus = false;
   m_state = nsIMsgParseMailMsgState::ParseBodyState;
 
   // setup handling of custom db headers, headers that are added to .msf files
@@ -478,30 +457,30 @@ nsParseMailMessageState::nsParseMailMessageState() {
   m_customDBHeaderValues = nullptr;
   nsCString customDBHeaders;  // not shown in search UI
   nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
-  if (pPrefBranch) {
-    pPrefBranch->GetCharPref("mailnews.customDBHeaders", customDBHeaders);
-    ToLowerCase(customDBHeaders);
-    if (customDBHeaders.Find("content-base") == -1)
-      customDBHeaders.InsertLiteral("content-base ", 0);
-    ParseString(customDBHeaders, ' ', m_customDBHeaders);
+  if (!pPrefBranch) {
+    return;
+  }
+  pPrefBranch->GetCharPref("mailnews.customDBHeaders", customDBHeaders);
+  ToLowerCase(customDBHeaders);
+  if (customDBHeaders.Find("content-base") == -1)
+    customDBHeaders.InsertLiteral("content-base ", 0);
+  ParseString(customDBHeaders, ' ', m_customDBHeaders);
 
-    // now add customHeaders
-    nsCString customHeadersString;  // shown in search UI
-    nsTArray<nsCString> customHeadersArray;
-    pPrefBranch->GetCharPref("mailnews.customHeaders", customHeadersString);
-    ToLowerCase(customHeadersString);
-    customHeadersString.StripWhitespace();
-    ParseString(customHeadersString, ':', customHeadersArray);
-    for (uint32_t i = 0; i < customHeadersArray.Length(); i++) {
-      if (!m_customDBHeaders.Contains(customHeadersArray[i]))
-        m_customDBHeaders.AppendElement(customHeadersArray[i]);
-    }
+  // now add customHeaders
+  nsCString customHeadersString;  // shown in search UI
+  nsTArray<nsCString> customHeadersArray;
+  pPrefBranch->GetCharPref("mailnews.customHeaders", customHeadersString);
+  ToLowerCase(customHeadersString);
+  customHeadersString.StripWhitespace();
+  ParseString(customHeadersString, ':', customHeadersArray);
+  for (uint32_t i = 0; i < customHeadersArray.Length(); i++) {
+    if (!m_customDBHeaders.Contains(customHeadersArray[i]))
+      m_customDBHeaders.AppendElement(customHeadersArray[i]);
+  }
 
-    if (m_customDBHeaders.Length()) {
-      m_customDBHeaderValues =
-          new struct message_header[m_customDBHeaders.Length()];
-      if (!m_customDBHeaderValues) m_customDBHeaders.Clear();
-    }
+  if (m_customDBHeaders.Length()) {
+    m_customDBHeaderValues =
+        new struct message_header[m_customDBHeaders.Length()];
   }
   Clear();
 }
@@ -512,12 +491,6 @@ nsParseMailMessageState::~nsParseMailMessageState() {
   delete[] m_customDBHeaderValues;
 }
 
-void nsParseMailMessageState::Init(uint64_t fileposition) {
-  m_state = nsIMsgParseMailMsgState::ParseBodyState;
-  m_position = fileposition;
-  m_newMsgHdr = nullptr;
-}
-
 NS_IMETHODIMP nsParseMailMessageState::Clear() {
   m_message_id.length = 0;
   m_references.length = 0;
@@ -572,21 +545,6 @@ NS_IMETHODIMP nsParseMailMessageState::GetState(nsMailboxParseState* aState) {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsParseMailMessageState::GetEnvelopePos(uint64_t* aEnvelopePos) {
-  NS_ENSURE_ARG_POINTER(aEnvelopePos);
-
-  *aEnvelopePos = m_envelope_pos;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsParseMailMessageState::SetEnvelopePos(uint64_t aEnvelopePos) {
-  m_envelope_pos = aEnvelopePos;
-  m_position = m_envelope_pos;
-  m_headerstartpos = m_position;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsParseMailMessageState::GetNewMsgHdr(nsIMsgDBHdr** aMsgHeader) {
   NS_ENSURE_ARG_POINTER(aMsgHeader);
   NS_IF_ADDREF(*aMsgHeader = m_newMsgHdr);
@@ -744,7 +702,6 @@ bool nsParseMailMessageState::IsEnvelopeLine(const char* buf,
 // We've found the start of the next message, so finish this one off.
 NS_IMETHODIMP nsParseMailMessageState::FinishHeader() {
   if (m_newMsgHdr) {
-    m_newMsgHdr->SetMessageOffset(m_envelope_pos);
     if (m_lastLineBlank) m_body_lines--;
     m_newMsgHdr->SetMessageSize(m_position - m_envelope_pos - m_lastLineBlank);
     m_newMsgHdr->SetLineCount(m_body_lines);
@@ -862,120 +819,116 @@ nsresult nsParseMailMessageState::ParseHeaders() {
   }
   while (buf < buf_end) {
     char* colon = PL_strnchr(buf, ':', buf_end - buf);
-    char* end;
     char* value = 0;
     struct message_header* header = 0;
     struct message_header receivedBy;
 
     if (!colon) break;
 
-    end = colon;
+    nsDependentCSubstring headerStr(buf, colon);
+    ToLowerCase(headerStr);
 
-    switch (buf[0]) {
-      case 'B':
+    // Obtain firstChar in headerStr. But if headerStr is empty, just set it to
+    // the colon. This is needed because First() asserts on an empty string.
+    char firstChar = !headerStr.IsEmpty() ? headerStr.First() : *colon;
+
+    // See RFC 5322 section 3.6 for min-max number for given header.
+    // If multiple headers exist we need to make sure to use the first one.
+
+    switch (firstChar) {
       case 'b':
-        if (!PL_strncasecmp("BCC", buf, end - buf)) header = &m_bccList;
+        if (headerStr.EqualsLiteral("bcc") && !m_bccList.length)
+          header = &m_bccList;
         break;
-      case 'C':
       case 'c':
-        if (!PL_strncasecmp("CC", buf, end - buf))
+        if (headerStr.EqualsLiteral("cc"))  // XXX: RFC 5322 says it's 0 or 1.
           header = GetNextHeaderInAggregate(m_ccList);
-        else if (!PL_strncasecmp("Content-Type", buf, end - buf))
+        else if (headerStr.EqualsLiteral("content-type"))
           header = &m_content_type;
         break;
-      case 'D':
       case 'd':
-        if (!PL_strncasecmp("Date", buf, end - buf))
+        if (headerStr.EqualsLiteral("date") && !m_date.length)
           header = &m_date;
-        else if (!PL_strncasecmp("Disposition-Notification-To", buf, end - buf))
+        else if (headerStr.EqualsLiteral("disposition-notification-to"))
           header = &m_mdn_dnt;
-        else if (!PL_strncasecmp("Delivery-date", buf, end - buf))
+        else if (headerStr.EqualsLiteral("delivery-date"))
           header = &m_delivery_date;
         break;
-      case 'F':
       case 'f':
-        if (!PL_strncasecmp("From", buf, end - buf)) header = &m_from;
+        if (headerStr.EqualsLiteral("from") && !m_from.length) {
+          header = &m_from;
+        }
         break;
-      case 'I':
       case 'i':
-        if (!PL_strncasecmp("In-Reply-To", buf, end - buf))
+        if (headerStr.EqualsLiteral("in-reply-to") && !m_in_reply_to.length)
           header = &m_in_reply_to;
         break;
-      case 'M':
       case 'm':
-        if (!PL_strncasecmp("Message-ID", buf, end - buf))
+        if (headerStr.EqualsLiteral("message-id") && !m_message_id.length)
           header = &m_message_id;
         break;
-      case 'N':
       case 'n':
-        if (!PL_strncasecmp("Newsgroups", buf, end - buf))
-          header = &m_newsgroups;
+        if (headerStr.EqualsLiteral("newsgroups")) header = &m_newsgroups;
         break;
-      case 'O':
       case 'o':
-        if (!PL_strncasecmp("Original-Recipient", buf, end - buf))
+        if (headerStr.EqualsLiteral("original-recipient"))
           header = &m_mdn_original_recipient;
         break;
-      case 'R':
+      case 'p':
+        // we could very well care what the priority header was when we
+        // remember its value. If so, need to remember it here. Also,
+        // different priority headers can appear in the same message,
+        // but we only remember the last one that we see. Applies also to
+        // x-priority checked below.
+        if (headerStr.EqualsLiteral("priority")) header = &m_priority;
+        break;
       case 'r':
-        if (!PL_strncasecmp("References", buf, end - buf))
+        if (headerStr.EqualsLiteral("references") && !m_references.length)
           header = &m_references;
-        else if (!PL_strncasecmp("Return-Path", buf, end - buf))
+        else if (headerStr.EqualsLiteral("return-path"))
           header = &m_return_path;
         // treat conventional Return-Receipt-To as MDN
         // Disposition-Notification-To
-        else if (!PL_strncasecmp("Return-Receipt-To", buf, end - buf))
+        else if (headerStr.EqualsLiteral("return-receipt-to"))
           header = &m_mdn_dnt;
-        else if (!PL_strncasecmp("Reply-To", buf, end - buf))
+        else if (headerStr.EqualsLiteral("reply-to") && !m_replyTo.length)
           header = &m_replyTo;
-        else if (!PL_strncasecmp("Received", buf, end - buf)) {
+        else if (headerStr.EqualsLiteral("received")) {
           header = &receivedBy;
           header->length = 0;
         }
         break;
-      case 'S':
       case 's':
-        if (!PL_strncasecmp("Subject", buf, end - buf) && !m_subject.length)
+        if (headerStr.EqualsLiteral("subject") && !m_subject.length)
           header = &m_subject;
-        else if (!PL_strncasecmp("Sender", buf, end - buf))
+        else if (headerStr.EqualsLiteral("sender") && !m_sender.length)
           header = &m_sender;
-        else if (!PL_strncasecmp("Status", buf, end - buf))
+        else if (headerStr.EqualsLiteral("status"))
           header = &m_status;
         break;
-      case 'T':
       case 't':
-        if (!PL_strncasecmp("To", buf, end - buf))
+        if (headerStr.EqualsLiteral("to"))  // XXX: RFC 5322 says it's 0 or 1.
           header = GetNextHeaderInAggregate(m_toList);
         break;
-      case 'X':
-        if (X_MOZILLA_STATUS2_LEN == end - buf &&
-            !PL_strncasecmp(X_MOZILLA_STATUS2, buf, end - buf) &&
-            !m_IgnoreXMozillaStatus && !m_mozstatus2.length)
+      case 'x':
+        if (headerStr.EqualsIgnoreCase(X_MOZILLA_STATUS2) &&
+            !m_mozstatus2.length)
           header = &m_mozstatus2;
-        else if (X_MOZILLA_STATUS_LEN == end - buf &&
-                 !PL_strncasecmp(X_MOZILLA_STATUS, buf, end - buf) &&
-                 !m_IgnoreXMozillaStatus && !m_mozstatus.length)
+        else if (headerStr.EqualsIgnoreCase(X_MOZILLA_STATUS) &&
+                 !m_mozstatus.length)
           header = &m_mozstatus;
-        else if (!PL_strncasecmp(HEADER_X_MOZILLA_ACCOUNT_KEY, buf,
-                                 end - buf) &&
+        else if (headerStr.EqualsIgnoreCase(HEADER_X_MOZILLA_ACCOUNT_KEY) &&
                  !m_account_key.length)
           header = &m_account_key;
-        // we could very well care what the priority header was when we
-        // remember its value. If so, need to remember it here. Also,
-        // different priority headers can appear in the same message,
-        // but we only remember the last one that we see.
-        else if (!PL_strncasecmp("X-Priority", buf, end - buf) ||
-                 !PL_strncasecmp("Priority", buf, end - buf))
+        else if (headerStr.EqualsLiteral("x-priority"))  // See case 'p' above.
           header = &m_priority;
-        else if (!PL_strncasecmp(HEADER_X_MOZILLA_KEYWORDS, buf, end - buf) &&
+        else if (headerStr.EqualsIgnoreCase(HEADER_X_MOZILLA_KEYWORDS) &&
                  !m_keywords.length)
           header = &m_keywords;
         break;
     }
-    if (!header && m_customDBHeaders.Length()) {
-      nsDependentCSubstring headerStr(buf, end);
 
-      ToLowerCase(headerStr);
+    if (!header && m_customDBHeaders.Length()) {
       size_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
       if (customHeaderIndex != m_customDBHeaders.NoIndex)
         header = &m_customDBHeaderValues[customHeaderIndex];
@@ -1067,7 +1020,7 @@ nsresult nsParseMailMessageState::ParseHeaders() {
         }
         // Someone might want the received header saved.
         if (m_customDBHeaders.Length()) {
-          if (m_customDBHeaders.Contains(NS_LITERAL_CSTRING("received"))) {
+          if (m_customDBHeaders.Contains("received"_ns)) {
             if (!m_receivedValue.IsEmpty()) m_receivedValue.Append(' ');
             m_receivedValue.Append(header->value, header->length);
           }
@@ -1081,6 +1034,9 @@ nsresult nsParseMailMessageState::ParseHeaders() {
   return NS_OK;
 }
 
+// Try and glean a sender and/or timestamp from the "From " line, to use
+// as last-ditch fallbacks if the message is missing "From"/"Sender" or
+// "Date" headers.
 nsresult nsParseMailMessageState::ParseEnvelope(const char* line,
                                                 uint32_t line_size) {
   const char* end;
@@ -1169,7 +1125,6 @@ nsresult nsParseMailMessageState::FinalizeHeaders() {
   char md5_data[50];
 
   uint32_t flags = 0;
-  uint32_t deltaToMozStatus = 0;
   nsMsgPriorityValue priorityFlags = nsMsgPriority::notSet;
   uint32_t labelFlags = 0;
 
@@ -1221,10 +1176,6 @@ nsresult nsParseMailMessageState::FinalizeHeaders() {
           (nsMsgPriorityValue)((flags & nsMsgMessageFlags::Priorities) >> 13);
       flags &= ~nsMsgMessageFlags::Priorities;
     }
-
-    deltaToMozStatus =
-        m_headerstartpos + (mozstatus->value - m_headers.GetBuffer()) -
-        (X_MOZILLA_STATUS_LEN + 2 /* for ": " */) - m_envelope_pos;
   }
 
   if (mozstatus2) {
@@ -1310,23 +1261,6 @@ nsresult nsParseMailMessageState::FinalizeHeaders() {
         labelFlags = ((flags & nsMsgMessageFlags::Labels) >> 25);
         m_newMsgHdr->SetLabel(labelFlags);
       }
-      NS_ASSERTION(!mozstatus || deltaToMozStatus < 0xffff,
-                   "unexpected deltaToMozStatus");
-      if (mozstatus &&
-          deltaToMozStatus < 0xffff) { /* Only use if fits in 16 bits. */
-        m_newMsgHdr->SetStatusOffset((uint16_t)deltaToMozStatus);
-        if (!m_IgnoreXMozillaStatus) {  // imap doesn't care about
-                                        // X-MozillaStatus
-          // TODO: Clarify, why is it necessary to query the value we've just
-          // set? Does querying trigger some kind of side effect? Or is the
-          // following assertion check the only need for querying? If it seems
-          // unnecessary, the following lines should be removed.
-          uint32_t offset;
-          (void)m_newMsgHdr->GetStatusOffset(&offset);
-          NS_ASSERTION(offset < 10000,
-                       "invalid status offset"); /* ### Debugging hack */
-        }
-      }
       if (sender) m_newMsgHdr->SetAuthor(sender->value);
       if (recipient == &m_newsgroups) {
         /* In the case where the recipient is a newsgroup, truncate the string
@@ -1562,7 +1496,6 @@ nsresult nsParseMailMessageState::FinalizeHeaders() {
 nsParseNewMailState::nsParseNewMailState() : m_disableFilters(false) {
   m_ibuffer = nullptr;
   m_ibuffer_size = 0;
-  m_ibuffer_fp = 0;
   m_numNotNewMessages = 0;
 }
 
@@ -1636,11 +1569,6 @@ void nsParseNewMailState::GetMsgWindow(nsIMsgWindow** aMsgWindow) {
 // This gets called for every message because libnet calls IncorporateBegin,
 // IncorporateWrite (once or more), and IncorporateComplete for every message.
 void nsParseNewMailState::DoneParsingFolder(nsresult status) {
-  /* End of file.  Flush out any partial line remaining in the buffer. */
-  if (m_ibuffer_fp > 0) {
-    ParseFolderLine(m_ibuffer, m_ibuffer_fp);
-    m_ibuffer_fp = 0;
-  }
   PublishMsgHeader(nullptr);
   if (m_mailDB)  // finished parsing, so flush db folder info
     UpdateDBFolderInfo();
@@ -1649,8 +1577,6 @@ void nsParseNewMailState::DoneParsingFolder(nsresult status) {
  any more. */
   PR_FREEIF(m_ibuffer);
   m_ibuffer_size = 0;
-  PR_FREEIF(m_obuffer);
-  m_obuffer_size = 0;
 }
 
 void nsParseNewMailState::OnNewMessage(nsIMsgWindow* msgWindow) {}
@@ -1667,10 +1593,6 @@ int32_t nsParseNewMailState::PublishMsgHeader(nsIMsgWindow* msgWindow) {
       m_newMsgHdr->OrFlags(nsMsgMessageFlags::New, &newFlags);
 
     if (!m_disableFilters) {
-      uint64_t msgOffset;
-      (void)m_newMsgHdr->GetMessageOffset(&msgOffset);
-      m_curHdrOffset = msgOffset;
-
       nsCOMPtr<nsIMsgIncomingServer> server;
       nsresult rv = m_rootFolder->GetServer(getter_AddRefs(server));
       NS_ENSURE_SUCCESS(rv, 0);
@@ -1732,7 +1654,7 @@ int32_t nsParseNewMailState::PublishMsgHeader(nsIMsgWindow* msgWindow) {
         }
       }
 
-      ApplyFilters(&moved, msgWindow, msgOffset);
+      ApplyFilters(&moved, msgWindow);
     }
     if (!moved) {
       if (m_mailDB) {
@@ -1753,16 +1675,6 @@ int32_t nsParseNewMailState::PublishMsgHeader(nsIMsgWindow* msgWindow) {
   return 0;
 }
 
-// We've found the start of the next message, so finish this one off.
-NS_IMETHODIMP nsParseNewMailState::FinishHeader() {
-  if (m_newMsgHdr) {
-    if (m_lastLineBlank) m_body_lines--;
-    m_newMsgHdr->SetMessageSize(m_position - m_envelope_pos - m_lastLineBlank);
-    m_newMsgHdr->SetLineCount(m_body_lines);
-  }
-  return NS_OK;
-}
-
 nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder** pTrashFolder) {
   nsresult rv = NS_ERROR_UNEXPECTED;
   if (!pTrashFolder) return NS_ERROR_NULL_POINTER;
@@ -1781,10 +1693,8 @@ nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder** pTrashFolder) {
   return rv;
 }
 
-void nsParseNewMailState::ApplyFilters(bool* pMoved, nsIMsgWindow* msgWindow,
-                                       uint64_t msgOffset) {
+void nsParseNewMailState::ApplyFilters(bool* pMoved, nsIMsgWindow* msgWindow) {
   m_msgMovedByFilter = m_msgCopiedByFilter = false;
-  m_curHdrOffset = msgOffset;
 
   if (!m_disableFilters) {
     nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
@@ -1796,10 +1706,11 @@ void nsParseNewMailState::ApplyFilters(bool* pMoved, nsIMsgWindow* msgWindow,
       if (downloadFolder) downloadFolder->GetURI(m_inboxUri);
       char* headers = m_headers.GetBuffer();
       uint32_t headersSize = m_headers.GetBufferPos();
+      nsAutoCString tok;
+      msgHdr->GetStringProperty("storeToken", getter_Copies(tok));
       if (m_filterList) {
         MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
-                ("(Local) Running filters on 1 message at offset %" PRIu64,
-                 msgOffset));
+                ("(Local) Running filters on 1 message (%s)", tok.get()));
         MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
                 ("(Local) Using filters from the original account"));
         (void)m_filterList->ApplyFiltersToHdr(
@@ -1808,8 +1719,7 @@ void nsParseNewMailState::ApplyFilters(bool* pMoved, nsIMsgWindow* msgWindow,
       }
       if (!m_msgMovedByFilter && m_deferredToServerFilterList) {
         MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
-                ("(Local) Running filters on 1 message at offset %" PRIu64,
-                 msgOffset));
+                ("(Local) Running filters on 1 message (%s)", tok.get()));
         MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
                 ("(Local) Using filters from the deferred to account"));
         (void)m_deferredToServerFilterList->ApplyFiltersToHdr(
@@ -1926,8 +1836,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
             // has a pending copy action, use the imap coalescer so that
             // we won't truncate the inbox before the copy fires.
             if (m_msgCopiedByFilter ||
-                StringBeginsWith(actionTargetFolderUri,
-                                 NS_LITERAL_CSTRING("imap:"))) {
+                StringBeginsWith(actionTargetFolderUri, "imap:"_ns)) {
               if (!m_moveCoalescer)
                 m_moveCoalescer =
                     new nsImapMoveCoalescer(m_downloadFolder, m_msgWindow);
@@ -1948,15 +1857,15 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
               if (!m_msgMovedByFilter /* == NS_FAILED(err) */) {
                 // XXX: Invoke MSG_LOG_TO_CONSOLE once bug 1135265 lands.
                 if (loggingEnabled) {
-                  (void)filter->LogRuleHitFail(
-                      filterAction, msgHdr, rv,
-                      NS_LITERAL_CSTRING("filterFailureMoveFailed"));
+                  (void)filter->LogRuleHitFail(filterAction, msgHdr, rv,
+                                               "filterFailureMoveFailed"_ns);
                 }
               }
             }
           } else {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
-                    ("(Local) Target folder is the same as source folder, skipping"));
+                    ("(Local) Target folder is the same as source folder, "
+                     "skipping"));
             rv = NS_OK;
           }
           *applyMore = false;
@@ -1967,11 +1876,6 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
 
           if (!actionTargetFolderUri.IsEmpty() &&
               !actionTargetFolderUri.Equals(uri)) {
-            nsCOMPtr<nsIMutableArray> messageArray(
-                do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-            if (NS_FAILED(rv) || !messageArray) break;
-            messageArray->AppendElement(msgHdr);
-
             nsCOMPtr<nsIMsgFolder> dstFolder;
             nsCOMPtr<nsIMsgCopyService> copyService;
             rv = GetExistingFolder(actionTargetFolderUri,
@@ -1986,22 +1890,22 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
 
             copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
             if (NS_SUCCEEDED(rv))
-              rv = copyService->CopyMessages(m_downloadFolder, messageArray,
+              rv = copyService->CopyMessages(m_downloadFolder, {&*msgHdr},
                                              dstFolder, false, nullptr,
                                              msgWindow, false);
 
             if (NS_FAILED(rv)) {
               // XXX: Invoke MSG_LOG_TO_CONSOLE once bug 1135265 lands.
               if (loggingEnabled) {
-                (void)filter->LogRuleHitFail(
-                    filterAction, msgHdr, rv,
-                    NS_LITERAL_CSTRING("filterFailureCopyFailed"));
+                (void)filter->LogRuleHitFail(filterAction, msgHdr, rv,
+                                             "filterFailureCopyFailed"_ns);
               }
             } else
               m_msgCopiedByFilter = true;
           } else {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
-                    ("(Local) Target folder is the same as source folder, skipping"));
+                    ("(Local) Target folder is the same as source folder, "
+                     "skipping"));
             break;
           }
         } break;
@@ -2026,11 +1930,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
           rv = msgHdr->OrFlags(nsMsgMessageFlags::Watched, &newFlags);
           break;
         case nsMsgFilterAction::MarkFlagged: {
-          nsCOMPtr<nsIMutableArray> messageArray(
-              do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-          if (NS_FAILED(rv) || !messageArray) break;
-          messageArray->AppendElement(msgHdr);
-          rv = m_downloadFolder->MarkMessagesFlagged(messageArray, true);
+          rv = m_downloadFolder->MarkMessagesFlagged({&*msgHdr}, true);
         } break;
         case nsMsgFilterAction::ChangePriority: {
           nsMsgPriorityValue filterPriority;
@@ -2040,11 +1940,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
         case nsMsgFilterAction::AddTag: {
           nsCString keyword;
           filterAction->GetStrValue(keyword);
-          nsCOMPtr<nsIMutableArray> messageArray(
-              do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-          if (NS_FAILED(rv) || !messageArray) break;
-          messageArray->AppendElement(msgHdr);
-          rv = m_downloadFolder->AddKeywordsToMessages(messageArray, keyword);
+          rv = m_downloadFolder->AddKeywordsToMessages({&*msgHdr}, keyword);
           break;
         }
         case nsMsgFilterAction::Label: {
@@ -2138,13 +2034,8 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
           rv = filterAction->GetStrValue(value);
           if (NS_FAILED(rv)) break;
 
-          nsCOMPtr<nsIMutableArray> messageArray(
-              do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-          if (NS_FAILED(rv) || !messageArray) break;
-          messageArray->AppendElement(msgHdr);
-
-          rv = customAction->Apply(messageArray, value, nullptr,
-                                   nsMsgFilterType::InboxRule, msgWindow);
+          rv = customAction->ApplyAction({&*msgHdr}, value, nullptr,
+                                         nsMsgFilterType::InboxRule, msgWindow);
         } break;
 
         default:
@@ -2161,7 +2052,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter* filter,
                static_cast<uint32_t>(rv)));
       if (loggingEnabled) {
         (void)filter->LogRuleHitFail(filterAction, msgHdr, rv,
-                                     NS_LITERAL_CSTRING("filterFailureAction"));
+                                     "filterFailureAction"_ns);
       }
     } else {
       MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
@@ -2237,9 +2128,8 @@ nsresult nsParseNewMailState::ApplyForwardAndReplyFilter(
         nsCOMPtr<nsIMsgComposeService> compService =
             do_GetService(NS_MSGCOMPOSESERVICE_CONTRACTID);
         if (compService) {
-          rv = compService->ReplyWithTemplate(m_msgToForwardOrReply,
-                                              m_replyTemplateUri[i].get(),
-                                              msgWindow, server);
+          rv = compService->ReplyWithTemplate(
+              m_msgToForwardOrReply, m_replyTemplateUri[i], msgWindow, server);
           if (NS_FAILED(rv)) {
             NS_WARNING("ReplyWithTemplate failed");
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,
@@ -2247,11 +2137,11 @@ nsresult nsParseNewMailState::ApplyForwardAndReplyFilter(
             if (rv == NS_ERROR_ABORT) {
               (void)m_filter->LogRuleHitFail(
                   m_ruleAction, m_msgToForwardOrReply, rv,
-                  NS_LITERAL_CSTRING("filterFailureSendingReplyAborted"));
+                  "filterFailureSendingReplyAborted"_ns);
             } else {
               (void)m_filter->LogRuleHitFail(
                   m_ruleAction, m_msgToForwardOrReply, rv,
-                  NS_LITERAL_CSTRING("filterFailureSendingReplyError"));
+                  "filterFailureSendingReplyError"_ns);
             }
           }
         }
@@ -2264,10 +2154,7 @@ nsresult nsParseNewMailState::ApplyForwardAndReplyFilter(
 }
 
 void nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr* msgHdr) {
-  nsCOMPtr<nsIMutableArray> messageArray(
-      do_CreateInstance(NS_ARRAY_CONTRACTID));
-  messageArray->AppendElement(msgHdr);
-  m_downloadFolder->MarkMessagesRead(messageArray, true);
+  m_downloadFolder->MarkMessagesRead({msgHdr}, true);
 }
 
 void nsParseNewMailState::MarkFilteredMessageUnread(nsIMsgDBHdr* msgHdr) {
@@ -2279,10 +2166,7 @@ void nsParseNewMailState::MarkFilteredMessageUnread(nsIMsgDBHdr* msgHdr) {
   } else {
     msgHdr->OrFlags(nsMsgMessageFlags::New, &newFlags);
   }
-  nsCOMPtr<nsIMutableArray> messageArray(
-      do_CreateInstance(NS_ARRAY_CONTRACTID));
-  messageArray->AppendElement(msgHdr);
-  m_downloadFolder->MarkMessagesRead(messageArray, false);
+  m_downloadFolder->MarkMessagesRead({msgHdr}, false);
 }
 
 nsresult nsParseNewMailState::EndMsgDownload() {
@@ -2334,7 +2218,6 @@ nsresult nsParseNewMailState::AppendMsgFromStream(nsIInputStream* fileStream,
     m_ibuffer = (char*)PR_Malloc(m_ibuffer_size);
     NS_ASSERTION(m_ibuffer != nullptr, "couldn't get memory to move msg");
   }
-  m_ibuffer_fp = 0;
 
   while (length > 0 && m_ibuffer) {
     uint32_t nRead;
@@ -2491,9 +2374,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr* mailHdr,
     nsCOMPtr<nsIMsgFolder> folder;
     nsresult rv = mailHdr->GetFolder(getter_AddRefs(folder));
     if (NS_SUCCEEDED(rv)) {
-      notifier->NotifyItemEvent(
-          folder, NS_LITERAL_CSTRING("UnincorporatedMessageMoved"), newHdr,
-          EmptyCString());
+      notifier->NotifyMsgUnincorporatedMoved(folder, newHdr);
     } else {
       NS_WARNING("Can't get folder for message that was moved.");
     }
