@@ -5,16 +5,23 @@
  * This tests that copied multiple messages in maildir are correct.
  */
 
+/* import-globals-from ../../../test/resources/POP3pump.js */
 load("../../../resources/POP3pump.js");
-ChromeUtils.import("resource://testing-common/mailnews/PromiseTestUtils.jsm");
+const { PromiseTestUtils } = ChromeUtils.import(
+  "resource://testing-common/mailnews/PromiseTestUtils.jsm"
+);
 
-var testSubjects = ["[Bug 397009] A filter will let me tag, but not untag",
-                    "Hello, did you receive my bugmail?"];
+var testSubjects = [
+  "[Bug 397009] A filter will let me tag, but not untag",
+  "Hello, did you receive my bugmail?",
+];
 
-Services.prefs.setCharPref("mail.serverDefaultStoreContractID",
-                           "@mozilla.org/msgstore/maildirstore;1");
+Services.prefs.setCharPref(
+  "mail.serverDefaultStoreContractID",
+  "@mozilla.org/msgstore/maildirstore;1"
+);
 
-add_task(function* runPump() {
+add_task(async function runPump() {
   // Test for multiple message copy for maildir.
   let storeID = "@mozilla.org/msgstore/maildirstore;1";
   gPOP3Pump.resetPluggableStore(storeID);
@@ -24,49 +31,51 @@ add_task(function* runPump() {
   // We want to test cross-server copy, so don't defer.
   gPOP3Pump.fakeServer.deferredToAccount = "";
 
-  gPOP3Pump.files = ["../../../data/bugmail1",
-                     "../../../data/draft1"];
-  yield gPOP3Pump.run();
+  gPOP3Pump.files = ["../../../data/bugmail1", "../../../data/draft1"];
+  await gPOP3Pump.run();
 
   // get message headers for the inbox folder
-  let inbox = gPOP3Pump.fakeServer
-                       .rootMsgFolder
-                       .getFolderWithFlags(Ci.nsMsgFolderFlags.Inbox);
+  let inbox = gPOP3Pump.fakeServer.rootMsgFolder.getFolderWithFlags(
+    Ci.nsMsgFolderFlags.Inbox
+  );
   dump("inbox is at " + inbox.filePath.path + "\n");
 
   // Accumulate messages to copy.
-  let messages = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
-  let enumerator = inbox.msgDatabase.EnumerateMessages();
+  let messages = [];
   let msgCount = 0;
-  while (enumerator.hasMoreElements()) {
+  for (let hdr of inbox.msgDatabase.EnumerateMessages()) {
     msgCount++;
-    let hdr = enumerator.getNext().QueryInterface(Ci.nsIMsgDBHdr);
-    messages.appendElement(hdr);
+    messages.push(hdr);
     Assert.equal(hdr.subject, testSubjects[msgCount - 1]);
   }
   Assert.equal(messages.length, 2);
 
   // Create a test folder on the Local Folders account.
   let testFolder = localAccountUtils.rootFolder
-                                    .QueryInterface(Ci.nsIMsgLocalMailFolder)
-                                    .createLocalSubfolder("test");
+    .QueryInterface(Ci.nsIMsgLocalMailFolder)
+    .createLocalSubfolder("test");
   dump("testFolder is at " + testFolder.filePath.path + "\n");
 
   // Copy messages to that folder.
   let promiseCopyListener = new PromiseTestUtils.PromiseCopyListener();
-  MailServices.copy.CopyMessages(inbox, messages, testFolder, false,
-                                 promiseCopyListener, null, false);
-  yield promiseCopyListener.promise;
+  MailServices.copy.copyMessages(
+    inbox,
+    messages,
+    testFolder,
+    false,
+    promiseCopyListener,
+    null,
+    false
+  );
+  await promiseCopyListener.promise;
 
   // Check the destination headers.
-  messages.clear();
-  enumerator = testFolder.msgDatabase.EnumerateMessages();
+  messages = [];
   msgCount = 0;
   let subjects = [];
-  while (enumerator.hasMoreElements()) {
+  for (let hdr of testFolder.msgDatabase.EnumerateMessages()) {
     msgCount++;
-    let hdr = enumerator.getNext().QueryInterface(Ci.nsIMsgDBHdr);
-    messages.appendElement(hdr);
+    messages.push(hdr);
     dump("Subject: " + hdr.subject + "\n");
     subjects.push(hdr.subject);
   }
@@ -79,16 +88,10 @@ add_task(function* runPump() {
   }
 
   // Make sure the body matches the message.
-  enumerator = testFolder.msgDatabase.EnumerateMessages();
-  while (enumerator.hasMoreElements()) {
-    let hdr = enumerator.getNext().QueryInterface(Ci.nsIMsgDBHdr);
+  for (let hdr of testFolder.msgDatabase.EnumerateMessages()) {
     let body = mailTestUtils.loadMessageToString(testFolder, hdr);
-    Assert.ok(body.indexOf(hdr.subject) >= 0);
+    Assert.ok(body.includes(hdr.subject));
   }
 
   gPOP3Pump = null;
 });
-
-function run_test() {
-  run_next_test();
-}
