@@ -26,6 +26,7 @@
 #include "nsMsgUtils.h"
 #include "nsTextFormatter.h"
 #include "mozilla/Services.h"
+#include "mozilla/intl/AppDateTimeFormat.h"
 
 static mozilla::LazyLogModule gMimeEmitterLogModule("MIME");
 
@@ -173,13 +174,12 @@ char* nsMimeBaseEmitter::MimeGetStringByName(const char* aHeaderName) {
   nsresult res = NS_OK;
 
   if (!m_headerStringBundle) {
-    static const char propertyURL[] = MIME_HEADER_URL;
-
     nsCOMPtr<nsIStringBundleService> sBundleService =
         mozilla::services::GetStringBundleService();
     if (sBundleService) {
-      res = sBundleService->CreateBundle(propertyURL,
+      res = sBundleService->CreateBundle(MIME_HEADER_URL,
                                          getter_AddRefs(m_headerStringBundle));
+      if (NS_FAILED(res)) return nullptr;
     }
   }
 
@@ -195,9 +195,8 @@ char* nsMimeBaseEmitter::MimeGetStringByName(const char* aHeaderName) {
     // if this is used as UCS-2 (e.g. cannot do nsString(utfStr);
     //
     return ToNewUTF8String(val);
-  } else {
-    return nullptr;
   }
+  return nullptr;
 }
 
 char* nsMimeBaseEmitter::MimeGetStringByID(int32_t aID) {
@@ -220,8 +219,8 @@ char* nsMimeBaseEmitter::MimeGetStringByID(int32_t aID) {
     if (NS_FAILED(res)) return nullptr;
 
     return ToNewUTF8String(val);
-  } else
-    return nullptr;
+  }
+  return nullptr;
 }
 
 //
@@ -356,7 +355,7 @@ nsMimeBaseEmitter::UtilityWriteCRLF(const char* buf) {
 
   uint32_t written;
   Write(nsDependentCString(buf), &written);
-  Write(NS_LITERAL_CSTRING(CRLF), &written);
+  Write(nsLiteralCString(CRLF), &written);
   return NS_OK;
 }
 
@@ -520,7 +519,7 @@ nsMimeBaseEmitter::UpdateCharacterSet(const char* aCharset) {
       // have to set content-type since it could have an embedded null byte
       mChannel->SetContentType(nsDependentCString(cBegin));
       if (PL_strcasecmp(aCharset, "US-ASCII") == 0) {
-        mChannel->SetContentCharset(NS_LITERAL_CSTRING("ISO-8859-1"));
+        mChannel->SetContentCharset("ISO-8859-1"_ns);
       } else {
         mChannel->SetContentCharset(nsDependentCString(aCharset));
       }
@@ -587,11 +586,8 @@ nsresult nsMimeBaseEmitter::GenerateDateString(const char* dateString,
   /**
    * See if the user wants to have the date displayed in the senders
    * timezone (including the timezone offset).
-   * We also evaluate the pref original_date which was introduced
-   * as makeshift in bug 118899.
    */
   bool displaySenderTimezone = false;
-  bool displayOriginalDate = false;
 
   nsCOMPtr<nsIPrefService> prefs =
       do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
@@ -602,10 +598,6 @@ nsresult nsMimeBaseEmitter::GenerateDateString(const char* dateString,
   NS_ENSURE_SUCCESS(rv, rv);
 
   dateFormatPrefs->GetBoolPref("date_senders_timezone", &displaySenderTimezone);
-  dateFormatPrefs->GetBoolPref("original_date", &displayOriginalDate);
-  // migrate old pref to date_senders_timezone
-  if (displayOriginalDate && !displaySenderTimezone)
-    dateFormatPrefs->SetBoolPref("date_senders_timezone", true);
 
   PRExplodedTime explodedMsgTime;
 
@@ -637,20 +629,24 @@ nsresult nsMimeBaseEmitter::GenerateDateString(const char* dateString,
 
   // If we want short dates, check if the message is from today, and if so
   // only show the time (e.g. 3:15 pm).
-  mozilla::nsDateFormatSelector dateFormat = mozilla::kDateFormatShort;
+  nsDateFormatSelectorComm dateFormat = kDateFormatShort;
   if (!showDateForToday &&
       explodedCurrentTime.tm_year == explodedCompTime.tm_year &&
       explodedCurrentTime.tm_month == explodedCompTime.tm_month &&
       explodedCurrentTime.tm_mday == explodedCompTime.tm_mday) {
     // same day...
-    dateFormat = mozilla::kDateFormatNone;
+    dateFormat = kDateFormatNone;
   }
 
   nsAutoString formattedDateString;
 
-  rv = mozilla::DateTimeFormat::FormatPRExplodedTime(
-      dateFormat, mozilla::kTimeFormatNoSeconds, &explodedCompTime,
-      formattedDateString);
+  mozilla::intl::DateTimeFormat::StyleBag style;
+  if (dateFormat == kDateFormatShort) {
+    style.date = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Short);
+  }
+  style.time = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Short);
+  rv = mozilla::intl::AppDateTimeFormat::Format(style, &explodedCompTime,
+                                                formattedDateString);
 
   if (NS_SUCCEEDED(rv)) {
     if (displaySenderTimezone) {
@@ -678,7 +674,7 @@ char* nsMimeBaseEmitter::GetLocalizedDateString(const char* dateString) {
   nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
 
   if (prefBranch)
-    prefBranch->GetBoolPref("mailnews.display.original_date",
+    prefBranch->GetBoolPref("mailnews.display.date_senders_timezone",
                             &displayOriginalDate);
 
   if (!displayOriginalDate) {
@@ -741,7 +737,7 @@ nsresult nsMimeBaseEmitter::WriteHeaderFieldHTML(const char* field,
     mHTMLHeaders.AppendLiteral("<b>");
   else
     mHTMLHeaders.AppendLiteral(
-        "<div class=\"headerdisplayname\" style=\"display:inline;\">");
+        "<div class=\"moz-header-display-name\" style=\"display:inline;\">");
 
   // Here is where we are going to try to L10N the tagName so we will always
   // get a field name next to an emitted header value. Note: Default will always
@@ -756,8 +752,8 @@ nsresult nsMimeBaseEmitter::WriteHeaderFieldHTML(const char* field,
     mHTMLHeaders.Append(field);
   else {
     mHTMLHeaders.Append(l10nTagName);
-    PR_FREEIF(l10nTagName);
   }
+  PR_FREEIF(l10nTagName);
 
   mHTMLHeaders.AppendLiteral(": ");
   if (mFormat == nsMimeOutput::nsMimeMessageSaveAs)
@@ -777,13 +773,17 @@ nsresult nsMimeBaseEmitter::WriteHeaderFieldHTML(const char* field,
 
 nsresult nsMimeBaseEmitter::WriteHeaderFieldHTMLPrefix(const nsACString& name) {
   if (((mFormat == nsMimeOutput::nsMimeMessageSaveAs) && (mFirstHeaders)) ||
-      ((mFormat == nsMimeOutput::nsMimeMessagePrintOutput) && (mFirstHeaders)))
+      ((mFormat == nsMimeOutput::nsMimeMessagePrintOutput) &&
+       (mFirstHeaders)) ||
+      ((mFormat == nsMimeOutput::nsMimeMessageBodyDisplay) && (mFirstHeaders)))
     /* DO NOTHING */;  // rhp: Do nothing...leaving the conditional like this so
                        // its easier to see the logic of what is going on.
   else {
-    mHTMLHeaders.AppendLiteral("<br><fieldset class=\"mimeAttachmentHeader\">");
+    mHTMLHeaders.AppendLiteral(
+        "<br><fieldset class=\"moz-mime-attachment-header\">");
     if (!name.IsEmpty()) {
-      mHTMLHeaders.AppendLiteral("<legend class=\"mimeAttachmentHeaderName\">");
+      mHTMLHeaders.AppendLiteral(
+          "<legend class=\"moz-mime-attachment-header-name\">");
       nsAppendEscapedHTML(name, mHTMLHeaders);
       mHTMLHeaders.AppendLiteral("</legend>");
     }
@@ -828,7 +828,11 @@ nsMimeBaseEmitter::WriteHTMLHeaders(const nsACString& name) {
 nsresult nsMimeBaseEmitter::DumpSubjectFromDate() {
   mHTMLHeaders.AppendLiteral(
       "<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" "
-      "class=\"header-part1\">");
+      "class=\"moz-header-part1");
+  if (mDocHeader) {
+    mHTMLHeaders.AppendLiteral(" moz-main-header");
+  }
+  mHTMLHeaders.AppendLiteral("\">");
 
   // This is the envelope information
   OutputGenericHeader(HEADER_SUBJECT);
@@ -857,7 +861,11 @@ nsresult nsMimeBaseEmitter::DumpToCC() {
   if (toField || ccField || bccField || newsgroupField) {
     mHTMLHeaders.AppendLiteral(
         "<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" "
-        "class=\"header-part2\">");
+        "class=\"moz-header-part2");
+    if (mDocHeader) {
+      mHTMLHeaders.AppendLiteral(" moz-main-header");
+    }
+    mHTMLHeaders.AppendLiteral("\">");
 
     if (toField) WriteHeaderFieldHTML(HEADER_TO, toField);
     if (ccField) WriteHeaderFieldHTML(HEADER_CC, ccField);
@@ -876,7 +884,11 @@ nsresult nsMimeBaseEmitter::DumpRestOfHeaders() {
 
   mHTMLHeaders.AppendLiteral(
       "<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" "
-      "class=\"header-part3\">");
+      "class=\"moz-header-part3");
+  if (mDocHeader) {
+    mHTMLHeaders.AppendLiteral(" moz-main-header");
+  }
+  mHTMLHeaders.AppendLiteral("\">");
 
   for (size_t i = 0; i < array->Length(); i++) {
     headerInfoType* headerInfo = array->ElementAt(i);
