@@ -358,37 +358,44 @@ nsresult nsMimeHtmlDisplayEmitter::StartAttachment(const nsACString& name,
                                                    const char* url,
                                                    bool aIsExternalAttachment) {
   nsresult rv = NS_OK;
+  nsCString uriString;
+
+  nsCOMPtr<nsIMsgMessageUrl> msgurl(do_QueryInterface(mURL, &rv));
+  if (NS_SUCCEEDED(rv)) {
+    // HACK: news urls require us to use the originalSpec. Everyone
+    // else uses GetURI to get the RDF resource which describes the message.
+    nsCOMPtr<nsINntpUrl> nntpUrl(do_QueryInterface(mURL, &rv));
+    if (NS_SUCCEEDED(rv) && nntpUrl)
+      rv = msgurl->GetOriginalSpec(uriString);
+    else
+      rv = msgurl->GetUri(uriString);
+  }
+
+  // we need to convert the attachment name from UTF-8 to unicode before
+  // we emit it.  The attachment name has already been rfc2047 processed
+  // upstream of us.  (Namely, mime_decode_filename has been called, deferring
+  // to nsIMimeHeaderParam.decodeParameter.)
+  // But we'll send it through decoding ourselves as well, since we do some
+  // more adjustments, such as removing spoofy chars.
+
+  nsCString decodedName(name);
+  nsCOMPtr<nsIMimeConverter> mimeConverter =
+      do_GetService("@mozilla.org/messenger/mimeconverter;1", &rv);
+
+  if (NS_SUCCEEDED(rv)) {
+    mimeConverter->DecodeMimeHeaderToUTF8(name, nullptr, false, true, decodedName);
+  }
+
   nsCOMPtr<nsIMsgHeaderSink> headerSink;
   rv = GetHeaderSink(getter_AddRefs(headerSink));
-
   if (NS_SUCCEEDED(rv) && headerSink) {
-    nsCString uriString;
-
-    nsCOMPtr<nsIMsgMessageUrl> msgurl(do_QueryInterface(mURL, &rv));
-    if (NS_SUCCEEDED(rv)) {
-      // HACK: news urls require us to use the originalSpec. Everyone
-      // else uses GetURI to get the RDF resource which describes the message.
-      nsCOMPtr<nsINntpUrl> nntpUrl(do_QueryInterface(mURL, &rv));
-      if (NS_SUCCEEDED(rv) && nntpUrl)
-        rv = msgurl->GetOriginalSpec(uriString);
-      else
-        rv = msgurl->GetUri(uriString);
-    }
-
-    // we need to convert the attachment name from UTF-8 to unicode before
-    // we emit it.  The attachment name has already been rfc2047 processed
-    // upstream of us.  (Namely, mime_decode_filename has been called, deferring
-    // to nsIMimeHeaderParam.decodeParameter.)
-    nsString unicodeHeaderValue;
-    CopyUTF8toUTF16(name, unicodeHeaderValue);
-
     headerSink->HandleAttachment(
         contentType, nsDependentCString(url) /* was escapedUrl */,
-        unicodeHeaderValue.get(), uriString, aIsExternalAttachment);
+        NS_ConvertUTF8toUTF16(decodedName).get(), uriString, aIsExternalAttachment);
   }
 
   // List the attachments for printing.
-  rv = StartAttachmentInBody(name, contentType, url);
+  rv = StartAttachmentInBody(decodedName, contentType, url);
 
   return rv;
 }
