@@ -75,7 +75,14 @@ static int MimeInlineText_initializeCharset(MimeObject* obj) {
   // Figure out an appropriate charset for this object.
   if (!text->charset && obj->headers) {
     if (obj->options && obj->options->override_charset) {
-      text->charset = strdup(obj->options->default_charset);
+      if (obj->options->default_charset) {
+        text->charset = strdup(obj->options->default_charset);
+      } else {
+        text->charsetOverridable = true;
+        text->inputAutodetect = true;
+        text->needUpdateMsgWinCharset = true;
+        text->charset = strdup("");
+      }
     } else {
       char* ct =
           MimeHeaders_get(obj->headers, HEADER_CONTENT_TYPE, false, false);
@@ -102,35 +109,14 @@ static int MimeInlineText_initializeCharset(MimeObject* obj) {
         text->charset = strdup("UTF-8");
 
       if (!text->charset) {
-        nsresult res;
-
         text->charsetOverridable = true;
-
-        nsCOMPtr<nsIPrefBranch> prefBranch(
-            do_GetService(NS_PREFSERVICE_CONTRACTID, &res));
-        if (NS_SUCCEEDED(res)) {
-          nsCOMPtr<nsIPrefLocalizedString> str;
-          if (NS_SUCCEEDED(prefBranch->GetComplexValue(
-                  "intl.charset.detector", NS_GET_IID(nsIPrefLocalizedString),
-                  getter_AddRefs(str)))) {
-            // Only if we can get autodetector name correctly, do we set this to
-            // true.
-            text->inputAutodetect = true;
-          }
-        }
+        text->inputAutodetect = true;
+        text->needUpdateMsgWinCharset = true;
 
         if (obj->options && obj->options->default_charset)
           text->charset = strdup(obj->options->default_charset);
-        else {
-          if (NS_SUCCEEDED(res)) {
-            nsString value;
-            NS_GetLocalizedUnicharPreferenceWithDefault(
-                prefBranch, "mailnews.view_default_charset", EmptyString(),
-                value);
-            text->charset = ToNewUTF8String(value);
-          } else
-            text->charset = strdup("");
-        }
+        else
+          text->charset = strdup("UTF-8");
       }
     }
   }
