@@ -125,12 +125,23 @@ typedef enum _StatesEnum {
 
 class nsICacheEntry;
 
+/**
+ * nsNNTPProtocol handles an NNTP connection. It manages the socket
+ * communications and state machine for issuing NNTP commands and parsing the
+ * responses.
+ * For some operations it can also be used as a nsIChannel (for example, to
+ * directly feed an article out to a nsDocShell for display).
+ * It cooperates with nsNntpMockChannel. When a command is completed, it
+ * informs the nsNntpIncomingServer by calling PrepareForNextUrl(), which
+ * sets up the next queued nsNntpMockChannel.
+ */
 class nsNNTPProtocol : public nsMsgProtocol,
                        public nsINNTPProtocol,
                        public nsITimerCallback,
                        public nsICacheEntryOpenCallback,
                        public nsIMsgAsyncPromptListener,
-                       public nsIProtocolProxyCallback {
+                       public nsIProtocolProxyCallback,
+                       public nsINamed {
  public:
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSINNTPPROTOCOL
@@ -138,6 +149,7 @@ class nsNNTPProtocol : public nsMsgProtocol,
   NS_DECL_NSITIMERCALLBACK
   NS_DECL_NSIMSGASYNCPROMPTLISTENER
   NS_DECL_NSIPROTOCOLPROXYCALLBACK
+  NS_DECL_NSINAMED
 
   // Creating a protocol instance requires the URL
   // need to call Initialize after we do a new of nsNNTPProtocol
@@ -207,15 +219,17 @@ class nsNNTPProtocol : public nsMsgProtocol,
   nsCOMPtr<nsIMsgNewsFolder> m_newsFolder;
   nsCOMPtr<nsIMsgWindow> m_msgWindow;
 
+  // Pipe endpoints when nsNNTPProtocol is being used as an nsIChannel.
+  // Used to stream article data onward to the consumer (in m_channelListener).
   nsCOMPtr<nsIAsyncInputStream> mDisplayInputStream;
   nsCOMPtr<nsIAsyncOutputStream> mDisplayOutputStream;
+
   RefPtr<nsMsgLineStreamBuffer>
       m_lineStreamBuffer;  // used to efficiently extract lines from the
                            // incoming data stream
   // the nsINntpURL that is currently running
   nsCOMPtr<nsINntpUrl> m_runningURL;
   bool m_connectionBusy;
-  bool m_fromCache;  // is this connection from the cache?
   PRTime m_lastActiveTimeStamp;
   nsNewsAction m_newsAction;
   nsCOMPtr<nsISupports> m_consumer;
