@@ -21,7 +21,6 @@
 #include "nsComponentManagerUtils.h"
 #include "nsMsgUtils.h"
 #include "mozilla/Services.h"
-#include "nsArrayUtils.h"
 
 // This file contains the news article download state machine.
 
@@ -103,8 +102,9 @@ nsresult nsNewsDownloader::DownloadNext(bool firstTimeP) {
       do_GetService(NS_NNTPSERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsCOMPtr<nsIURI> uri;
   return nntpService->FetchMessage(m_folder, m_keyToDownload, m_window, nullptr,
-                                   this, nullptr);
+                                   this, getter_AddRefs(uri));
 }
 
 bool DownloadNewsArticlesToOfflineStore::GetNextHdrToRetrieve() {
@@ -119,9 +119,7 @@ bool DownloadNewsArticlesToOfflineStore::GetNextHdrToRetrieve() {
 
   while (NS_SUCCEEDED(rv = m_headerEnumerator->HasMoreElements(&hasMore)) &&
          hasMore) {
-    nsCOMPtr<nsISupports> supports;
-    rv = m_headerEnumerator->GetNext(getter_AddRefs(supports));
-    m_newsHeader = do_QueryInterface(supports);
+    rv = m_headerEnumerator->GetNext(getter_AddRefs(m_newsHeader));
     NS_ENSURE_SUCCESS(rv, false);
     uint32_t hdrFlags;
     m_newsHeader->GetFlags(&hdrFlags);
