@@ -18,8 +18,6 @@
 #include "nsIContentHandler.h"
 #include "nsICacheStorage.h"
 
-#include "nsICommandLineHandler.h"
-
 class nsIURI;
 class nsIUrlListener;
 
@@ -28,7 +26,6 @@ class nsNntpService : public nsINntpService,
                       public nsIMsgMessageFetchPartService,
                       public nsIProtocolHandler,
                       public nsIMsgProtocolInfo,
-                      public nsICommandLineHandler,
                       public nsIContentHandler {
  public:
   NS_DECL_THREADSAFE_ISUPPORTS
@@ -38,7 +35,6 @@ class nsNntpService : public nsINntpService,
   NS_DECL_NSIMSGPROTOCOLINFO
   NS_DECL_NSICONTENTHANDLER
   NS_DECL_NSIMSGMESSAGEFETCHPARTSERVICE
-  NS_DECL_NSICOMMANDLINEHANDLER
 
   // nsNntpService
   nsNntpService();
@@ -59,10 +55,11 @@ class nsNntpService : public nsINntpService,
   nsresult GetMessageFromUrl(nsIURI* aUrl, nsIMsgWindow* aMsgWindow,
                              nsISupports* aDisplayConsumer);
   // a convenience routine used to put together news urls
-  nsresult ConstructNntpUrl(const char* urlString, nsIUrlListener* aUrlListener,
+  nsresult ConstructNntpUrl(const nsACString& urlString,
+                            nsIUrlListener* aUrlListener,
                             nsIMsgWindow* aMsgWindow,
-                            const char* originalMessageUri, int32_t action,
-                            nsIURI** aUrl);
+                            const nsACString& originalMessageUri,
+                            int32_t action, nsIURI** aUrl);
   nsresult CreateNewsAccount(const char* aHostname, bool aIsSecure,
                              int32_t aPort, nsIMsgIncomingServer** aServer);
   nsresult GetServerForUri(nsIURI* aUri, nsINntpIncomingServer** aProtocol);
@@ -70,11 +67,10 @@ class nsNntpService : public nsINntpService,
   nsresult RunNewsUrl(nsIURI* aUrl, nsIMsgWindow* aMsgWindow,
                       nsISupports* aConsumer);
   // a convenience routine to go from folder uri to msg folder
-  nsresult GetFolderFromUri(const char* uri, nsIMsgFolder** folder);
-  nsresult DecomposeNewsMessageURI(const char* aMessageURI,
+  nsresult GetFolderFromUri(const nsACString&, nsIMsgFolder** folder);
+  nsresult DecomposeNewsMessageURI(const nsACString& aMessageURI,
                                    nsIMsgFolder** aFolder, nsMsgKey* aMsgKey);
 
-  bool mPrintingOperation;        // Flag for printing operations
   bool mOpenAttachmentOperation;  // Flag for opening attachments
 
   nsCOMPtr<nsICacheStorage> mCacheStorage;  // the cache storage used by news
