@@ -204,6 +204,10 @@ function resetFolder(folder) {
 }
 
 function do_check_transaction(real, expected) {
+  if (Array.isArray(real)) {
+    real = real.at(-1);
+  }
+
   // real.them may have an extra QUIT on the end, where the stream is only
   // closed after we have a chance to process it and not them. We therefore
   // excise this from the list
@@ -241,8 +245,8 @@ var articleTextListener = {
   finished: false,
 
   QueryInterface: ChromeUtils.generateQI([
-    Ci.nsIStreamListener,
-    Ci.nsIRequestObserver,
+    "nsIStreamListener",
+    "nsIRequestObserver",
   ]),
 
   // nsIRequestObserver
@@ -271,3 +275,8 @@ var articleTextListener = {
 registerCleanupFunction(function() {
   load("../../../resources/mailShutdown.js");
 });
+
+// Somehow profile-after-change is not triggered in xpcshell tests, here we
+// manually run the getService, so that correct nntp modules are loaded
+// according to the pref values.
+Cc["@mozilla.org/messenger/nntp-module-loader;1"].getService();
