@@ -1,17 +1,17 @@
 // Protocol tests for NNTP. These actually aren't too important, but their main
 // purpose is to make sure that maild is working properly and to provide
-// examples for how using maild. They also help make sure that I coded nntpd.js
+// examples for how using maild. They also help make sure that I coded Nntpd.jsm
 // right, both logically and for RFC compliance.
 // TODO:
 // * We need to hook up mochitest,
 // * TLS negotiation.
 
-// The basic daemon to use for testing nntpd.js implementations
+// The basic daemon to use for testing Nntpd.jsm implementations
 var daemon = setupNNTPDaemon();
 
 // NNTP SERVER TESTS
 // -----------------
-// Functions in order as defined in nntpd.js. Each function tests the URLs
+// Functions in order as defined in Nntpd.jsm. Each function tests the URLs
 // that are located over the implementation of nsNNTPProtocol::LoadURL and
 // added in bug 400331. Furthermore, they are tested in rough order as they
 // would be expected to be used in a session. If more URL types are modified,
@@ -57,21 +57,28 @@ function testRFC977() {
     ]);
 
     // Test - news expiration
-    test = "news:GROUP?list-ids";
-    server.resetTest();
-    setupProtocolTest(NNTP_PORT, prefix + "test.filter?list-ids");
-    server.performTest();
-    transaction = server.playTransaction();
-    do_check_transaction(transaction, ["MODE READER", "listgroup test.filter"]);
+    if (!Services.prefs.getBoolPref("mailnews.nntp.jsmodule", false)) {
+      test = "news:GROUP?list-ids";
+      server.resetTest();
+      setupProtocolTest(NNTP_PORT, prefix + "test.filter?list-ids");
+      server.performTest();
+      transaction = server.playTransaction();
+      do_check_transaction(transaction, [
+        "MODE READER",
+        "listgroup test.filter",
+      ]);
+    }
 
     // Test - posting
-    test = "news with post";
-    server.resetTest();
-    var url = create_post(prefix, "postings/post1.eml");
-    setupProtocolTest(NNTP_PORT, url);
-    server.performTest();
-    transaction = server.playTransaction();
-    do_check_transaction(transaction, ["MODE READER", "POST"]);
+    if (!Services.prefs.getBoolPref("mailnews.nntp.jsmodule", false)) {
+      test = "news with post";
+      server.resetTest();
+      var url = create_post(prefix, "postings/post1.eml");
+      setupProtocolTest(NNTP_PORT, url);
+      server.performTest();
+      transaction = server.playTransaction();
+      do_check_transaction(transaction, ["MODE READER", "POST"]);
+    }
   } catch (e) {
     dump("NNTP Protocol test " + test + " failed for type RFC 977:\n");
     try {
@@ -99,7 +106,7 @@ function testConnectionLimit() {
   var prefix = "news://localhost:" + NNTP_PORT + "/";
 
   // To test make connections limit, we run two URIs simultaneously.
-  var url = URLCreator.newURI(prefix + "*");
+  var url = Services.io.newURI(prefix + "*");
   _server.loadNewsUrl(url, null, null);
   setupProtocolTest(NNTP_PORT, prefix + "TSS1@nntp.invalid");
   server.performTest();
@@ -134,11 +141,17 @@ function testReentrantClose() {
   };
   // Nice multi-step command--we can close while executing this URL if we are
   // careful.
-  var url = URLCreator.newURI("news://localhost:" + NNTP_PORT + "/test.filter");
+  var url = Services.io.newURI(
+    "news://localhost:" + NNTP_PORT + "/test.filter"
+  );
   url.QueryInterface(Ci.nsIMsgMailNewsUrl);
   url.RegisterListener(listener);
 
-  _server.loadNewsUrl(url, null, null);
+  _server.loadNewsUrl(url, null, {
+    QueryInterface: ChromeUtils.generateQI(["nsIStreamListener"]),
+    onStartRequest() {},
+    onStopRequest() {},
+  });
   server.performTest("GROUP");
   dump("Stopping server\n");
   gThreadManager.currentThread.dispatch(
@@ -171,12 +184,8 @@ function testManyConnections() {
       }
     },
   };
-  let groups = _server.rootFolder.subFolders;
-  while (groups.hasMoreElements()) {
-    groups
-      .getNext()
-      .QueryInterface(Ci.nsIMsgFolder)
-      .getNewMessages(null, listener);
+  for (let group of _server.rootFolder.subFolders) {
+    group.getNewMessages(null, listener);
     listener.ran++;
   }
   server.performTest();
