@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -54,16 +53,12 @@ var gFilterActionStrings = [
 
 // A temporary filter with the current state of actions in the UI.
 var gTempFilter = null;
-// A nsIArray of the currently defined actions in the order they will be run.
+// nsIMsgRuleAction[] - the currently defined actions in the order they will be run.
 var gActionListOrdered = null;
 
 var gFilterEditorMsgWindow = null;
 
-var nsMsgFilterAction = Ci.nsMsgFilterAction;
-var nsMsgFilterType = Ci.nsMsgFilterType;
-var nsIMsgRuleAction = Ci.nsIMsgRuleAction;
-var nsMsgSearchScope = Ci.nsMsgSearchScope;
-
+window.addEventListener("DOMContentLoaded", filterEditorOnLoad, { once: true });
 document.addEventListener("dialogaccept", onAccept);
 
 function filterEditorOnLoad() {
@@ -137,9 +132,9 @@ function filterEditorOnLoad() {
         // for everything else, it's MoveToFolder
         var filterAction = gFilter.createAction();
         filterAction.type =
-          getScopeFromFilterList(gFilterList) == nsMsgSearchScope.newsFilter
-            ? nsMsgFilterAction.Delete
-            : nsMsgFilterAction.MoveToFolder;
+          getScopeFromFilterList(gFilterList) == Ci.nsMsgSearchScope.newsFilter
+            ? Ci.nsMsgFilterAction.Delete
+            : Ci.nsMsgFilterAction.MoveToFolder;
         gFilter.appendAction(filterAction);
         initializeDialog(gFilter);
       } else if ("copiedFilter" in args) {
@@ -158,12 +153,7 @@ function filterEditorOnLoad() {
         }
 
         // copy the search terms
-        for (let i = 0; i < copiedFilter.searchTerms.length; i++) {
-          let searchTerm = copiedFilter.searchTerms.queryElementAt(
-            i,
-            Ci.nsIMsgSearchTerm
-          );
-
+        for (let searchTerm of copiedFilter.searchTerms) {
           let newTerm = newFilter.createTerm();
           newTerm.attrib = searchTerm.attrib;
           newTerm.op = searchTerm.op;
@@ -193,7 +183,7 @@ function filterEditorOnLoad() {
   if (!gFilter) {
     // This is a new filter. Set to both Incoming and Manual contexts.
     gFilterTypeSelector.setType(
-      nsMsgFilterType.Incoming | nsMsgFilterType.Manual
+      Ci.nsMsgFilterType.Incoming | Ci.nsMsgFilterType.Manual
     );
   }
 
@@ -209,7 +199,7 @@ function filterEditorOnLoad() {
 function onEnterInSearchTerm(event) {
   if (event.ctrlKey || (Services.appinfo.OS == "Darwin" && event.metaKey)) {
     // If accel key (Ctrl on Win/Linux, Cmd on Mac) was held too, accept the dialog.
-    document.getElementById("FilterEditor").acceptDialog();
+    document.querySelector("dialog").acceptDialog();
   } else {
     // If only plain Enter was pressed, add a new rule line.
     onMore(event);
@@ -249,7 +239,7 @@ function duplicateFilterNameExists(filterName) {
 function getScopeFromFilterList(filterList) {
   if (!filterList) {
     dump("yikes, null filterList\n");
-    return nsMsgSearchScope.offlineMail;
+    return Ci.nsMsgSearchScope.offlineMail;
   }
   return filterList.folder.server.filterScope;
 }
@@ -284,37 +274,37 @@ function initializeFilterTypeSelector() {
 
     /**
      * Returns the currently set filter type (checkboxes) in terms
-     * of a Ci.nsMsgFilterType value.
+     * of a Ci.Ci.nsMsgFilterType value.
      */
     getType() {
-      let type = nsMsgFilterType.None;
+      let type = Ci.nsMsgFilterType.None;
 
       if (this.checkBoxManual.checked) {
-        type |= nsMsgFilterType.Manual;
+        type |= Ci.nsMsgFilterType.Manual;
       }
 
       if (this.checkBoxIncoming.checked) {
         if (this.menulistIncoming.selectedItem == this.menuitemAfterPlugins) {
-          type |= nsMsgFilterType.PostPlugin;
+          type |= Ci.nsMsgFilterType.PostPlugin;
         } else if (
-          getScopeFromFilterList(gFilterList) == nsMsgSearchScope.newsFilter
+          getScopeFromFilterList(gFilterList) == Ci.nsMsgSearchScope.newsFilter
         ) {
-          type |= nsMsgFilterType.NewsRule;
+          type |= Ci.nsMsgFilterType.NewsRule;
         } else {
-          type |= nsMsgFilterType.InboxRule;
+          type |= Ci.nsMsgFilterType.InboxRule;
         }
       }
 
       if (this.checkBoxArchive.checked) {
-        type |= nsMsgFilterType.Archive;
+        type |= Ci.nsMsgFilterType.Archive;
       }
 
       if (this.checkBoxOutgoing.checked) {
-        type |= nsMsgFilterType.PostOutgoing;
+        type |= Ci.nsMsgFilterType.PostOutgoing;
       }
 
       if (this.checkBoxPeriodic.checked) {
-        type |= nsMsgFilterType.Periodic;
+        type |= Ci.nsMsgFilterType.Periodic;
       }
 
       return type;
@@ -324,29 +314,29 @@ function initializeFilterTypeSelector() {
      * Sets the checkboxes to represent the filter type passed in.
      *
      * @param aType  the filter type to set in terms
-     *               of Ci.nsMsgFilterType values.
+     *               of Ci.Ci.nsMsgFilterType values.
      */
     setType(aType) {
       // If there is no type (event) requested, force "when manually run"
-      if (aType == nsMsgFilterType.None) {
-        aType = nsMsgFilterType.Manual;
+      if (aType == Ci.nsMsgFilterType.None) {
+        aType = Ci.nsMsgFilterType.Manual;
       }
 
-      this.checkBoxManual.checked = aType & nsMsgFilterType.Manual;
+      this.checkBoxManual.checked = aType & Ci.nsMsgFilterType.Manual;
 
       this.checkBoxIncoming.checked =
-        aType & (nsMsgFilterType.PostPlugin | nsMsgFilterType.Incoming);
+        aType & (Ci.nsMsgFilterType.PostPlugin | Ci.nsMsgFilterType.Incoming);
 
       this.menulistIncoming.selectedItem =
-        aType & nsMsgFilterType.PostPlugin
+        aType & Ci.nsMsgFilterType.PostPlugin
           ? this.menuitemAfterPlugins
           : this.menuitemBeforePlugins;
 
-      this.checkBoxArchive.checked = aType & nsMsgFilterType.Archive;
+      this.checkBoxArchive.checked = aType & Ci.nsMsgFilterType.Archive;
 
-      this.checkBoxOutgoing.checked = aType & nsMsgFilterType.PostOutgoing;
+      this.checkBoxOutgoing.checked = aType & Ci.nsMsgFilterType.PostOutgoing;
 
-      this.checkBoxPeriodic.checked = aType & nsMsgFilterType.Periodic;
+      this.checkBoxPeriodic.checked = aType & Ci.nsMsgFilterType.Periodic;
       const periodMinutes = gFilterList.folder.server.getIntValue(
         "periodicFilterRateMinutes"
       );
@@ -393,7 +383,7 @@ function initializeDialog(filter) {
     gFilterActionList.appendChild(newActionRow);
     newActionRow.setAttribute(
       "value",
-      filterAction.type == nsMsgFilterAction.Custom
+      filterAction.type == Ci.nsMsgFilterAction.Custom
         ? filterAction.customId
         : gFilterActionStrings[filterAction.type]
     );
@@ -424,7 +414,7 @@ function ensureActionRow() {
 // move to overlay
 function saveFilter() {
   // See if at least one filter type (activation event) is selected.
-  if (gFilterType == nsMsgFilterType.None) {
+  if (gFilterType == Ci.nsMsgFilterType.None) {
     Services.prompt.alert(
       window,
       gFilterBundle.getString("mustHaveFilterTypeTitle"),
@@ -581,7 +571,7 @@ function saveFilter() {
   }
 
   gFilter.filterType = gFilterType;
-  saveSearchTerms(gFilter.searchTerms, gFilter);
+  gFilter.searchTerms = saveSearchTerms(gFilter.searchTerms, gFilter);
 
   if (isNewFilter) {
     // new filter - insert into gFilterList
@@ -698,57 +688,14 @@ function AssignMeaningfulName() {
     stub = gFilterBundle.getString("matchAllFilterName");
   } else {
     // Assign a name based on the first search term.
-    let searchValue = termRoot.searchvalue;
-    let selIndex = searchValue.getAttribute("selectedIndex");
-    let children = searchValue.children;
-    let activeItem = children[selIndex];
-    let attribs = Ci.nsMsgSearchAttrib;
-
-    // Term, Operator and Value are the three parts of a filter match
-    // Term and Operator are easy to retrieve
     let term = termRoot.searchattribute.label;
     let operator = termRoot.searchoperator.label;
-
-    // Values are either popup menu items or edit fields.
-    // For popup menus use activeItem.label; for
-    // edit fields, activeItem.value
-    let value;
-    switch (Number(termRoot.searchattribute.value)) {
-      case attribs.Priority:
-      case attribs.MsgStatus:
-      case attribs.Keywords:
-      case attribs.HasAttachmentStatus:
-      case attribs.JunkStatus:
-      case attribs.JunkScoreOrigin:
-        if (activeItem) {
-          value = activeItem.label;
-        } else {
-          value = "";
-        }
-        break;
-
-      default:
-        try {
-          value = activeItem.value;
-        } catch (ex) {
-          // We should never get here, but for safety's sake,
-          // let's name the filter "Untitled Filter".
-          stub = gFilterBundle.getString("untitledFilterName");
-          // Do not 'Return'. Instead fall through and deal with the untitled filter below.
-        }
-        break;
-    }
-    // We are now ready to name the filter.
-    // If at this point stub is empty, we know that this is not a Match All Filter
-    // and is not an "untitledFilterName" Filter, so assign it a name using
-    // a string format from the Filter Bundle.
-    if (!stub) {
-      stub = gFilterBundle.getFormattedString("filterAutoNameStr", [
-        term,
-        operator,
-        value,
-      ]);
-    }
+    let value = termRoot.searchvalue.getReadableValue();
+    stub = gFilterBundle.getFormattedString("filterAutoNameStr", [
+      term,
+      operator,
+      value,
+    ]);
   }
 
   // Whatever name we have used, 'uniquify' it.
@@ -756,7 +703,7 @@ function AssignMeaningfulName() {
   let count = 1;
   while (duplicateFilterNameExists(tempName)) {
     count++;
-    tempName = stub + " " + count;
+    tempName = `${stub} ${count}`;
   }
   gFilter.filterName = tempName;
 }
@@ -801,8 +748,7 @@ function doHelpButton() {
 
 function getCustomActions() {
   if (!gCustomActions) {
-    gCustomActions = [];
-    gCustomActions = [...MailServices.filters.getCustomActions()];
+    gCustomActions = MailServices.filters.getCustomActions();
   }
 }
 
@@ -836,17 +782,17 @@ function setFilterScope(aFilterType, aFilterList) {
 // hierarchy of contexts, with incoming the most restrictive,
 // followed by manual and post-plugin.
 function getFilterScope(aServerFilterScope, aFilterType, aFilterList) {
-  if (aFilterType & nsMsgFilterType.Incoming) {
+  if (aFilterType & Ci.nsMsgFilterType.Incoming) {
     return aServerFilterScope;
   }
 
   // Manual or PostPlugin
   // local mail allows body and junk types
-  if (aServerFilterScope == nsMsgSearchScope.offlineMailFilter) {
-    return nsMsgSearchScope.offlineMail;
+  if (aServerFilterScope == Ci.nsMsgSearchScope.offlineMailFilter) {
+    return Ci.nsMsgSearchScope.offlineMail;
   }
   // IMAP and NEWS online don't allow body
-  return nsMsgSearchScope.onlineManual;
+  return Ci.nsMsgSearchScope.onlineManual;
 }
 
 /**
