@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -57,7 +56,6 @@ searchTermContainer.prototype = {
 
     this.booleanAnd = val.booleanAnd;
     this.matchAll = val.matchAll;
-    return val;
   },
 
   // searchscope - just forward to the searchattribute
@@ -72,7 +70,6 @@ searchTermContainer.prototype = {
     if (searchAttribute) {
       searchAttribute.searchScope = val;
     }
-    return val;
   },
 
   saveId(element, slot) {
@@ -93,21 +90,18 @@ searchTermContainer.prototype = {
   },
   set searchattribute(val) {
     this.saveId(val, "internalSearchAttributeId");
-    return val;
   },
   get searchoperator() {
     return this.getElement("internalSearchOperatorId");
   },
   set searchoperator(val) {
     this.saveId(val, "internalSearchOperatorId");
-    return val;
   },
   get searchvalue() {
     return this.getElement("internalSearchValueId");
   },
   set searchvalue(val) {
     this.saveId(val, "internalSearchValueId");
-    return val;
   },
 
   booleanNodes: null,
@@ -116,7 +110,6 @@ searchTermContainer.prototype = {
   },
   set booleanAnd(val) {
     this.internalBooleanAnd = val;
-    return val;
   },
 
   save() {
@@ -154,16 +147,16 @@ searchTermContainer.prototype = {
   },
 };
 
-var nsIMsgSearchTerm = Ci.nsIMsgSearchTerm;
-
 function initializeSearchWidgets() {
   gSearchBooleanRadiogroup = document.getElementById("booleanAndGroup");
   gSearchTermList = document.getElementById("searchTermList");
 
   // initialize some strings
-  var bundle = document.getElementById("bundle_search");
-  gMoreButtonTooltipText = bundle.getString("moreButtonTooltipText");
-  gLessButtonTooltipText = bundle.getString("lessButtonTooltipText");
+  var bundle = Services.strings.createBundle(
+    "chrome://messenger/locale/search.properties"
+  );
+  gMoreButtonTooltipText = bundle.GetStringFromName("moreButtonTooltipText");
+  gLessButtonTooltipText = bundle.GetStringFromName("lessButtonTooltipText");
 }
 
 function initializeBooleanWidgets() {
@@ -198,7 +191,7 @@ function initializeBooleanWidgets() {
 
 function initializeSearchRows(scope, searchTerms) {
   for (let i = 0; i < searchTerms.length; i++) {
-    let searchTerm = searchTerms.queryElementAt(i, nsIMsgSearchTerm);
+    let searchTerm = searchTerms[i];
     createSearchRow(i, scope, searchTerm, false);
     gTotalSearchTerms++;
   }
@@ -519,17 +512,19 @@ function removeSearchRow(index) {
   gSearchTerms.splice(index, 1);
 }
 
-// save the search terms from the UI back to the actual search terms
-// searchTerms: nsIMutableArray of terms
-// termOwner:   object which can contain and create the terms
-//              (will be unnecessary if we just make terms creatable
-//               via XPCOM)
+/**
+ * Save the search terms from the UI back to the actual search terms.
+ * @param {nsIMsgSearchTerm[]} searchTerms - Array of terms
+ * @param {Object} termOwner - Object which can contain and create the terms
+ *   e.g. a nsIMsgSearchSession (will be unnecessary if we just make terms
+ *   creatable via XPCOM).
+ * @returns {nsIMsgSearchTerm[]} The filtered searchTerms.
+ */
 function saveSearchTerms(searchTerms, termOwner) {
   var matchAll = gSearchBooleanRadiogroup.value == "matchAll";
   var i;
-  for (i = 0; i < gSearchRemovedTerms.length; i++) {
-    searchTerms.removeElementAt(searchTerms.indexOf(0, gSearchRemovedTerms[i]));
-  }
+
+  searchTerms = searchTerms.filter(t => !gSearchRemovedTerms.includes(t));
 
   for (i = 0; i < gSearchTerms.length; i++) {
     try {
@@ -548,11 +543,12 @@ function saveSearchTerms(searchTerms, termOwner) {
         // but we need to make the array longer anyway
         termOwner.appendTerm(searchTerm);
       }
-      searchTerms.replaceElementAt(searchTerm, i);
+      searchTerms[i] = searchTerm;
     } catch (ex) {
       dump("** Error saving element " + i + ": " + ex + "\n");
     }
   }
+  return searchTerms;
 }
 
 function onReset(event) {
