@@ -26,6 +26,7 @@
 #include "prmem.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Logging.h"
+#include "mozilla/intl/AppDateTimeFormat.h"
 #include <ctype.h>
 
 // Marker for EOF or failure during read
@@ -64,7 +65,19 @@ NS_IMETHODIMP nsMsgFilterList::CreateFilter(const nsAString& name,
   return NS_OK;
 }
 
-NS_IMPL_GETSET(nsMsgFilterList, LoggingEnabled, bool, m_loggingEnabled)
+NS_IMETHODIMP nsMsgFilterList::SetLoggingEnabled(bool enabled) {
+  if (!enabled) {
+    // Disabling logging has side effect of closing logfile (if open).
+    SetLogStream(nullptr);
+  }
+  m_loggingEnabled = enabled;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgFilterList::GetLoggingEnabled(bool* enabled) {
+  *enabled = m_loggingEnabled;
+  return NS_OK;
+}
 
 NS_IMETHODIMP nsMsgFilterList::GetListId(nsACString& aListId) {
   aListId.Assign(m_listId);
@@ -124,34 +137,20 @@ nsresult nsMsgFilterList::EnsureLogFile(nsIFile* file) {
   return NS_OK;
 }
 
-nsresult nsMsgFilterList::TruncateLog() {
-  // This will flush and close the stream.
-  nsresult rv = SetLogStream(nullptr);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIFile> file;
-  rv = GetLogFile(getter_AddRefs(file));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  file->Remove(false);
-
-  return EnsureLogFile(file);
-}
-
 NS_IMETHODIMP nsMsgFilterList::ClearLog() {
   bool loggingEnabled = m_loggingEnabled;
 
-  // disable logging while clearing
-  m_loggingEnabled = false;
+  // disable logging while clearing (and close logStream if open).
+  SetLoggingEnabled(false);
 
-#ifdef DEBUG
-  nsresult rv =
-#endif
-      TruncateLog();
-  NS_ASSERTION(NS_SUCCEEDED(rv), "failed to truncate filter log");
-
-  m_loggingEnabled = loggingEnabled;
+  nsCOMPtr<nsIFile> file;
+  if (NS_SUCCEEDED(GetLogFile(getter_AddRefs(file)))) {
+    file->Remove(false);
+    // Recreate the file, with just the html header.
+    EnsureLogFile(file);
+  }
 
+  SetLoggingEnabled(loggingEnabled);
   return NS_OK;
 }
 
@@ -161,12 +160,8 @@ nsresult nsMsgFilterList::GetLogFile(nsIFile** aFile) {
   // XXX todo
   // the path to the log file won't change
   // should we cache it?
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetFolder(getter_AddRefs(folder));
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsCOMPtr<nsIMsgIncomingServer> server;
-  rv = folder->GetServer(getter_AddRefs(server));
+  nsresult rv = m_folder->GetServer(getter_AddRefs(server));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCString type;
@@ -174,7 +169,7 @@ nsresult nsMsgFilterList::GetLogFile(nsIFile** aFile) {
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool isServer = false;
-  rv = folder->GetIsServer(&isServer);
+  rv = m_folder->GetIsServer(&isServer);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // for news folders (not servers), the filter file is
@@ -211,10 +206,10 @@ nsresult nsMsgFilterList::GetLogFile(nsIFile** aFile) {
     rv = server->GetLocalPath(aFile);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = (*aFile)->AppendNative(NS_LITERAL_CSTRING("filterlog.html"));
+    rv = (*aFile)->AppendNative("filterlog.html"_ns);
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  return EnsureLogFile(*aFile);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -233,9 +228,7 @@ NS_IMETHODIMP
 nsMsgFilterList::SetLogStream(nsIOutputStream* aLogStream) {
   // if there is a log stream already, close it
   if (m_logStream) {
-    // will flush
-    nsresult rv = m_logStream->Close();
-    NS_ENSURE_SUCCESS(rv, rv);
+    m_logStream->Close();  // will flush
   }
 
   m_logStream = aLogStream;
@@ -246,23 +239,26 @@ NS_IMETHODIMP
 nsMsgFilterList::GetLogStream(nsIOutputStream** aLogStream) {
   NS_ENSURE_ARG_POINTER(aLogStream);
 
-  nsresult rv;
-
-  if (!m_logStream) {
+  if (!m_logStream && m_loggingEnabled) {
     nsCOMPtr<nsIFile> logFile;
-    rv = GetLogFile(getter_AddRefs(logFile));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    // append to the end of the log file
-    rv = MsgNewBufferedFileOutputStream(getter_AddRefs(m_logStream), logFile,
-                                        PR_CREATE_FILE | PR_WRONLY | PR_APPEND,
-                                        0666);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    if (!m_logStream) return NS_ERROR_FAILURE;
+    nsresult rv = GetLogFile(getter_AddRefs(logFile));
+    if (NS_SUCCEEDED(rv)) {
+      // Make sure it exists and has it's initial header.
+      rv = EnsureLogFile(logFile);
+      if (NS_SUCCEEDED(rv)) {
+        // append to the end of the log file
+        rv = MsgNewBufferedFileOutputStream(
+            getter_AddRefs(m_logStream), logFile,
+            PR_CREATE_FILE | PR_WRONLY | PR_APPEND, 0666);
+      }
+    }
+    if (NS_FAILED(rv)) {
+      m_logStream = nullptr;
+    }
   }
 
-  NS_ADDREF(*aLogStream = m_logStream);
+  // Always returns NS_OK. The stream can be null.
+  NS_IF_ADDREF(*aLogStream = m_logStream);
   return NS_OK;
 }
 
@@ -354,8 +350,7 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
           if (NS_FAILED(rv)) {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,
                     ("(Auto) Applying filter actions failed"));
-            LogFilterMessage(
-                NS_LITERAL_STRING("Applying filter actions failed"), filter);
+            LogFilterMessage(u"Applying filter actions failed"_ns, filter);
           } else {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
                     ("(Auto) Applying filter actions succeeded"));
@@ -370,8 +365,7 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
           if (NS_FAILED(matchTermStatus)) {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,
                     ("(Auto) Filter evaluation failed"));
-            LogFilterMessage(NS_LITERAL_STRING("Filter evaluation failed"),
-                             filter);
+            LogFilterMessage(u"Filter evaluation failed"_ns, filter);
           }
           if (!result)
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
@@ -390,7 +384,7 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
     MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,
             ("(Auto) Filter run failed (%" PRIx32 ")", static_cast<uint32_t>(rv)));
     // clang-format on
-    LogFilterMessage(NS_LITERAL_STRING("Filter run failed"), nullptr);
+    LogFilterMessage(u"Filter run failed"_ns, nullptr);
   }
   return rv;
 }
@@ -446,16 +440,15 @@ static const unsigned int sNumFilterFileAttribTable =
 int nsMsgFilterList::ReadChar(nsIInputStream* aStream) {
   char newChar;
   uint32_t bytesRead;
-  nsresult rv = aStream->Read(&newChar, 1, &bytesRead);
-  if (NS_FAILED(rv) || !bytesRead) return EOF_CHAR;
   uint64_t bytesAvailable;
-  rv = aStream->Available(&bytesAvailable);
-  if (NS_FAILED(rv))
-    return EOF_CHAR;
-  else {
-    if (m_startWritingToBuffer) m_unparsedFilterBuffer.Append(newChar);
-    return (unsigned char)newChar;  // Make sure the char is unsigned.
-  }
+  nsresult rv = aStream->Available(&bytesAvailable);
+  if (NS_FAILED(rv) || bytesAvailable == 0) return EOF_CHAR;
+
+  rv = aStream->Read(&newChar, 1, &bytesRead);
+  if (NS_FAILED(rv) || !bytesRead) return EOF_CHAR;
+
+  if (m_startWritingToBuffer) m_unparsedFilterBuffer.Append(newChar);
+  return (unsigned char)newChar;  // Make sure the char is unsigned.
 }
 
 int nsMsgFilterList::SkipWhitespace(nsIInputStream* aStream) {
@@ -584,9 +577,9 @@ nsresult nsMsgFilterList::LoadTextFilters(
         break;
       case nsIMsgFilterList::attribLogging:
         m_loggingEnabled = StrToBool(value);
-        m_unparsedFilterBuffer
-            .Truncate();  // we are going to buffer each filter as we read them,
-                          // make sure no garbage is there
+        // We are going to buffer each filter as we read them.
+        // Make sure no garbage is there
+        m_unparsedFilterBuffer.Truncate();
         m_startWritingToBuffer = true;  // filters begin now
         break;
       case nsIMsgFilterList::attribName:  // every filter starts w/ a name
@@ -746,13 +739,10 @@ NS_IMETHODIMP nsMsgFilterList::ParseCondition(nsIMsgFilter* aFilter,
   nsresult err = NS_OK;
   const char* curPtr = aCondition;
   if (!strcmp(aCondition, "ALL")) {
-    nsMsgSearchTerm* newTerm = new nsMsgSearchTerm;
-
-    if (newTerm) {
-      newTerm->m_matchAll = true;
-      aFilter->AppendTerm(newTerm);
-    }
-    return (newTerm) ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    RefPtr<nsMsgSearchTerm> newTerm = new nsMsgSearchTerm;
+    newTerm->m_matchAll = true;
+    aFilter->AppendTerm(newTerm);
+    return NS_OK;
   }
 
   while (!done) {
@@ -793,21 +783,18 @@ NS_IMETHODIMP nsMsgFilterList::ParseCondition(nsIMsgFilter* aFilter,
     } else
       break;
     if (termDup) {
-      nsMsgSearchTerm* newTerm = new nsMsgSearchTerm;
-
-      if (newTerm) {
-        /* Invert nsMsgSearchTerm::EscapeQuotesInStr() */
-        for (char *to = termDup, *from = termDup;;) {
-          if (*from == '\\' && from[1] == '"') from++;
-          if (!(*to++ = *from++)) break;
-        }
-        newTerm->m_booleanOp = (ANDTerm) ? nsMsgSearchBooleanOp::BooleanAND
-                                         : nsMsgSearchBooleanOp::BooleanOR;
-
-        err = newTerm->DeStreamNew(termDup, PL_strlen(termDup));
-        NS_ENSURE_SUCCESS(err, err);
-        aFilter->AppendTerm(newTerm);
+      RefPtr<nsMsgSearchTerm> newTerm = new nsMsgSearchTerm;
+      // Invert nsMsgSearchTerm::EscapeQuotesInStr()
+      for (char *to = termDup, *from = termDup;;) {
+        if (*from == '\\' && from[1] == '"') from++;
+        if (!(*to++ = *from++)) break;
       }
+      newTerm->m_booleanOp = (ANDTerm) ? nsMsgSearchBooleanOp::BooleanAND
+                                       : nsMsgSearchBooleanOp::BooleanOR;
+
+      err = newTerm->DeStreamNew(termDup, PL_strlen(termDup));
+      NS_ENSURE_SUCCESS(err, err);
+      aFilter->AppendTerm(newTerm);
       PR_FREEIF(termDup);
     } else
       break;
@@ -1108,11 +1095,9 @@ nsresult nsMsgFilterList::ComputeArbitraryHeaders() {
     rv = GetFilterAt(index, getter_AddRefs(filter));
     if (!(NS_SUCCEEDED(rv) && filter)) continue;
 
-    nsCOMPtr<nsIMutableArray> searchTerms;
-    uint32_t numSearchTerms = 0;
-    filter->GetSearchTerms(getter_AddRefs(searchTerms));
-    if (searchTerms) searchTerms->GetLength(&numSearchTerms);
-    for (uint32_t i = 0; i < numSearchTerms; i++) {
+    nsTArray<RefPtr<nsIMsgSearchTerm>> searchTerms;
+    filter->GetSearchTerms(searchTerms);
+    for (uint32_t i = 0; i < searchTerms.Length(); i++) {
       filter->GetTerm(i, &attrib, nullptr, nullptr, nullptr, arbitraryHeader);
       if (!arbitraryHeader.IsEmpty()) {
         if (m_arbitraryHeaders.IsEmpty())
@@ -1137,19 +1122,11 @@ NS_IMETHODIMP nsMsgFilterList::GetArbitraryHeaders(nsACString& aResult) {
 
 NS_IMETHODIMP nsMsgFilterList::FlushLogIfNecessary() {
   // only flush the log if we are logging
-  bool loggingEnabled = false;
-  nsresult rv = GetLoggingEnabled(&loggingEnabled);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (loggingEnabled) {
-    nsCOMPtr<nsIOutputStream> logStream;
-    rv = GetLogStream(getter_AddRefs(logStream));
-    if (NS_SUCCEEDED(rv) && logStream) {
-      rv = logStream->Flush();
-      NS_ENSURE_SUCCESS(rv, rv);
-    }
+  if (m_loggingEnabled && m_logStream) {
+    nsresult rv = m_logStream->Flush();
+    NS_ENSURE_SUCCESS(rv, rv);
   }
-  return rv;
+  return NS_OK;
 }
 
 #define LOG_ENTRY_START_TAG "<p>\n"
@@ -1159,16 +1136,24 @@ NS_IMETHODIMP nsMsgFilterList::FlushLogIfNecessary() {
 
 NS_IMETHODIMP nsMsgFilterList::LogFilterMessage(const nsAString& message,
                                                 nsIMsgFilter* filter) {
+  if (!m_loggingEnabled) {
+    return NS_OK;
+  }
   nsCOMPtr<nsIOutputStream> logStream;
-  nsresult rv = GetLogStream(getter_AddRefs(logStream));
-  NS_ENSURE_SUCCESS(rv, rv);
+  GetLogStream(getter_AddRefs(logStream));
+  if (!logStream) {
+    // Logging is on, but we failed to access the filter logfile.
+    // For completeness, we'll return an error, but we don't expect anyone
+    // to ever check it - logging failures shouldn't stop anything else.
+    return NS_ERROR_FAILURE;
+  }
 
   nsCOMPtr<nsIStringBundleService> bundleService =
       mozilla::services::GetStringBundleService();
   NS_ENSURE_TRUE(bundleService, NS_ERROR_UNEXPECTED);
 
   nsCOMPtr<nsIStringBundle> bundle;
-  rv = bundleService->CreateBundle(
+  nsresult rv = bundleService->CreateBundle(
       "chrome://messenger/locale/filter.properties", getter_AddRefs(bundle));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1190,9 +1175,10 @@ NS_IMETHODIMP nsMsgFilterList::LogFilterMessage(const nsAString& message,
   PRExplodedTime exploded;
   nsString dateValue;
   PR_ExplodeTime(PR_Now(), PR_LocalTimeParameters, &exploded);
-  mozilla::DateTimeFormat::FormatPRExplodedTime(mozilla::kDateFormatShort,
-                                                mozilla::kTimeFormatSeconds,
-                                                &exploded, dateValue);
+  mozilla::intl::DateTimeFormat::StyleBag style;
+  style.date = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Short);
+  style.time = mozilla::Some(mozilla::intl::DateTimeFormat::Style::Long);
+  mozilla::intl::AppDateTimeFormat::Format(style, &exploded, dateValue);
 
   // HTML-escape the log for security reasons.
   // We don't want someone to send us a message with a subject with
