@@ -594,7 +594,7 @@ def target_tasks_promote_desktop(full_task_graph, parameters, graph_config):
 
     def filter(task):
         # Bug 1758507 - geckoview ships in the promote phase
-        if "mozilla-esr" not in parameters["project"] and is_geckoview(
+        if not parameters["release_type"].startswith("esr") and is_geckoview(
             task, parameters
         ):
             return True
@@ -993,13 +993,6 @@ def target_tasks_build_linux64_clang_trunk_perf(
     return [l for l, t in full_task_graph.tasks.items() if filter(t.label)]
 
 
-# Run Updatebot's cron job 4 times daily.
-@_target_task("updatebot_cron")
-def target_tasks_updatebot_cron(full_task_graph, parameters, graph_config):
-    """Select tasks required to run Updatebot's cron job"""
-    return ["updatebot-cron"]
-
-
 @_target_task("customv8_update")
 def target_tasks_customv8_update(full_task_graph, parameters, graph_config):
     """Select tasks required for building latest d8/v8 version."""
