@@ -2,8 +2,17 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/
 
+# ALL CHANGES TO THIS FILE MUST HAVE REVIEW FROM A MARIONETTE PEER!
+#
+# The Marionette Python client is used out-of-tree with various builds of
+# Firefox. Removing a preference from this file will cause regressions,
+# so please be careful and get review from a Testing :: Marionette peer
+# before you make any changes to this file.
+
 from __future__ import absolute_import
 
+import codecs
+import json
 import os
 import sys
 import tempfile
@@ -14,9 +23,9 @@ from copy import deepcopy
 
 import mozversion
 
-from mozdevice import DMError
 from mozprofile import Profile
 from mozrunner import Runner, FennecEmulatorRunner
+import six
 from six import reraise
 
 from . import errors
@@ -24,6 +33,8 @@ from . import errors
 
 class GeckoInstance(object):
     required_prefs = {
+        # Make sure Shield doesn't hit the network.
+        "app.normandy.api_url": "",
         # Increase the APZ content response timeout in tests to 1 minute.
         # This is to accommodate the fact that test environments tends to be slower
         # than production environments (with the b2g emulator being the slowest of them
@@ -31,19 +42,37 @@ class GeckoInstance(object):
         # and causing false-positive test failures. See bug 1176798, bug 1177018,
         # bug 1210465.
         "apz.content_response_timeout": 60000,
-
-        # Do not send Firefox health reports to the production server
+        # Disable geolocation ping (#1)
+        "browser.region.network.url": "",
+        # Don't pull Top Sites content from the network
+        "browser.topsites.contile.enabled": False,
+        # Disable UI tour
+        "browser.uitour.pinnedTabUrl": "http://%(server)s/uitour-dummy/pinnedTab",
+        "browser.uitour.url": "http://%(server)s/uitour-dummy/tour",
+        # Disable captive portal
+        "captivedetect.canonicalURL": "",
+        # Defensively disable data reporting systems
         "datareporting.healthreport.documentServerURI": "http://%(server)s/dummy/healthreport/",
-
-        # Do not show datareporting policy notifications which can interfer with tests
+        "datareporting.healthreport.logging.consoleEnabled": False,
+        "datareporting.healthreport.service.enabled": False,
+        "datareporting.healthreport.service.firstRun": False,
+        "datareporting.healthreport.uploadEnabled": False,
+        # Do not show datareporting policy notifications which can interfere with tests
+        "datareporting.policy.dataSubmissionEnabled": False,
         "datareporting.policy.dataSubmissionPolicyBypassNotification": True,
-
+        # Automatically unload beforeunload alerts
+        "dom.disable_beforeunload": True,
+        # Enabling the support for File object creation in the content process.
+        "dom.file.createInChild": True,
+        # Disable the ProcessHangMonitor
         "dom.ipc.reportProcessHangs": False,
-
         # No slow script dialogs
         "dom.max_chrome_script_run_time": 0,
         "dom.max_script_run_time": 0,
-
+        # DOM Push
+        "dom.push.connection.enabled": False,
+        # Disable dialog abuse if alerts are triggered too quickly
+        "dom.successive_dialog_time_limit": 0,
         # Only load extensions from the application and user profile
         # AddonManager.SCOPE_PROFILE + AddonManager.SCOPE_APPLICATION
         "extensions.autoDisableScopes": 0,
@@ -52,75 +81,91 @@ class GeckoInstance(object):
         "extensions.getAddons.cache.enabled": False,
         # Disable intalling any distribution add-ons
         "extensions.installDistroAddons": False,
-        # Make sure Shield doesn't hit the network.
-        "app.normandy.api_url": "",
-        "extensions.showMismatchUI": False,
         # Turn off extension updates so they don't bother tests
         "extensions.update.enabled": False,
         "extensions.update.notifyUser": False,
+        # Redirect various extension update URLs
+        "extensions.blocklist.detailsURL": (
+            "http://%(server)s/extensions-dummy/blocklistDetailsURL"
+        ),
+        "extensions.blocklist.itemURL": "http://%(server)s/extensions-dummy/blocklistItemURL",
+        "extensions.hotfix.url": "http://%(server)s/extensions-dummy/hotfixURL",
+        "extensions.systemAddon.update.url": "http://%(server)s/dummy-system-addons.xml",
+        "extensions.update.background.url": (
+            "http://%(server)s/extensions-dummy/updateBackgroundURL"
+        ),
+        "extensions.update.url": "http://%(server)s/extensions-dummy/updateURL",
         # Make sure opening about:addons won"t hit the network
-        "extensions.webservice.discoverURL": "http://%(server)s/dummy/discoveryURL",
-
+        "extensions.getAddons.discovery.api_url": "data:, ",
+        "extensions.getAddons.get.url": "http://%(server)s/extensions-dummy/repositoryGetURL",
+        "extensions.getAddons.search.browseURL": (
+            "http://%(server)s/extensions-dummy/repositoryBrowseURL"
+        ),
         # Allow the application to have focus even it runs in the background
         "focusmanager.testmode": True,
-
         # Disable useragent updates
         "general.useragent.updates.enabled": False,
-
+        # Disable geolocation ping (#2)
+        "geo.provider.network.url": "",
         # Always use network provider for geolocation tests
         # so we bypass the OSX dialog raised by the corelocation provider
         "geo.provider.testing": True,
         # Do not scan Wifi
         "geo.wifi.scan": False,
-
-        # No hang monitor
-        "hangmonitor.timeout": 0,
-
-        "javascript.options.showInConsole": True,
-
-        # Enable Marionette component
-        # (deprecated and can be removed when Firefox 60 ships)
-        "marionette.enabled": True,
-        "marionette.defaultPrefs.enabled": True,
-
-        # Disable recommended automation prefs in CI
-        "marionette.prefs.recommended": False,
-
+        # Ensure webrender is on, no need for environment variables
+        "gfx.webrender.all": True,
+        # Disable idle-daily notifications to avoid expensive operations
+        # that may cause unexpected test timeouts.
+        "idle.lastDailyNotification": -1,
+        # Disable Firefox accounts ping
+        "identity.fxaccounts.auth.uri": "https://{server}/dummy/fxa",
+        # Disable download and usage of OpenH264, and Widevine plugins
+        "media.gmp-manager.updateEnabled": False,
+        # Disable the GFX sanity window
+        "media.sanity-test.disabled": True,
         "media.volume_scale": "0.01",
-
+        # Disable connectivity service pings
+        "network.connectivity-service.enabled": False,
         # Do not prompt for temporary redirects
         "network.http.prompt-temp-redirect": False,
-        # Disable speculative connections so they aren"t reported as leaking when they"re
-        # hanging around
-        "network.http.speculative-parallel-limit": 0,
         # Do not automatically switch between offline and online
         "network.manage-offline-status": False,
         # Make sure SNTP requests don't hit the network
         "network.sntp.pools": "%(server)s",
-
+        # Privacy and Tracking Protection
+        "privacy.trackingprotection.enabled": False,
+        # Disable recommended automation prefs in CI
+        "remote.prefs.recommended": False,
+        # Don't do network connections for mitm priming
+        "security.certerrors.mitm.priming.enabled": False,
         # Tests don't wait for the notification button security delay
         "security.notification_enable_delay": 0,
-
         # Ensure blocklist updates don't hit the network
         "services.settings.server": "http://%(server)s/dummy/blocklist/",
-
         # Disable password capture, so that tests that include forms aren"t
         # influenced by the presence of the persistent doorhanger notification
         "signon.rememberSignons": False,
-
         # Prevent starting into safe mode after application crashes
         "toolkit.startup.max_resumed_crashes": -1,
-
-        # We want to collect telemetry, but we don't want to send in the results
-        "toolkit.telemetry.server": "https://%(server)s/dummy/telemetry/",
-
-        # Enabling the support for File object creation in the content process.
-        "dom.file.createInChild": True,
+        # Disable most telemetry pings
+        "toolkit.telemetry.server": "https://%(server)s/telemetry-dummy/",
     }
 
-    def __init__(self, host=None, port=None, bin=None, profile=None, addons=None,
-                 app_args=None, symbols_path=None, gecko_log=None, prefs=None,
-                 workspace=None, verbose=0, headless=False):
+    def __init__(
+        self,
+        host=None,
+        port=None,
+        bin=None,
+        profile=None,
+        addons=None,
+        app_args=None,
+        symbols_path=None,
+        gecko_log=None,
+        prefs=None,
+        workspace=None,
+        verbose=0,
+        headless=False,
+    ):
         self.runner_class = Runner
         self.app_args = app_args or []
         self.runner = None
@@ -180,14 +225,16 @@ class GeckoInstance(object):
         self._update_profile(value)
 
     def _update_profile(self, profile=None, profile_name=None):
-        """Check if the profile has to be created, or replaced
+        """Check if the profile has to be created, or replaced.
 
         :param profile: A Profile instance to be used.
         :param name: Profile name to be used in the path.
         """
         if self.runner and self.runner.is_running():
-            raise errors.MarionetteException("The current profile can only be updated "
-                                             "when the instance is not running")
+            raise errors.MarionetteException(
+                "The current profile can only be updated "
+                "when the instance is not running"
+            )
 
         if isinstance(profile, Profile):
             # Only replace the profile if it is not the current one
@@ -199,11 +246,14 @@ class GeckoInstance(object):
             profile_path = profile
 
             # If a path to a profile is given then clone it
-            if isinstance(profile_path, basestring):
+            if isinstance(profile_path, six.string_types):
                 profile_args["path_from"] = profile_path
                 profile_args["path_to"] = tempfile.mkdtemp(
-                    suffix=u".{}".format(profile_name or os.path.basename(profile_path)),
-                    dir=self.workspace)
+                    suffix=u".{}".format(
+                        profile_name or os.path.basename(profile_path)
+                    ),
+                    dir=self.workspace,
+                )
                 # The target must not exist yet
                 os.rmdir(profile_args["path_to"])
 
@@ -213,7 +263,8 @@ class GeckoInstance(object):
             else:
                 profile_args["profile"] = tempfile.mkdtemp(
                     suffix=u".{}".format(profile_name or "mozrunner"),
-                    dir=self.workspace)
+                    dir=self.workspace,
+                )
                 profile = Profile(**profile_args)
                 profile.create_new = True
 
@@ -250,18 +301,19 @@ class GeckoInstance(object):
             args["preferences"].update(self.prefs)
 
         if self.verbose:
-            level = "TRACE" if self.verbose >= 2 else "DEBUG"
+            level = "Trace" if self.verbose >= 2 else "Debug"
             args["preferences"]["marionette.log.level"] = level
             args["preferences"]["marionette.logging"] = level
 
         if "-jsdebugger" in self.app_args:
-            args["preferences"].update({
-                "devtools.browsertoolbox.panel": "jsdebugger",
-                "devtools.debugger.remote-enabled": True,
-                "devtools.chrome.enabled": True,
-                "devtools.debugger.prompt-connection": False,
-                "marionette.debugging.clicktostart": True,
-            })
+            args["preferences"].update(
+                {
+                    "devtools.browsertoolbox.panel": "jsdebugger",
+                    "devtools.chrome.enabled": True,
+                    "devtools.debugger.prompt-connection": False,
+                    "devtools.debugger.remote-enabled": True,
+                }
+            )
 
         if self.addons:
             args["addons"] = self.addons
@@ -278,8 +330,10 @@ class GeckoInstance(object):
             instance_class = apps[app]
         except (IOError, KeyError):
             exc, val, tb = sys.exc_info()
-            msg = 'Application "{0}" unknown (should be one of {1})'
-            reraise(NotImplementedError, msg.format(app, apps.keys()), tb)
+            msg = 'Application "{0}" unknown (should be one of {1})'.format(
+                app, list(apps.keys())
+            )
+            reraise(NotImplementedError, NotImplementedError(msg), tb)
 
         return instance_class(*args, **kwargs)
 
@@ -291,25 +345,36 @@ class GeckoInstance(object):
     def _get_runner_args(self):
         process_args = {
             "processOutputLine": [NullOutput()],
+            "universal_newlines": True,
         }
 
         if self.gecko_log == "-":
-            process_args["stream"] = sys.stdout
+            if hasattr(sys.stdout, "buffer"):
+                process_args["stream"] = codecs.getwriter("utf-8")(sys.stdout.buffer)
+            else:
+                process_args["stream"] = codecs.getwriter("utf-8")(sys.stdout)
         else:
             process_args["logfile"] = self.gecko_log
 
         env = os.environ.copy()
 
+        # Store all required preferences for tests which need to create clean profiles.
+        required_prefs_keys = list(self.required_prefs.keys())
+        env["MOZ_MARIONETTE_REQUIRED_PREFS"] = json.dumps(required_prefs_keys)
+
         if self.headless:
             env["MOZ_HEADLESS"] = "1"
             env["DISPLAY"] = "77"  # Set a fake display.
 
         # environment variables needed for crashreporting
         # https://developer.mozilla.org/docs/Environment_variables_affecting_crash_reporting
-        env.update({"MOZ_CRASHREPORTER": "1",
-                    "MOZ_CRASHREPORTER_NO_REPORT": "1",
-                    "MOZ_CRASHREPORTER_SHUTDOWN": "1",
-                    })
+        env.update(
+            {
+                "MOZ_CRASHREPORTER": "1",
+                "MOZ_CRASHREPORTER_NO_REPORT": "1",
+                "MOZ_CRASHREPORTER_SHUTDOWN": "1",
+            }
+        )
 
         return {
             "binary": self.binary,
@@ -317,7 +382,7 @@ class GeckoInstance(object):
             "cmdargs": ["-no-remote", "-marionette"] + self.app_args,
             "env": env,
             "symbols_path": self.symbols_path,
-            "process_args": process_args
+            "process_args": process_args,
         }
 
     def close(self, clean=False):
@@ -357,37 +422,38 @@ class GeckoInstance(object):
 
 class FennecInstance(GeckoInstance):
     fennec_prefs = {
-        # Enable output of dump()
+        # Enable output for dump() and chrome console API
         "browser.dom.window.dump.enabled": True,
-
-        # Disable Android snippets
-        "browser.snippets.enabled": False,
-        "browser.snippets.syncPromo.enabled": False,
-        "browser.snippets.firstrunHomepage.enabled": False,
-
-        # Disable safebrowsing components
-        "browser.safebrowsing.blockedURIs.enabled": False,
-        "browser.safebrowsing.downloads.enabled": False,
-        "browser.safebrowsing.passwords.enabled": False,
-        "browser.safebrowsing.malware.enabled": False,
-        "browser.safebrowsing.phishing.enabled": False,
-
+        "devtools.console.stdout.chrome": True,
+        # Disable safe browsing / tracking protection updates
+        "browser.safebrowsing.update.enabled": False,
         # Do not restore the last open set of tabs if the browser has crashed
         "browser.sessionstore.resume_from_crash": False,
-
         # Disable e10s by default
         "browser.tabs.remote.autostart": False,
-
         # Do not allow background tabs to be zombified, otherwise for tests that
         # open additional tabs, the test harness tab itself might get unloaded
         "browser.tabs.disableBackgroundZombification": True,
     }
 
-    def __init__(self, emulator_binary=None, avd_home=None, avd=None,
-                 adb_path=None, serial=None, connect_to_running_emulator=False,
-                 package_name=None, *args, **kwargs):
+    def __init__(
+        self,
+        emulator_binary=None,
+        avd_home=None,
+        avd=None,
+        adb_path=None,
+        serial=None,
+        connect_to_running_emulator=False,
+        package_name=None,
+        env=None,
+        *args,
+        **kwargs
+    ):
+        required_prefs = deepcopy(FennecInstance.fennec_prefs)
+        required_prefs.update(kwargs.get("prefs", {}))
+
         super(FennecInstance, self).__init__(*args, **kwargs)
-        self.required_prefs.update(FennecInstance.fennec_prefs)
+        self.required_prefs.update(required_prefs)
 
         self.runner_class = FennecEmulatorRunner
         # runner args
@@ -396,6 +462,7 @@ class FennecInstance(GeckoInstance):
         self.avd_home = avd_home
         self.adb_path = adb_path
         self.avd = avd
+        self.env = env
         self.serial = serial
         self.connect_to_running_emulator = connect_to_running_emulator
 
@@ -420,36 +487,34 @@ class FennecInstance(GeckoInstance):
             if self.connect_to_running_emulator:
                 self.runner.device.connect()
             self.runner.start()
-        except Exception as e:
-            exc, val, tb = sys.exc_info()
-            message = "Error possibly due to runner or device args: {}"
-            reraise(exc, message.format(e.message), tb)
-        # gecko_log comes from logcat when running with device/emulator
-        logcat_args = {
-            "filterspec": "Gecko",
-            "serial": self.runner.device.dm._deviceSerial
-        }
-        if self.gecko_log == "-":
-            logcat_args["stream"] = sys.stdout
-        else:
-            logcat_args["logfile"] = self.gecko_log
-        self.runner.device.start_logcat(**logcat_args)
+        except Exception:
+            exc_cls, exc, tb = sys.exc_info()
+            reraise(
+                exc_cls,
+                exc_cls("Error possibly due to runner or device args: {}".format(exc)),
+                tb,
+            )
 
-        # forward marionette port (localhost:2828)
-        self.runner.device.dm.forward(
+        # forward marionette port
+        self.runner.device.device.forward(
             local="tcp:{}".format(self.marionette_port),
-            remote="tcp:{}".format(self.marionette_port))
+            remote="tcp:{}".format(self.marionette_port),
+        )
 
     def _get_runner_args(self):
         process_args = {
             "processOutputLine": [NullOutput()],
+            "universal_newlines": True,
         }
 
+        env = {} if self.env is None else self.env.copy()
+
         runner_args = {
             "app": self.package_name,
             "avd_home": self.avd_home,
             "adb_path": self.adb_path,
             "binary": self.emulator_binary,
+            "env": env,
             "profile": self.profile,
             "cmdargs": ["-marionette"] + self.app_args,
             "symbols_path": self.symbols_path,
@@ -474,89 +539,107 @@ class FennecInstance(GeckoInstance):
         super(FennecInstance, self).close(clean)
         if clean and self.runner and self.runner.device.connected:
             try:
-                self.runner.device.dm.remove_forward(
-                    "tcp:{}".format(self.marionette_port))
+                self.runner.device.device.remove_forwards(
+                    "tcp:{}".format(self.marionette_port)
+                )
                 self.unresponsive_count = 0
-            except DMError:
+            except Exception:
                 self.unresponsive_count += 1
                 traceback.print_exception(*sys.exc_info())
 
 
 class DesktopInstance(GeckoInstance):
     desktop_prefs = {
-        # Disable application updates
-        "app.update.enabled": False,
-
-        # Enable output of dump()
+        # Disable Firefox old build background check
+        "app.update.checkInstallTime": False,
+        # Disable automatically upgrading Firefox
+        #
+        # Note: Possible update tests could reset or flip the value to allow
+        # updates to be downloaded and applied.
+        "app.update.disabledForTesting": True,
+        # !!! For backward compatibility up to Firefox 64. Only remove
+        # when this Firefox version is no longer supported by the client !!!
+        "app.update.auto": False,
+        # Don't show the content blocking introduction panel
+        # We use a larger number than the default 22 to have some buffer
+        # This can be removed once Firefox 69 and 68 ESR and are no longer supported.
+        "browser.contentblocking.introCount": 99,
+        # Enable output for dump() and chrome console API
         "browser.dom.window.dump.enabled": True,
-
+        "devtools.console.stdout.chrome": True,
         # Indicate that the download panel has been shown once so that whichever
         # download test runs first doesn"t show the popup inconsistently
         "browser.download.panel.shown": True,
-
         # Do not show the EULA notification which can interfer with tests
         "browser.EULA.override": True,
-
+        # Disable Activity Stream telemetry pings
+        "browser.newtabpage.activity-stream.telemetry": False,
         # Always display a blank page
         "browser.newtabpage.enabled": False,
-
         # Background thumbnails in particular cause grief, and disabling thumbnails
         # in general can"t hurt - we re-enable them when tests need them
         "browser.pagethumbnails.capturing_disabled": True,
-
-        # Disable safebrowsing components
-        "browser.safebrowsing.blockedURIs.enabled": False,
-        "browser.safebrowsing.downloads.enabled": False,
-        "browser.safebrowsing.passwords.enabled": False,
-        "browser.safebrowsing.malware.enabled": False,
-        "browser.safebrowsing.phishing.enabled": False,
-
+        # Disable safe browsing / tracking protection updates
+        "browser.safebrowsing.update.enabled": False,
         # Disable updates to search engines
         "browser.search.update": False,
-
         # Do not restore the last open set of tabs if the browser has crashed
         "browser.sessionstore.resume_from_crash": False,
-
         # Don't check for the default web browser during startup
         "browser.shell.checkDefaultBrowser": False,
-
-        # Disable e10s by default
-        "browser.tabs.remote.autostart": False,
-
+        # Disable session restore infobar
+        "browser.startup.couldRestoreSession.count": -1,
         # Needed for branded builds to prevent opening a second tab on startup
         "browser.startup.homepage_override.mstone": "ignore",
         # Start with a blank page by default
         "browser.startup.page": 0,
-
-        # Disable browser animations
-        "toolkit.cosmeticAnimations.enabled": False,
-
-        # Do not warn on exit when multiple tabs are open
+        # Don't unload tabs when available memory is running low
+        "browser.tabs.unloadOnLowMemory": False,
+        # Do not warn when closing all open tabs
         "browser.tabs.warnOnClose": False,
         # Do not warn when closing all other open tabs
         "browser.tabs.warnOnCloseOtherTabs": False,
         # Do not warn when multiple tabs will be opened
         "browser.tabs.warnOnOpen": False,
-
+        # Don't show the Bookmarks Toolbar on any tab (the above pref that
+        # disables the New Tab Page ends up showing the toolbar on about:blank).
+        "browser.toolbars.bookmarks.visibility": "never",
         # Disable the UI tour
         "browser.uitour.enabled": False,
-
+        # Turn off Merino suggestions in the location bar so as not to trigger network
+        # connections.
+        "browser.urlbar.merino.endpointURL": "",
         # Turn off search suggestions in the location bar so as not to trigger network
         # connections.
         "browser.urlbar.suggest.searches": False,
-
-        # Turn off the location bar search suggestions opt-in.  It interferes with
-        # tests that don't expect it to be there.
-        "browser.urlbar.userMadeSearchSuggestionsChoice": True,
-
+        # Don't warn when exiting the browser
+        "browser.warnOnQuit": False,
         # Disable first-run welcome page
         "startup.homepage_welcome_url": "about:blank",
         "startup.homepage_welcome_url.additional": "",
     }
 
     def __init__(self, *args, **kwargs):
+        required_prefs = deepcopy(DesktopInstance.desktop_prefs)
+        required_prefs.update(kwargs.get("prefs", {}))
+
         super(DesktopInstance, self).__init__(*args, **kwargs)
-        self.required_prefs.update(DesktopInstance.desktop_prefs)
+        self.required_prefs.update(required_prefs)
+
+
+class ThunderbirdInstance(GeckoInstance):
+    def __init__(self, *args, **kwargs):
+        super(ThunderbirdInstance, self).__init__(*args, **kwargs)
+        try:
+            # Copied alongside in the test archive
+            from .thunderbirdinstance import thunderbird_prefs
+        except ImportError:
+            try:
+                # Coming from source tree through virtualenv
+                from thunderbirdinstance import thunderbird_prefs
+            except ImportError:
+                thunderbird_prefs = {}
+        self.required_prefs.update(thunderbird_prefs)
 
 
 class NullOutput(object):
@@ -565,11 +648,13 @@ class NullOutput(object):
 
 
 apps = {
-    'fennec': FennecInstance,
-    'fxdesktop': DesktopInstance,
+    "fennec": FennecInstance,
+    "fxdesktop": DesktopInstance,
+    "thunderbird": ThunderbirdInstance,
 }
 
 app_ids = {
-    '{aa3c5121-dab2-40e2-81ca-7ea25febc110}': 'fennec',
-    '{ec8030f7-c20a-464f-9b0e-13a3a9e97384}': 'fxdesktop',
+    "{aa3c5121-dab2-40e2-81ca-7ea25febc110}": "fennec",
+    "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}": "fxdesktop",
+    "{3550f703-e582-4d05-9a08-453d09bdfdc6}": "thunderbird",
 }
